#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# D-Case Weaver 1.1.1
#
# Copyright (c) 2012-2014 JST DEOS R&D Center
#
# Licensed under The MIT License
#

import base64
import cgi
import StringIO
import syslog
import tarfile
import time

def debug (msg):
    syslog.syslog(syslog.LOG_DEBUG, msg)

class BadRequestException (Exception):
    pass

class SaveAsImage (object):
    def __init__(self):
        pass

    def __call__(self, environ, start_response):
        headers = []
        status = "200 OK"
        output = ""

        try:
            q = dict(
                cgi.parse_qsl(
                    environ["wsgi.input"].read(
                        int(environ.get("CONTENT_LENGTH", 0)))))

            if "BaseName" not in q or "Length" not in q:
                raise BadRequestException()

            basename = q["BaseName"]
            length = int(q["Length"])

            if length == 1:
                if "Data0" not in q:
                    raise BadRequestException()
                data = base64.b64decode(q["Data0"])
                headers, output = self.singleMode(data, basename)

            elif 1 < length:
                dlist = []
                for i in range(length):
                    key = "Data%d" % i
                    if key not in q:
                        raise BadRequestException()
                    dlist.append(base64.b64decode(q[key]))
                headers, output = self.multiMode(dlist, basename)

            else:
                raise BadRequestException()


        except BadRequestException:
            status = "400 Bad Request"
            headers.append(("Content-type", "text/plain"))
            output = "400 Bad Request"


        headers.append(('Cache-Control','no-cache'))

        start_response(status, headers)

        return output


    def singleMode (self, data, basename):
        output = data

        headers = [
            ("Content-type", "image/png"),
            ("Content-Disposition", "attachment; filename=%s.png" % basename),
            ("Content-Length", str(len(output)))]

        return headers, output


    def multiMode (self, dlist, basename):
        f = StringIO.StringIO()

        tar = tarfile.open(mode="w:gz", fileobj=f)
        #tar = tarfile.open(mode="w", fileobj=f)

        i = 0
        for data in dlist:
            sio = StringIO.StringIO(data)

            name = "%s(%d).png" % (basename, i)
            debug(name)

            info = tarfile.TarInfo(name)
            info.size = sio.len
            info.mtime = time.time()

            tar.addfile(info, sio)

            i = i + 1

        tar.close()

        output = f.getvalue()

        headers = [
            ("Content-type", "apprication/x-tar"),
            ("Content-Disposition", "attachment; filename=%s.tgz" % basename),
            ("Content-Length", str(len(output)))]

        return headers, output


class SaveAsImageOrg (object):
    def __init__(self):
        pass

    def __call__(self, environ, start_response):
        headers = [('Cache-Control','no-cache')]
        status = "200 OK"
        output = ""

        try:
            q = dict(
                cgi.parse_qsl(
                    environ["wsgi.input"].read(
                        int(environ.get("CONTENT_LENGTH", 0)))))

            if "Data" not in q or "Name" not in q:
                raise BadRequestException()

            output = base64.b64decode(q["Data"])

            headers.append(("Content-type", "image/png"))
            headers.append(
                ("Content-Disposition", "attachment; filename=%s" % q["Name"]))
            headers.append(("Content-Length", str(len(output))))


        except BadRequestException:
            status = "400 Bad Request"
            headers.append(("Content-type", "text/plain"))
            output = "400 Bad Request"

        start_response(status, headers)
        return output

application = SaveAsImage()
