/*
 * D-Case Weaver 1.1.1
 *
 * Copyright (c) 2012-2014 JST DEOS R&D Center
 *
 * Licensed under The MIT License
 */
(
  function (window) {
    var deosc = {};

    function Parser (ns) {
      this.setNamespace(ns);
    }

    $.extend(
      Parser.prototype,
      {
        setNamespace: function (ns) {
          this.ns = ns;
        },
        getChildren: function (el, localName) {
          var self = this;
          return $(el).children().filter(
            function () {
              if (this.namespaceURI != self.ns) return false;
              if (this.localName != localName) return false;
              return true;
            }
          );
        }
      });

    deosc = {
      Namespace: {
        xmlns: "http://www.w3.org/2000/xmlns/",
        DCASE201006: "http://www.dependable-os.net/2010/06/dcase",
        DCASE201311: "http://www.dependable-os.net/2013/11/dcase",
        DRE: "http://www.dependable-os.net/dre",
        DSCRIPT: "http://www.dependable-os.net/d-script"
      },
      util: {
        Parser: Parser
      },
      path: {
        basename: function (path) {
          return path.replace(/.*\//, "");
        },
        dirname: function (path) {
          var dirname = path.replace(/\/[^/]*$/, "");
          return (dirname.length == 0) ? "/" : dirname;
        }
      }
    };

    window.deosc = deosc;
  }
) (window);
(
function (window) {
  var Rect = function () {
    this.left = 0;
    this.top = 0;
    this.right = 0;
    this.bottom = 0;
    this.set.apply(this, arguments);
  };
  $.extend(
    Rect.prototype,
    {
      set: function () {
        if (arguments.length == 4) {
          this.left = arguments[0];
          this.top = arguments[1];
          this.right = arguments[2];
          this.bottom = arguments[3];
        } else if (arguments.length == 1
                   && arguments[0] instanceof Rect) {
          this.left = arguments[0].left;
          this.top = arguments[0].top;
          this.right = arguments[0].right;
          this.bottom = arguments[0].bottom;
        }
      },
      width: function () {
        return this.right - this.left;
      },
      height: function () {
        return this.bottom - this.top;
      },
      centerX: function () {
        return this.left + this.width() / 2;
      },
      centerY: function () {
        return this.top + this.height() / 2;
      },
      contains: function () {
        var l, t, r, b;
        if (arguments.length == 2) {
          return (this.left <= arguments[0]
                  && arguments[0] <= this.right
                  && this.top <= arguments[1]
                  && arguments[1] <= this.bottom);
        } else if (arguments.length == 4) {
          l = arguments[0];
          t = arguments[1];
          r = arguments[2];
          b = arguments[3];
        } else if (arguments.length == 1
                   && arguments[0] instanceof Rect) {
          l = arguments[0].left;
          t = arguments[0].top;
          r = arguments[0].right;
          b = arguments[0].bottom;
        } else {
          return false;
        }
        return this.contains(l, t) && this.contains(l, b)
          && this.contains(r, t) && this.contains(r, b);
      },
      intersects: function () {
        var l, t, r, b;
        if (arguments.length == 4) {
          l = arguments[0];
          t = arguments[1];
          r = arguments[2];
          b = arguments[3];
        } else if (arguments.length == 1
                   && arguments[0] instanceof Rect) {
          l = arguments[0].left;
          t = arguments[0].top;
          r = arguments[0].right;
          b = arguments[0].bottom;
        } else {
          return false;
        }
        return this.left < r && l < this.right
          && this.top < b && t < this.bottom;
      },
      intersect: function () {
        if (!this.intersects.apply(this, arguments)) return false;

        var rect;
        if (arguments.length == 4) {
          rect = new Rect(arguments[0], arguments[1],
                          arguments[2], arguments[3]);
        } else if (arguments.length == 1
                   && arguments[0] instanceof Rect) {
          rect = arguments[0];
        } else {
          return false;
        }

        this.left = Math.max(this.left, rect.left);
        this.top = Math.max(this.top, rect.top);
        this.right = Math.min(this.right, rect.right);
        this.bottom = Math.min(this.bottom, rect.bottom);

        return true;
      },
      union: function () {
        var rect;
        if (arguments.length == 2) {
          rect = new Rect(arguments[0], arguments[1],
                          arguments[0], arguments[1]);
        } else if (arguments.length == 4) {
          rect = new Rect(arguments[0], arguments[1],
                          arguments[2], arguments[3]);
        } else if (arguments.length == 1
                   && arguments[0] instanceof Rect) {
          rect = arguments[0];
        } else {
          return;
        }
        this.left = Math.min(this.left, rect.left);
        this.top = Math.min(this.top, rect.top);
        this.right = Math.max(this.right, rect.right);
        this.bottom = Math.max(this.bottom, rect.bottom);
      },
      offset: function (x, y) {
        this.left += x;
        this.right += x;
        this.top += y;
        this.bottom += y;
      },
      toString: function () {
        return "[" + this.left + ", " + this.top + ", "
          + this.right + ", " + this.bottom + "]";
      }
    }
  );

  if (!window.deosc) window.deosc = {};
  window.deosc.Rect = Rect;
}
)(window);
function draggable(options) {
  var self = this;
  var anchor;
  var html = $("html")[0];
  var dragging = false;
  var mouse = {
    position: {
      x: 0, y: 0
    }
  };
  var position = {
    x: 0, y: 0
  };

  if ($(self).css("position") != "absolute"
      && $(self).css("position") != "relative") {
    $(self).css({position: "relative"});
  }

  anchor = (!options || !options.anchor) ? this : options.anchor;

  var mousedown = function (e) {
    dragging = true;
    mouse.position.x = e.pageX;
    mouse.position.y = e.pageY;
    var rect = self.getBoundingClientRect();
    position.x = rect.left;
    position.y = rect.top;

    $(html).mouseup(mouseup);
    $(html).mousemove(mousemove);
    $(html).mouseout(mouseout);

    return false;
  };

  var mousemove = function (e) {
    if (dragging) {
      var dx = e.pageX - mouse.position.x;
      var dy = e.pageY - mouse.position.y;
      var x = position.x + dx;
      var y = position.y + dy;

      if (options && options.bound) {
        if(options.bound == "outer") {
          var w = $(self).width();
          var h = $(self).height();
          var r = x + w;
          var b = y + h;
          var pw = $(self.parentNode).width();
          var ph = $(self.parentNode).height();
          var rect = self.parentNode.getBoundingClientRect();
          var pl = rect.left;
          var pt = rect.top;
          var pr = pl + pw;
          var pb = pt + ph;

          if (pl < x) x = pl;
          if (pt < y) y = pt;
          if (r < pr) x = pr - w;
          if (b < pb) y = pb - h;
          $(self).css({left: x - pl, top: y - pt});
        } else if (options.bound == "inner") {
          //TODO:
        }
      } else {
        $(self).css({left: x, top: y});
      }
    }
    return true;
  };

  var mouseup = function (e) {
    dragging = false;
    $(html).unbind("mouseup", mouseup);
    $(html).unbind("mouseout", mouseout);
    $(html).unbind("mousemove", mousemove);
  };

  var mouseout = function (e) {
    if (e.target != html) return;
    dragging = false;
    $(html).unbind("mouseup", mouseup);
    $(html).unbind("mouseout", mouseout);
    $(html).unbind("mousemove", mousemove);
  };

  $(anchor).mousedown(mousedown);
}

function create_table(list) {
  var table = $("<table>")[0];
  var ilength = list.length;
  for (var i = 0; i < ilength; i++) {
    var tr = $("<tr>")[0];
    $(tr).addClass("tr-" + i);
    if (i == ilength - 1) $(tr).addClass("tr-end");
    (i % 2) ? $(tr).addClass("tr-even") : $(tr).addClass("tr-odd");

    var jlength = list[i].length;
    for (var j = 0; j < jlength; j++) {
      var data = list[i][j];
      var td = $("<td>")[0];
      $(td).addClass("td-" + j);
      if (j == jlength - 1) $(td).addClass("td-end");

      if (data instanceof HTMLElement) {
        $(td).append(data);
      } else if (!(data instanceof Object)) { //number or string
        $(td).text(data);
      }
      $(tr).append(td);
    }
    $(table).append(tr);
  }
  return table;
}

function post(url, data) {
  var form = $("<form method='POST'>")
    .attr("action", url)[0];
  for(var key in data) {
    var input = $("<input type='hidden'>").attr('name', key).attr('value', data[key])[0];
    $(form).append(input);
  }

  $(form).hide();
  $("body").append(form);
  form.submit();
  $(form).remove();
}

function download(obj) {
  post("cgi-bin/download.cgi", obj);
}

function isString(obj) {
  return (obj instanceof String || typeof obj == "string");
}

function isNumber(obj) {
  return (obj instanceof Number || typeof obj == "number");
}

function Color() {
  this.r = 0;
  this.g = 0;
  this.b = 0;

  if (arguments.length == 1) {
    var color = arguments[0];
    if (isString(color)) {
      if (color[0] == "#") {
        if (color.length == 4) {
          this.r = 255 * parseInt(color[1], 16) / 15;
          this.g = 255 * parseInt(color[2], 16) / 15;
          this.b = 255 * parseInt(color[3], 16) / 15;
        } else if (color.length == 7) {
          this.r = parseInt(color.substring(1, 3), 16);
          this.g = parseInt(color.substring(3, 5), 16);
          this.b = parseInt(color.substring(5, 7), 16);
        }
      }
    }
  } else if (arguments.length == 3) {
    if(isNumber(arguments[0])) this.r = this.bound(arguments[0]);
    if(isNumber(arguments[1])) this.g = this.bound(arguments[1]);
    if(isNumber(arguments[2])) this.b = this.bound(arguments[2]);
  }
}

Color.prototype.toString = function () {
  var str_r = this.r.toString(16); if (str_r.length == 1) str_r = "0" + str_r;
  var str_g = this.g.toString(16); if (str_g.length == 1) str_g = "0" + str_g;
  var str_b = this.b.toString(16); if (str_b.length == 1) str_b = "0" + str_b;
  return "#" + str_r + str_g + str_b;
};

Color.prototype.red = function (r) {
  if (r) this.r = r;
  return this.r;
};

Color.prototype.green = function (g) {
  if (g) this.g = g;
  return this.g;
};

Color.prototype.blue = function (b) {
  if (b) this.b = b;
  return this.b;
};

Color.prototype.bound = function (c) {
  c = Math.floor(c);
  if (255 < c) return 255;
  if (c < 0) return 0;
  return c;
};

Color.prototype.multi = function (c) {
  var r, g, b;
  if (c instanceof Color) {
    r = this.bound(this.r * c.red());
    g = this.bound(this.g * c.green());
    b = this.bound(this.b * c.blue());
  } else if (isNumber(c)) {
    r = this.bound(this.r * c);
    g = this.bound(this.g * c);
    b = this.bound(this.b * c);
  } else {
    r = this.r;
    g = this.g;
    b = this.b;
  }
  return new Color(r, g, b);
};

Color.prototype.add = function (c) {
  var r, g, b;
  if (c instanceof Color) {
    r = this.bound(this.r + c.red());
    g = this.bound(this.g + c.green());
    b = this.bound(this.b + c.blue());
  } else if (isNumber(c)) {
    r = this.bound(this.r + c);
    g = this.bound(this.g + c);
    b = this.bound(this.b + c);
  } else {
    r = this.r;
    g = this.g;
    b = this.b;
  }
  return new Color(r, g, b);
};
(
function (window) {
  var requestAnimationFrame = (
    function() {
      return window.requestAnimationFrame ||
        window.webkitRequestAnimationFrame ||
        window.mozRequestAnimationFrame ||
        window.msRequestAnimationFrame ||
        window.oRequestAnimationFrame ||
        function(f) { return window.setTimeout(f, 1000 / 60); };
    }());

  var ViewRoot = function () {};
  $.extend(
    ViewRoot.prototype,
    {
      init: function (parent) {
        var self = this;
        this.width = 0;
        this.height = 0;
        this.childView = null;
        this.requestTraversalsTimerID = null;
        this.wrapContent = false;

        this.canvas = $("<canvas>")[0];

        this.parent = parent ? parent : $("body")[0];
        $(this.parent).append(this.canvas);

        this.eventQueue = [];

        function eventFunc (e) {
          if (e.button == 2) {
            if (e.type != "contextmenu") return;
            e.preventDefault();
          }

          var rect = self.canvas.getBoundingClientRect();
          var event = {
            event: e,
            type: e.type,
            x: e.pageX - rect.left,
            y: e.pageY - rect.top,
            view: null
          };

          var ret = self.childView.dispatchMouseEvent(event);

          if (!event.cursorStyle) event.cursorStyle = "auto";

          if (self.cursorStyle != event.cursorStyle) {
            self.cursorStyle = event.cursorStyle;
            $(self.canvas).css({cursor: self.cursorStyle});
          }
        }

        function requestEventFunc (e) {
          if (self.eventFuncTimerID) return;

          self.eventFuncTimerID = requestAnimationFrame(
            function () {
              if (self.eventQueue.length == 0) return;
              var e = self.eventQueue.shift();
              eventFunc(e);
              self.eventFuncTimerID = null;
              if (0 < self.eventQueue.length) requestEventFunc();
            }
          );
        }

        $(this.canvas).bind(
          "mousedown mouseup mouseout mousemove contextmenu",
          function (e) {
            var i;

            e.preventDefault();

            for (i = 0; i < self.eventQueue.length; i++) {
              if (e.type == self.eventQueue[i].type) return;
            }

            self.eventQueue.push(e);

            requestEventFunc();
          }
        );

        return this;
      },
      setView: function (child) {
        this.childView = child;
        child.setParent(this);
        this.invalidate();
      },
      invalidate: function () {
        this.requestTraversals();
      },
      requestTraversals: function () {
        if (this.requestTraversalsTimerID != null) return;

        this.requestTraversalsTimerID = requestAnimationFrame(
          function (self) {
            return function () {
              self.performTraversals();
              self.requestTraversalsTimerID = null;
            };
          } (this));
      },
      performTraversals: function () {
        var child = this.childView;
        var date, t0, t1;

        child.measure($(this.parent).width(), $(this.parent).height());

        if (this.wrapContent) {
          this.width = child.getMeasuredWidth();
          this.height = child.getMeasuredHeight();
        } else {
          this.width = $(this.parent).width();
          this.height = $(this.parent).height();
        }

        child.layout(0, 0, child.getMeasuredWidth(), child.getMeasuredHeight());

        $(this.canvas).attr({width: this.width, height: this.height});
        var ctx = this.canvas.getContext("2d");
        ctx.clearRect(0, 0, this.width, this.height);

        child.setDisplayArea(new deosc.Rect(0, 0, this.width, this.height));

        child.draw(ctx);
      },
      getCanvas: function () {
        return this.canvas;
      },
      destroy: function () {
        View.prototype.destroy.apply(this);
        $(this.canvas).remove();
      },
      mousemove: function (x, y) {
        var rect = this.canvas.getBoundingClientRect();
        this.childView.mousemove(x - rect.left, y - rect.top);
      },
      mouseout: function (x, y) {
        var rect = this.canvas.getBoundingClientRect();
        this.childView.mouseout(x - rect.left, y - rect.top);
      },
      mousedown: function (x, y) {
        var rect = this.canvas.getBoundingClientRect();
        this.childView.mousedown(x - rect.left, y - rect.top);
      },
      mouseup: function (x, y) {
        var rect = this.canvas.getBoundingClientRect();
        this.childView.mouseup(x - rect.left, y - rect.top);
      },
      getBoundingClientRect: function () {
        var rect = this.canvas.getBoundingClientRect();
        return new deosc.Rect(rect.left, rect.top, rect.right, rect.bottom);
      }
    }
  );

  if (!window.deosc.canvas) window.deosc.canvas = {};
  window.deosc.canvas.ViewRoot = ViewRoot;
  window.deosc.canvas.createViewRoot = function (elem) {
    return new ViewRoot().init(elem);
  };
}
)(window);
(
function (window) {
  var View = function () {};
  View.count = 0;
  View.LayoutMode = {
    MATCH_PARENT: 1,
    WRAP_CONTENT: 2
  };
  View.prototype = {
    constructor: View,
    init: function () {
      this.left = 0;
      this.top = 0;
      this.right = 0;
      this.bottom = 0;
      this.layoutParams = {
        x: 0, y: 0, width: 0, height: 0,
        mode: View.LayoutMode.WRAP_CONTENT
      };
      this.backgroundColor = null;
      this.parentView = null;
      this.invalidFlag = true;
      this.isMousedown = false;
      this.isMouseover = false;
      this.mouse = "out";
      this.isDraggable = false;
      this.isClickable = false;
      this.scalse = 1;
      this.measured = {width: 0, height: 0};
      this.padding = {left:0 , top: 0, right: 0, bottom: 0};
      this.viewID = View.count++;
      this.cursorStyles = {};
      return this;
    },
    destroy: function () {
      this.parentView.removeView(this);
    },
    width: function () {
      return this.right - this.left;
    },
    height: function () {
      return this.bottom - this.top;
    },
    setPosition: function (x, y) {
      this.layoutParams.x = x;
      this.layoutParams.y = y;
      this.invalid();
    },
    setSize: function (w, h) {
      this.layoutParams.width = w;
      this.layoutParams.height = h;
      this.invalid();
    },
    setLayoutParams: function (params) {
      this.layoutParams = {x: 0, y: 0, width: 0, height: 0};
      $.extend(this.layoutParams, params);
      this.invalid();
    },
    getLayoutParams: function () {
      return this.layoutParams;
    },
    setPadding: function (l, t, r, b) {
      this.padding = {left:l , top: t, right: r, bottom: b};
    },
    draw: function (ctx) {
      var i;

      if (this.isHidden) return;

      ctx.save();

      ctx.save();
      if (this.backgroundColor) {
        ctx.fillStyle = this.backgroundColor;
        ctx.fillRect(0, 0, this.width(), this.height());
      }
      ctx.restore();

      ctx.save();
      this.onDraw(ctx);
      ctx.restore();

      if (this.debug) {
        ctx.save();
        ctx.strokeStyle = "rgb(255, 0, 0)";
        ctx.strokeRect(this.left, this.top, this.width(), this.height());
        ctx.restore();
      }

      this.drawChildren(ctx);

      ctx.restore();
    },
    drawChildren: function (ctx) {
    },
    onDraw: function (ctx) {
    },
    measure: function (width, height) {
      if (this.isHidden) {
        this.measured.width = 0;
        this.measured.height = 0;
        return;
      }
      this.onMeasure(width, height);
    },
    onMeasure: function (width, height) {
    },
    getMeasuredWidth: function () {
      return this.measured.width;
    },
    getMeasuredHeight: function () {
      return this.measured.height;
    },
    layout: function (l, t, r, b) {
      if (this.isHidden) return;
      this.setFrame(l, t, r, b);
      this.onLayout(l, t, r, b);
    },
    onLayout: function (l, t, r, b) {
    },
    setFrame: function (l, t, r, b) {
      this.left = l;
      this.top = t;
      this.right = r;
      this.bottom = b;
    },
    setBackgroundColor: function (color) {
      this.backgroundColor = color;
      this.invalidate();
    },
    setScale: function (scale) {
      this.scale = scale;
    },
    invalid: function () {
      this.invalidate();
    },
    invalidate: function () {
      this.invalidFlag = true;
      if (this.parentView) this.parentView.invalidate();
    },
    setParent: function(parent) {
      this.parentView = parent;
    },
    getViewRoot: function () {
      if (this.parentView == null) {
        return null;
      } else if (this.parentView instanceof deosc.canvas.ViewRoot) {
        return this.parentView;
      } else {
        return this.parentView.getViewRoot();
      }
    },
    show: function () {
      this.isHidden = false;
      this.invalidate();
    },
    hide: function () {
      this.isHidden = true;
      this.invalidate();
    },
    setDisplayArea: function (rect) {
      this.display = new deosc.Rect(rect);
    },
    getCanvas: function () {
      return this.parentView.getCanvas();
    },
    isHit: function (x, y) {
      var rect = new deosc.Rect(this.left, this.top, this.right, this.bottom);
      return rect.contains(x, y);
    },
    dispatchMouseEvent: function (e) {
      return this.onMouseEvent(e);
    },
    onMouseEvent: function (e) {
      var ret = false;

      if (e.type == "mousedown") {
        if (this.isHit(e.x, e.y)) {
          this.isMousedown = true;
          if (this.isDraggable) {
            this.orgX = this.layoutParams.x;
            this.orgY = this.layoutParams.y;
            this.dragX = e.x;
            this.dragY = e.y;
          }

          ret = this.onMousedown(e.x, e.y, e.event);

          if (this.isClickable || this.isDraggable)
            ret = true;
        }
      } else if (e.type == "mouseup") {
        if (this.isHit(e.x, e.y)) {
          if (this.isMousedown) {
            ret = this.onMouseup(e.x, e.y, e.event);

            if (this.isClickable) {
              ret = this.onClick(e.x, e.y, e.event);
            }

            this.clearMouseStatus();

            if (this.isClickable || this.isDraggable)
              ret = true;
          }
        }
        if (this.isMousedown && this.isDraggable) {
          ret = this.onMouseup(e.x, e.y);
          this.clearMouseStatus();
          ret = true;
        }
      } else if (e.type == "mouseout") {
        ret = this.onMouseout();
        this.clearMouseStatus();
      } else if (e.type == "mousemove") {
        if (this.isMousedown && this.isDraggable) {
          e.isDrag = true;
        }
        if (this.isHit(e.x, e.y)) {
          if (this.isMouseover) {
            ret = this.onMousemove(e.x, e.y, e.event);
          } else {
            ret = this.onMouseover(e.x, e.y, e.event);
            this.isMouseover = true;
          }
        } else {
          if (this.isMouseover) {
            ret = this.onMouseout(e.x, e.y, e.event);
            this.isMouseover = false;
          }
        }
        if (this.isMousedown && this.isDraggable) {
          ret = this.onDrag(e.x, e.y, e.event);
        }

      } else if (e.type == "contextmenu") {
        if (this.isHit(e.x, e.y)) {
          ret = this.onContextMenu(e.x, e.y, e.event);
        }
      }

      if (!e.cursorStyle) {
        if (this.isMousedown && this.isDraggable && this.cursorStyles.drag) {
          e.cursorStyle = this.cursorStyles.drag;
        } else if (!e.isDrag && this.isMouseover && this.cursorStyles.hover) {
          e.cursorStyle = this.cursorStyles.hover;
        }
      }

      if (this.isMousedown && this.isDraggable) {
        e.isDrag = true;
      }

      return ret;
    },
    clearMouseStatus: function () {
      this.isMousedown = false;
      this.isMouseover = false;
    },
    setOnClickListener: function (l) {
      this.onClickListener = l;
    },
    setOnDragListener: function (l) {
      this.onDragListener = l;
    },
    setOnMousemoveListener: function (l) {
      this.onMousemoveListener = l;
    },
    setOnMousedownListener: function (l) {
      this.onMousedownListener = l;
    },
    setOnMouseupListener: function (l) {
      this.onMouseupListener = l;
    },
    setOnMouseoverListener: function (l) {
      this.onMouseoverListener = l;
    },
    setOnMouseoutListener: function (l) {
      this.onMouseoutListener = l;
    },
    setOnContextMenuListener: function (l) {
      this.onContextMenuListener = l;
    },
    onDrag: function (x, y) {
      this.setPosition(
        this.orgX + x - this.dragX,
        this.orgY + y - this.dragY
      );
      if (this.onDragListener)
        return this.onDragListener(x, y);

      return true;
    },
    onClick: function (x, y, event) {
      if (this.onClickListener)
        return this.onClickListener(x, y, event);
      return true;
    },
    onFocusIn: function (x, y, event) {
    },
    onFocusOut: function (x, y, event) {
    },
    onMousedown: function (x, y, event) {
      if (this.onMousedownListener)
        return this.onMousedownListener(x, y, event);
      return false;
    },
    onMouseup: function (x, y, event) {
      if (this.onMouseupListener)
        return this.onMouseupListener(x, y, event);
      return false;
    },
    onMouseover: function (x, y, event) {
      if (this.onMouseoverListener)
        return this.onMouseoverListener(x, y, event);
      return false;
    },
    onMouseout: function (x, y, event) {
      if (this.onMouseoutListener)
        return this.onMouseoutListener(x, y, event);
      return false;
    },
    onMousemove: function (x, y, event) {
      if (this.onMousemoveListener)
        return this.onMousemoveListener(x, y, event);
      return false;
    },
    onContextMenu: function (x, y, event) {
      if (this.onContextMenuListener)
        return this.onContextMenuListener(x, y, event);
      return false;
    },
    draggable: function (draggable) {
      this.isDraggable = draggable;
    },
    clickable: function (clickable) {
      this.isClickable = clickable;
    },
    getBoundingClientRect: function () {
      var parentRect = this.parentView.getBoundingClientRect();
      return new deosc.Rect(
        this.left + parentRect.left,
        this.top + parentRect.top,
        this.right + parentRect.left,
        this.bottom + parentRect.top
      );
    },
    setCursorStyles: function (styles) {
      this.cursorStyles = styles;
    }
  };
  if (!window.deosc.canvas) window.deosc.canvas = {};
  window.deosc.canvas.View = View;
  window.deosc.canvas.createView = function () {
    var view = new View();
    view.init();
    return view;
  };
}
)(window);
(
function (window) {
  var ViewGroup = function () {
  };
  ViewGroup.prototype = new deosc.canvas.View;
  $.extend(
    ViewGroup.prototype,
    {
      constructor: ViewGroup,
      init: function () {
        deosc.canvas.View.prototype.init.apply(this);
        this.childViews = [];
        return this;
      },
      destroy: function () {
        deosc.canvas.View.prototype.destroy.apply(this);
      },
      drawChildren: function (ctx) {
        var display = new deosc.Rect(this.display);
        display.intersect(
          this.left, this.top, this.right, this.bottom
        );
        display.offset(-this.left, -this.top);

        for (var i = 0; i < this.childViews.length; i++) {
          var child = this.childViews[i];
          var childRect
            = new deosc.Rect(child.left, child.top, child.right, child.bottom);
          if (!childRect.intersects(display)) {
            continue;
          }

          child.setDisplayArea(display);

          ctx.save();
          ctx.beginPath();
          ctx.rect(child.left, child.top, child.width(), child.height());
          ctx.clip();

          ctx.translate(child.left, child.top);
          child.draw(ctx);
          ctx.restore();
        }
      },
      addView: function (child) {
        this.childViews.push(child);
        child.setParent(this);
        this.invalid();
      },
      removeView: function (child) {
        var i;
        var tmp = [];
        for (i = 0; i < this.childViews.length; i++) {
          if (child != this.childViews[i]) {
            tmp.push(this.childViews[i]);
          }
        }
        this.childViews = tmp;
        this.invalid();
      },
      removeAllViews: function () {
        this.childViews = [];
        this.invalidate();
      },
      dispatchMouseEvent: function (e) {
        var ex = e.x;
        var ey = e.y;
        e.x -= this.left;
        e.y -= this.top;
        for (var i = this.childViews.length - 1; 0 <= i; i--) {
          if (this.childViews[i].isHidden) continue;
          if (this.childViews[i].dispatchMouseEvent(e)) return true;
        }
        e.x = ex;
        e.y = ey;
        return deosc.canvas.View.prototype.dispatchMouseEvent.call(this, e);
      }
    }
  );

  if (!window.deosc.canvas) window.deosc.canvas = {};
  window.deosc.canvas.ViewGroup = ViewGroup;
  window.deosc.canvas.createViewGroup = function () {
    var view = new ViewGroup();
    view.init();
    return view;
  };
}
)(window);
(
function (window) {
  var FrameLayout = function () {};
  FrameLayout.prototype = new deosc.canvas.ViewGroup;
  $.extend(
    FrameLayout.prototype,
    {
      constructor: FrameLayout,
      init: function () {
        deosc.canvas.ViewGroup.prototype.init.apply(this);
      },
      onMeasure: function (width, height) {
        var pd = this.padding;
        var maxWidth = this.layoutParams.width;
        var maxHeight = this.layoutParams.height;

        for (var i = 0; i < this.childViews.length; i++) {
          var child = this.childViews[i];
          child.measure(width - (pd.left + pd.right),
                        height - (pd.top + pd.bottom));
          maxWidth = Math.max(maxWidth, child.getMeasuredWidth());
          maxHeight = Math.max(maxHeight, child.getMeasuredHeight());
        }

        if (this.layoutParams.mode ==
           deosc.canvas.View.LayoutMode.MATCH_PARENT) {
          this.measured.width = width;
          this.measured.height = height;
        } else {
          this.measured.width = maxWidth + pd.left + pd.right;
          this.measured.height = maxHeight + pd.top + pd.bottom;
        }
      },
      onLayout: function (l, t, r, b) {
        var child, left, top, right, bottom, width, height, cw, ch,
        vgravity, hgravity;
        var pd = this.padding;

        width = this.getMeasuredWidth();
        height = this.getMeasuredHeight();

        left = pd.left;
        top = pd.top;
        for (var i = 0; i < this.childViews.length; i++) {
          child = this.childViews[i];
          cw = child.getMeasuredWidth();
          ch = child.getMeasuredHeight();

          right = left + child.getMeasuredWidth();
          bottom = top + child.getMeasuredHeight();

          vgravity = "top";
          hgravity = "left";
          if (child.gravity) {
            if (child.gravity.match(/middle/)) {
              vgravity = "middle";
            } else if (child.gravity.match(/bottom/)) {
              vgravity = "bottom";
            }
            if (child.gravity.match(/center/)) {
              hgravity = "center";
            } else if (child.gravity.match(/right/)) {
              hgravity = "right";
            }
          }

          if (vgravity == "middle") {
            top = Math.floor((height - pd.top - pd.bottom - ch) / 2) + pd.top;
            bottom = top + ch;
          } else if (vgravity == "bottom") {
            bottom = height - pd.bottom;
            top = bottom - ch;
          } else {
            top = pd.top;
            bottom = top + ch;
          }

          if (hgravity == "center") {
            left = Math.floor((width - pd.left - pd.right - cw) / 2) + pd.left;
            right = left + cw;
          } else if (hgravity == "right") {
            right = width - pd.right;
            left = right - cw;
          } else {
            left = pd.left;
            right = left + cw;
          }

          child.layout(left, top, right, bottom);
        }
      }
    }
  );

  if (!window.deosc.canvas) window.deosc.canvas = {};
  window.deosc.canvas.FrameLayout = FrameLayout;
  window.deosc.canvas.createFrameLayout = function () {
    var view = new FrameLayout();
    view.init();
    return view;
  };
}
)(window);
(
function (window) {
  var AbsoluteLayout = function () {};
  AbsoluteLayout.prototype = new deosc.canvas.ViewGroup;
  $.extend(
    AbsoluteLayout.prototype,
    {
      constructor: AbsoluteLayout,
      init: function () {
        deosc.canvas.ViewGroup.prototype.init.apply(this);
      },
      onMeasure: function (width, height) {
        var pd = this.padding;
        var maxWidth = this.layoutParams.width;
        var maxHeight = this.layoutParams.height;

        for (var i = 0; i < this.childViews.length; i++) {
          var child = this.childViews[i];
          child.measure(width - (pd.left + pd.right),
                        height - (pd.top + pd.bottom));
          var lp = child.getLayoutParams();
          var left = lp.x;
          var top = lp.y;
          maxWidth = Math.max(maxWidth, left + child.getMeasuredWidth());
          maxHeight = Math.max(maxHeight, top + child.getMeasuredHeight());
        }
        if (this.layoutParams.mode ==
            deosc.canvas.View.LayoutMode.MATCH_PARENT) {
          this.measured.width = width;
          this.measured.height = height;
        } else {
          this.measured.width = maxWidth + pd.left + pd.right;
          this.measured.height = maxHeight + pd.top + pd.bottom;
        }
      },
      onLayout: function (l, t, r, b) {
        for (var i = 0; i < this.childViews.length; i++) {
          var child = this.childViews[i];
          var lp = child.getLayoutParams();
          var pd = this.padding;
          var left = pd.left + lp.x;
          var top = pd.top + lp.y;
          var right = left + child.getMeasuredWidth();
          var bottom = top + child.getMeasuredHeight();
          child.layout(left, top, right, bottom);
        }
      }
    }
  );

  if (!window.deosc.canvas) window.deosc.canvas = {};
  window.deosc.canvas.AbsoluteLayout = AbsoluteLayout;
  window.deosc.canvas.createAbsoluteLayout = function () {
    var view = new AbsoluteLayout();
    view.init();
    return view;
  };
}
)(window);
(
function (window) {
  function VLinearLayout () {
    var self = this;

    self.onMeasure = function (w, h) {
      var i, child;
      var height = 0;
      var maxWidth = self.layoutParams.width;
      var maxHeight = self.layoutParams.height;
      var pd = this.padding;

      for (i = 0; i < self.childViews.length; i++) {
        child = self.childViews[i];
        child.measure(w, h);
        maxWidth = Math.max(maxWidth, child.getMeasuredWidth());
        height += child.getMeasuredHeight();
      }
      maxHeight = Math.max(maxHeight, height);

      self.measured.width = maxWidth + pd.left + pd.right;
      self.measured.height = maxHeight + pd.top + pd.bottom;
    };

    self.onLayout = function (l, t, r, b) {
      var child, left, top, right, bottom, i;
      var pd = self.padding;
      left = pd.left;
      top = pd.top;
      for (i = 0; i < self.childViews.length; i++) {
        child = self.childViews[i];
        right = left + child.getMeasuredWidth();
        bottom = top + child.getMeasuredHeight();

        if (child.gravity == "right") {
          var w = self.getMeasuredWidth();
          var cw = child.getMeasuredWidth();
          right = w - pd.right;
          left = right - cw;
        } else if (child.gravity == "center") {
          var w = self.getMeasuredWidth();
          var cw = child.getMeasuredWidth();
          w -= pd.left + pd.right;
          left = (w - cw) / 2 + pd.left;
          right = left + cw;
        } else {
          left = pd.left;
          right = left + child.getMeasuredWidth();
        }

        child.layout(left, top, right, bottom);
        top = bottom;
      }
    };

    function constructor () {
      deosc.canvas.ViewGroup.prototype.init.apply(self, arguments);
    }

    constructor();
  }

  function VLinearLayoutPrototype () {}
  VLinearLayoutPrototype.prototype = deosc.canvas.ViewGroup.prototype;
  VLinearLayout.prototype = new VLinearLayoutPrototype;
  VLinearLayout.constructor = VLinearLayout;


  function HLinearLayout () {
    var self = this;

    self.onMeasure = function (w, h) {
      var i, child;
      var width = 0;
      var maxWidth = this.layoutParams.width;
      var maxHeight = this.layoutParams.height;
      var pd = this.padding;

      for (i = 0; i < self.childViews.length; i++) {
        child = self.childViews[i];
        child.measure(w, h);
        maxHeight = Math.max(maxHeight, child.getMeasuredHeight());
        width += child.getMeasuredWidth();
      }
      maxWidth = Math.max(maxWidth, width);

      self.measured.width = maxWidth + pd.left + pd.right;
      self.measured.height = maxHeight + pd.top + pd.bottom;
    };

    self.onLayout = function (l, t, r, b) {
      var child, left, top, right, bottom, i;
      var pd = self.padding;
      left = pd.left;
      top = pd.top;
      for (i = 0; i < self.childViews.length; i++) {
        child = self.childViews[i];
        right = left + child.getMeasuredWidth();
        bottom = top + child.getMeasuredHeight();

        if (child.gravity == "bottom") {
          var h = self.getMeasuredHeight();
          var ch = child.getMeasuredHeight();
          bottom = h - pd.bottom;
          top = bottom - ch;
        } else if (child.gravity == "middle") {
          var h = self.getMeasuredHeight();
          var ch = child.getMeasuredHeight();
          h -= pd.top + pd.bottom;
          top = (h - ch) / 2 + pd.top;
          bottom = top + ch;
        } else {
          top = pd.top;
          bottom = top + child.getMeasuredHeight();
        }

        child.layout(left, top, right, bottom);
        left = right;
      }
    };

    function constructor () {
      deosc.canvas.ViewGroup.prototype.init.apply(self, arguments);
    }

    constructor();
  }

  function HLinearLayoutPrototype () {}
  HLinearLayoutPrototype.prototype = deosc.canvas.ViewGroup.prototype;
  HLinearLayout.prototype = new HLinearLayoutPrototype;
  HLinearLayout.constructor = HLinearLayout;

  window.deosc.canvas.LinearLayout = VLinearLayout;
  window.deosc.canvas.createLinearLayout = function () {
    return new VLinearLayout();
  };
  window.deosc.canvas.VLinearLayout = VLinearLayout;
  window.deosc.canvas.createVLinearLayout = window.deosc.canvas.createLinearLayout;
  window.deosc.canvas.HLinearLayout = HLinearLayout;
  window.deosc.canvas.createHLinearLayout = function () {
    return new HLinearLayout();
  };
}
)(window);
(
function (window) {
  var TextView = function () {};
  TextView.prototype = new deosc.canvas.View;
  $.extend(
    TextView.prototype,
    {
      constructor: TextView,
      init: function () {
        deosc.canvas.View.prototype.init.apply(this);
        this.text = "";
        this.fontColor = "#000";
        this.fontSize = 20;
        this.fontFamily = "sans-serif";
        this.isValidMeasure = false;
        this.cache = null;
        this.cacheEnable = false;
      },
      setText: function (text) {
        this.text = text ? text : "";
        this.isValidMeasure = false;
        this.cache = null;
        this.invalid();
      },
      setFontSize: function (size) {
        this.fontSize = size;
        this.isValidMeasure = false;
        this.cache = null;
        this.invalid();
      },
      setFontFamily: function (family) {
        this.fontFamily = family;
        this.isValidMeasure = false;
        this.cache = null;
        this.invalid();
      },
      setFontColor: function (color) {
        this.fontColor = color;
        this.cache = null;
      },
      onMeasure: function (w, h) {
        if (this.isValidMeasure) return;
        this.isValidMeasure = true;

        var ctx = this.getCanvas().getContext("2d");
        var lineHieght = Math.floor(this.fontSize * 1.2);
        var maxWidth = this.layoutParams.width;
        var maxHeight = this.layoutParams.height;

        var lines = this.lineBreak(this.text);
        this.breakedLines = lines;

        maxHeight = Math.max(maxHeight, lines.length * lineHieght);

        ctx.textBaseline = "top";
        ctx.font = this.fontSize + "px " + this.fontFamily;

        for (var i = 0; i < lines.length; i++) {
          var metrics = ctx.measureText(lines[i]);
          maxWidth = Math.max(maxWidth, metrics.width);
        }

        var pd = this.padding;
        this.measured.width = maxWidth + pd.left + pd.right;
        this.measured.height = maxHeight + pd.top + pd.bottom;
      },
      onDraw: function (ctx) {
        var self = this;

        function draw (ctx) {
          var lineHieght = Math.floor(self.fontSize * 1.2);
          var lines = self.breakedLines;
          ctx.fillStyle = self.fontColor;
          ctx.textBaseline = "top";
          ctx.font = self.fontSize + "px " + self.fontFamily;

          var pd = self.padding;
          var left = pd.left;
          var top = pd.top;
          for (var i = 0; i < lines.length; i++) {
            ctx.fillText(lines[i], left, top);
            top += lineHieght;
          }
        }

        if (this.cacheEnable) {
          if (!this.cache) {
            this.cache = $("<canvas>")
              .attr({
                      width: this.measured.width,
                      height: this.measured.height
                    })[0];

            draw(this.cache.getContext("2d"));
          }
          ctx.drawImage(this.cache, 0, 0);
        } else {
          draw(ctx);
        }
      },
      lineBreak: function (str) {
        function createBreakList (l) {
          var c, i, j;
          var breakChars = [" ",",","　","、","。"];
          var breakList = [];
          for (i = 0; i < l.length; i++) {
            c = l.charAt(i);
            if (breakChars.indexOf(c) != -1) {
              breakList.push(i);
            }
          }
          breakList.push(l.length);
          return breakList;
        }

        var MAX_LEN = 32;
        var lines = [], i, j, l, offset, breakList;
        var tmpLines = str.split("\n");
        for (i = 0; i < tmpLines.length; i++) {
          l = tmpLines[i];
          if (MAX_LEN < l.length) {
            breakList = createBreakList(l);
            offset = 0;
            for (j = 0; j < breakList.length; j++) {
              if ((MAX_LEN < breakList[j] - offset) ||
                  (breakList.length <= j + 1) ||
                  (MAX_LEN < breakList[j + 1] - offset)) {
                lines.push(l.substring(offset, breakList[j] + 1));
                offset = breakList[j] + 1;
              }
            }
          } else {
            lines.push(l);
          }
        }
        return lines;
      }
    }
  );

  if (!window.deosc.canvas) window.deosc.canvas = {};
  window.deosc.canvas.TextView = TextView;
  window.deosc.canvas.createTextView = function () {
    var view = new TextView();
    view.init();
    return view;
  };
}
) (window);(
  function (window) {
    function ImageView () {
      var self = this, mImage, mSrc, mDest, mUrl;

      self.setImageUrl = function (url) {
        if (mUrl == url) return;
        mUrl = url;
        if (mUrl) {
          mImage.src = mUrl;
          mImage.onload = function () {
            self.invalidate();
          };
          mImage.onerror = function () {
            mUrl = null;
            self.invalidate();
          };
        } else {
          mUrl = null;
          self.invalidate();
        }
      };

      self.setImage = function (img) {
        mImage = img;
        mUrl = $(mImage).attr("src");
        self.invalidate();
      };

      self.onMeasure = function (width, height) {
        var sw, sh, dw, dh, pad = self.padding;
        sw = mImage.naturalWidth;
        sh = mImage.naturalHeight;
        dw = self.layoutParams.width;
        dh = self.layoutParams.height;

        mSrc = {x: 0, y: 0, w: sw, h: sh};
        if (sw == 0 || sh == 0) {
          mDest = {x: 0, y: 0, w: 0, h: 0};
        } else {

          if (dw == 0) dw = sw;
          if (dh == 0) dh = sh;

          if (sh * dw <= dh * sw) {
            mDest = {x: 0, y: 0, w: dw, h: (sh * dw / sw)};
          } else {
            mDest = {x: 0, y: 0, w: (sw * dh / sh), h: dh};
          }
        }
        self.measured.width = mDest.w + pad.left + pad.right;
        self.measured.height = mDest.h + pad.top + pad.bottom;
      };

      self.onDraw = function (ctx) {
        var pad = self.padding;

        if (!mUrl) return;

        ctx.drawImage(
          mImage,
          mSrc.x, mSrc.y, mSrc.w, mSrc.h,
          mDest.x + pad.left, mDest.y + pad.top, mDest.w, mDest.h);
      };

      function constructor () {
        deosc.canvas.View.prototype.init.apply(self);

        mImage = new Image();
        mSrc = {x: 0, y: 0, w: 0, h: 0};
        mDest = {x: 0, y: 0, w: 0, h: 0};
      }

      constructor();
    }

    function ImageViewPrototype () {}
    ImageViewPrototype.prototype = deosc.canvas.View.prototype;
    ImageView.prototype = new ImageViewPrototype;
    ImageView.constructor = ImageView;

    window.deosc.canvas.ImageView = ImageView;
  }
) (window);
(
function (window) {
  var clipIcon = $("<img>").attr("src", "images/clip16.png")[0];

  function DCaseNodeView (node) {
    var self = this, mContentView, mDCaseNode, mDrawable, mStatus, mAnimation,
    mBackgroundColor, mBorderColor, mMessage;

    deosc.canvas.ViewGroup.prototype.init.apply(self);

    function createDrawable (type) {
      if (type == "Goal") {
        return new RectNodeDrawable();
      } else if (type == "Evidence") {
        return new EllipseNodeDrawable();
      } else if (type == "Strategy") {
        return new ParallelogramNodeDrawable();
      } else if (type == "Context") {
        return new RoundRectNodeDrawable();
      } else if (type == "Justification") {
        return new RoundRectNodeDrawable();
      } else if (type == "Assumption") {
        return new EllipseNodeDrawable();
      } else if (type == "Undeveloped") {
        return new DiamondNodeDrawable();
      } else if (type == "Module") {
        return new ModuleNodeDrawable();
      } else if (type == "Contract") {
        return new ContractNodeDrawable();
      } else if (type == "Monitor") {
        return new EllipseNodeDrawable();
      } else if (type == "Parameter") {
        return new RoundRectNodeDrawable();
      } else if (type == "Pattern") {
        return new RoundRectNodeDrawable();
      } else if (type == "Action") {
        return new EllipseNodeDrawable();
      } else if (type == "External") {
        return new ModuleNodeDrawable();
      } else {
        return new RectNodeDrawable();
      }
    }

    self.setDCaseNode = function (node) {
      var type, name;

      mDCaseNode = node;
      self.node = node; //TODO: getDCaseNode()

      mDrawable = createDrawable(node.getType());
      mDrawable.setColor(mBackgroundColor);

      mContentView.setDCaseNode(node);
    };

    self.getType = function () {
      return mDCaseNode ? mDCaseNode.getType() : null;
    };

    self.setTypeHidden = function (hidden) {//TODO: hideType()?
      mContentView.setTypeHidden(hidden);
    };

    self.setDescriptionHidden = function (hidden) {//TODO: hideDescription()?
      mContentView.setDescriptionHidden(hidden);
    };

    self.setOnExpandModuleListener = function (l) {
      mContentView.setOnExpandModuleListener(l);
    };

    self.setOnCollapseModuleListener = function (l) {
      mContentView.setOnCollapseModuleListener(l);
    };

    self.expandModule = function () {
      mContentView.expandModule();
    };

    self.collapseModule = function () {
      mContentView.collapseModule();
    };

    self.isExpandModule = function () {
      return mContentView.isExpandModule();
    };

    self.setBackgroundColor = function (color) {
      mBackgroundColor = color;
      mDrawable.setColor(color);
      self.invalidate();
    };

    self.setBorderColor = function (color) {
      mBorderColor = color;
      mDrawable.setBorderColor(color);
      self.invalidate();
    };

    self.setMessage = function (msg) {
      if (msg && msg.length == 0) msg = null;
      if (mMessage != msg) {
        mMessage = msg;
        mContentView.onChangeDCaseNode();
      }
    };

    self.getMessage = function () {
      return mMessage;
    };

    self.setStatus = function (status) {
      var colors = {
        "normal": "#fff",
        "running": "#ff8",
        "satisfied": "#8af",
        "warning": "#ff8",
        "error": "#f88"
      };
      if (mStatus != status) {
        var color = (status in colors) ? colors[status] : "white";
        if (mAnimation) mAnimation.stop();
        if (status == "normal") {
          mAnimation = new ColorAnimation(self, colors[mStatus], color, 5000);
          mAnimation.start();
          mStatus = status;
        } else {
          self.setBackgroundColor(color);
          mStatus = status;
        }
      }
    };

    self.updateNode = function () {
      mDrawable = createDrawable(node.getType());
      mDrawable.setColor(mBackgroundColor);
      mDrawable.setBorderColor(mBorderColor);
      mContentView.onChangeDCaseNode();
    };

    self.onClick = function(x, y, event) {
      if (self.onClickListener) {
        self.onClickListener(event);
      }
    };

    self.onMeasure = function () {
      mContentView.measure();

      mDrawable.setContentSize(
        mContentView.getMeasuredWidth(), mContentView.getMeasuredHeight());

      self.measured.width = mDrawable.getWidth();
      self.measured.height = mDrawable.getHeight();
    };

    self.onLayout = function () {
      var bounds = mDrawable.getContentBounds();
      mContentView.layout(bounds.left, bounds.top, bounds.right, bounds.bottom);
    };

    self.onDraw = function (ctx) {
      mDrawable.draw(ctx);
    };

    function constructor (node) {
      mStatus = "normal";
      mBackgroundColor = "#fff";
      mBorderColor = "#000";
      mContentView = new ContentView(self);
      self.addView(mContentView);
      self.setDCaseNode(node);
    }

    constructor(node);
  }

  function DCaseNodeViewPrototype () {}
  DCaseNodeViewPrototype.prototype = deosc.canvas.ViewGroup.prototype;
  DCaseNodeView.prototype = new DCaseNodeViewPrototype;
  DCaseNodeView.constructor = DCaseNodeView;


  function ContentView (nodeView) {
    var self = this, mDCaseNodeView, mDCaseNode, mTypeView, mNameView, mDescView, mAttachmentView,
    mMessageView, mModuleView,
    mExpirationDateView, mResponsibilityIcon, mResponsibilityNameView,
    mType, mName, mDesc, mMessage, mAttachment, mValidUntil, mModule,
    mResponsibility, mDescriptionHidden, mTypeHidden;

    self.setDCaseNode = function (node) {
      mDCaseNode = node;
      mModuleView.setDCaseNode(node);
      self.onChangeDCaseNode();
    };

    self.setTypeHidden = function (hidden) {
      mTypeHidden = hidden;
    };

    self.setDescriptionHidden = function (hidden) {
      mDescriptionHidden = hidden;
    };

    self.setOnExpandModuleListener = function (l) {
      mModuleView.setOnExpandListener(l);
    };

    self.setOnCollapseModuleListener = function (l) {
      mModuleView.setOnCollapseListener(l);
    };

    self.expandModule = function () {
      mModuleView.expandModule();
    };

    self.collapseModule = function () {
      mModuleView.collapseModule();
    };

    self.isExpandModule = function () {
      return mModuleView.isExpandModule();
    };

    self.onChangeDCaseNode = function () {
      mType = mDCaseNode.getType();
      mName = mDCaseNode.getName();

      mTypeView.setText(mType + ": ");
      mNameView.setText(mName);

      if (mType == "Undeveloped") {
        mDesc = "";
      } else {
        mDesc = mDCaseNode.getDescription(true);
      }
      mDescView.setText(mDesc);

      mValidUntil = mDCaseNode.getValidUntil();
      if (mValidUntil) {
        try {
          var date = new Date(Date.parse(mValidUntil));
          mValidUntil = date.toLocaleDateString();
        } catch (x) {
          alert("catch");
          mValidUntil = "";
        }
      } else {
        mValidUntil = "";
      }
      mExpirationDateView.setText(mValidUntil);

      mResponsibility = mDCaseNode.getResponsibility();
      if (mResponsibility) {
        if (!mResponsibility.name) mResponsibility.name = "";
        if (!mResponsibility.address) mResponsibility.address = "";
        if (!mResponsibility.icon) mResponsibility.icon = "";
      } else {
        mResponsibility = {name: "", address: "", icon: ""};
      }

      mResponsibilityNameView.setText(mResponsibility.name);

      mResponsibilityIcon.setImageUrl(mResponsibility.icon);

      mMessage = mDCaseNodeView.getMessage();
      if (!mMessage) {
        mMessage = mDCaseNode.getMessage();
      }
      if (!mMessage) mMessage = "";
      mMessageView.setText(mMessage);

      mModule = "";
      mAttachment = mDCaseNode.getAttachment();
      if (mAttachment) {
        if (!mAttachment.match(/^https?:/)) {
          if (mType == "Goal" || mType == "Module") {
            mModule = mAttachment;
            if (mModule.match(/\/([^/]+)$/))
              mModule = RegExp.$1;
            mAttachment = "";
          }
        }
      } else {
        mAttachment = "";
      }

      mModuleView.hide();
      mAttachmentView.hide();
      if (mAttachment.length) {
        mAttachmentView.show();
      } else if (mModule.length) {
        mModuleView.show();
      }

      mModuleView.onChangeDCaseNode();
    };

    self.onMeasure = function () {
      var i, width, height, type, desc, w,
      isResNameHidden, isResIconHidden, isValidUntilHidden;

      for (i = 0; i < self.childViews.length; i++) {
        self.childViews[i].measure();
      }

      if (mTypeHidden) {
        width = mNameView.getMeasuredWidth();
        height = mNameView.getMeasuredHeight();
      } else {
        width = mTypeView.getMeasuredWidth() + mNameView.getMeasuredWidth();
        height = Math.max(mTypeView.getMeasuredHeight(),
                          mNameView.getMeasuredHeight());
      }

      if (!mDescriptionHidden && mType != "Undeveloped") {
        if (!mDescView.isHidden) {
          width = Math.max(width, mDescView.getMeasuredWidth());
          height += 8 + mDescView.getMeasuredHeight();
        }

        isValidUntilHidden = mExpirationDateView.isHidden ||
          mValidUntil.length == 0;

        isResNameHidden = mResponsibilityNameView.isHidden ||
          mResponsibility.name.length == 0;

        isResIconHidden = mResponsibilityIcon.isHidden ||
          mResponsibility.icon.length == 0;

        if (!isResIconHidden) {
          width = Math.max(width, mResponsibilityIcon.getMeasuredWidth());
          height += 8 + mResponsibilityIcon.getMeasuredHeight();
        }

        if (!isValidUntilHidden || !isResNameHidden) {
          w = mExpirationDateView.getMeasuredWidth()
            + mResponsibilityNameView.getMeasuredWidth();

          if (!isValidUntilHidden && !isResNameHidden) {
            w += 8;
          }
          width = Math.max(width, w);


          height += (isResIconHidden ? 8 : 0)
            + Math.max(mExpirationDateView.getMeasuredHeight(),
                       mResponsibilityNameView.getMeasuredHeight());
        }

        if (!mMessageView.isHidden && mMessage.length) {
          width = Math.max(width, mMessageView.getMeasuredWidth());
          height += 8 + mMessageView.getMeasuredHeight();
        }

        if (!mAttachmentView.isHidden && mAttachment.length) {
          width = Math.max(width, mAttachmentView.getMeasuredWidth());
          height += 8 + mAttachmentView.getMeasuredHeight();
        }

        if (!mModuleView.isHidden && mModule.length) {
          width = Math.max(width, mModuleView.getMeasuredWidth());
          height += 8 + mModuleView.getMeasuredHeight();
        }
      }

      self.measured.width = width;
      self.measured.height = height;
    };

    self.onLayout = function (l, t, r, b) {
      var left, top, right, bottom, b1, b2,
      isResNameHidden, isResIconHidden, isValidUntilHidden;

      left = top = right = bottom = 0;
      b1 = b2 = 0;

      if (!mTypeHidden) {
        right = left + mTypeView.getMeasuredWidth();
        b1 = top + mTypeView.getMeasuredHeight();
        mTypeView.layout(left, top, right, b1);
      }

      left = right;
      right = left + mNameView.getMeasuredWidth();
      b2 = top + mNameView.getMeasuredHeight();
      mNameView.layout(left, top, right, b2);

      bottom = Math.max(b1, b2);

      if (!mDescriptionHidden) {
        if (!mDescView.isHidden) {
          left = 0;
          right = left + mDescView.getMeasuredWidth();
          top = bottom + 8;
          bottom = top + mDescView.getMeasuredHeight();
          mDescView.layout(left, top, right, bottom);
        }

        isValidUntilHidden = mExpirationDateView.isHidden ||
          mValidUntil.length == 0;

        isResNameHidden = mResponsibilityNameView.isHidden ||
          mResponsibility.name.length == 0;

        isResIconHidden = mResponsibilityIcon.isHidden ||
          mResponsibility.icon.length == 0;

        if (!isResIconHidden) {
          right = self.getMeasuredWidth();;
          left = right - mResponsibilityIcon.getMeasuredWidth();
          top = bottom + 8;
          bottom = top + mResponsibilityIcon.getMeasuredHeight();
          mResponsibilityIcon.layout(left, top, right, bottom);
        }

        b1 = b2 = bottom;

        if (!isValidUntilHidden) {
          left = 0;
          right = left + mExpirationDateView.getMeasuredWidth();
          top = bottom + (isResIconHidden ? 8 : 0);
          b1 = top + mExpirationDateView.getMeasuredHeight();
          mExpirationDateView.layout(left, top, right, b1);
        }

        if (!isResNameHidden) {
          right = self.getMeasuredWidth();
          left = right - mResponsibilityNameView.getMeasuredWidth();
          top = bottom + (isResIconHidden ? 8 : 0);
          b2 = top + mResponsibilityNameView.getMeasuredHeight();
          mResponsibilityNameView.layout(left, top, right, b2);
        }

        bottom = Math.max(b1, b2);

        if (!mMessageView.isHidden && mMessage.length) {
          left = 0;
          right = left + mMessageView.getMeasuredWidth();
          top = bottom + 8;
          bottom = top + mMessageView.getMeasuredHeight();
          mMessageView.layout(left, top, right, bottom);
        }

        if (!mAttachmentView.isHidden && mAttachment.length) {
          left = 0;
          right = left + mAttachmentView.getMeasuredWidth();
          top = bottom + 8;
          bottom = top + mAttachmentView.getMeasuredHeight();
          mAttachmentView.layout(left, top, right, bottom);
        }

        if (!mModuleView.isHidden && mModule.length) {
          left = 0;
          right = left + mModuleView.getMeasuredWidth();
          top = bottom + 8;
          bottom = top + mModuleView.getMeasuredHeight();
          mModuleView.layout(left, top, right, bottom);
        }
      }
    };

    function constructor (nodeView) {
      deosc.canvas.ViewGroup.prototype.init.apply(self);

      mDCaseNodeView = nodeView;
      mDCaseNode = nodeView.node;

      mTypeView = deosc.canvas.createTextView();
      mTypeView.setFontSize(12);

      mNameView = deosc.canvas.createTextView();
      mNameView.setFontSize(12);

      mDescView = deosc.canvas.createTextView();
      mDescView.setFontSize(12);

      mExpirationDateView = deosc.canvas.createTextView();
      mExpirationDateView.setFontSize(12);

      mResponsibilityIcon = new deosc.canvas.ImageView();
      mResponsibilityIcon.setLayoutParams({width: 40, height: 40});

      mResponsibilityNameView = deosc.canvas.createTextView();
      mResponsibilityNameView.setFontSize(12);

      mMessageView = deosc.canvas.createTextView();
      mMessageView.setFontSize(12);

      mAttachmentView = new deosc.canvas.ImageView();
      mAttachmentView.setImageUrl("images/clip16.png");
      mAttachmentView.setImage(clipIcon);
      mAttachmentView.clickable(true);
      mAttachmentView.setCursorStyles({hover: "pointer"});
      mAttachmentView.setOnClickListener(
        function () {
          window.open(mAttachment, "_blank", "");
        });

      mModuleView = new ModuleView();

      self.addView(mTypeView);
      self.addView(mNameView);
      self.addView(mDescView);
      self.addView(mExpirationDateView);
      self.addView(mResponsibilityIcon);
      self.addView(mResponsibilityNameView);
      self.addView(mMessageView);
      self.addView(mAttachmentView);
      self.addView(mModuleView);
    }

    constructor(nodeView);
  }

  function ContentViewPrototype () {}
  ContentViewPrototype.prototype = deosc.canvas.ViewGroup.prototype;
  ContentView.prototype = new ContentViewPrototype;
  ContentView.constructor = ContentView;


  function NodeDrawable () {
    var self = this, mWidth, mHeight,
    mPadding = {left: 0, top: 0, right: 0, bottom: 0}, mColor = "#fff",
    mBorder = {width: 2, color: "#000"};

    self.setColor = function (color) {
      mColor = color;
    };

    self.getColor = function () {
      return mColor;
    };

    self.setBorderColor = function (color) {
      mBorder.color = color;
    };

    self.getBorderColor = function () {
      return mBorder.color;
    };

    self.getBorder = function () {
      return mBorder;
    };

    self.draw = function (ctx) {};

    self.setContentSize = function (width, height) {
      mWidth = width + mPadding.left + mPadding.right;
      mHeight = height + mPadding.top + mPadding.bottom;
    };

    self.setWidth = function (width) {
      mWidth = width;
    };

    self.getWidth = function () {
      return mWidth;
    };

    self.setHeight = function (height) {
      mHeight = height;
    };

    self.getHeight = function () {
      return mHeight;
    };

    self.getPadding = function () {
      return mPadding;
    };

    self.setPadding = function (left, top, right, bottom) {
      mPadding = {left: left, top: top, right: right, bottom: bottom};
    };

    self.getContentBounds = function () {
      return {
        left: mPadding.left, top: mPadding.top,
        right: mPadding.left + mWidth, bottom: mPadding.top + mHeight
      };
    };
  }


  function RectNodeDrawable () {
    var self = this;

    NodeDrawable.call(self);

    self.draw = function (ctx) {
      var border, margin, l, r, t, b;
      border = self.getBorder();
      margin = Math.floor(border.width / 2);
      l = margin;
      t = margin;
      r = self.getWidth() - margin;
      b = self.getHeight() - margin;
      ctx.fillStyle = self.getColor();
      ctx.strokeStyle = border.color;
      ctx.beginPath();
      ctx.moveTo(l, t);
      ctx.lineTo(r, t);
      ctx.lineTo(r, b);
      ctx.lineTo(l, b);
      ctx.closePath();
      ctx.fill();
      ctx.stroke();
    };

    function constructor () {
      self.setPadding(8, 8, 8, 8);
    }

    constructor();
  }

  function RectNodeDrawablePrototype () {}
  RectNodeDrawablePrototype.prototype = NodeDrawable.prototype;
  RectNodeDrawable.prototype = new RectNodeDrawablePrototype;
  RectNodeDrawable.constructor = RectNodeDrawable;


  function ParallelogramNodeDrawable () {
    var self = this;

    NodeDrawable.call(self);

    self.draw = function (ctx) {
      var border, margin, l, r, t, b;
      border = self.getBorder();
      margin = Math.floor(border.width / 2);
      l = margin;
      t = margin;
      r = self.getWidth() - margin;
      b = self.getHeight() - margin;
      ctx.fillStyle = self.getColor();
      ctx.strokeStyle = border.color;
      ctx.beginPath();
      ctx.moveTo(l + 10, t);
      ctx.lineTo(r, t);
      ctx.lineTo(r - 10, b);
      ctx.lineTo(l, b);
      ctx.closePath();
      ctx.fill();
      ctx.stroke();
    };

    function constructor () {
      self.setPadding(18, 8, 18, 8);
    }

    constructor();
  }

  function ParallelogramNodeDrawablePrototype () {}
  ParallelogramNodeDrawablePrototype.prototype = NodeDrawable.prototype;
  ParallelogramNodeDrawable.prototype = new ParallelogramNodeDrawablePrototype;
  ParallelogramNodeDrawable.constructor = ParallelogramNodeDrawable;


  function RoundRectNodeDrawable () {
    var self = this;

    NodeDrawable.call(self);

    self.draw = function (ctx) {
      var border, margin, r, b, radius;
      radius = 10;
      border = self.getBorder();
      margin = Math.floor(border.width / 2);
      r = self.getWidth();
      b = self.getHeight();
      ctx.fillStyle = self.getColor();
      ctx.strokeStyle = border.color;
      ctx.beginPath();
      ctx.arc(radius, radius, radius - margin,
              -Math.PI, - 0.5 * Math.PI, false);
      ctx.arc(r - radius, radius, radius - margin,
              - 0.5 * Math.PI, 0, false);
      ctx.arc(r - radius, b - radius, radius - margin,
              0, 0.5 * Math.PI, false);
      ctx.arc(radius, b - radius, radius - margin,
              0.5 * Math.PI, Math.PI, false);
      ctx.closePath();
      ctx.fill();
      ctx.stroke();
    };

    function constructor () {
      self.setPadding(8, 8, 8, 8);
    }

    constructor();
  }

  function RoundRectNodeDrawablePrototype () {}
  RoundRectNodeDrawablePrototype.prototype = NodeDrawable.prototype;
  RoundRectNodeDrawable.prototype = new RoundRectNodeDrawablePrototype;
  RoundRectNodeDrawable.constructor = RoundRectNodeDrawable;


  function EllipseNodeDrawable () {
    var self = this;

    NodeDrawable.call(self);

    self.setContentSize = function (width, height) {
      var outer_w, outer_h, ph, pv;

      outer_w = width * Math.sqrt(2);
      outer_h = height * Math.sqrt(2);

      ph = Math.floor((outer_w - width) / 2) + 8;
      pv = Math.floor((outer_h - height) / 2) + 8;

      self.setPadding(ph, pv, ph, pv);

      self.setWidth(width + ph * 2);
      self.setHeight(height + pv * 2);
    };

    self.draw = function (ctx) {
      var border, margin, w, h, max, min;
      border = self.getBorder();
      margin = Math.floor(border.width / 2);
      w = self.getWidth();
      h = self.getHeight();
      if (w == 0 || h == 0) return;
      min = w < h ? w : h;
      max = w > h ? w : h;
      ctx.fillStyle = self.getColor();
      ctx.strokeStyle = border.color;
      ctx.beginPath();
      ctx.scale(w/max, h/max);
      ctx.arc(max/2, max/2, max/2 - margin, 0, 2 * Math.PI, false);
      ctx.closePath();
      ctx.fill();
      ctx.stroke();
    };

    function constructor () {
    }

    constructor();
  }

  function EllipseNodeDrawablePrototype () {}
  EllipseNodeDrawablePrototype.prototype = NodeDrawable.prototype;
  EllipseNodeDrawable.prototype = new EllipseNodeDrawablePrototype;
  EllipseNodeDrawable.constructor = EllipseNodeDrawable;


  function DiamondNodeDrawable () {
    var self = this;

    NodeDrawable.call(self);

    self.setContentSize = function (width, height) {
      var outer_w, outer_h, ph, pv, digree;

      digree = 24;

      outer_h = height + width * Math.tan(digree * (Math.PI / 180));
      outer_w = outer_h / Math.tan(digree * (Math.PI / 180));

      ph = Math.floor((outer_w - width) / 2) + 8;
      pv = Math.floor((outer_h - height) / 2) + 8;

      self.setPadding(ph, pv, ph, pv);

      self.setWidth(width + ph * 2);
      self.setHeight(height + pv * 2);
    };

    self.draw = function (ctx) {
      var border, margin, w, h, l, c, r, t, m, b;
      border = self.getBorder();
      margin = Math.floor(border.width / 2);
      w = self.getWidth();
      h = self.getHeight();
      l = Math.floor(margin);
      c = Math.floor(w / 2);
      r = Math.floor(w - margin);
      t = Math.floor(margin);
      m = Math.floor(h / 2);
      b = Math.floor(h - margin);

      ctx.fillStyle = self.getColor();
      ctx.strokeStyle = border.color;
      ctx.beginPath();
      ctx.moveTo(c, t);
      ctx.lineTo(r, m);
      ctx.lineTo(c, b);
      ctx.lineTo(l, m);
      ctx.closePath();
      ctx.fill();
      ctx.stroke();
    };

    function constructor () {
    }

    constructor();
  }

  function DiamondNodeDrawablePrototype () {}
  DiamondNodeDrawablePrototype.prototype = NodeDrawable.prototype;
  DiamondNodeDrawable.prototype = new DiamondNodeDrawablePrototype;
  DiamondNodeDrawable.constructor = DiamondNodeDrawable;


  function ModuleNodeDrawable () {
    var self = this;

    NodeDrawable.call(self);

    self.draw = function (ctx) {
      var border, margin, l, r, t, b, tab_w, tab_h;
      border = self.getBorder();
      margin = Math.floor(border.width / 2);
      l = margin;
      t = margin;
      r = self.getWidth() - margin;
      b = self.getHeight() - margin;
      tab_w = Math.floor(self.getWidth() / 3);
      tab_h = 8;

      ctx.fillStyle = self.getColor();
      ctx.strokeStyle = border.color;
      ctx.beginPath();

      ctx.lineTo(l + tab_w, t + tab_h);
      ctx.lineTo(l + tab_w, t);
      ctx.lineTo(l, t);

      ctx.lineTo(l, b);
      ctx.lineTo(r, b);
      ctx.lineTo(r, t + tab_h);
      ctx.lineTo(l, t + tab_h);

      ctx.fill();
      ctx.stroke();
    };

    function constructor () {
      self.setPadding(8, 16, 8, 8);
    }

    constructor();
  }

  function ModuleNodeDrawablePrototype () {}
  ModuleNodeDrawablePrototype.prototype = NodeDrawable.prototype;
  ModuleNodeDrawable.prototype = new ModuleNodeDrawablePrototype;
  ModuleNodeDrawable.constructor = ModuleNodeDrawable;


  function ContractNodeDrawable () {
    var self = this;

    NodeDrawable.call(self);

    self.draw = function (ctx) {
      var border, margin, l, r, t, b, tab_w, tab_h;
      border = self.getBorder();
      margin = Math.floor(border.width / 2);
      l = margin;
      t = margin;
      r = self.getWidth() - margin;
      b = self.getHeight() - margin;
      tab_w = Math.floor(self.getWidth() / 3);
      tab_h = 8;

      ctx.fillStyle = self.getColor();
      ctx.strokeStyle = border.color;
      ctx.beginPath();

      ctx.moveTo(l + tab_w, t + tab_h);
      ctx.lineTo(l + tab_w, t);
      ctx.lineTo(l, t);
      ctx.lineTo(l, t + tab_h);

      ctx.moveTo(l, t + tab_h);
      ctx.lineTo(l, b - tab_h);
      ctx.lineTo(r, b - tab_h);
      ctx.lineTo(r, t + tab_h);
      ctx.lineTo(l, t + tab_h);

      ctx.moveTo(r, b - tab_h);
      ctx.lineTo(r, b);
      ctx.lineTo(r - tab_w, b);
      ctx.lineTo(r - tab_w, b - tab_h);

      ctx.fill();
      ctx.stroke();
    };

    function constructor () {
      self.setPadding(8, 16, 8, 16);
    }

    constructor();
  }

  function ContractNodeDrawablePrototype () {}
  ContractNodeDrawablePrototype.prototype = NodeDrawable.prototype;
  ContractNodeDrawable.prototype = new ContractNodeDrawablePrototype;
  ContractNodeDrawable.constructor = ContractNodeDrawable;


  function ModuleView () {
    var self = this, mDCase, mExpandView, mCollapseView, mTreeView, mIsExpand,
    mOnExpandListener, mOnCollapseListener, mNodeName, mAttachmentView, mNode;

    deosc.canvas.LinearLayout.call(self);

    self.setDCaseNode = function (node) {
      mNode = node;
      self.onChangeDCaseNode();
    };

    self.addView = function (view) {
      var id = self.viewID;
      deosc.canvas.LinearLayout.prototype.addView.call(self, view);
    };

    self.setDCase = function (dcase, nodeName) {
      mDCase = dcase;
      mNodeName = nodeName;
      if (mIsExpand) {
        if (mTreeView) self.removeView(mTreeView);
        mTreeView = createTreeView(mDCase);
        self.addView(mTreeView);
      }
    };

    self.setOnExpandListener = function (l) {
      mOnExpandListener = l;
    };

    self.setOnCollapseListener = function (l) {
      mOnCollapseListener = l;
    };

    self.expandModule = function () {
      onExpand();
    };

    self.collapseModule = function () {
      onCollapse();
    };

    self.isExpandModule = function () {
      return mIsExpand;
    };

    self.onChangeDCaseNode = function () {
      var a = mNode.getAttachment(), type = mNode.getType();
      if (a) {
        if (type == "Goal") {
          if (a.match(/([^/]+\/[^/]+)$/)) {
            a = RegExp.$1;
          }
        } else if (type == "Module") {
          if (a.match(/\/([^/]+)$/)) {
            a = RegExp.$1;
          }
        }
      }
      mAttachmentView.setText(a);
    };

    function constructor () {
      var hlinear;

      mExpandView = new deosc.canvas.ImageView();
      mExpandView
        .setImage($("<img>").attr("src", "images/expand24.png")[0]);
      mExpandView.clickable(true);
      mExpandView.setOnClickListener(onExpand);

      mCollapseView = new deosc.canvas.ImageView();
      mCollapseView
        .setImage($("<img>").attr("src", "images/collapse24.png")[0]);
      mCollapseView.clickable(true);
      mCollapseView.setOnClickListener(onCollapse);
      mCollapseView.hide();

      mAttachmentView = deosc.canvas.createTextView();
      mAttachmentView.setFontSize(12);
      mAttachmentView.gravity = "middle";

      hlinear = new deosc.canvas.HLinearLayout();
      hlinear.addView(mExpandView);
      hlinear.addView(mCollapseView);
      hlinear.addView(mAttachmentView);

      self.addView(hlinear);
    }

    function onExpand () {
      if (mOnExpandListener) mOnExpandListener.call(self);
      mIsExpand = true;
      mExpandView.hide();
      mCollapseView.show();
      if (mDCase) {
        mTreeView = createTreeView(mDCase);
        self.addView(mTreeView);
      }
    }

    function onCollapse () {
      if (mOnCollapseListener) mOnCollapseListener.call(self);
      mIsExpand = false;
      mExpandView.show();
      mCollapseView.hide();
      if (mTreeView) {
        self.removeView(mTreeView);
        mTreeView = null;
      }
    }

    function createTreeView (module) {
      var nodes, i, rootNode, ret, dcase, tree;

      nodes = module.getRootNodes();

      rootNode = nodes[0];

      if (mNodeName) {
        ret = true;
        for (i = 0; i < nodes.length && ret; i++) {
          ret = module.foreach(
            nodes[i],
            function () {
              if (this.getName() == mNodeName) {
                rootNode = this;
                return false;
              } else {
                return true;
              }
            });
        }
      }

      dcase = new deosc.DCase();

      module.foreach(
        rootNode,
        function () {
          if (this instanceof deosc.DCaseNode) {
            if (!dcase.findDCaseNodeByID(this.getID())) {
              dcase.addDCaseNode(this);
            }
          } else if (this instanceof deosc.DCaseLink) {
            if (!dcase.findDCaseLinkByID(this.getID())) {
              dcase.addDCaseLink(this);
            }
          }
        });

      tree = new deosc.canvas.createDCaseTreeView();
      tree.setDCase(dcase);
      tree.draggable(false);

      return tree;
    }

    constructor();
  }

  function ModuleViewPrototype () {}
  ModuleViewPrototype = deosc.canvas.LinearLayout;
  ModuleView.prototype = new ModuleViewPrototype;
  ModuleView.constructor = ModuleView;


  function ColorAnimation (view, from, to, duration) {
    this.view = view;
    this.from = new Color(from);
    this.to = new Color(to);
    this.duration = duration;
  };
  $.extend(
    ColorAnimation.prototype,
    {
      start: function () {
        var self = this;
        function animation () {
          var t, color;
          self.tid = null;

          t = new Date().getTime() - self.start;

          if (t < self.duration) {
            color = self.from
              .multi((self.duration - t) / self.duration)
              .add(self.to.multi(t / self.duration))
              .toString();

            self.view.setBackgroundColor(color);

            self.tid = setTimeout(animation, 10);
          } else {
            self.view.setBackgroundColor(self.to.toString());
          }
        }
        this.start = new Date().getTime();
        animation();
      },
      stop: function () {
        if (this.tid) clearInterval(this.tid);
      }
    });


  if (!window.deosc.canvas) window.deosc.canvas = {};
  window.deosc.canvas.DCaseNodeView = DCaseNodeView;
  window.deosc.canvas.createDCaseNodeView = function (node) {
    return new DCaseNodeView(node);
  };
}
)(window);
(
function (window) {
  var DCaseLinkView = function () {
  };
  DCaseLinkView.ArrowType = {
    NONE:0, FILL:1, NO_FILL:2
  };
  DCaseLinkView.prototype = new deosc.canvas.View;
  $.extend(
    DCaseLinkView.prototype,
    {
      init: function (link) {
        deosc.canvas.View.prototype.init.apply(this);
        this.link = link;
        this.lineWidth = 2;
        this.color = "#000";
        this.axes = [[0, 0], [0, 0]];
        this.onClickListener = null;
        this.arrowType = DCaseLinkView.ArrowType.FILL;
        this.clickable(true);
      },
      setAxes: function () {
        this.axes = arguments;
      },
      setColor: function (color) {
        this.color = color;
        this.invalidate();
      },
      onMeasure: function (width, height) {
        var pd = this.padding;
        var lp = this.getLayoutParams();
        var maxWidth = Math.max(this.axes[0][0], this.axes[1][0]);
        var maxHeight = Math.max(this.axes[0][1], this.axes[1][1]);
        maxWidth = Math.max(maxWidth, lp.width);
        maxHeight = Math.max(maxHeight, lp.height);

        this.measured.width = maxWidth + pd.left + pd.right;
        this.measured.height = maxHeight + pd.top + pd.bottom;
      },
      onDraw: function (ctx) {
        var fromX, fromY, toX, toY, type;

        if (this.axes.length < 2) return;
        ctx.save();
        ctx.strokeStyle = this.color;
        ctx.beginPath();
        ctx.moveTo(this.axes[0][0], this.axes[0][1]);
        for (var i = 1; i < this.axes.length; i++) {
          ctx.lineTo(this.axes[i][0], this.axes[i][1]);
        }
        ctx.stroke();
        ctx.restore();

        fromX = this.axes[0][0];
        fromY = this.axes[0][1];
        toX = this.axes[this.axes.length - 1][0];
        toY = this.axes[this.axes.length - 1][1];

        ctx.save();
        ctx.translate(toX, toY);
        ctx.rotate(Math.atan2(toY - fromY, toX - fromX));

        type = null;
        if (this.link) type = this.link.getType();
        if (!type) type = "SupportedBy";

        if (type == "SupportedBy" || type == "InContextOf") {
          ctx.beginPath();
          ctx.moveTo(-8, 4);
          ctx.lineTo(0, 0);
          ctx.lineTo(-8, -4);
          ctx.closePath();
          ctx.fillStyle = type == "InContextOf" ? "#fff" : this.color;
          ctx.fill();
        } else if (type == "Responsibility") {
          ctx.beginPath();
          ctx.moveTo(-4, 4);
          ctx.lineTo(0, 0);
          ctx.lineTo(-4, -4);
          ctx.moveTo(-8, 4);
          ctx.lineTo(-4, 0);
          ctx.lineTo(-8, -4);
        }

        ctx.strokeStyle = this.color;
        ctx.stroke();
        ctx.restore();
      },
      isHit: function (x, y) {
        if (!deosc.canvas.View.prototype.isHit.apply(this, arguments)) return false;
        var x0 = this.axes[0][0] + this.left;
        var y0 = this.axes[0][1] + this.top;
        var x1 = this.axes[1][0] + this.left;
        var y1 = this.axes[1][1] + this.top;
        var a = y0 - y1;
        var b = -(x0 - x1);
        var c = x0 * y1 - x1 * y0;
        return 4 * Math.sqrt(a*a + b*b) > Math.abs(a * x + b * y + c);
      },
      setOnClickListener: function (l) {
        this.onClickListener = l;
      },
      onClick: function(x, y, event) {
        if (this.onClickListener) {
          this.onClickListener(x, y, event);
        }
      },
      onContextMenu: function(x, y, event) {
        if (this.onContextMenuListener) {
          return this.onContextMenuListener(x, y, event);
        }
        return false;
      },
      onMouseover: function (x, y, event) {
        document.body.style.cursor = "pointer";
      },
      onMouseout: function (x, y, event) {
        document.body.style.cursor = "default";
      }
    }
  );

  if (!window.deosc.canvas) window.deosc.canvas = {};
  window.deosc.canvas.DCaseLinkView = DCaseLinkView;
  window.deosc.canvas.createDCaseLinkView = function (link) {
    var view = new DCaseLinkView();
    view.init(link);
    return view;
  };
}
)(window);(
function (window) {
  var DCaseTreeView = function () {};
  DCaseTreeView.prototype = new deosc.canvas.ViewGroup;
  $.extend(
    DCaseTreeView.prototype,
    {
      constructor: DCaseTreeView,
      init: function () {
        deosc.canvas.ViewGroup.prototype.init.apply(this);
        this.dcase = null;
        this.draggable(true);
        this.onNodeContextMenuListener = null;
        this.onClickLinkListener = null;
        this.onLinkContextMenuListener = null;
        this.setCursorStyles({drag: "move"});
      },
      arrange: function () {
        this.requestArrange = true;
        this.invalidate();
      },
      onArrange: function (w, h) {
        var tree = this;

        function ContextNodeBox () {
          var self = this;

          self._children = [];

          self.measure = function () {
            var i, c, w = 0, h = 0;
            for (i = 0; i < self._children.length; i++) {
              c = self._children[i];
              c.measure();
              w = Math.max(w, c.getMeasuredWidth());
              h += c.getMeasuredHeight();
            }
            self._measuredWidth = w;
            self._measuredHeight = h;
          };

          self.layout = function (_l, _t, _r, _b) {
            var i, c, w = 0, h = 0, l = _l, t = _t, r, b;
            for (i = 0; i < self._children.length; i++) {
              c = self._children[i];
              r = l + c.getMeasuredWidth();
              b = t + c.getMeasuredHeight();
              c.layout(l, t, r, b);
              t = b;
            }
          };

          self.getMeasuredWidth = function () {
           return self._measuredWidth;
          };

          self.getMeasuredHeight = function () {
           return self._measuredHeight;
          };

          self.addChild = function (c) {
            self._children.push(c);
          };

          self.removeChild = function (c) {
            var index = self._children.indexOf(c);
            if (index != 1)
              self._children.splice(index, 1);
          };

          self.hasChild = function () {
            return self._children.length != 0;
          };
        }

        function ChildNodeBox () {
          var self = this;

          self._children = [];

          self.measure = function () {
            var i, c, w = 0, h = 0;
            for (i = 0; i < self._children.length; i++) {
              c = self._children[i];
              c.measure();
              w += c.getMeasuredWidth();
              h = Math.max(h, c.getMeasuredHeight());
            }
            self._measuredWidth = w;
            self._measuredHeight = h;
          };

          self.layout = function (_l, _t, _r, _b) {
            var i, c, w = 0, h = 0, l = _l, t = _t, r, b;
            for (i = 0; i < self._children.length; i++) {
              c = self._children[i];
              r = l + c.getMeasuredWidth();
              b = t + c.getMeasuredHeight();
              c.layout(l, t, r, b);
              l = r;
            }
          };

          self.getMeasuredWidth = function () {
           return self._measuredWidth;
          };

          self.getMeasuredHeight = function () {
           return self._measuredHeight;
          };

          self.addChild = function (c) {
            self._children.push(c);
          };

          self.removeChild = function (c) {
            var index = self._children.indexOf(c);
            if (index != 1)
              self._children.splice(index, 1);
          };

          self.hasChild = function () {
            return self._children.length != 0;
          };
        }

        function SubTree (node, tree) {
          var self = this,
          CONTEXT_LINK_WIDTH = 20,
          CHILD_LINK_HEIGHT = 20,
          NODE_MARGIN = 2;

          self._node = node;
          self._tree = tree;
          self._view = null;
          self._children = [];
          self._childNodeBox = new ChildNodeBox();
          self._contextNodeBox = new ContextNodeBox();
          self._measuredWidth = 0;
          self._measuredHeight = 0;

          self.getNode = function () {
            return self._node;
          };

          self.measure = function () {
            var nodeOuterW, nodeOuterH,
            nodeL, nodeR, nodeT, nodeB, nodeW, nodeH,
            cldL, cldR, cldT, cldB, cldW, cldH,
            ctxL, ctxR, ctxT, ctxB, ctxW, ctxH;

            nodeL = nodeT = nodeR = nodeB = 0;
            nodeW = self._view.getMeasuredWidth();
            nodeH = self._view.getMeasuredHeight();
            nodeOuterW = nodeW + 2 * NODE_MARGIN;
            nodeOuterH = nodeH + 2 * NODE_MARGIN;

            self._childNodeBox.measure();
            cldL = cldT = cldR = cldB = 0;
            cldW = self._childNodeBox.getMeasuredWidth();
            cldH = self._childNodeBox.getMeasuredHeight();

            self._contextNodeBox.measure();
            ctxL = ctxT = ctxR = ctxB = 0;
            ctxW = self._contextNodeBox.getMeasuredWidth();
            ctxH = self._contextNodeBox.getMeasuredHeight();

            // calc vertical layout
            if (self._contextNodeBox.hasChild()) {
              if (nodeOuterH < ctxH) {
                nodeT = Math.floor((ctxH - nodeOuterH) / 2);
              } else {
                ctxT = Math.floor((nodeOuterH - ctxH) / 2);
              }
              ctxB = ctxT + ctxH;
            }

            nodeT += NODE_MARGIN;
            nodeB = nodeT + nodeH;

            if (self._childNodeBox.hasChild()) {
              cldT = Math.max(nodeB + NODE_MARGIN + CHILD_LINK_HEIGHT, ctxB);
              cldB = cldT + cldH;
            }

            // calc horizontal layout
            if (self._childNodeBox.hasChild()) {
              if (nodeOuterW < cldW) {
                nodeL = Math.floor((cldW - nodeOuterW) / 2);
              } else {
                cldL = Math.floor((nodeOuterW - cldW) / 2);
              }
              cldR = cldL + cldW;
            }

            nodeL += NODE_MARGIN;
            nodeR = nodeL + nodeW;

            if (self._contextNodeBox.hasChild()) {
              ctxL = nodeR + NODE_MARGIN + CONTEXT_LINK_WIDTH;
              ctxR = ctxL + ctxW;
            }

            self._nodeLayout = {l: nodeL, t: nodeT, r: nodeR, b: nodeB};
            self._childLayout = {l: cldL, t: cldT, r: cldR, b: cldB};
            self._contextLayout = {l: ctxL, t: ctxT, r: ctxR, b: ctxB};

            self._measuredWidth = Math.max(nodeR + NODE_MARGIN, ctxR, cldR);
            self._measuredHeight = Math.max(nodeB + NODE_MARGIN, ctxB, cldB);
          };

          self.layout = function (l, t, r, b) {
            self._view.layoutParams.x = l + self._nodeLayout.l;
            self._view.layoutParams.y = t + self._nodeLayout.t;

            self._childNodeBox.layout(
              l + self._childLayout.l,
              t + self._childLayout.t,
              l + self._childLayout.r,
              t + self._childLayout.b
            );

            self._contextNodeBox.layout(
              l + self._contextLayout.l,
              t + self._contextLayout.t,
              l + self._contextLayout.r,
              t + self._contextLayout.b
            );
          };

          self.getMeasuredWidth = function () {
            return self._measuredWidth;
          };

          self.getMeasuredHeight = function () {
            return self._measuredHeight;
          };

          self._isContextType = function (type) {
            return ["Context", "Justification", "Assumption", "Parameter"]
              .indexOf(type) != -1;
          };

          self.addChild = function (subTree) {
            if (self._isContextType(subTree.getNode().getType())) {
              self._contextNodeBox.addChild(subTree);
            } else {
              self._childNodeBox.addChild(subTree);
            }
          };

          self.removeChild = function (subTree) {
            if (self._isContextType(subTree.getNode().getType())) {
              self._contextNodeBox.removeChild(subTree);
            } else {
              self._childNodeBox.removeChild(subTree);
            }
            self._tree.unregisterDCaseNode(subTree.getNode());
          };

          function constructor () {
            var children, child, i, st;

            self._tree.registerDCaseNode(self._node);

            self._view = self._tree.findDCaseNodeViewByID(self._node.getID());

            children = self._node.getChildNodes();
            for (i = 0; i < children.length; i++) {
              child = children[i];
              if (!self._tree.isRegisterd(child))
                self.addChild(new SubTree(child, self._tree));
            }
          }

          constructor();
        }

        function Tree (tree) {
          var self = this,
          PADDING = 20;

          self._tree = tree;
          self._rootTrees = [];
          self._registerdNodes = [];
          self._measuredWidth = 0;
          self._measuredHeight = 0;

          self.measure = function () {
            var i, st, w, h;

            w = 0;
            for (i = 0; i < self._rootTrees.length; i++) {
              st = self._rootTrees[i];
              st.measure();
              w += st.getMeasuredWidth();
              h = Math.max(h, st.getMeasuredHeight());
            }

            w += 2 * PADDING;
            h += 2 * PADDING;

            self._tree.setSize(w, h);

            self._measuredWidth = w;
            self._measuredHeight = h;
          };

          self.layout = function (w, h) {
            var l, t, r, b, i, st;

            l = PADDING, t = PADDING;

            if (self._measuredWidth < w) {
              l = Math.floor((w - self._measuredWidth) / 2) + PADDING;
            }

            for (i = 0; i < self._rootTrees.length; i++) {
              st = self._rootTrees[i];
              r = l + st.getMeasuredWidth();
              b = t + st.getMeasuredHeight();
              st.layout(l, t, r, b);
              l = r;
            }
          };

          self.registerDCaseNode = function (node) {
            self._registerdNodes.push(node);
          };

          self.unregisterDCaseNode = function (node) {
            var index = self._registerdNodes.indexOf(node);
            if (index != -1)
              self._registerNodes.splice(index, 1);
          };

          self.isRegisterd = function (node) {
            var index = self._registerdNodes.indexOf(node);
            return index != -1;
          };

          // TODO
          self.findDCaseNodeViewByID = function (id) {
            return tree.findDCaseNodeViewByID(id);
          };

          function constructor () {
            var i, rootNodes;

            rootNodes = tree.dcase.getRootNodes();
            if (rootNodes.length == 0) return;

            for (i = 0; i < rootNodes.length; i++) {
              self._rootTrees.push(new SubTree(rootNodes[i], self));
            }
          }

          constructor();
        }

        var t = new Tree(this);
        t.measure();
        t.layout(w, h);
      },
      onMeasure: function (w, h) {
        var maxWidth = 0, maxHeight = 0;
        var i, view, lp, width, height;
        var padding = 20;

        for (i = 0; i < this.childViews.length; i++) {
          view = this.childViews[i];
          if (view instanceof deosc.canvas.DCaseNodeView) {
            view.measure();
          }
        }

        if (this.requestArrange) {
          this.requestArrange = false;
          this.onArrange(w, h);
        }

        for (i = 0; i < this.childViews.length; i++) {
          view = this.childViews[i];
          if (view instanceof deosc.canvas.DCaseNodeView) {
            lp = view.getLayoutParams();
            width = view.getMeasuredWidth()+ lp.x;
            height = view.getMeasuredHeight() + lp.y;
            maxWidth = Math.max(maxWidth, width);
            maxHeight = Math.max(maxHeight, height);
          }
        }

        if (!w) w = 0;
        if (!h) h = 0;

        maxWidth = Math.max(w, maxWidth + 2 * padding);
        maxHeight = Math.max(h, maxHeight + 2 * padding);

        this.measured.width = maxWidth;
        this.measured.height = maxHeight;
      },
      onLayout: function (l, t, r, b) {
        var key;
        var i, view;

        for (i = 0; i < this.childViews.length; i++) {
          view = this.childViews[i];
          if (view instanceof deosc.canvas.DCaseNodeView) {
            this.layoutNode(view, l, t, r, b);
          }
        }

        for (i = 0; i < this.childViews.length; i++) {
          view = this.childViews[i];
          if (view instanceof deosc.canvas.DCaseLinkView) {
            this.layoutLink(view, l, t, r, b);
          }
        }
      },
      layoutNode: function (view, l, t, r, b) {
        var lp = view.getLayoutParams();
        var pd = this.padding;
        var left = pd.left + lp.x;
        var top = pd.top + lp.y;
        var right = left + view.getMeasuredWidth();
        var bottom = top + view.getMeasuredHeight();
        view.layout(left, top, right, bottom);
      },
      layoutLink: function (view, l, t, r, b) {
        var sv = this.findDCaseNodeViewByID(view.link.getSource());
        var tv = this.findDCaseNodeViewByID(view.link.getTarget());
        if (!sv) return;
        if (!tv) return;

        var sx, sy, tx, ty;
        if (["Context", "Justification", "Assumption", "Parameter"]
            .indexOf(tv.getType()) != -1) {
          sx = sv.right;
          sy = Math.floor(sv.top + sv.height() / 2);
          tx = tv.left;
          ty = Math.floor(tv.top + tv.height() / 2);
        } else {
          sx = Math.floor(sv.left + sv.width() / 2);
          sy = sv.bottom;
          tx = Math.floor(tv.left + tv.width() / 2);
          ty = tv.top;
        }

        var margin = 10;
        var left = Math.min(sx, tx) - margin;
        var top = Math.min(sy, ty) - margin;
        var right = Math.max(sx, tx) + margin;
        var bottom = Math.max(sy, ty) + margin;
        view.setAxes([sx - left, sy - top], [tx - left, ty - top]);

        view.layout(left, top, right, bottom);
      },
      setOnNodeContextMenuListener: function (l) {
        this.onNodeContextMenuListener = l;
      },
      setOnMouseoverNodeListener: function (l) {
        this.onMouseoverNodeListener = l;
      },
      setOnMouseoutNodeListener: function (l) {
        this.onMouseoutNodeListener = l;
      },
      setOnLinkContextMenuListener: function (l) {
        this.onLinkContextMenuListener = l;
      },
      setOnClickLinkListener: function (l) {
        this.onClickLinkListener = l;
      },
      setDCase: function (dcase) {
        this.dcase = dcase;
        this.updateDCase();
      },
      updateDCase: function () {
        var i, view, nodes, links;

        this.removeAllViews();

        nodes = this.dcase.getDCaseNodes();
        for (i = 0; i < nodes.length; i++) {
          view = deosc.canvas.createDCaseNodeView(nodes[i]);
          this.addDCaseNodeView(view);
        }

        links = this.dcase.getDCaseLinks();
        for (i = 0; i < links.length; i++) {
          view = deosc.canvas.createDCaseLinkView(links[i]);
          this.addDCaseLinkView(view);
        }

        this.arrange();

        this.invalidate();
      },
      addDCaseNodeView: function (view) {
        var self = this;
        function onNodeContextMenu(x, y) {
          if (self.onNodeContextMenuListener) {
            return self.onNodeContextMenuListener.apply(this, arguments);
          }
          return false;
        }
        function onMouseoverNode(x, y) {
          if (self.onMouseoverNodeListener) {
            self.onMouseoverNodeListener.apply(this, arguments);
          }
        }
        function onMouseoutNode(x, y) {
          if (self.onMouseoutNodeListener) {
            self.onMouseoutNodeListener.apply(this, arguments);
          }
        }

        view.setOnContextMenuListener(onNodeContextMenu);
        view.setOnMouseoverListener(onMouseoverNode);
        view.setOnMouseoutListener(onMouseoutNode);

        this.addView(view);

        this.invalidate();
      },
      addDCaseLinkView: function (view) {
        var self = this;

        function onClick (x, y) {
          if (self.onClickLinkListener) {
            return self.onClickLinkListener.apply(this, arguments);
          }
          return false;
        }

        function onLinkContextMenu(x, y) {
          if (self.onLinkContextMenuListener) {
            return self.onLinkContextMenuListener.apply(this, arguments);
          }
          return false;
        }

        view.setOnClickListener(onClick);

        view.setOnContextMenuListener(onLinkContextMenu);

        this.addView(view);

        this.invalidate();
      },
      removeDCaseNodeView: function (view) {
        this.removeView(view);
        view.destroy();

        this.invalidate();
      },
      removeDCaseLinkView: function (view) {
        this.removeView(view);
        view.destroy();

        this.invalidate();
      },
      clear: function () {
        for (var i = 0; i < this.childViews.length; i++) {
          var view = this.childViews[i];
          if (view instanceof deosc.canvas.DCaseNodeView) {
            this.removeDCaseNodeView(view);
          } else if (view instanceof deosc.canvas.DCaseNodeLink) {
            this.removeDCaseLinkView(view);
          }
        }
      },
      update: function () {
        var views = {}, id, nodes, links, i, node, link, view;
        for (i = 0; i < this.childViews.length; i++) {
          view = this.childViews[i];
          if (view instanceof deosc.canvas.DCaseNodeView) {
            views[this.childViews[i].node.getID()] = this.childViews[i];
          } else if (view instanceof deosc.canvas.DCaseLinkView) {
            views[this.childViews[i].link.getID()] = this.childViews[i];
          }
        }

        for (id in views) {
          this.removeView(views[id]);
        }

        nodes = this.dcase.getDCaseNodes();
        for (i = 0; i < nodes.length; i++) {
          node = nodes[i];
          id = node.getID();
          if (id in views) {
            view = views[id];
            view.node = node;
            view.updateNode();
          } else {
            view = deosc.canvas.createDCaseNodeView(node);
          }
          this.addDCaseNodeView(view);
        }

        links = this.dcase.getDCaseLinks();
        for (i = 0; i < links.length; i++) {
          link = links[i];
          id = link.getID();
          if (id in views) {
            view = views[id];
            view.link = link;
          } else {
            view = deosc.canvas.createDCaseLinkView(link);
          }
          this.addDCaseLinkView(view);
        }

        this.invalidate();
      },
      setTypeHidden: function (hide) {
        for (var i = 0; i < this.childViews.length; i++) {
          var view = this.childViews[i];
          if (view instanceof deosc.canvas.DCaseNodeView) {
            view.setTypeHidden(hide);
          }
        }
      },
      setDescriptionHidden: function (hide) {
        for (var i = 0; i < this.childViews.length; i++) {
          var view = this.childViews[i];
          if (view instanceof deosc.canvas.DCaseNodeView) {
            view.setDescriptionHidden(hide);
          }
        }
      },
      onContextMenu: function(x, y, event) {
        if (this.onContextMenuListener) {
          return this.onContextMenuListener(x, y, event);
        }
        return false;
      },
      findDCaseNodeViewByID: function (id) {
        var i, view;
        for (i = 0; i < this.childViews.length; i++) {
          view = this.childViews[i];
          if (view instanceof deosc.canvas.DCaseNodeView && view.node.getID() == id) {
            return view;
          }
        }
        return null;
      },
      findDCaseLinkViewByID: function (id) {
        var i, view;
        for (i = 0; i < this.childViews.length; i++) {
          view = this.childViews[i];
          if (view instanceof deosc.canvas.DCaseLinkView && view.link.getID() == id) {
            return view;
          }
        }
        return null;
      }
    }
  );

  if (!window.deosc.canvas) window.deosc.canvas = {};
  window.deosc.canvas.DCaseTreeView = DCaseTreeView;
  window.deosc.canvas.createDCaseTreeView = function (elem, fix) {
    var tree = new DCaseTreeView();
    tree.init();
    return tree;
  };
}
)(window);
(
function (window) {
  var Point = function () {
  };
  Point.prototype = new deosc.canvas.View;
  $.extend(
    Point.prototype,
    {
      init: function () {
        deosc.canvas.View.prototype.init.apply(this);
        this.color = "#f00";
        this.draggable(true);
        this.isFix = false;
        this.setCursorStyles({drag: "move", hover: "pointer"});
      },
      onMeasure: function (w, h) {
        this.measured.width = 16;
        this.measured.height = 16;
      },
      onDraw: function (ctx) {
        ctx.fillStyle = this.color;
        ctx.beginPath();
        ctx.arc(8, 8, 4, 0, 2 * Math.PI, true);
        ctx.fill();
        if (!this.isFix) {
          ctx.fillStyle = "#fff";
          ctx.beginPath();
          ctx.arc(8, 8, 2, 0, 2 * Math.PI, true);
          ctx.fill();
        }
      },
      setOnDragStartListener: function (l) {
        this.onDragStartListener = l;
      },
      setOnDragListener: function (l) {
        this.onDragListener = l;
      },
      setOnDragEndListener: function (l) {
        this.onDragEndListener = l;
      },
      onDrag: function (x, y) {
        var dx, dy;

        if (this.isFix) return;

        deosc.canvas.View.prototype.onDrag.apply(this, arguments);

        dx = x - this.dragX;
        dy = y - this.dragY;

        if (this.onDragListener)
          this.onDragListener(this.orgX + dx, this.orgY + dy);
      },
      onMousedown: function (x, y) {
        var dx, dy;
        dx = x - this.dragX;
        dy = y - this.dragY;
        if (this.onDragStartListener)
          this.onDragStartListener(this.orgX + dx, this.orgY + dy);
      },
      onMouseup: function (x, y) {
        var dx, dy;
        dx = x - this.dragX;
        dy = y - this.dragY;
        if (this.onDragEndListener)
          this.onDragEndListener(this.orgX + dx, this.orgY + dy);
      },
      setPosition: function (x, y) {
        this.setLayoutParams({x: x - 8, y: y - 8});
        this.invalidate();
      },
      getPosition: function () {
        return {x: this.layoutParams.x + 8, y: this.layoutParams.y + 8};
      },
      fixPoint: function (fix) {
        this.isFix = fix;
      }
    }
  );

  var DCaseLinkEditor = function () {
  };
  DCaseLinkEditor.prototype = new deosc.canvas.AbsoluteLayout();
  $.extend(
    DCaseLinkEditor.prototype,
    {
      init: function () {
        var self = this;

        deosc.canvas.AbsoluteLayout.prototype.init.apply(this);
        this.clickable(true);

        this.linkView = deosc.canvas.createDCaseLinkView();
        this.linkView.setColor("#f00");
        this.addView(this.linkView);

        this.sourcePoint = new Point();
        this.sourcePoint.init();
        this.sourcePoint.setOnDragListener(
          function (x, y) {
            self.onDragSourcePoint(x, y);
          });
        this.sourcePoint.setOnDragStartListener(
          function (x, y) {
            self.onDragStartSourcePoint(x, y);
          });
        this.sourcePoint.setOnDragEndListener(
          function (x, y) {
            self.onDragEndSourcePoint(x, y);
          });
        this.addView(this.sourcePoint);

        this.targetPoint = new Point();
        this.targetPoint.init();
        this.targetPoint.setOnDragListener(
          function (x, y) {
            self.onDragTargetPoint(x, y);
          });
        this.targetPoint.setOnDragStartListener(
          function (x, y) {
            self.onDragStartTargetPoint(x, y);
          });
        this.targetPoint.setOnDragEndListener(
          function (x, y) {
            self.onDragEndTargetPoint(x, y);
          });
        this.addView(this.targetPoint);
      },
      fixSourcePoint: function (fix) {
        this.sourcePoint.fixPoint(fix);
      },
      fixTargetPoint: function (fix) {
        this.targetPoint.fixPoint(fix);
      },
      setSourcePosition: function (x, y) {
        this.sourcePoint.setPosition(x, y);
        this.onChangeSourcePosition(x, y);
      },
      setTargetPosition: function (x, y) {
        this.targetPoint.setPosition(x, y);
        this.onChangeTargetPosition(x, y);
      },
      getSourcePosition: function () {
        return this.sourcePoint.getPosition();
      },
      getTargetPosition: function () {
        return this.targetPoint.getPosition();
      },
      setOnCompleteListener: function (l) {
        this.onCompleteListener = l;
      },
      setOnDragStartSourcePointListener: function (l) {
        this.onDragStartSourcePointListener = l;
      },
      setOnDragSourcePointListener: function (l) {
        this.onDragSourcePointListener = l;
      },
      setOnDragEndSourcePointListener: function (l) {
        this.onDragEndSourcePointListener = l;
      },
      setOnDragStartTargetPointListener: function (l) {
        this.onDragStartTargetPointListener = l;
      },
      setOnDragTargetPointListener: function (l) {
        this.onDragTargetPointListener = l;
      },
      setOnDragEndTargetPointListener: function (l) {
        this.onDragEndTargetPointListener = l;
      },
      onClick: function () {
        this.onCompleteListener();
      },
      onContextMenu: function () {
        this.onCompleteListener();
        return true;
      },
      onDragSourcePoint: function (x, y) {
        this.onChangeSourcePosition(x, y);
        if (this.onDragSourcePointListener)
          this.onDragSourcePointListener();
      },
      onDragStartSourcePoint: function () {
        if (this.onDragStartSourcePointListener)
          this.onDragStartSourcePointListener();
      },
      onDragEndSourcePoint: function () {
        if (this.onDragEndSourcePointListener)
          this.onDragEndSourcePointListener();
      },
      onChangeSourcePosition: function (x, y) {
        var sx, sy, tx, ty, lp, axes, lpx, lpy, w, h, margin = 10;
        lp = this.linkView.layoutParams;
        axes = this.linkView.axes;
        sx = x;
        sy = y;
        tx = lp.x + axes[1][0];
        ty = lp.y + axes[1][1];
        lpx = Math.min(sx, tx);
        lpy = Math.min(sy, ty);
        w = Math.abs(sx - tx);
        h = Math.abs(sy - ty);
        this.linkView.setLayoutParams(
          {
            x: lpx - margin,
            y: lpy - margin,
            width: w + margin * 2,
            height: h + margin * 2
          });
        this.linkView.setAxes(
          [sx - lpx + margin, sy - lpy + margin],
          [tx - lpx + margin, ty - lpy + margin]);
      },
      onDragTargetPoint: function (x, y) {
        this.onChangeTargetPosition(x, y);
        if (this.onDragTargetPointListener)
          this.onDragTargetPointListener();
      },
      onDragStartTargetPoint: function () {
        if (this.onDragStartTargetPointListener)
          this.onDragStartTargetPointListener();
      },
      onDragEndTargetPoint: function () {
        if (this.onDragEndTargetPointListener)
          this.onDragEndTargetPointListener();
      },
      onChangeTargetPosition: function (x, y) {
        var sx, sy, tx, ty, lp, axes, lpx, lpy, w, h, margin = 10;
        lp = this.linkView.layoutParams;
        axes = this.linkView.axes;
        sx = lp.x + axes[0][0];
        sy = lp.y + axes[0][1];
        tx = x;
        ty = y;
        lpx = Math.min(sx, tx);
        lpy = Math.min(sy, ty);
        w = Math.abs(sx - tx);
        h = Math.abs(sy - ty);
        this.linkView.setLayoutParams(
          {
            x: lpx - margin,
            y: lpy - margin,
            width: w + margin * 2,
            height: h + margin * 2
          });
        this.linkView.setAxes(
          [sx - lpx + margin, sy - lpy + margin],
          [tx - lpx + margin, ty - lpy + margin]);
      }
    }
  );

  if (!window.deosc.canvas) window.deosc.canvas = {};

  window.deosc.canvas.DCaseLinkEditor = DCaseLinkEditor;

  window.deosc.canvas.createDCaseLinkEditor = function () {
    var view = new DCaseLinkEditor();
    view.init();
    return view;
  };
}
) (window);(
  function (window) {

    function View (elem) {
      var self = this, mElem, mOnClickListener, mOnDoubleClickListener,
      mOnMouseOverListener, mOnContextMenuListener, mOnChangeListener,
      mChildren = [], mParent, mBindList, mGravity, mTextAlign, mVerticalAlign;

      self.setElement = function (elem) {
        mElem = elem;
      };

      self.getElement = function () {
        return mElem;
      };

      self.getChildAt = function (index) {
        if (mChildren.length <= index) return null;
        return mChildren[index];
      };

      self.getChildCount = function () {
        return mChildren.length;
      };

      self.addChild = function (view, index) {
        if (!isNaN(index) && index < mChildren.length) {
          $(mChildren[index].getElement()).before(view.getElement());
          mChildren.splice(index, 0, view);
        } else {
          $(mElem).append(view.getElement());
          mChildren.push(view);
        }
        view.setParent(self);

        if (mTextAlign)
          $(view.getElement()).css("text-align", mTextAlign);
        if (mVerticalAlign)
          $(view.getElement()).css("vertical-align", mVerticalAlign);

        view.attachedToWindow();
        self.requestLayout();
      };

      self.removeChild = function (view) {
        var i;
        for (i = 0; i < mChildren.length; i++) {
          if (mChildren[i] == view) {
            view.setParent(null);
            $(view.getElement()).remove();
            mChildren.splice(i, 1);
            view.detachedFromWindow();
            self.requestLayout();
            return;
          }
        }
      };

      self.setParent = function (parent) {
        mParent = parent;
      };

      self.getParent = function () {
        return mParent;
      };

      self.setWidth = function (width) {
        $(mElem).css({width: width});
      };

      self.setHeight = function (height) {
        $(mElem).css({height: height});
      };

      self.setSize = function (width, height) {
        $(mElem).css({width: width, height: height});
        self.requestLayout();
      };

      self.addStyle = function (style) {
        $(mElem).addClass(style);
        self.requestLayout();
      };

      self.removeStyle = function (style) {
        $(mElem).removeClass(style);
        self.requestLayout();
      };

      self.setPadding = function (padding) {
        $(mElem).css("padding", padding);
      };

      self.setMargin = function (margin) {
        $(mElem).css("margin", margin);
      };

      self.setGravity = function (gravity) {
        var params;

        mGravity = gravity;

        params = gravity.split(" ");

        if (params.indexOf("center") != -1) {
          mTextAlign = "center";
          mVerticalAlign = "middle";
        }

        if (params.indexOf("left") != -1) {
          mTextAlign = "left";
        } else if (params.indexOf("right") != -1) {
          mTextAlign = "right";
        }

        if (params.indexOf("top") != -1) {
          mVerticalAlign = "top";
        } else if (params.indexOf("bottom") != -1) {
          mVerticalAlign = "bottom";
        }

        if (mTextAlign)
          $(mElem).children("*").css("text-align", mTextAlign);
        if (mVerticalAlign)
          $(mElem).children("*").css("vertical-align", mVerticalAlign);
      };

      self.empty = function () {
        $(mElem).empty();
        mChildren.length = 0;
      };

      self.setText = function (text) {
        $(mElem).text(text);
        self.requestLayout();
      };

      self.getText = function () {
        return $(mElem).text();
      };

      self.setHtml = function (html) {
        $(mElem).html(html);
        self.requestLayout();
      };

      self.getHtml = function () {
        return $(mElem).html();
      };

      self.requestLayout = function () {
        if (mParent)
          mParent.requestLayout();
      };

      self.layout = function () {
        var i;

        for (i = 0; i < mChildren.length; i++) {
          mChildren[i].layout();
        }

        self.onLayout();
      };

      self.onLayout = function () {
      };

      self.attachedToWindow = function () {
        var i;

        self.bind(mBindList.join(" "));

        self.onAttachedToWindow();

        for (i = 0; i < mChildren.length; i++) {
          mChildren[i].attachedToWindow();
        }
      };

      self.onAttachedToWindow = function () {
      };

      self.detachedFromWindow = function () {
        var i;

        for (i = 0; i < mChildren.length; i++) {
          mChildren[i].detachedFromWindow();
        }

        self.onDetachedFromWindow();
      };

      self.onDetachedFromWindow = function () {
      };

      self.setOnClickListener = function (l) {
        mOnClickListener = l;
        if (l) self.bind("click");
      };

      self.setOnDoubleClickListener = function (l) {
        mOnDoubleClickListener = l;
        if (l) self.bind("dblclick");
      };

      self.setOnMouseOverListener = function (l) {
        mOnMouseOverListener = l;
        if (l) self.bind("mouseover");
      };

      self.setOnContextMenuListener = function (l) {
        mOnContextMenuListener = l;
        if (l) self.bind("contextmenu");
      };

      self.setOnChangeListener = function (l) {
        mOnChangeListener = l;
        if (l) self.bind("change");
      };

      self.checked = function (checked) {
        $(mElem).prop("checked", checked);
        if (checked) {
          $(mElem).addClass("checked");
        } else {
          $(mElem).removeClass("checked");
        }
      };

      self.isChecked = function () {
        return $(mElem).prop("checked");
      };

      self.disabled = function (disabled) {
        $(mElem).prop("disabled", disabled);
        if (disabled) {
          $(mElem).addClass("disabled");
        } else {
          $(mElem).removeClass("disabled");
        }
      };

      self.isDisabled = function () {
        return $(mElem).prop("disabled");
      };

      self.readonly = function (readonly) {
        $(mElem).prop("readonly", readonly);
        if (readonly) {
          $(mElem).addClass("readonly");
        } else {
          $(mElem).removeClass("readonly");
        }
      };

      self.isReadonly = function () {
        return $(mElem).prop("readonly");
      };

      self.selected = function (selected) {
        $(mElem).prop("selected", selected);
        if (selected) {
          $(mElem).addClass("selected");
        } else {
          $(mElem).removeClass("selected");
        }
      };

      self.isSelected = function () {
        return $(mElem).prop("selected");
      };

      self.show = function () {
        $(mElem).show();
      };

      self.hide = function () {
        $(mElem).hide();
      };

      self.bind = function (types) {
        var i, list, type;

        list = types.split(" ");

        for (i = 0; i < list.length; i++) {
          type = list[i];

          self.unbind(type);

          if (type == "click") {
            $(mElem).click(self.onClick);
          } else if (type == "dblclick") {
            $(mElem).bind("dblclick", self.onDoubleClick);
          } else if (type == "mouseover") {
            $(mElem).mouseover(self.onMouseOver);
          } else if (type == "contextmenu") {
            $(mElem).bind("contextmenu", self.onContextMenu);
          } else if (type == "change") {
            $(mElem).bind("change", self.onChange);
          }

          if (mBindList.indexOf(type) == -1) mBindList.push(type);
        }
      };

      self.unbind = function (types) {
        var i, list, type, index;

        list = types.split(" ");

        for (i = 0; i < list.length; i++) {
          type = list[i];
          if (type == "click") {
            $(mElem).unbind("click", self.onClick);
          } else if (type == "dblclick") {
            $(mElem).unbind("dblclick", self.onDoubleClick);
          } else if (type == "mouseover") {
            $(mElem).unbind("mouseover", self.onMouseOver);
          } else if (type == "contextmenu") {
            $(mElem).unbind("contextmenu", self.onContextMenu);
          } else if (type == "change") {
            $(mElem).unbind("change", self.onChange);
          }

          index = mBindList.indexOf(type);
          if (index != -1) mBindList.splice(index, 1);
        }
      };

      self.onClick = function (event) {
        if (self.isDisabled()) return;
        if (mOnClickListener) mOnClickListener.call(self, event);
      };

      self.onDoubleClick = function (event) {
        if (self.isDisabled()) return;
        if (mOnDoubleClickListener) mOnDoubleClickListener.call(self, event);
      };

      self.onMouseOver = function (event) {
        if (self.isDisabled()) return;
        if (mOnMouseOverListener) mOnMouseOverListener.call(self, event);
      };

      self.onContextMenu = function (event) {
        if (self.isDisabled()) return;
        if (mOnContextMenuListener) mOnContextMenuListener.call(self, event);
      };

      self.onChange = function (event) {
        if (self.isDisabled()) return;
        if (mOnChangeListener) mOnChangeListener.call(self, event);
      };

      self.draggable = function (draggable, option) {
        var elem, handle, selfPos, mousePos;

        function onStart (e) {
          var rect;

          window.document.body.style.cursor = "default";

          mousePos = {x: e.pageX, y: e.pageY};
          selfPos = $(elem).offset();
          $("html").mousemove(onMove).mouseup(onEnd);

          e.stopPropagation();
          e.preventDefault();
        }

        function onMove (e) {
          var dx, dy, left, top;

          dx = e.pageX - mousePos.x;
          dy = e.pageY - mousePos.y;

          $(elem).offset({left: selfPos.left + dx, top: selfPos.top + dy});

          e.stopPropagation();
          e.preventDefault();
        }

        function onEnd (e) {

          window.document.body.style.cursor = "auto";

          $("html").unbind("mousemove", onMove).unbind("mouseup", onEnd);
          e.stopPropagation();
          e.preventDefault();
        }

        elem = self.getElement();

        if (draggable) {
          handle = self.getElement();

          if (option) {
            if (option.handle) handle = option.handle.getElement();
          }

          $(handle).mousedown(onStart);

        } else {
          $(elem).unbind("mousedown", onStart);
          $("html").unbind("mousemove", onMove).unbind("mouseup", onEnd);
        }
      };

      self.resizable = function (resizable, option) {
        var elem, handle, resize, width, height, mousePos;

        function onStart (e) {
          var rect;

          mousePos = {x: e.pageX, y: e.pageY};
          width = $(elem).width();
          height = $(elem).height();

          $("html").mousemove(onMove).mouseup(onEnd);

          e.stopPropagation();
        }

        function onMove (e) {
          var dx, dy, left, top, w, h;

          dx = e.pageX - mousePos.x;
          dy = e.pageY - mousePos.y;

          w = width + dx;
          if (w < 0) w = 0;
          h = height + dy;
          if (h < 0) h = 0;

          $(elem).width(w);
          $(elem).height(h);

          e.stopPropagation();
        }

        function onEnd (e) {
          $("html").unbind("mousemove", onMove).unbind("mouseup", onEnd);
          e.stopPropagation();
        }

        elem = self.getElement();

        if (resizable) {
          if (option) {
            if (option.handle) handle = option.handle.getElement();
          }

          if (!handle) {
            handle = $("<div>")
              .addClass("resize-handle")
              .appendTo(elem)[0];
          }

          $(handle).mousedown(onStart);

        } else {
          $(handle).unbind("mousedown", onStart).hide();
          $("html").unbind("mousemove", onMove).unbind("mouseup", onEnd);
        }
      };

      function constructor (elem) {
        if (elem) {
          mElem = elem;
        } else {
          mElem = $("<div>")[0];
        }
        $(mElem).addClass("view");

        mBindList = [];
      };

      constructor(elem);
    } //View


    function InputText () {
      var self = this;

      View.call(
        self,
        $("<input type='text'>")
          .css({width: 200})[0]);

      self.getText = function () {
        return $(self.getElement()).val();
      };

      self.setText = function (val) {
        $(self.getElement()).val(val);
      };
    }

    function InputTextPrototype () {}
    InputTextPrototype.prototype = View.prototype;
    InputText.prototype = new InputTextPrototype;
    InputText.constructor = InputText;


    function InputPassword () {
      var self = this;

      View.call(
        self,
        $("<input type='password'>")
          .css({width: 200})[0]);

      self.getText = function () {
        return $(self.getElement()).val();
      };

      self.setText = function (val) {
        $(self.getElement()).val(val);
      };
    }

    function InputPasswordPrototype () {}
    InputPasswordPrototype.prototype = View.prototype;
    InputPassword.prototype = new InputPasswordPrototype;
    InputPassword.constructor = InputPassword;


    function TableView () {
      var self = this;

      View.call(self, $("<table>")[0]);

      self.addRow = function (cols, index) {
        var i, tr, td, col, v;

        tr = new View($("<tr>")[0]);

        for (i = 0; i < cols.length; i++) {
          col = cols[i];

          td = new View($("<td>")[0]);

          if (typeof col == "string" || col instanceof String) {
            v = new deosc.View();
            v.setHtml(col);
            td.addChild(v);
          } else if (!isNaN(col)) {
            v = new deosc.View();
            v.setText(col);
            td.addChild(v);
          } else if (col instanceof View) {
            td.addChild(col);
          } else if (col instanceof Element) {
            v = new deosc.View(col);
            td.addChild(v);
          } else if (!col) {

          } else {//TODO:
            td.addChild(col);
          }

          tr.addChild(td);
        }

        self.addChild(tr, index);
      };

      self.getRow = function (i) {
        return self.getChildAt(i);
      };

      self.getItem = function (i, j) {
        var tr, td;

        tr = self.getChildAt(i);
        if (!tr) return null;

        td = tr.getChildAt(j);
        if (!td) return null;

        return td.getChildAt(0);
      };

      function constructor () {
      }

      constructor();
    }

    function TableViewPrototype () {}
    TableViewPrototype.prototype = View.prototype;
    TableView.prototype = new TableViewPrototype;
    TableView.constructor = TableView;


    function SelectView () {
      var self = this;

      deosc.View.call(
        self,
        $("<select>").css({width: 200})[0]);

      self.setValue = function (val) {
        $(self.getElement()).val(val);
      };

      self.getValue = function () {
        return $(self.getElement()).val();
      };

      self.addItem = function (label, value) {
        var elem;

        elem = new deosc.View(
          $("<option>").text(label).val(value));

        self.addChild(elem);
      };

      self.removeAllItems = function () {
        self.empty();
      };

      function constructor() {
      }

      constructor();
    }

    function SelectViewPrototype () {}
    SelectViewPrototype.prototype = View.prototype;
    SelectView.prototype = new SelectViewPrototype;
    SelectView.constructor = SelectView;


    function Textarea () {
      var self = this;

      View.call(self, $("<textarea>"));

      self.getText = function () {
        return $(self.getElement()).val();
      };

      self.setText = function (val) {
        $(self.getElement()).val(val);
      };
    }

    function TextareaPrototype () {}
    TextareaPrototype.prototype = View.prototype;
    Textarea.prototype = new TextareaPrototype;
    Textarea.constructor = Textarea;


    function ListItem () {
      var self = this, mData, mColumns, mView;

      self.addColumn = function (obj) {
        mColumns.push(obj);
      };

      self.addColumns = function (columns) {
        var i;
        for (i = 0; i < columns.length; i++) {
          self.addColumn(columns[i]);
        }
      };

      self.setColumns = function (columns) {
        mColumns.length = 0;
        self.addColumns(columns);
      };

      self.empty = function () {
        mColumns.length = 0;
        mData = null;
      };

      self.getColumn = function (index) {
        return mColumns[index];
      };

      self.getColumnCount = function (index) {
        return mColumns.length;
      };

      self.setData = function (data) {
        mData = data;
      };

      self.getData = function () {
        return mData;
      };

      self.setView = function (view) {
        mView = view;
      };

      self.getView = function () {
        return mView;
      };

      function constructor () {
        mColumns = [];
      }

      constructor();
    }

    function ListModel () {
      var self = this, mHeaderItem, mItems;

      self.setHeaderItem = function (item) {
        mHeaderItem = item;
      };

      self.getHeaderItem = function () {
        return mHeaderItem;
      };

      self.add = function (item) {
        mItems.push(item);
      };

      self.foreach = function (callback) {
        var i;

        for (i = 0; i < mItems.length; i++) {
          if (callback.call(mItems[i], i) === false) break;
        }
      };

      self.getItemAt = function (i) {
        return (0 <= i && i < mItems.length) ? mItems[i] : null;
      };

      self.getItemCount = function () {
        return mItems.length;
      };

      self.indexOf = function (item) {
        return mItems.indexOf(item);
      };

      self.removeItem = function (item) {
        var index = mItems.indexOf(item);
        if (index != -1)
          mItems.splice(index, 1);
      };

      function constructor () {
        mItems = [];
      }

      constructor();
    }


    function ListView () {
      var self = this, mList, mTableView, mTHeadView, mTBodyView, mModel,
      mOnClickItemListener, mCurrent;

      deosc.View.call(self);

      self.setModel = function (model) {
        mModel = model;
        self.update();
      };

      self.setCurrent = function (index) {
        mCurrent = index;
      };

      self.getCurrent = function () {
        return mCurrent;
      };

      self.getCurrentItem = function () {
        return mCurrent == -1 ? null : mModel.getItemAt(mCurrent);
      };

      self.setOnClickItemListener = function (l) {
        mOnClickItemListener = l;
      };

      self.onLayout = function () {
        var main, thead, tbody, tr, tds, ths, trs, i, width, w;
        main = self.getElement();
        thead = mTHeadView.getElement();
        tbody = mTBodyView.getElement();
        mList.setHeight($(main).innerHeight());
      };

      self.update = function () {

        function createHeader () {
          var item, i, tr, th, obj;

          mTHeadView.empty();

          item = mModel.getHeaderItem();
          if (!item) return;

          tr = new deosc.View($("<tr>")[0]);
          item.setView(tr);

          for (i = 0; i < item.getColumnCount(); i++) {
            th = new deosc.View($("<th>")[0]);
            obj = item.getColumn(i);
            if (obj) {
              if (typeof obj == "string" || obj instanceof String) {
                th.setText(obj);
              } else if (!isNaN(obj)) {
                th.setText(obj);
              } else if (obj instanceof View) {
                th.addChild(obj);
              } else if (obj instanceof Element) {
                $(th.getElement()).append(obj);
              } else {//TODO:
                th.addChild(obj);
              }
            }
            tr.addChild(th);
          }
          mTHeadView.addChild(tr);
        }

        createHeader();

        mTBodyView.empty();

        mModel.foreach(
          function (index) {
            var item = this, i, tr, td, obj;

            tr = new deosc.View($("<tr>")[0]);
            item.setView(tr);

            for (i = 0; i < item.getColumnCount(); i++) {
              td = new deosc.View($("<td>")[0]);
              obj = item.getColumn(i);
              if (obj) {
                if (typeof obj == "string" || obj instanceof String) {
                  td.setText(obj);
                } else if (!isNaN(obj)) {
                  td.setText(obj);
                } else if (obj instanceof View) {
                  td.addChild(obj);
                } else if (obj instanceof Element) {
                  $(td.getElement()).append(obj);
                } else {//TODO:
                  td.addChild(obj);
                }
              }
              tr.addChild(td);
            }

            tr.addStyle("list-item");
            tr.setOnClickListener(
              function () {
                onClickItem(index);
              });

            mTBodyView.addChild(tr);
          });
      };

      function constructor () {
        mCurrent = -1;

        mTHeadView = new deosc.View($("<thead>")[0]);
        mTBodyView = new deosc.View($("<tbody>")[0]);

        mTableView = new deosc.View($("<table>")[0]);
        mTableView.setWidth("100%");
        mTableView.addChild(mTBodyView);
        mTableView.addChild(mTHeadView);

        mList = new deosc.View();
        mList.addStyle("list");
        mList.addChild(mTableView);

        self.addStyle("list-view");
        self.addChild(mList);
      }

      function onClickItem(index) {
        mCurrent = index;

        mModel.foreach(
          function (index, depth) {
            this.getView().checked(index == mCurrent);
          });

        if (mOnClickItemListener) mOnClickItemListener(index);
      }

      constructor();
    }

    function ListViewPrototype () {}
    ListViewPrototype.prototype = View.prototype;
    ListView.prototype = new ListViewPrototype;
    ListView.constructor = ListView;


    function TreeListItem () {
      var self = this, mParentItem, mChildItems, mColumns, mData,
      mIcon, mCollapsable, mCollapse, mView;

      self.clearColumns = function () {
        mColumns.length = 0;
      };

      self.addColumn = function (obj) {
        mColumns.push(obj);
      };

      self.getColumn = function (index) {
        return mColumns[index];
      };

      self.getColumnCount = function (index) {
        return mColumns.length;
      };

      self.setParentItem = function (item) {
        mParentItem = item;
      };

      self.getParentItem = function () {
        return mParentItem;
      };

      self.addChildItem = function (item) {
        mChildItems.push(item);
      };

      self.getChildItems = function () {
        return mChildItems;
      };

      self.removeChildItem = function (item) {
        var i, child;
        for (i = 0; i < mChildItems.length; i++) {
          if (mChildItems[i] == item) {
            mChildItems.splice(i, 1);
            break;
          }
        }
      };

      self.setData = function (data) {
        mData = data;
      };

      self.getData = function () {
        return mData;
      };

      self.setIcon = function (icon) {
        mIcon = icon;
      };

      self.getIcon = function () {
        return mIcon;
      };

      self.collapse = function (collapse) {
        mCollapse = collapse;
      };

      self.isCollapse = function () {
        return mCollapse;
      };

      self.collapsable = function (enable) {
        mCollapsable = enable;
      };

      self.isCollapsable = function () {
        return mCollapsable;
      };

      self.setView = function (view) {
        mView = view;
      };

      self.getView = function () {
        return mView;
      };

      function constructor () {
        mChildItems = [];
        mColumns = [];
        mCollapsable = false;
        mCollapse = true;
      }

      constructor();
    }


    function TreeListItemView (item) {
      var self = this, mCollapseExpandView, mCollapseView, mExpandView,
      mIconView, mIndentView, mOnExpandListener, mOnCollapseListener,
      mIndent, mItem;

      deosc.View.call(self, $("<tr>")[0]);

      self.addColumn = function (obj) {
        var td, view;

        td = new deosc.View($("<td>")[0]);
        if (self.getChildCount() == 0) {
          td.addStyle("hbox");
          td.addChild(mIndentView);
          td.addChild(mCollapseExpandView);
          td.addChild(mIconView);
        }
        self.addChild(td);

        if (obj) {
          if (typeof obj == "string" || obj instanceof String || !isNaN(obj)) {
            view = new deosc.View();
            view.setText(obj);
          } else if (obj instanceof View) {
            view = obj;
          } else if (obj instanceof Element) {
            view = new deosc.View(obj);
          } else {//TODO:
            view = obj;
          }
          td.addChild(view);
          td.setGravity("center left");
        }
      };

      self.setIndent = function (indent) {
        mIndent = indent;
        update();
      };

      self.isCollapse = function () {
        return mItem.isCollapse();
      };

      self.isCollapsable = function () {
        return mItem.isCollapsable();
      };

      self.setOnExpandListener = function (l) {
        mOnExpandListener = l;
      };

      self.setOnCollapseListener = function (l) {
        mOnCollapseListener = l;
      };

      function constructor (item) {
        var i;

        self.addStyle("list-item");

        mIndentView = new deosc.View();

        mCollapseExpandView = new deosc.View();
        $(mCollapseExpandView.getElement()).css({width: 24});

        mCollapseView = new deosc.View();
        mCollapseView.addStyle("icon-collapse");
        mCollapseView.setOnClickListener(onCollapse);
        mCollapseExpandView.addChild(mCollapseView);

        mExpandView = new deosc.View();
        mExpandView.addStyle("icon-expand");
        mExpandView.setOnClickListener(onExpand);
        mCollapseExpandView.addChild(mExpandView);

        mIconView = new deosc.View();
        mIconView.addStyle(item.getIcon());

        mItem = item;
        for (i = 0; i < item.getColumnCount(); i++) {
          self.addColumn(item.getColumn(i));
        }

        update();
      }

      function update () {
        if (self.isCollapsable()) {
          if (self.isCollapse()) {
            mCollapseView.hide();
            mExpandView.show();
          } else {
            mCollapseView.show();
            mExpandView.hide();
          }
        } else {
          mCollapseView.hide();
          mExpandView.hide();
        }

        $(mIndentView.getElement()).css({width: mIndent * 16});
      }

      function onExpand (event) {
        if (mOnExpandListener) mOnExpandListener();
        event.stopPropagation();
      };

      function onCollapse (event)  {
        if (mOnCollapseListener) mOnCollapseListener();
        event.stopPropagation();
      };

      constructor(item);
    }

    function TreeListItemViewPrototype () {}
    TreeListItemViewPrototype.prototype = View.prototype;
    TreeListItemView.prototype = new TreeListItemViewPrototype;
    TreeListItemView.constructor = TreeListItemView;


    function TreeListModel () {
      var self = this, mRootItem, mExitForeach, mItems, mFilter, mHeaderItem;

      self.setHeaderItem = function (item) {
        mHeaderItem = item;
      };

      self.getHeaderItem = function () {
        return mHeaderItem;
      };

      self.getRootItem = function () {
        return mRootItem;
      };

      self.add = function (parent, item) {
        if (!parent) parent = mRootItem;
        item.setParentItem(parent);
        parent.addChildItem(item);
        mItems.push(item);
      };

      self.remove = function (item) {
        self.foreach(
          function () {
            if (this == item) {
              this.getParentItem().removeChildItem(item);
              self.exitForeach();
            }
          });
      };

      self.setFilter = function (filter) {
        mFilter = filter;
      };

      self.foreach = function (callback) {
        var l = [], i, items;

        function isLoop (item) {
          var i = 0;
          for (i = 0; i < l.length; i++) {
            if (item == l[i]) return true;
          }
          return false;
        }

        function _walk (item, depth) {
          var i, ret, items;

          if (mExitForeach) return;

          if (isLoop(item)) return;

          if (mFilter && !mFilter(item)) return;

          l.push(item);

          ret = callback.call(item, self.indexOf(item), depth);
          if (mExitForeach || ret === false) return;

          items = item.getChildItems();

          for (i = 0; i < items.length; i++) {
            _walk(items[i], depth + 1);
          }
        }

        mExitForeach = false;

        items = mRootItem.getChildItems();

        for (i = 0; i < items.length; i++) {
          _walk(items[i], 0);
        }
      };

      self.exitForeach = function () {
        mExitForeach = true;
      };

      self.sort = function (sort) {
        mRootItem.getChildItems().sort(sort);

        self.foreach(
          function (index, depth) {
            this.getChildItems().sort(sort);
          });
      };

      self.getItemAt = function (i) {
        return (0 <= i && i < mItems.length) ? mItems[i] : null;
      };

      self.getItemCount = function () {
        return mItems.length;
      };

      self.indexOf = function (item) {
        return mItems.indexOf(item);
      };

      self.collapsedItem = function (index) {
      };

      self.expandedItem = function (index) {
      };

      function constructor () {
        mItems = [];
        mRootItem = new TreeListItem();
      }

      constructor();
    }


    function TreeListView () {
      var self = this, mList, mTableView, mTHeadView, mTBodyView, mModel,
      mOnClickItemListener, mOnCollapseItemListener, mOnExpandItemListener,
      mCurrent;

      deosc.View.call(self);

      self.setModel = function (model) {
        mModel = model;
        self.update();
      };

      self.setCurrent = function (index) {
        mCurrent = index;
      };

      self.getCurrent = function () {
        return mCurrent;
      };

      self.getCurrentItem = function () {
        return mCurrent == -1 ? null : mModel.getItemAt(mCurrent);
      };

      self.setOnClickItemListener = function (l) {
        mOnClickItemListener = l;
      };

      self.setOnCollapseItemListener = function (l) {
        mOnCollapseItemListener = l;
      };

      self.setOnExpandItemListener = function (l) {
        mOnExpandItemListener = l;
      };

      self.onLayout = function () {
        var main, thead, tbody, tr, tds, ths, trs, i;
        main = self.getElement();
        thead = mTHeadView.getElement();
        tbody = mTBodyView.getElement();
        mList.setHeight($(main).innerHeight());
      };

      self.update = function () {

        function createHeader () {
          var item, i, tr, th, obj;

          mTHeadView.empty();

          item = mModel.getHeaderItem();
          if (!item) return;

          tr = new deosc.View($("<tr>")[0]);
          item.setView(tr);

          for (i = 0; i < item.getColumnCount(); i++) {
            th = new deosc.View($("<th>")[0]);
            obj = item.getColumn(i);
            if (obj) {
              if (typeof obj == "string" || obj instanceof String) {
                th.setText(obj);
              } else if (!isNaN(obj)) {
                th.setText(obj);
              } else if (obj instanceof View) {
                th.addChild(obj);
              } else if (obj instanceof Element) {
                $(th.getElement()).append(obj);
              } else {//TODO:
                th.addChild(obj);
              }
            }
            tr.addChild(th);
          }
          mTHeadView.addChild(tr);
        }

        createHeader();

        mTBodyView.empty();

        mModel.foreach(
          function (index, depth) {
            var item = this, itemView;

            itemView = new TreeListItemView(item);
            item.setView(itemView);

            itemView.setOnClickListener(
              function () {
                onClickItem(index);
              });

            itemView.setOnCollapseListener(
              function () {
                onCollapseItem(index);
              });

            itemView.setOnExpandListener(
              function () {
                onExpandItem(index);
              });

            itemView.setIndent(depth);

            mTableView.addChild(itemView);

            return itemView.isCollapsable() && !itemView.isCollapse();
          });
      };

      function constructor () {
        mCurrent = -1;

        mTHeadView = new deosc.View($("<thead>")[0]);
        mTBodyView = new deosc.View($("<tbody>")[0]);

        mTableView = new deosc.View($("<table>")[0]);
        mTableView.setWidth("100%");
        mTableView.addChild(mTBodyView);
        mTableView.addChild(mTHeadView);

        mList = new deosc.View();
        mList.addStyle("list");
        mList.addChild(mTableView);

        self.addStyle("list-view");
        self.addChild(mList);
      }

      function onClickItem(index) {
        mCurrent = index;

        mModel.foreach(
          function (index, depth) {
            var view;
            view = this.getView();
            if (view)
              view.checked(index == mCurrent);
          });

        if (mOnClickItemListener) mOnClickItemListener(index);
      }

      function onCollapseItem (index) {
        mModel.getItemAt(index).collapse(true);
        mModel.collapsedItem(index);
        self.update();
        if (mOnCollapseItemListener) mOnCollapseItemListener(index);
      }

      function onExpandItem (index) {
        mModel.getItemAt(index).collapse(false);
        mModel.expandedItem(index);
        self.update();
        if (mOnExpandItemListener) mOnExpandItemListener(index);
      }

      constructor();
    }

    function TreeListViewPrototype () {}
    TreeListViewPrototype.prototype = View.prototype;
    TreeListView.prototype = new TreeListViewPrototype;
    TreeListView.constructor = TreeListView;


    window.deosc.View = View;
    window.deosc.InputText = InputText;
    window.deosc.InputPassword = InputPassword;
    window.deosc.Textarea = Textarea;
    window.deosc.TableView = TableView;
    window.deosc.SelectView = SelectView;
    window.deosc.ListItem = ListItem;
    window.deosc.ListModel = ListModel;
    window.deosc.ListView = ListView;
    window.deosc.TreeListItem = TreeListItem;
    window.deosc.TreeListModel = TreeListModel;
    window.deosc.TreeListView = TreeListView;
  }
) (window);(function (window) {

   function DialogBase () {
     var self = this;

     deosc.View.call(self);

     self.addDialog = function (dialog) {
       self.addChild(dialog);
       self.show();
       dialog.open();
       update();
     };

     self.removeDialog = function (dialog) {
       self.removeChild(dialog);
       update();
     };

     function constructor () {
       $(self.getElement()).addClass("dialog-base");
       update();
     }

     function update () {
       var count, i;

       if (self.getChildCount() == 0) {
         self.hide();
       } else {
         self.show();
         count = self.getChildCount();
         for (i = 0; i < count - 1; i++) {
           self.getChildAt(i).inactive();
         }
         self.getChildAt(count - 1).active();
       }
     }

     constructor();
   };

   function Dialog (opt) {
     var self = this, mHeader, mBody, mFooter, mResizeHandle, mCloseButton,
     mTitle, mOKButton, mCancelButton, mInactive, mTID, mOnOKListener,
     mOnCancelListener, mOnCloseListener;


     var mEvents = ["mousedown", "mouseup", "mousemove"];

     deosc.View.call(self);

     self.setSize = function (width, height) {
       var root = self.getElement();
       var header = mHeader.getElement();
       var body = mBody.getElement();
       var footer = mFooter.getElement();

       height = height - $(header).outerHeight() - $(footer).outerHeight();

       $(body).css({width: width, height: height});
     };

     function onEvent (e) {
       e.stopPropagation();
     }

     self.open = function () {
       self.onCreate();
       self.layout();
       center();
       self.onStart();
     };

     function center () {
       var parent, my, left, top;

       parent = self.getParent().getElement().getBoundingClientRect();
       my = self.getElement().getBoundingClientRect();

       left = Math.floor((parent.width - my.width) / 2) - parent.left;
       top = Math.floor((parent.height - my.height) / 2) - parent.top;

       if (left < 0) left = 0;
       if (top < 0) top = 0;

       $(self.getElement()).offset({left: left, top: top});
     }

     self.close = function () {
       onClose();
       self.getParent().removeDialog(self);
     };

     self.setContent = function (widget) {
       mBody.empty();
       mBody.addChild(widget);
     };

     self.getContent = function () {
       return mBody.getChildAt(0);
     };

     self.setTitle = function (title) {
       mTitle.setText(title);
     };

     self.setOnOKListener = function (l) {
       mOnOKListener = l;
     };

     self.setOnCancelListener = function (l) {
       mOnCancelListener = l;
     };

     self.setOnCloseListener = function (l) {
       mOnCloseListener = l;
     };

    self.getFooter = function () {
       return mFooter;
     };

     self.dialogPopup = function (dialog) {
       self.getParent().addDialog(dialog);
     };

     self.active = function () {
       mInactive.hide();
       self.onStart();
     };

     self.inactive = function () {
       mInactive.show();
       self.onStop();
     };

     self.requestLayout = function () {
       if (!mTID) {
         mTID = setTimeout(
           function () {
             self.layout();
             mTID = null;
           }, 0);
       }
     };

     self.onOK = function () {
       self.close();
     };

     self.onCancel = function () {
       self.close();
     };

     self.onCreate = function () {
     };

     self.onStart = function () {
     };

     self.onStop = function () {
     };

     self.onClose = function () {
     };

     function constructor (opt) {
       $(self.getElement()).addClass("dialog");

       mHeader = new deosc.View();
       mHeader.addStyle("dialog-header");
       self.addChild(mHeader);

       mCloseButton = new deosc.View();
       mCloseButton.addStyle("dialog-close-button2");
       mCloseButton.setOnClickListener(self.close);
       mHeader.addChild(mCloseButton);

       mTitle = new deosc.View();
       mTitle.addStyle("dialog-title");
       mTitle.setText("Title");
       mHeader.addChild(mTitle);

       mBody = new deosc.View();
       mBody.addStyle("dialog-body");
       self.addChild(mBody);

       mFooter = new deosc.View();
       mFooter.addStyle("dialog-footer");
       self.addChild(mFooter);

       mOKButton = new deosc.View($("<button>")[0]);
       mOKButton.setText("OK");
       mOKButton.setOnClickListener(onOK);
       mFooter.addChild(mOKButton);

       mCancelButton = new deosc.View($("<button>")[0]);
       mCancelButton.setText("Cancel");
       mCancelButton.setOnClickListener(onCancel);
       mFooter.addChild(mCancelButton);

       if (opt && opt.resizable) {
         mResizeHandle = new deosc.View();
         mResizeHandle.addStyle("resize-handle");
         self.addChild(mResizeHandle);
         mBody.resizable(true, {handle: mResizeHandle});
       }

       mInactive = new deosc.View();
       $(mInactive.getElement()).css(
         {
           position: "absolute",
           width: "100%", height: "100%",
           left: 0, top: 0
         });
       mInactive.hide();
       self.addChild(mInactive);

       self.draggable(true, {handle: mHeader});
     }

     function onOK () {
       if (mOnOKListener) {
         mOnOKListener.call(self);
       } else {
         self.onOK();
       }
     }

     function onCancel () {
       if (mOnCancelListener) {
         mOnCancelListener.call(self);
       } else {
         self.onCancel();
       }
     }

     function onClose () {
       if (mOnCloseListener) {
         mOnCloseListener.call(self);
       } else {
         self.onClose();
       }
     }

     constructor(opt);
   }

   window.deosc.DialogBase = DialogBase;
   window.deosc.Dialog = Dialog;
}) (window);
(
  function (window) {
    function Menu () {
      var self = this, mMenuItems, mParentMenuItem, mBase, mData;

      deosc.View.call(self, $("<table>")[0]);

      self.addMenuItem = function (item, index) {
        if (!isNaN(index) && index < mMenuItems.length) {
          mMenuItems.splice(index, 0, item);
          self.addChild(item, index);
        } else {
          mMenuItems.push(item);
          self.addChild(item);
        }
        item.setParentMenu(self);
      };

      self.getMenuItems = function () {
        return mMenuItems;
      };

      self.setParentMenuItem = function (item) {
        mParentMenuItem = item;
      };

      self.setPosition = function (left, top) {
        $(self.getElement()).css({left: left, top: top});
      };

      self.init = function (base, data) {
        var i;

        mBase = base;
        mData = data;

        self.hide();

        base.addChild(self);

        self.onInit(data);

        for (i = 0; i < mMenuItems.length; i++) {
          mMenuItems[i].init(base, data);
        }
      };

      self.open = function () {
        var i;

        self.show();

        self.onOpen();

        $(self.getElement()).css(getPosition());

        for (i = 0; i < mMenuItems.length; i++) {
          mMenuItems[i].open();
        }
      };

      self.close = function () {
        var i;

        self.onClose();

        self.hide();

        for (i = 0; i < mMenuItems.length; i++) {
          mMenuItems[i].close();
        }
      };

      self.onInit = function (data) {
      };

      self.onOpen = function () {
      };

      self.onClose = function () {
      };

      function constructor () {
        mMenuItems = [];
        $(self.getElement()).addClass("context-menu");
      }

      function getPosition () {
        var left, top, baseRect, menuRect, itemRect, l, t;

        baseRect = mBase.getElement().getBoundingClientRect();
        menuRect = self.getElement().getBoundingClientRect();

        if (mParentMenuItem) {
          itemRect = mParentMenuItem.getElement().getBoundingClientRect();
        } else {
          l = mData.position.left;
          t = mData.position.top;
          itemRect = {
            left: l, top: t, right: l, bottom: t, width: 0, height: 0
          };
        }

        if (baseRect.right < itemRect.right + menuRect.width) {
          left = itemRect.left - menuRect.width;
        } else {
          left = itemRect.right;
        }

        if (baseRect.bottom < itemRect.top + menuRect.height) {
          top = baseRect.bottom - menuRect.height;
        } else {
          top = itemRect.top;
        }

        left -= baseRect.left;
        top -= baseRect.top;

        return {left: left, top: top};
      }

      constructor();
    }

    Menu.Visibility = {
      VISIBLE: 0,
      DISABLE: 1,
      GONE: 2
    };

    function MenuItem (label, callback) {
      var self = this, mLabel, mCallback, mMenu, mLabelView, mArrowView,
      mParentMenu, mData, mData2;

      deosc.View.call(self, $("<tr>")[0]);

      self.setLabel = function (label) {
        mLabel = label;
      };

      self.getLabel = function () {
        return mLabel;
      };

      self.setCallback = function (callback) {
        mCallback = callback;
      };

      self.getCallback = function () {
        return mCallback;
      };

      self.setMenu = function (menu) {
        mMenu = menu;
        mMenu.setParentMenuItem(self);
      };

      self.getMenu = function () {
        return mMenu;
      };

      self.setData = function (data) {
        mData2 = data;
      };

      self.getData = function () {
        return mData2;
      };

      self.setParentMenu = function (menu) {
        mParentMenu = menu;
      };

      self.init = function (base, data) {
        mData = data;

        self.setOnClickListener(onClick);
        self.setOnMouseOverListener(onMouseOver);

        self.selected(false);

        self.onInit(data);

        if (mMenu) mMenu.init(base, data);
      };

      self.open = function () {
        $(mArrowView.getElement())
          .removeClass(
            "context-menu-item-arrow context-menu-item-arrow-none")
          .addClass(mMenu ?
                    "context-menu-item-arrow" :
                    "context-menu-item-arrow-none");
        self.onOpen();
      };

      self.close = function () {
        if (mMenu) mMenu.close();
        self.selected(false);
        self.onClose();
      };

      self.onInit = function (data) {
      };

      self.onOpen = function () {
      };

      self.onClose = function () {
      };

      function constructor (label, callback) {
        self.setLabel(label);
        self.setCallback(callback);

        $(self.getElement())
          .addClass("context-menu-item");

        mLabelView = new deosc.View($("<td>")[0]);
        mLabelView.setText(self.getLabel());


        mArrowView = new deosc.View(
          $("<td>")
          .addClass("context-menu-item-arrow")[0]);

        self.addChild(mLabelView);
        self.addChild(mArrowView);
      }

      function onClick () {
        if (mCallback) mCallback.call(self, mData);
      }

      function onMouseOver () {
        var i, items;

        items = mParentMenu.getMenuItems();
        for (i = 0; i < items.length; i++) {
          if (items[i] != self) items[i].close();
        }

        if (mMenu) mMenu.open();

        self.selected(true);
      }

      constructor(label, callback);
    }


    function MenuBase () {
      var self = this, mMenu, mData, mCallback, mOnCloseListener, mMap;

      deosc.View.call(self);

      self.open = function (menu, data, callback) {
        var menuView, menuElem, pos;

        mCallback = callback;

        self.empty();
        self.show();


        var x = data.position.left;
        menu.init(self, data);

        menu.open();
      };

      self.close = function () {
        onClose();
      };

      function initMap (menu) {
        function scan (menu) {
          var i, items, submenu, m;

          mMap.push({menu: menu, widget: null, status: false});

          items = menu.getMenuItems();

          for (i = 0; i < items.length; i++) {

            mMap.push({item: items[i], widget: null, status: false});

            submenu = items[i].getMenu();
            if (submenu) {
              scan(submenu);
            }
          }
        }

        mMap = [];

        scan(menu);
      }

      function createMenuView (menu) {
        var menuView, items, i, m;

        menu.onOpen(mData);

        menuView = new MenuView(menu);

        items = menu.getMenuItems();

        for (i = 0; i < items.length; i++) {
          menuView.addMenuItem(createMenuItemView(items[i]));
        }

        m = findMap(menu);
        if (m) m.widget = menuView;

        return menuView;
      }

      function createMenuItemView (item) {
        var itemView, menuView, m;

        item.onOpen(mData);

        itemView = new MenuItemView(item);

        itemView.setOnMouseOverListener(
          function () {
            var menu = item.getMenu(), pos, m;
            if (menu) {
              m = findMap(menu);
              if (!m || !m.widget) {
                menuView = createMenuView(menu);
                self.addChild(menuView);

                pos = calcPosition(
                  getPosition(itemView),
                  getPosition(menuView));

                menuView.setPosition(pos.left, pos.top);
              }
            }
            update(item);
          });

        itemView.setOnClickListener(
          function () {
            var callback = item.getCallback();
            if (!item.getMenu() && callback) {
              callback(item.getLabel(), mData);
            }
          });

        m = findMap(item);
        if (m) m.widget = itemView;

        return itemView;
      }

      function getPosition (widget) {
        var rect = widget.getElement().getBoundingClientRect();
        var baseRect = self.getElement().getBoundingClientRect();

        return {
          left: rect.left - baseRect.left,
          top: rect.top - baseRect.top,
          right: rect.right - baseRect.left,
          bottom: rect.bottom - baseRect.top,
          width: rect.width,
          height: rect.height
        };
      }

      function calcPosition (item, menu) {
        var baseElem, menuElem, base, w, h, left, top;

        baseElem = self.getElement();
        w = $(baseElem).innerWidth();
        h = $(baseElem).innerHeight();
        base = {
          left: 0, top: 0, right: w, bottom: h, width: w, height: h
        };

        if (base.right < item.right + menu.width) {
          left = item.left - menu.width;
        } else {
          left = item.right;
        }

        if (base.bottom < item.top + menu.height) {
          top = base.bottom - menu.height;
        } else {
          top = item.top;
        }

        return {left: left, top: top};
      }

      function findMap (x) {
        var i, m;
        for (i = 0; i < mMap.length; i++) {
          m = mMap[i];
          if (m.menu == x || m.item == x) return m;
        }
        return null;
      }

      function clearMapStatus () {
        var i, m;
        for (i = 0; i < mMap.length; i++) {
          mMap[i].status = false;
        }
      }

      function updateMapStatus (item) {
        var m, menu;

        function scan (menu) {
          var i, items, submenu, m;
          items = menu.getMenuItems();
          for (i = 0; i < items.length; i++) {
            if (items[i] == item) {
              m = findMap(items[i]);
              if (m) m.status = true;
              m = findMap(menu);
              if (m) m.status = true;
              return true;
            }
            submenu = items[i].getMenu();
            if (submenu) {
              if (scan(submenu)) {
                m = findMap(items[i]);
                if (m) m.status = true;
                m = findMap(menu);
                if (m) m.status = true;
                return true;
              }
            }
          }
          return false;
        }

        clearMapStatus();

        menu = item.getMenu();
        if (menu) {
          m = findMap(menu);
          if (m) m.status = true;
        }

        scan(mMenu);
      }

      function update(item) {
        var i, j, m, m2, items;

        updateMapStatus(item);

        for (i = 0; i < mMap.length; i++) {
          m = mMap[i];
          if (m.menu) {
            if (!m.status) {
              self.removeChild(m.widget);
              m.widget = null;

              items = m.menu.getMenuItems();
              for (j = 0; j < items.length; j++) {
                m2 = findMap(item[j]);
                if (m2) m2.widget = null;
              }
            }
          }
        }

        for (i = 0; i < mMap.length; i++) {
          m = mMap[i];
          if (m.item && m.widget) {
            if (m.status) {
              m.widget.selected(true);
            } else {
              m.widget.selected(false);
            }
          }
        }
      }

      function removeMap (m) {
      }

      function constructor () {
        $(self.getElement())
          .addClass("menu-base")
          .addClass("context-menu-base")
          .click(
            function (e) {
              e.preventDefault();
              onClose();
            })
          .bind(
            "contextmenu",
            function (e) {
              e.preventDefault();
              self.hide();
              onClose();
            })
          .hide();
      }

      function onClose () {
        self.empty();
        self.hide();
        if (mCallback) mCallback();
      }

      constructor();
    }

    function MenuView (menu) {
      var self = this, mMenu;

      deosc.View.call(self, $("<table>")[0]);

      self.addMenuItem = function (item) {
        self.addChild(item);
      };

      self.setPosition = function (x, y) {
        $(self.getElement()).css({left: x, top: y});
      };

      function constructor (menu) {
        mMenu = menu;

        $(self.getElement())
          .addClass("context-menu");
      }

      constructor(menu);
    }

    function MenuItemView (item) {
      var self = this, mItem, mLabel, mArrow, mArrowCanvas,
      mOnClickListener, mOnMouseOverListener;

      deosc.View.call(self, $("<tr>")[0]);

      function constructor (item) {
        var canvas, v;

        mItem = item;

        $(self.getElement())
          .addClass("context-menu-item");

        self.setOnClickListener(onClick);
        self.setOnMouseOverListener(onMouseOver);

        mLabel = new deosc.View($("<td>")[0]);
        mLabel.setText(mItem.getLabel());

        v = mItem.getVisibility();
        if (v == Menu.Visibility.DISABLE) {
          self.disabled(true);
          self.show();
        } else if (v == Menu.Visibility.GONE) {
          self.disabled(false);
          self.hide();
        } else {
          self.disabled(false);
          self.show();
        }


        mArrow = new deosc.View($("<td>")[0]);

        mArrowCanvas = $("<canvas>")
          .addClass("context-menu-item-arrow")
          .attr({width: 16, height: 16})[0];

        $(mArrow.getElement()).append(mArrowCanvas);

        var ctx = mArrowCanvas.getContext("2d");
        ctx.save();
        if (mItem.getMenu()) {
          ctx.fillStyle = "whitesmoke";
          ctx.beginPath();
          ctx.moveTo(5, 5);
          ctx.lineTo(12, 8);
          ctx.lineTo(5, 11);
          ctx.closePath();
          ctx.fill();
        } else {
          ctx.clearRect(0, 0, 16, 16);
        }
        ctx.restore();

        self.addChild(mLabel);
        self.addChild(mArrow);
      }

      function onClick () {
        if (self.isDisabled()) return;
        if (mOnClickListener) mOnClickListener();
      }

      function onMouseOver () {
        if (self.isDisabled()) return;
        if (mOnMouseOverListener) mOnMouseOverListener();
      }

      constructor(item);
    }

    window.deosc.Menu = Menu;
    window.deosc.MenuItem = MenuItem;
    window.deosc.MenuBase = MenuBase;

  }
) (window);
(
  function (window) {
    function DCaseParameters (dcase, elem) {
      var self = this, mDCase = dcase, mElem = elem;

      self.getElement = function () {
        return mElem;
      };

      self.setParameters = function (params) {
        var i, e;
        $(mElem).empty();
        if (!params) return;
        for (name in params) {
          self.setParameter(name, params[name]);
        }
      };

      self.getParameters = function () {
        var elems, e, attrs, i, key, val, params = {}, name,
        keys = ["name", "type", "min", "max", "digit", "inc", "items", "value"];

        $(mElem).children().each(
          function () {
            if (this.namespaceURI != mElem.namespaceURI) return;
            if (this.localName != "parameter") return;

            attrs = {};
            for (i = 0; i < keys.length; i++) {
              key = keys[i];
              val = $(this).attr(key);
              if (val) {
                if (key == "items") {
                  attrs[key] = val.split(",");
                } else {
                  attrs[key] = val;
                }
              }
            }

            if (attrs.name) {
              name = attrs.name;
              delete attrs.name;
              params[name] = attrs;
            }
          });

        return Object.keys(params).length == 0 ? null : params;
      };

      self.setParameter = function (name, attrs) {
        var elems, e;

        elems = $(mElem).children().filter(
          function () {
            if (this.namespaceURI != mElem.namespaceURI) return false;
            if (this.localName != "parameter") return false;
            return $(this).attr("name") == name;
          });

        if (elems.length == 0) {
          e = mDCase.createElement("parameter");
          $(e).attr("name", name).attr(attrs).appendTo(mElem);
        } else {
          $(elems.get(0)).attr(attrs);
        }
      };

      self.getParameter = function (name) {
        var params = self.getParameters();
        return (params && (name in params)) ? params : null;
      };

      self.removeParameter = function (name) {
        $(mElem).children().each(
          function () {
            if (this.namespaceURI != mElem.namespaceURI) return;
            if (this.localName != "parameter") return;
            if ($(this).attr("name") == name) {
              $(this).remove();
            }
          });
      };

      function constructor () {
        if (!mElem)
          mElem = mDCase.createElement("parameters");
      }

      constructor();
    } /* DCaseParameters */


    function DCaseCommon () {
      var self = this, mDoc, mElem, mDescElem, mPropertiesElem,
      mResponsibilityElem, mParameters, doNothing = function () {};

      self.setElement = function (elem) {
        var ns;
        mElem = elem;
        mDoc = elem ? elem.ownerDocument : null;
        ns = mDoc.documentElement.namespaceURI;
        switch (ns) {
        case deosc.Namespace.DCASE201006:
          self.setSiblingOrder = setUserdef001;
          self.getSiblingOrder = getUserdef001;
          self.setMessage = setUserdef002;
          self.getMessage = getUserdef002;
          self.setParameterizedDesc = setUserdef005;
          self.getParameterizedDesc = getUserdef005;
          self.setParameters = setParametersDCASE201006;
          self.getParameters = getParametersDCASE201006;
          self.setCMISInfo = setCMISInfo;
          self.getCMISInfo = getCMISInfo;
          self.setModuleRefSource = setUserdef011;
          self.getModuleRefSource = getUserdef011;
          self.setModuleParent = setUserdef013;
          self.getModuleParent = getUserdef013;
          self.setFlag = setUserdef015;
          self.getFlag = getUserdef015;
          self.setDSBenchProject = setDSBenchProject;
          self.getDSBenchProject = getDSBenchProject;
          self.setResponsibility = setUserdef012;
          self.getResponsibility = getUserdef012;
          self.setValidUntil = setValidUntil;
          self.getValidUntil = getValidUntil;
          break;
        case deosc.Namespace.DCASE201311:
        default:
          self.setSiblingOrder = setSiblingOrder;
          self.getSiblingOrder = getSiblingOrder;
          self.setMessage = setMessage;
          self.getMessage = getMessage;
          self.setParameterizedDesc = setParameterizedDesc;
          self.getParameterizedDesc = getParameterizedDesc;
          self.setParameters = setParameters;
          self.getParameters = getParameters;
          self.setCMISInfo = setCMISInfo;
          self.getCMISInfo = getCMISInfo;
          self.setModuleRefSource = setModuleRefSource;
          self.getModuleRefSource = getModuleRefSource;
          self.setModuleParent = setModuleParent;
          self.getModuleParent = getModuleParent;
          self.setFlag = setFlag;
          self.getFlag = getFlag;
          self.setDSBenchProject = setDSBenchProject;
          self.getDSBenchProject = getDSBenchProject;
          self.setResponsibility = setResponsibility;
          self.getResponsibility = getResponsibility;
          self.setValidUntil = setValidUntil;
          self.getValidUntil = getValidUntil;
          break;
        }
        update();
      };

      self.getElement = function () {
        return mElem;
      };

      self.getNamespace = function () {
        return mElem.namespaceURI;
      };

      self.setID = function (id) {
        $(mElem).attr("id", id);
      };

      self.getID = function () {
        return $(mElem).attr("id");
      };

      self.setName = function (name) {
        $(mElem).attr("name", name);
      };

      self.getName = function () {
        return $(mElem).attr("name");
      };

      function generateDescription () {
        var desc, params, name, type, p;

        type = self.getType ? self.getType() : null;
        if (type && type == "Parameter") {
          params = self.getParameters();
          desc = [];
          if (params) {
            for (name in params) {
              p = params[name];
              desc.push(name+":"+(p.type?p.type:"")+"="+(p.value?p.value:""));
            }
          }
          return desc.join("\n");
        }

        desc = self.getParameterizedDesc();
        if (desc) {
          params = self.getParameters();

          if (self instanceof deosc.DCaseNode) {
            if (!params) params = {};
            self.scanAncestor(
              function () {
                var i, nodes = this.getChildNodes(), n, name;
                for (i = 0; i < nodes.length; i++) {
                  n = nodes[i];
                  if (n.getType() == "Parameter") {
                    p = n.getParameters();
                    for (name in p) {
                      if (!(name in params)) {
                        params[name] = p[name];
                      }
                    }
                  }
                }
              });
          }

          if (params) {
            for (name in params) {
              desc = desc.replace("{"+name+"}", params[name].value);
            }
          }
          return desc;
        }

        return null;
      }

      self.setDescription = function (desc) {
        if (!mDescElem) {
          mDescElem = mDoc.createElementNS(
            mDoc.documentElement.namespaceURI, "dcase:description");

          $(mElem).append(mDescElem);
        }
        if (desc && 0 < desc.length) {
          $(mDescElem).text(desc);
        } else {
          $(mDescElem).empty();
        }
      };

      self.getDescription = function (parameterize) {
        var desc, pdesc;
        desc = mDescElem ? $(mDescElem).text() : "";
        if (parameterize) {
          pdesc = generateDescription();
          if (pdesc) desc = pdesc;
        }
        return desc;
      };

      self.setProperty = function (name, value) {
        var propertyElem, i, node;

        if (value == null ||
            ((typeof value == 'string' || value instanceof String) && value.length == 0)) {
          self.removeProperty(name);
          return;
        }

        if (!mPropertiesElem) {
          mPropertiesElem = mDoc.createElementNS(
            mDoc.documentElement.namespaceURI, "dcase:properties");

          $(mElem).append(mPropertiesElem);
        }

        for (i = 0; i < mPropertiesElem.childNodes.length; i++) {
          node = mPropertiesElem.childNodes[i];
          if (node.nodeType == 1 && node.localName == "property" &&
              $(node).attr("name") == name) {
            propertyElem = node;
            break;
          }
        }

        if (!propertyElem) {
          propertyElem = mDoc.createElementNS(
            mDoc.documentElement.namespaceURI, "dcase:property");
          $(propertyElem).attr("name", name);

          $(mPropertiesElem).append(propertyElem);
        }

        $(propertyElem).attr("value", value);
      };

      self.getProperty = function (name, defaultValue) {
        var i, node;
        if (mPropertiesElem) {
          for (i = 0; i < mPropertiesElem.childNodes.length; i++) {
            node = mPropertiesElem.childNodes[i];
            if (node.nodeType == 1 && node.localName == "property") {
              if ($(node).attr("name") == name) {
                return $(node).attr("value");
              }
            }
          }
        }
        return 2 <= arguments.length ? defaultValue : null;
      };

      self.removeProperty = function (name) {
        var i, node, node2;

        if (mPropertiesElem) {
          for (i = 0; i < mPropertiesElem.childNodes.length; i++) {
            node = mPropertiesElem.childNodes[i];
            if (node.nodeType == 1 && node.localName == "property" &&
                $(node).attr("name") == name) {
              $(node).remove();
            }
          }
        }
      };

      self.setAttachment = function (val) {
        self.setProperty("Attachment", val);
      };

      self.getAttachment = function () {
        return self.getProperty("Attachment");
      };

      function setUserdef001 (val) {
        self.setProperty("Userdef001", val);
      }

      function getUserdef001 (def) {
        var val = self.getProperty("Userdef001", def);
        return val ? Number(val) : val;
      }

      function setSiblingOrder (val) {
        self.setProperty("SiblingOrder", val);
      }

      function getSiblingOrder (def) {
        var val = self.getProperty("SiblingOrder", def);
        return val ? Number(val) : val;
      }

      function setUserdef002 (val) {
        self.setProperty("Userdef002", val);
      }

      function getUserdef002 () {
        return self.getProperty("Userdef002");
      }

      function setMessage (val) {
        self.setProperty("Message", val);
      }

      function getMessage () {
        return self.getProperty("Message");
      }

      function setUserdef005 (val) {
        self.setProperty("Userdef005", val);
      }

      function getUserdef005 () {
        return self.getProperty("Userdef005");
      }

      function setParameterizedDesc (val) {
        self.setProperty("ParameterizedDesc", val);
      }

      function getParameterizedDesc () {
        return self.getProperty("ParameterizedDesc");
      }

      function setParametersDCASE201006 (params) {
        var name, vals = {}, defs = {};

        if (params) {
          for (name in params) {
            vals[name] = params[name].value;
            defs[name] = params[name];
            delete defs[name].value;
          }
        }

        setUserdef007(vals);
        setUserdef009(defs);
      }

      function getParametersDCASE201006 () {
        var params, p, keys, vals, val, name;

        params = getUserdef009();
        if (!params) params = {};

        vals = getUserdef007();
        for (name in vals) {
          val = vals[name];
          if (name in params) {
            params[name].value = val;
          } else {
            params[name] = {value: val};
          }
        }

        return Object.keys(params).length == 0 ? null : params;
      }

      function setUserdef007 (vals) {
        var l = [], str;
        if (vals)
          $.each(vals, function (key, val) { l.push(key + "=" + val); });
        str = l.length == 0 ? null : l.join(",");
        self.setProperty("Userdef007", str);
      }

      function getUserdef007 () {
        var vals = {}, _vals;
        _vals = self.getProperty("Userdef007");
        if (_vals) {
          $.each(
            _vals.split(","),
            function (index, val) {
              if (val.match(/\s*([^=]+)\s*=\s*(.+)\s*/)) {
                vals[RegExp.$1] = RegExp.$2;
              }
            });
        }
        return Object.keys(vals).length == 0 ? null : vals;
      }

      function setUserdef009 (defs) {
        var names = [], _defs = [], str = null;
        if (defs) {
          $.each(
            defs,
            function (name, def) {
              var i, key, l = [], keys;

              if (!("type" in def)) return;
              if (["string", "raw", "int", "double", "enum"]
                  .indexOf(def.type) == -1) return;

              if (def.type == "double") {
                keys = ["type", "min", "max", "digit", "inc"];
              } else if (def.type == "enum") {
                keys = ["type"];
              } else {
                keys = ["type", "min", "max"];
              }

              l.push("name="+name);
              for (i = 0; i < keys.length; i++) {
                key = keys[i];
                if (key in def) {
                  l.push(key+"="+def[key]);
                }
              }

              if (def.type == "enum") {
                if (!("items" in def)) return;
                l.push("items={"+def.items.join(",")+"}");
              }

              names.push(name);
              _defs.push(l.join(","));
            });
          if (0 < names.length)
            str = [names.join(",")].concat(_defs).join(";");
        }
        self.setProperty("Userdef009", str);
      }

      function getUserdef009 () {
        var defs = {}, str, l;
        str = self.getProperty("Userdef009");
        if (str) {
          l = str.split(";");
          if (2 <= l.length) {
            $.each(
              l.slice(1),
              function (index, val) {
                var obj1 = {}, obj2 = {}, keys, name, items;

                if (val.match(/items={(.*)}/)) {
                  obj1["items"] = RegExp.$1.split(",");
                  val = val.replace(/items={.*}/, "");
                }

                $.each(
                  val.split(","),
                  function (index, val) {
                    var k, v;
                    if (val.match(/\s*([^=]+)\s*=\s*(.+)\s*/)) {
                      obj1[RegExp.$1] = RegExp.$2;
                    }
                  });

                name = obj1.name; delete obj1.name;

                if (obj1.type == "double") {
                  keys = ["type", "min", "max", "digit", "inc"];
                } else if (obj1.type == "enum") {
                  keys = ["type", "items"];
                } else {
                  keys = ["type", "min", "max"];
                }

                $.each(
                  keys,
                  function (index, key) {
                    if (key in obj1) obj2[key] = obj1[key];
                  });

                defs[name] = obj2;
              });
          }
        }
        return Object.keys(defs).length == 0 ? null : defs;
      }

      function setParameters (params) {
        var dcase;
        if (!mParameters) {
          if (!params) return;
          dcase = self instanceof DCase ? self : self.getDCase();
          mParameters = new DCaseParameters(dcase);
          $(self.getElement()).append(mParameters.getElement());
        }
        mParameters.setParameters(params);
      }

      function getParameters () {
        var params = null;
        if (mParameters)
          params = mParameters.getParameters();
        return params;
      }

      function setCMISInfo (infolist) {
        var l = [], str;
        if (infolist) {
          $.each(
            infolist,
            function (index, info) {
              l.push("{"+info.name+","+info.id+","+info.workspace+"}");
            });
        }
        str = l.length == 0 ? null : "ECM:"+l.join(",");
        self.setProperty("Userdef010", str);
      }

      function getCMISInfo () {
        var i, infolist = [], l, str;
        str = self.getProperty("Userdef010");
        if (str && str.match(/^ECM:(.*)/)) {
          l = RegExp.$1.match(/(\{[^\}]*\})/g);
          if (l) {
            $.each(
              l, function (index, val) {
                var info = {};
                if (val.match(/\{([^,]*),([^,]*),([^,]*)\}/)) {
                  info.name = RegExp.$1;
                  info.id = RegExp.$2;
                  info.workspace = RegExp.$3;
                  if (info.id.match(/^[^;]*;([0-9]*\.[0-9]*)$/))
                    info.version = RegExp.$1;
                  infolist.push(info);
                }
              });
          }
        }
        return infolist.length == 0 ? null : infolist;
      }

      function setUserdef011 (list) {
        var val = list && 0 < list.length ? list.join(";") : null;
        self.setProperty("Userdef011", val);
      }

      function getUserdef011 () {
        var str = self.getProperty("Userdef011");
        return str ? str.split(";") : [];
      }

      function setModuleRefSource (list) {
        var val = list && 0 < list.length ? list.join(";") : null;
        self.setProperty("RefSource", val);
      }

      function getModuleRefSource () {
        var str  = self.getProperty("RefSource");
        return str ? str.split(";") : [];
      }

      function setUserdef013 (val) {
        self.setProperty("Userdef013", val);
      }

      function getUserdef013 () {
        return self.getProperty("Userdef013");
      }

      function setModuleParent (val) {
        self.setProperty("Parent", val);
      }

      function getModuleParent () {
        return self.getProperty("Parent");
      }

      function setUserdef015 (val) {
        self.setProperty("Userdef015", val);
      }

      function getUserdef015 () {
        return self.getProperty("Userdef015");
      }

      function setFlag (val) {
        self.setProperty("Flag", val);
      }

      function getFlag () {
        return self.getProperty("Flag");
      }

      function setDSBenchProject (proj) {
        var l = [], key, str;
        if (proj) {
          for (key in proj) {
            l.push(key + ":" + proj[key]);
          }
        }
        str = l.length == 0 ? null : l.join(",");
        self.setProperty("Userdef016", str);
      }

      function getDSBenchProject () {
        var str, proj = {};
        str = self.getProperty("Userdef016");
        if (str) {
          $.each(
            str.split(","),
            function (index, val) {
              if (val.match(/^([^:]+):(.+)$/)) {
                proj[RegExp.$1] = RegExp.$2;
              }
            });
        }
        return Object.keys(proj).length == 0 ? null : proj;
      }

      function setValidUntil (val) {
        self.setProperty("ValidUntil", val);
      }

      function getValidUntil () {
        return self.getProperty("ValidUntil");
      }

      function setResponsibility (res) {
        var key, l, i, keys = ["name", "address", "icon"];

        if (!mResponsibilityElem) {
          mResponsibilityElem = mDoc.createElementNS(
            mDoc.documentElement.namespaceURI, "dcase:responsibility");
          $(mResponsibilityElem).appendTo(mElem);
        }
        if (res) {
          for (i = 0; i < keys.length; i++) {
            if (keys[i] in res) {
              $(mResponsibilityElem).attr(keys[i], res[keys[i]]);
            } else {
              $(mResponsibilityElem).removeAttr(keys[i]);
            }
          }
        } else {
          $(mResponsibilityElem).remove();
          mResponsibilityElem = null;
        }
      }

      function getResponsibility () {
        var res = {}, i, val, keys = ["name", "address", "icon"], valid;
        if (mResponsibilityElem) {
          valid = false;
          for (i = 0; i < keys.length; i++) {
            val = $(mResponsibilityElem).attr(keys[i]);
            if (val) valid = true;
            res[keys[i]] = val ? val : "";
          }
          if (!valid) res = null;
        }
        return res;
      }

      function setUserdef012 (res) {
        var l = [], str, i, keys = ["name", "address", "icon"];
        if (res) {
          for (i = 0; i < keys.length; i++) {
            l.push(keys[i] in res ? res[keys[i]] : "");
          }
        }
        str = l.length == 0 ? null : l.join(";");
        if (str == ";;") str = null;
        self.setProperty("Userdef012", str);
      }

      function getUserdef012 () {
        var res = {}, str, l = [], i, keys = ["name", "address", "icon"];

        str = self.getProperty("Userdef012");
        if (!str) return null;

        l = str.split(";").concat("","","").slice(0, 3);

        for (i = 0; i < keys.length; i++) {
          res[keys[i]] = l[i];
        }
        return Object.keys(res).length == 0 ? null : res;
      }

      self.onUpdate = function () {
      };

      self.convertDCASE201006toDCASE201311 = function () {
        var i, val,
        convert_table = [
          ["Userdef001", "SiblingOrder"],
          ["Userdef002", "Message"],
          ["Userdef003", "Requirement"],
          ["Userdef005", "ParameterizedDesc"],
          ["Userdef006", "script"],
          ["Userdef011", "RefSource"],
          ["Userdef013", "Parent"],
          ["Userdef014", "Interval"],
          ["Userdef015", "Flag"]
        ];

        for (i = 0; i < convert_table.length; i++) {
          val = self.getProperty(convert_table[i][0]);
          if (val && !self.getProperty(convert_table[i][1])) {
            self.setProperty(convert_table[i][0], null);
            self.setProperty(convert_table[i][1], val);
          }
        }

        val = getUserdef012();
        if (val && !getResponsibility()) {
          setUserdef012(null);
          setResponsibility(val);
        }

        val = getParametersDCASE201006();
        if (val && !getParameters()) {
          setParametersDCASE201006(null);
          setParameters(val);
        }

        if (self instanceof DCaseNode) {
          switch (self.getType()) {
          case "Userdef001": self.setType("External"); break;
          case "Userdef004": self.setType("Assumption"); break;
          case "Userdef005": self.setType("Module"); break;
          case "Userdef006": self.setType("Contract"); break;
          case "System": self.setType("Pattern"); break;
          case "Polisy": self.setType("Action"); break;
          }
        } else if (self instanceof DCaseLink) {
          switch (self.getType()) {
          case "Link001": self.setType("SupportedBy"); break;
          case "Link002": self.setType("InContextOf"); break;
          case "Link003": self.setType("Responsibility"); break;
          }
        }
      };

      function update () {
        var parser, elems, dcase;

        dcase = self instanceof DCase ? self : self.getDCase();

        parser = new deosc.util.Parser(mDoc.documentElement.namespaceURI);
        elems = parser.getChildren(mElem, "description");
        if (0 < elems.length) mDescElem = elems[0];

        elems = parser.getChildren(mElem, "properties");
        if (0 < elems.length) mPropertiesElem = elems[0];

        elems = parser.getChildren(mElem, "parameters");
        if (0 < elems.length) mParameters = new DCaseParameters(mDoc, elems[0]);

        elems = parser.getChildren(mElem, "responsibility");
        if (0 < elems.length) mResponsibilityElem = elems[0];

        self.onUpdate();
      }

    } /* DCaseCommon */


    function DCase (doc) {
      var self = this, mDoc = doc, mNodes, mLinks, mNodesElem, mLinksElem,
      mDescElem;

      DCaseCommon.call(self, mDoc ? mDoc.documentElement : null);

      self.getDCaseNodes = function () {
        return mNodes;
      };

      self.getDCaseLinks = function () {
        return mLinks;
      };

      self.addDCaseNode = function (node, index) {
        if (typeof index == "number" && index < mNodes.length) {
          $(mNodes[index].getElement()).before(node.getElement());
          mNodes.splice(index, 0, node);
        } else {
          $(mNodesElem).append(node.getElement());
          mNodes.push(node);
        }
      };

      self.addDCaseLink = function (link, index) {
        if (typeof index == "number" && index < mLinks.length) {
          $(mLinks[index].getElement()).before(link.getElement());
          mLinks.splice(index, 0, link);
        } else {
          $(mLinksElem).append(link.getElement());
          mLinks.push(link);
        }
      };

      self.removeDCaseNode = function (node) {
        var i, index, elem;

        $(node.getElement()).remove();

        index = mNodes.indexOf(node);
        if (index != -1) mNodes.splice(index, 1);
      };

      self.removeDCaseLink = function (link) {
        var i, index, elem;

        $(link.getElement()).remove();

        index = mLinks.indexOf(link);
        if (index != -1) mLinks.splice(index, 1);
      };

      self.getParentNodes = function (node) {
        var links, i, nodes = [];

        links = self.findDCaseLinksByTarget(node.getID());

        for (i = 0; i < links.length; i++) {
          nodes.push(self.findDCaseNodeByID(links[i].getSource()));
        }

        return nodes;
      };

      self.getChildNodes = function (node) {
        var i, links, nodes = [], defaultOrder = {};

        links = self.findDCaseLinksBySource(node.getID());

        for (i = 0; i < links.length; i++) {
          defaultOrder[links[i].getID()] = i;
        }

        links.sort(
          function (l1, l2) {
            var o1, o2;
            o1 = l1.getSiblingOrder(Number.MAX_VALUE);
            o2 = l2.getSiblingOrder(Number.MAX_VALUE);
            if (o1 == o2) {
              return defaultOrder[l1.getID()] - defaultOrder[l2.getID()];
            } else {
              return o1 - o2;
            }
          });

        for (i = 0; i < links.length; i++) {
          nodes.push(self.findDCaseNodeByID(links[i].getTarget()));
        }

        return nodes;
      };

      self.createElement = function (tag) {
        return mDoc.createElementNS(
          mDoc.documentElement.namespaceURI, "dcase:"+tag);
      };

      self.findDCaseNodeByID = function (id) {
        var i;
        for (i = 0; i < mNodes.length; i++) {
          if (id == mNodes[i].getID()) return mNodes[i];
        }
        return null;
      };

      self.findDCaseNodesByName = function (name) {
        var i, node, ret = [];
        for (i = 0; i < mNodes.length; i++) {
          node = mNodes[i];
          if (name == node.getName()) ret.push(node);
        }
        return ret;
      };

      self.findDCaseLinkByID = function (id) {
        var i;
        for (i = 0; i < mLinks.length; i++) {
          if (id == mLinks[i].getID()) return mLinks[i];
        }
        return null;
      };

      self.findDCaseLinksByName = function (name) {
        var i, link, ret = [];
        for (i = 0; i < mLinks.length; i++) {
          link = mLinks[i];
          if (name == link.getName()) ret.push(link);
        }
        return ret;
      };

      self.findDCaseLinksBySource = function (source) {
        var i, ret = [];
        for (i = 0; i < mLinks.length; i++) {
          if (source == mLinks[i].getSource()) ret.push(mLinks[i]);
        }
        return ret;
      };

      self.findDCaseLinksByTarget = function (target) {
        var i, ret = [];
        for (i = 0; i < mLinks.length; i++) {
          if (target == mLinks[i].getTarget()) ret.push(mLinks[i]);
        }
        return ret;
      };

      self.getRootNodes = function () {
        var rootNodes = [], i, links, nodes = [], loopedNode;

        for (i = 0; i < mNodes.length; i++) {
          links = self.findDCaseLinksByTarget(mNodes[i].getID());
          if (links.length == 0) rootNodes.push(mNodes[i]);
          nodes.push(mNodes[i]);
        }

        for (i = 0; i < rootNodes.length; i++) {
          rootNodes[i].scanDescendant(
            function () {
              var index = nodes.indexOf(this);
              if (index != -1) nodes.splice(index, 1);
            });
        }

        while (0 < nodes.length) {
          loopedNode = null;
          for (i = 0; i < nodes.length; i++) {
            if (nodes[i].isLoop()) {
              loopedNode = nodes[i];
              break;
            }
          }

          if (!loopedNode) break;

          rootNodes.push(loopedNode);

          loopedNode.scanDescendant(
            function () {
              var index = nodes.indexOf(this);
              if (index != -1) nodes.splice(index, 1);
            });
        }

        rootNodes.sort(
          function (n1, n2) {
            return mNodes.indexOf(n1) - mNodes.indexOf(n2);
          });

        return rootNodes;
      };

      self.getNewID = function () {
        return deosc.Util.getUID();
      };

      self.getNewName = function (prefix) {
        var items = mNodes.concat(mLinks);
        var exp = "^" + prefix + "([0-9]+)$";
        var re = new RegExp(exp);
        var max = 0;
        for (var i = 0; i < items.length; i++) {
          if (items[i].getName().match(re)) {
            var n = RegExp.$1;
            n = parseInt(n);
            if (max < n) max = n;
          }
        }
        return prefix + ++max;
      };


      self.foreach = function (node, callback) {
        var nodes = [], rnodes, i;

        function isLoop (node) {
          var i, id  = node.getID();
          for (i = 0; i < nodes.length; i++) {
            if (id == nodes[i].getID()) return true;
          }
          return false;
        }

        function scanNode (node) {
          var i, links;
          if (isLoop(node)) return true;

          if (callback.call(node) === false) return false;

          nodes.push(node);

          links = self.findDCaseLinksBySource(node.getID());

          for (i = 0; i < links.length; i++) {
            if (!scanLink(links[i])) return false;
          }

          return true;
        }

        function scanLink (link) {
          var node;

          if (callback.call(link) === false) return false;

          node = self.findDCaseNodeByID(link.getTarget());
          if (node) {
            if (!scanNode(node)) return false;
          }

          return true;
        }

        if (!node) {
          rnodes = self.getRootNodes();
          if (rnodes.length == 0) return false;
          for (i = 0; i < rnodes.length; i++) {
            if (!scanNode(rnodes[i])) return false;
          }
          return true;
        } else {
          return scanNode(node);
        }
      };

      self.isLoop = function (node) {
        var scanedNodes = [];

        function isLoop (node) {
          return scanedNodes.indexOf(node.getID()) != -1;
        }

        function scan (node) {
          var i, children;

          if (isLoop(node)) return false;

          scanedNodes.push(node.getID());

          children = node.getChildNodes();

          for (i = 0; i < children.length; i++) {
            if (!scan(children[i])) return false;
          }

          return true;
        }

        if (!node) return false;

        return !scan(node);
      };

      self.scanDescendant = function (node, callback) {
        var scanedNodes = [];

        function isLoop (node) {
          return scanedNodes.indexOf(node.getID()) != -1;
        }

        function scan (node) {
          var i, children;

          if (isLoop(node)) return true;

          if (callback.call(node) === false) return false;

          scanedNodes.push(node.getID());

          children = node.getChildNodes();

          for (i = 0; i < children.length; i++) {
            if (!scan(children[i])) return false;
          }

          return true;
        }

        if (node) scan(node);
      };

      self.scanAncestor = function (node, callback) {
        var scanedNodes = [];

        function isLoop (node) {
          return scanedNodes.indexOf(node.getID()) != -1;
        }

        function scan (node) {
          var i, parents;

          if (isLoop(node)) return true;

          if (callback.call(node) === false) return false;

          scanedNodes.push(node.getID());

          parents = node.getParentNodes();

          for (i = 0; i < parents.length; i++) {
            if (!scan(parents[i])) return false;
          }

          return true;
        }

        if (node) scan(node);
      };

      function constructor () {
        var elem, propertiesElem, descElem, nodesElem, linksElem, i;

        mNodes = [];
        mLinks = [];

        if (!mDoc) {
          mDoc = document.implementation.createDocument(
            deosc.Namespace.DCASE201311, "dcase:dcase", null);

          elem = mDoc.documentElement;

          elem.setAttributeNS(
            deosc.Namespace.xmlns, "xmlns:dcase", deosc.Namespace.DCASE201311);
          elem.setAttributeNS(
            deosc.Namespace.xmlns, "xmlns:dre", deosc.Namespace.DRE);
          elem.setAttribute("id", self.getNewID());

          descElem = self.createElement("description");
          propertiesElem = self.createElement("properties");
          nodesElem = self.createElement("nodes");
          linksElem = self.createElement("links");

          $(elem)
            .append(descElem)
            .append(propertiesElem)
            .append(nodesElem)
            .append(linksElem);

          self.setElement(elem);
        } else {
          self.setElement(mDoc.documentElement);
          if (mDoc.documentElement.namespaceURI
              == deosc.Namespace.DCASE201311) {
            self.convertDCASE201006toDCASE201311();
            for (i = 0; i < mNodes.length; i++) {
              mNodes[i].convertDCASE201006toDCASE201311();
            }
            for (i = 0; i < mLinks.length; i++) {
              mLinks[i].convertDCASE201006toDCASE201311();
            }
          }
        }
      }

      self.onUpdate = function() {
        var dcaseElem, nodesElems, linksElems, nodeElems, linkElems,
        i, parser;

        parser = new deosc.util.Parser(mDoc.documentElement.namespaceURI);

        dcaseElem = self.getElement();

        nodesElems = parser.getChildren(dcaseElem, "nodes");
        if (0 < nodesElems.length) {
          mNodesElem = nodesElems[0];
          nodeElems = parser.getChildren(mNodesElem, "node");

          mNodes.length = 0;
          for (i = 0; i < nodeElems.length; i++) {
            mNodes.push(new DCaseNode(self, nodeElems[i]));
          }
        } else {
          mNodesElem = null;
        }

        linksElems = parser.getChildren(dcaseElem, "links");
        if (0 < linksElems.length) {
          mLinksElem = linksElems[0];
          linkElems = parser.getChildren(mLinksElem, "link");

          mLinks.length = 0;
          for (i = 0; i < linkElems.length; i++) {
            mLinks.push(new DCaseLink(self, linkElems[i]));
          }
        } else {
          mLinksElem = null;
        }

        debug();
      };

      self.format = function () {
        var i, j, root, elems, es,
        ns = mDoc.documentElement.namespaceURI,
        ns_dre = deosc.Namespace.DRE,
        dcase_order = [
          {ns: ns, tag: "description"},
          {ns: ns, tag: "properties"},
          {ns: ns, tag: "parameters"},
          {ns: ns, tag: "responsibility"},
          {ns: ns, tag: "nodes"},
          {ns: ns, tag: "links"}],
        nodes_order = [{ns: ns, tag: "node"}],
        links_order = [{ns: ns, tag: "link"}],
        node_order = [
          {ns: ns, tag: "description"},
          {ns: ns, tag: "properties"},
          {ns: ns, tag: "parameters"},
          {ns: ns, tag: "responsibility"},
          {ns: ns_dre, tag: "d-script"}],
        link_order = [
          {ns: ns, tag: "description"},
          {ns: ns, tag: "properties"}],
        params_order = [{ns: ns, tag: "parameter"}],
        props_order = [
          {ns: ns, tag: "property", attrs: [{tag: "name", text: "Userdef001"}]},
          {ns: ns, tag: "property", attrs: [{tag: "name", text: "Userdef002"}]},
          {ns: ns, tag: "property", attrs: [{tag: "name", text: "Userdef003"}]},
          {ns: ns, tag: "property", attrs: [{tag: "name", text: "Userdef004"}]},
          {ns: ns, tag: "property", attrs: [{tag: "name", text: "Userdef005"}]},
          {ns: ns, tag: "property", attrs: [{tag: "name", text: "Userdef006"}]},
          {ns: ns, tag: "property", attrs: [{tag: "name", text: "Userdef007"}]},
          {ns: ns, tag: "property", attrs: [{tag: "name", text: "Userdef008"}]},
          {ns: ns, tag: "property", attrs: [{tag: "name", text: "Userdef009"}]},
          {ns: ns, tag: "property", attrs: [{tag: "name", text: "Userdef010"}]},
          {ns: ns, tag: "property", attrs: [{tag: "name", text: "Userdef011"}]},
          {ns: ns, tag: "property", attrs: [{tag: "name", text: "Userdef012"}]},
          {ns: ns, tag: "property", attrs: [{tag: "name", text: "Userdef013"}]},
          {ns: ns, tag: "property", attrs: [{tag: "name", text: "Userdef014"}]},
          {ns: ns, tag: "property", attrs: [{tag: "name", text: "Userdef015"}]},
          {ns: ns, tag: "property", attrs: [{tag: "name", text: "Userdef016"}]},
          {ns: ns, tag: "property", attrs: [{tag: "name", text: "Attachment"}]},
          {ns: ns, tag: "property", attrs: [{tag: "name", text: "Message"}]},
          {ns: ns, tag: "property", attrs: [{tag: "name", text: "Requirement"}]},
          {ns: ns, tag: "property", attrs: [{tag: "name", text: "ParameterizedDesc"}]},
          {ns: ns, tag: "property", attrs: [{tag: "name", text: "Parent"}]},
          {ns: ns, tag: "property", attrs: [{tag: "name", text: "RefSource"}]},
          {ns: ns, tag: "property", attrs: [{tag: "name", text: "Flag"}]},
          {ns: ns, tag: "property", attrs: [{tag: "name", text: "Interval"}]},
          {ns: ns, tag: "property", attrs: [{tag: "name", text: "Score"}]},
          {ns: ns, tag: "property", attrs: [{tag: "name", text: "Weight"}]},
          {ns: ns, tag: "property", attrs: [{tag: "name", text: "Stakeholder"}]},
          {ns: ns, tag: "property", attrs: [{tag: "name", text: "RiskAnalysis"}]},
          {ns: ns, tag: "property", attrs: [{tag: "name", text: "IsNormal"}]},
          {ns: ns, tag: "property", attrs: [{tag: "name", text: "LeafNode"}]},
          {ns: ns, tag: "property", attrs: [{tag: "name", text: "I"}]},
          {ns: ns, tag: "property", attrs: [{tag: "name", text: "J"}]},
          {ns: ns, tag: "property", attrs: [{tag: "name", text: "SiblingOrder"}]},
          {ns: ns, tag: "property", attrs: [{tag: "name", text: "ValidUntil"}]}
        ],
        node_attr_order = [
          {tag: "type"}, {tag: "id"}, {tag: "name"}, {tag: "subtype"},
          {tag: "status"}
        ],
        link_attr_order = [
          {tag: "type"}, {tag: "id"}, {tag: "name"},
          {tag: "source"}, {tag: "target"}, {tag: "status"}
        ],
        prop_attr_order = [
          {tag: "name"}, {tag: "value"}
        ],
        param_attr_order = [
          {tag: "type"}, {tag: "min"}, {tag: "max"}, {tag: "digit"},
          {tag: "inc"}, {tag: "items"}, {tag: "value"}
        ],
        res_attr_order = [
          {tag: "name"}, {tag: "address"}, {tag: "icon"}
        ],
        dscript_order = [
          {ns: ns_dre, tag: "full-name"},
          {ns: ns_dre, tag: "values"}],
        dscript_values_order = [{ns: ns_dre, tag: "value"}],
        dscript_value_attr_order = [{tag: "name"}];

        function sortChildNodes (elem, func) {
          var i, list = [], def = [];
          for (i = 0; i < elem.childNodes.length; i++) {
            list.push(elem.childNodes[i]);
            def.push(elem.childNodes[i]);
          }
          list.sort(
            function (a, b) {
              var ret = func(a, b);
              return ret == 0 ? def.indexOf(a) - def.indexOf(b) : ret;
            });
          for (i = 0; i < list.length; i++) elem.appendChild(list[i]);
        }

        function sortAttributes (elem, func) {
          var i, list = [];
          for (i = 0; i < elem.attributes.length; i++)
            list.push(elem.attributes[i]);
          list.sort(func);
          for (i = 0; i < list.length; i++) elem.removeAttributeNode(list[i]);
          for (i = 0; i < list.length; i++) elem.setAttributeNode(list[i]);
        }

        function orderIndex (order, e) {
          var i, j, k, c, attr1, attr2, match;
          if (e.nodeType != 1) return Infinity;
          for (i = 0; i < order.length; i++) {
            c = order[i];
            if (c.ns != e.namespaceURI) continue;
            if (c.tag != e.localName) continue;
            if ("attrs" in c) {
              for (j = 0; j < c.attrs.length; j++) {
                attr1 = c.attrs[j];
                attr2 = e.getAttributeNode(attr1.tag);
                match = false;
                if (!attr2) break;
                if (attr1.text != attr2.nodeValue) break;
                match = true;
              }
              if (!match) continue;
            }
            return i;
          }
          return Infinity;
        }

        function attrOrderIndex (order, e) {
          var i, j, k, c, attr1, attr2, match;
          if (e.nodeType != 2) return Infinity;
          for (i = 0; i < order.length; i++) {
            c = order[i];
            if (c.tag != e.localName) continue;
            return i;
          }
          return Infinity;
        }

        function children (e, ns, tag) {
          var i, c, ret = [];
          for (i = 0; i < e.childNodes.length; i++) {
            c = e.childNodes[i];
            if (ns != c.namespaceURI) continue;
            if (tag != c.localName) continue;
            ret.push(c);
          }
          return ret;
        }

        function sort_dcase (c1, c2) {
          return orderIndex(dcase_order, c1) - orderIndex(dcase_order, c2);
        }

        function sort_props (c1, c2) {
          return orderIndex(props_order, c1) - orderIndex(props_order, c2);
        }

        function sort_params (c1, c2) {
          return orderIndex(params_order, c1) - orderIndex(params_order, c2);
        }

        function sort_nodes (c1, c2) {
          return orderIndex(nodes_order, c1) - orderIndex(nodes_order, c2);
        }

        function sort_links (c1, c2) {
          return orderIndex(links_order, c1) - orderIndex(links_order, c2);
        }

        function sort_node (c1, c2) {
          return orderIndex(node_order, c1) - orderIndex(node_order, c2);
        }

        function sort_link (c1, c2) {
          return orderIndex(link_order, c1) - orderIndex(link_order, c2);
        }

        function sort_node_attr (c1, c2) {
          return attrOrderIndex(node_attr_order, c1)
            - attrOrderIndex(node_attr_order, c2);
        }

        function sort_link_attr (c1, c2) {
          return attrOrderIndex(link_attr_order, c1)
            - attrOrderIndex(link_attr_order, c2);
        }

        function sort_prop_attr (c1, c2) {
          return attrOrderIndex(prop_attr_order, c1)
            - attrOrderIndex(prop_attr_order, c2);
        }

        function sort_param_attr (c1, c2) {
          return attrOrderIndex(param_attr_order, c1)
            - attrOrderIndex(param_attr_order, c2);
        }

        function sort_res_attr (c1, c2) {
          return attrOrderIndex(res_attr_order, c1)
            - attrOrderIndex(res_attr_order, c2);
        }

        function sort_dscript (c1, c2) {
          return orderIndex(dscript_order, c1) - orderIndex(dscript_order, c2);
        }

        function sort_dscript_values (c1, c2) {
          return orderIndex(dscript_values_order, c1)
            - orderIndex(dscript_values_order, c2);
        }

        function sort_dscript_values_attr (c1, c2) {
          return attrOrderIndex(dscript_value_attr_order, c1)
            - attrOrderIndex(dscript_value_attr_order, c2);
        }

        function find_and_format_properties (e) {
          var elems, es, i, j;
          elems = children(e, ns, "properties");
          for (i = 0; i < elems.length; i++) {
            sortChildNodes(elems[i], sort_props);
            es = children(elems[i], ns, "property");
            for (j = 0; j < es.length; j++)
              sortAttributes(es[j], sort_prop_attr);
          }
        }

        function find_and_format_parameters (e) {
          var elems, es, i, j;
          elems = children(e, ns, "parameters");
          for (i = 0; i < elems.length; i++) {
            sortChildNodes(elems[i], sort_params);
            es = children(elems[i], ns, "parameter");
            for (j = 0; j < es.length; j++)
              sortAttributes(es[j], sort_param_attr);
          }
        }

        function find_and_format_responsibility (e) {
          var elems, i;
          elems = children(e, ns, "responsibility");
          for (i = 0; i < elems.length; i++)
            sortAttributes(elems[i], sort_res_attr);
        }

        function find_and_format_dscript (e) {
          var elems, i;
          function find_and_format_values (e) {
            var elems, i, j;
            elems = children(e, ns_dre, "values");
            for (i = 0; i < elems.length; i++) {
              sortChildNodes(elems[i], sort_dscript_values);
              es = children(elems[i], ns_dre, "value");
              for (j = 0; j < es.length; j++)
                sortAttributes(es[j], sort_dscript_values_attr);
            }
          }
          elems = children(e, ns_dre, "d-script");
          for (i = 0; i < elems.length; i++) {
            sortChildNodes(elems[i], sort_dscript);
            find_and_format_values(elems[i]);
          }
        }

        //dcase
        sortChildNodes(self.getElement(), sort_dcase);
        find_and_format_properties(self.getElement());
        find_and_format_parameters(self.getElement());
        find_and_format_responsibility(self.getElement());

        //dcase/nodes
        elems = children(self.getElement(), ns, "nodes");
        for (i = 0; i < elems.length; i++)
          sortChildNodes(elems[i], sort_nodes);

        //dcase/links
        elems = children(self.getElement(), ns, "links");
        for (i = 0; i < elems.length; i++)
          sortChildNodes(elems[i], sort_links);

        for (i = 0; i < mNodes.length; i++) {
          //dcase/nodes/node
          sortChildNodes(mNodes[i].getElement(), sort_node);
          sortAttributes(mNodes[i].getElement(), sort_node_attr);
          find_and_format_properties(mNodes[i].getElement());
          find_and_format_parameters(mNodes[i].getElement());
          find_and_format_responsibility(mNodes[i].getElement());
          find_and_format_dscript(mNodes[i].getElement());
        }

        for (i = 0; i < mLinks.length; i++) {
          //dcase/links/link
          sortChildNodes(mLinks[i].getElement(), sort_link);
          sortAttributes(mLinks[i].getElement(), sort_link_attr);
          find_and_format_properties(mLinks[i].getElement());
          find_and_format_parameters(mLinks[i].getElement());
        }
      };

      self.toXML = function () {
        var serializer, xml;
        serializer = new XMLSerializer();
        xml = serializer.serializeToString(self.getElement());
        return '<?xml version="1.0" encoding="UTF-8"?>\n'+xml;
      };

      self.toFormatXML = function () {
        var xml;
        self.format();
        xml = self.toXML();
        xml = xml.replace(/>[\n\s]*</g, ">\n<");
        return xml;
      };

      function indexOf (childNodes, childNode) {
        var i;
        for (i = 0; i < childNodes.length; i++)
          if (childNodes[i] == childNode) return i;
        return -1;
      }

      function debug () {
        self.debug = {
          doc: mDoc,
          nodes: mNodes,
          links: mLinks
        };
      }

      constructor();
    } /* D-Case */


    $.extend(
      DCase,
      {
        NODE_TYPE: [
          "Goal", "Evidence", "Strategy", "Context", "Justification",
          "Assumption", "Undeveloped", "Module", "Contract", "Monitor",
          "Parameter", "Action", "External"
        ],
        SELECTABLE_NODE_TYPE: {
          Goal: ["Evidence", "Strategy", "Context", "Justification", "Assumption", "Undeveloped", "Module", "Contract", "Monitor", "Parameter", "Action", "External"],
          Evidence: ["Context", "Assumption", "Module", "Contract", "Parameter", "Action", "External"],
          Strategy: ["Goal", "Context", "Assumption", "Module", "Contract", "Parameter", "Action", "External"],
          Context: ["Assumption", "Module", "Contract", "Action", "External"],
          Justification: ["Assumption", "Module", "Contract", "Action", "External"],
          Assumption: ["Goal", "Evidence", "Strategy", "Context", "Justification", "Assumption", "Undeveloped", "Module", "Contract", "Monitor", "Parameter", "Action", "External"],
          Undeveloped: ["Assumption", "Module", "Contract", "Action", "External"],
          Module: ["Goal", "Evidence", "Strategy", "Context", "Justification", "Assumption", "Undeveloped", "Module", "Contract", "Monitor", "Parameter", "Action", "External"],
          Contract: ["Goal", "Evidence", "Strategy", "Context", "Justification", "Assumption", "Undeveloped", "Module", "Contract", "Monitor", "Parameter", "Action", "External"],
          Monitor: ["Context", "Assumption", "Module", "Contract", "Parameter", "Action", "External"],
          Parameter: ["Assumption", "Module", "Contract", "Action", "External"],
          Action: ["Goal", "Evidence", "Strategy", "Context", "Justification", "Assumption", "Undeveloped", "Module", "Contract", "Monitor", "Parameter", "Action", "External"],
          External: ["Goal", "Evidence", "Strategy", "Context", "Justification", "Assumption", "Undeveloped", "Module", "Contract", "Monitor", "Parameter", "Action", "External"]
        },
        NODE_NAME_PREFIX: {
          Goal: "G_", Evidence: "E_", Strategy: "S_", Context: "C_", Justification: "J_", Assumption: "A_", Undeveloped: "U_", Module: "D_", Contract: "Cr_",
          Monitor: "M_", Parameter: "P_", Action: "Ac_", External: "Ex_"
        },
        LINK_NAME_PREFIX: "LINK_",
        getNodeNamePrefix: function (type) {
          if (type in DCase.NODE_NAME_PREFIX) {
            return DCase.NODE_NAME_PREFIX[type];
          } else {
            return type;
          }
        }
      }
    );


    function DCaseNode (dcase, elem) {
      var self = this, mDCase, mDescElem, mPropertiesElem;

      DCaseCommon.call(self, elem);

      self.setDCase = function (dcase) {
        mDCase = dcase;
      };

      self.getDCase = function () {
        return mDCase;
      };

      self.setType = function (type) {
        self.setSubType(null);
        if (type == "Parameter") {
          switch (mDCase.getNamespace()) {
          case deosc.Namespace.DCASE201006:
            type = "System";
            $(self.getElement()).attr("type", "System");
            break;
          case deosc.Namespace.DCASE201311:
          default:
            $(self.getElement()).attr("type", "Pattern");
            self.setSubType("Parameter");
            break;
          }
        } else {
          $(self.getElement()).attr("type", type);
        }
      };

      self.getType = function () {
        var type = $(self.getElement()).attr("type");
        if (type == "System" || (type == "Pattern" && self.getSubType() == "Parameter")) {
          type = "Parameter";
        }
        return type;
      };

      self.setSubType = function (type) {
        $(self.getElement()).attr("subtype", type);
      };

      self.getSubType = function () {
        return $(self.getElement()).attr("subtype");
      };

      self.setAttributes = function (parameters) {
        var map = [
          {key: "id", func: "setID", def: ""},
          {key: "name", func: "setName", def: ""},
          {key: "type", func: "setType", def: ""},
          {key: "description", func: "setDescription", def: ""},
          {key: "attachment", func: "setAttachment"},
          {key: "message", func: "setMessage"},
          {key: "parameterizedDesc", func: "setParameterizedDesc"},
          {key: "parameters", func: "setParameters"},
          {key: "cmisInfo", func: "setCMISInfo"},
          {key: "moduleRefSource", func: "setModuleRefSource"},
          {key: "moduleParent", func: "setModuleParent"},
          {key: "flag", func: "setFlag"},
          {key: "dsBenchProject", func: "setDSBenchProject"},
          {key: "validUntil", func: "setValidUntil"},
          {key: "responsibility", func: "setResponsibility"}
        ], i, m;

        for (i = 0; i < map.length; i++) {
          m = map[i];
          if (m.key in parameters) {
            self[m.func](parameters[m.key]);
          } else {
            self[m.func](m.def);
          }
        }
      };

      self.getAttributes = function () {
        var map = [
          {key: "id", func: "getID"},
          {key: "name", func: "getName"},
          {key: "type", func: "getType"},
          {key: "description", func: "getDescription"},
          {key: "attachment", func: "getAttachment"},
          {key: "message", func: "getMessage"},
          {key: "parameterizedDesc", func: "getParameterizedDesc"},
          {key: "parameters", func: "getParameters"},
          {key: "cmisInfo", func: "getCMISInfo"},
          {key: "moduleRefSource", func: "getModuleRefSource"},
          {key: "moduleParent", func: "getModuleParent"},
          {key: "flag", func: "getFlag"},
          {key: "dsBenchProject", func: "getDSBenchProject"},
          {key: "validUntil", func: "getValidUntil"},
          {key: "responsibility", func: "getResponsibility"}
        ], i, m, val, ret = {};

        for (i = 0; i < map.length; i++) {
          m = map[i];
          val = self[m.func]();
          if (val != null) {
            ret[m.key] = val;
          }
        }

        return ret;
      };

      self.getParentNodes = function () {
        return mDCase ? mDCase.getParentNodes(self) : [];
      };

      self.getChildNodes = function () {
        return mDCase ? mDCase.getChildNodes(self) : [];
      };

      self.isLoop = function () {
        return mDCase ? mDCase.isLoop(self) : false;
      };

      self.scanDescendant = function (callback) {
        if (mDCase) mDCase.scanDescendant(self, callback);
      };

      self.scanAncestor = function (callback) {
        if (mDCase) mDCase.scanAncestor(self, callback);
      };

      function constructor (dcase, elem) {
        self.setDCase(dcase);
        if (!elem) {
          elem = dcase.createElement("node");
        }
        self.setElement(elem);
      }

      self.onUpdate = function () {
      };

      constructor(dcase, elem);
    } /* DCaseNode */


    function DCaseLink (dcase, elem) {
      var self = this, mDCase;

      DCaseCommon.call(self, elem);

      self.setDCase = function (dcase) {
        mDCase = dcase;
      };

      self.getDCase = function () {
        return mDCase;
      };

      self.setType = function (type) {
        $(self.getElement()).attr("type", type);
      };

      self.getType = function () {
        return $(self.getElement()).attr("type");
      };

      self.setSource = function (source) {
        $(self.getElement()).attr("source", source);
      };

      self.getSource = function () {
        return $(self.getElement()).attr("source");
      };

      self.setTarget = function (target) {
        $(self.getElement()).attr("target", target);
      };

      self.getTarget = function () {
        return $(self.getElement()).attr("target");
      };

      self.setAttributes = function (attrs) {
        var l = [
          {key: "id", func: "setID", def: ""},
          {key: "name", func: "setName", def: ""},
          {key: "type", func: "setType", def: ""},
          {key: "source", func: "setSource"},
          {key: "target", func: "setTarget"},
          {key: "attachment", func: "setAttachment"},
          {key: "siblingOrder", func: "setSiblingOrder"}
        ], i, o;

        for (i = 0; i < l.length; i++) {
          o = l[i];
          if (o.key in attrs) {
            self[o.func](attrs[o.key]);
          } else {
            self[o.func](o.def);
          }
        }
      };

      self.getAttributes = function () {
        var l = [
          {key: "id", func: "getID"},
          {key: "name", func: "getName"},
          {key: "type", func: "getType"},
          {key: "source", func: "getSource"},
          {key: "target", func: "getTarget"},
          {key: "attachment", func: "getAttachment"},
          {key: "siblingOrder", func: "getSiblingOrder"}
        ], i, o, val, ret = {};

        for (i = 0; i < l.length; i++) {
          o = l[i];
          val = self[o.func]();
          if (val != null) {
            ret[o.key] = val;
          }
        }

        return ret;
      };

      function constructor (dcase, elem) {
        self.setDCase(dcase);
        if (!elem) {
          elem = mDCase.createElement("link");
        }
        self.setElement(elem);
      }

      constructor(dcase, elem);
    } /* DCaseLink */

    DCaseLink.LINK_TYPE = {
      Goal: "SupportedBy", Evidence: "SupportedBy", Strategy: "SupportedBy",
      Context: "InContextOf", Justification: "InContextOf",
      Assumption: "InContextOf", Undeveloped: "SupportedBy",
      Module: "SupportedBy", Contract: "SupportedBy",
      Monitor: "SupportedBy", Parameter: "InContextOf", Pattern: "InContextOf",
      Action: "SupportedBy", External: "SupportedBy"
    };

    DCaseLink.getLinkType = function (target) {
      if (target in DCaseLink.LINK_TYPE) {
        return DCaseLink.LINK_TYPE[target];
      } else {
        return "SupportedBy";
      }
    };

    window.deosc.DCase = DCase;
    window.deosc.DCaseNode = DCaseNode;
    window.deosc.DCaseLink = DCaseLink;
  }
) (window);
(
  function (window) {
    function ToolBarItem () {
      var self = this, mID;

      deosc.View.call(self);

      self.setID = function (id) {
        mID = id;
      };

      self.getID = function () {
        return mID;
      };

      self.onAttachedToWindow = function () {
        var max = 0, i, elem, h, padding, height, parentH;

        parentH = $(self.getParent().getElement()).innerHeight();

        for (i = 0; i < self.getChildCount(); i++) {
          h = $(self.getChildAt(i).getElement()).outerHeight(true);
          max = max < h ? h : max;
        }

        padding = Math.floor((parentH - max) / 2);

        $(self.getElement()).css(
          {
            "padding-top": padding,
            "padding-bottom": parentH - max - padding,
            height: max
          });
        $(self.getElement()).css({"padding-bottom": padding});

      };

      function constructor () {
        $(self.getElement())
          .css(
            {
              display: "inline-block",
              padding: "0 8px",
              "border-left": "1px solid #fff",
              "border-right": "1px solid #ccc",
              "vertical-align": "middle"
            });
      }

      constructor();
    }

    function ToolBarItemPrototype () {}
    ToolBarItemPrototype.prototype = deosc.View.prototype;
    ToolBarItem.prototype = new ToolBarItemPrototype;
    ToolBarItem.constructor = ToolBarItem;


    function ToolBar () {
      var self = this, mTID;

      deosc.View.call(self);

      self.getChildByID = function (id) {
        var i, w;
        for (i = 0; i < self.getChildCount(); i++) {
          w = self.getChildAt(i);
          if (w.getID() == id) return w;
        }
        return null;
      };

      self.addItem = function (item) {
        self.addChild(item);
      };

      self.addItems = function (items) {
        var i;
        for (i = 0; i < items.length; i++) {
          self.addItem(items[i]);
        }
      };

      self.requestLayout = function () {
        if (!mTID) {
          mTID = setTimeout(
            function () {
              self.layout();
              mTID = null;
            }, 0);
        }
      };

      function constructor () {
        $(self.getElement())
          .addClass("tool-bar")
          .css(
            {
              "background-color": "#eee",
              "border-top": "1px solid #fff",
              "border-bottom": "1px solid #aaa",
              position: "relative",
              height: 38
            });
      }

      constructor();
    }

    function ToolBarPrototype () {}
    ToolBarPrototype.prototype = deosc.View.prototype;
    ToolBar.prototype = new ToolBarPrototype;
    ToolBar.constructor = ToolBar;


    window.deosc.ToolBar = ToolBar;
    window.deosc.ToolBarItem = ToolBarItem;
  }
) (window);(
  function (window) {

    function Preferences () {
      var self = this, mWeaver, mName, mStorage;

      self.setDCaseWeaver = function (weaver) {
        mWeaver = weaver;
      };

      self.getPreference = function (name) {
        return new Preference(self, name);
      };

      self.setValue = function (name, key, value) {
        mStorage = getStorage();
        if (name in mStorage) {
          mStorage[name][key] = value;
        } else {
          mStorage[name] = {};
          mStorage[name][key] = value;
        }
        window.localStorage.preference = JSON.stringify(mStorage);
      };

      self.getValue = function (name, key, defaultValue) {
        mStorage = getStorage();
        if (name in mStorage && key in mStorage[name])
          return mStorage[name][key];

        return defaultValue ? defaultValue : null;
      };

      self.onCreate = function () {
        var menuItem, menuBarItem;

        menuItem = new deosc.MenuItem("Preferences", onPreferences);
        menuBarItem = mWeaver.getMenuBar().getMenuBarItem("Edit");
        menuBarItem.addMenuItem(menuItem);
      };

      function getStorage () {
        try {
          return JSON.parse(localStorage.preference);
        } catch (e) {
          return {};
        }
      }

      function onPreferences () {
        var dialog;

        dialog = new PreferenceDialog(mWeaver);
        mWeaver.sendEvent(
          {
            what: deosc.DCaseWeaver.Event.OPEN_PREFERENCE_DIALOG,
            data: dialog
          });
        setTimeout(
          function () {
            mWeaver.dialogPopup(dialog);
          }, 1);
      }
    }

    function Preference (prefs, name) {
      var self = this, mPreferences, mName;

      self.setValue = function (key, value) {
        mPreferences.setValue(mName, key, value);
      };

      self.getValue = function (key, defaultValue) {
        return mPreferences.getValue(mName, key, defaultValue);
      };

      self.getName = function () {
        return mName;
      };

      function constructor (prefs, name) {
        mPreferences = prefs;
        mName = name;
      }

      constructor(prefs, name);
    }


    function PreferenceDialog (weaver) {
      var self = this, mWeaver = weaver, mStorage, mMain, mListeners = [];

      deosc.Dialog.call(self);

      self.addView = function (view, pref) {
        var name;

        view.setPadding("0 0 8px 32px");

        name = new deosc.View();
        name.setText(pref.getName());
        name.setPadding("8px 8px");

        mMain.addChild(name);
        mMain.addChild(view);
      };

      self.getPreference = function (name) {
        return {
          getName: function () {
            return name;
          },
          setValue: function (key, value) {
            if (!(name in mStorage)) {
              mStorage[name] = {};
            }
            mStorage[name][key] = value;
          },
          getValue: function (key, def) {
            var val;
            if (name in mStorage && key in mStorage[name])
              val = mStorage[name][key];
            return val ? val : def ? def : null;
          }
        };
      };

      self.setOnSaveListener = function (l) {
        mListeners.push(l);
      };

      self.onOK = function () {
        var i;
        for (i = 0; i < mListeners.length; i++) {
          mListeners[i]();
        }
        window.localStorage.preference = JSON.stringify(mStorage);
        self.close();
      };

      function constructor () {
        try {
          mStorage = JSON.parse(localStorage.preference);
        } catch (e) {
          mStorage = {};
        }

        mMain = new deosc.View();
        mMain.setHeight(300);
        $(mMain.getElement()).css("overflow-y", "auto");

        self.setTitle("Preference");
        self.setContent(mMain);
      }

      constructor();
    }

    window.deosc.Preferences = Preferences;
  }
) (window);(
  function (window) {

    function EventQueue () {
      var self = this, mQueue;

      self.push = function (event) {
        var i, e;
        for (i = 0; i < mQueue.length; i++) {
          e = mQueue[i];
          if (e.what == event.what) {
            mQueue[i] = event;
            return;
          }
        }
        mQueue.push(event);
      };

      self.pop = function () {
        return mQueue.shift();
      };

      self.foreach = function (callback) {
        while (0 < mQueue.length) {
          callback(mQueue.shift());
        }
      };

      function constructor () {
        mQueue = [];
      }

      constructor ();
    }


    function DCaseWeaver (options) {
      var self = this, mPlugins, mMain, mHeader, mBody, mTree, mMenuBar,
      mToolBar, mMenuBackground, mFileManager, mDCase, mIsCreated, mMenuBase,
      mDialogBase, mPreferences,
      mUndoStack, mRedoStack,
      mDCaseTreeMenu, mDCaseNodeMenu, mDCaseLinkMenu,
      mMetadata, mEventQueue, mEventTID, mOptions = options;

      self.create = function () {
        var i, plugin;

        mEventQueue = new EventQueue();

        $(window).resize(onResize);

        mMain = new deosc.View();
        $(mMain.getElement()).appendTo("body");

        mHeader = new deosc.View();
        $(mHeader.getElement()).addClass("deosc-header");
        mMain.addChild(mHeader);

        mBody = new deosc.View();
        $(mBody.getElement()).addClass("deosc-main");
        mMain.addChild(mBody);

        mDialogBase = new deosc.DialogBase();
        mMain.addChild(mDialogBase);

        mMenuBase = new deosc.MenuBase();
        mMain.addChild(mMenuBase);

        mMenuBar = new MenuBar(mMenuBase);
        mMenuBar.getMenuBarItem("File");
        mMenuBar.getMenuBarItem("Edit");
        mHeader.addChild(mMenuBar);

        mToolBar = new deosc.ToolBar();
        mHeader.addChild(mToolBar);

        mTree = new deosc.DCaseTreeView(self);
        mTree.setOnContextMenuListener(onClickDCaseTree);
        mTree.setOnNodeContextMenuListener(onClickDCaseNode);
        mTree.setOnLinkContextMenuListener(onClickDCaseLink);
        mTree.setOnClickLinkListener(onClickDCaseLink2);

        mDCaseTreeMenu = new deosc.Menu(self);
        mDCaseNodeMenu = new deosc.Menu(self);
        mDCaseLinkMenu = new deosc.Menu(self);

        mBody.addChild(mTree);

        mPreferences = new deosc.Preferences();

        //size is still not fixed now.
        //onResize();

        setTimeout(
          function () {
            var i, plugin, fm;

            self.addPlugin(new UndoPlugin(mTree));

            for (i = 0; i < readyCallbacks.length; i++) {
              readyCallbacks[i](self);
            }

            self.addPlugin(mPreferences);

            fm = self.getFileManager();
            if (fm) fm.onCreate();

            for (i = 0; i < mPlugins.length; i++) {
              plugin = mPlugins[i];
              plugin.onCreate();
            }

            mIsCreated = true;

            onResize();
          }, 0);
      };

      self.getOptions = function () {
        return mOptions;
      };

      self.setDCase = function (dcase, metadata) {
        mTree.setDCase(dcase);
        self.setMetadata(metadata);
        self.resetUndo();
      };

      self.getDCase = function () {
        return mTree.getDCase();
      };

      self.setMetadata = function (metadata) {
        mMetadata = metadata;
      };

      self.getMetadata = function () {
        return mMetadata;
      };

      self.getBody = function () {
        return mBody;
      };

      self.getMenuBar = function () {
        return mMenuBar;
      };

      self.getToolBar = function () {
        return mToolBar;
      };

      self.getPreferences = function () {
        return mPreferences;
      };

      self.setFileManager = function (fm) {
        mFileManager = fm;
        fm.setDCaseWeaver(self);
      };

      self.getFileManager = function () {
        return mFileManager;
      };

      self.getDCaseTreeView = function () {
        return mTree;
      };

      self.getDCaseTreeMenu = function () {
        return mDCaseTreeMenu;
      };

      self.getDCaseNodeMenu = function () {
        return mDCaseNodeMenu;
      };

      self.getDCaseLinkMenu = function () {
        return mDCaseLinkMenu;
      };

      self.addPlugin = function (plugin) {
        if (checkPlugin(plugin)) {
          mPlugins.push(plugin);
          plugin.setDCaseWeaver(self);

          if (mIsCreated) {
            plugin.onCreate();
          }
        }
      };

      self.addChild = function (widget) {
        mBody.addChild(widget);
      };

      self.sendEvent = function (event) {
        mEventQueue.push(event);
        if (mEventTID) return;
        mEventTID = setTimeout(
          function () {
            mEventTID = null;
            mEventQueue.foreach(dispatchEvent);
          }, 0);
      };

      self.menuPopup = function (menu, target, event) {
        var data = {
          position: {left: event.pageX, top: event.pageY},
          target: target,
          event: event
        };

        mMenuBase.open(menu, data);
      };

      self.dialogPopup = function (dialog) {
        mDialogBase.addDialog(dialog);
      };

      self.undo = function () {
        var i, commands;

        if (mUndoStack.length == 0) return;

        commands = mUndoStack.pop();
        for (i = commands.length - 1; 0 <= i; i--) {
          commands[i].undo();
        }
        mRedoStack.push(commands);
      };

      self.redo = function () {
        var i, commands;

        if (mRedoStack.length == 0) return;

        commands = mRedoStack.pop();
        for (i = 0; i < commands.length; i++) {
          commands[i].execute();
        }
        mUndoStack.push(commands);
      };

      self.command = function (commands) {
        var i;
        if (!(commands instanceof Array)) commands = [commands];

        for (i = 0; i < commands.length; i++) {
          commands[i].execute();
        }
        mUndoStack.push(commands);
        mRedoStack.length = 0;
      };

      self.resetUndo = function () {
        mUndoStack.length = 0;
        mRedoStack.length = 0;
      };

      self.canUndo = function () {
        return mUndoStack != 0;
      };

      self.canRedo = function () {
        return mRedoStack != 0;
      };

      self.getUndoStatus = function () {
        return {undo: self.canUndo(), redo: self.canRedo()};
      };

      function constructor () {
        mPlugins = [];
        mIsCreated = false;
        mUndoStack = [];
        mRedoStack = [];
      }

      function checkPlugin (plugin) {
        return true;
      }

      function dispatchEvent (event) {
        var i, plugin, fm;

        mTree.onEvent(event);

        fm = self.getFileManager();
        if (fm && fm.onEvent) {
          fm.onEvent(event);
        }

        for (i = 0; i < mPlugins.length; i++) {
          plugin = mPlugins[i];
          if (plugin.onEvent) {
            plugin.onEvent(event);
          }
        }
      }

      function onClickDCaseTree (x, y, event) {
        self.menuPopup(mDCaseTreeMenu, this, event);
        return true;
      }

      function onClickDCaseNode (x, y, event) {
        self.menuPopup(mDCaseNodeMenu, this, event);
        return true;
      }

      function onClickDCaseLink (x, y, event) {
        self.menuPopup(mDCaseLinkMenu, this, event);
        return true;
      }

      function onClickDCaseLink2 (x, y) {
        self.sendEvent(
          {
            what: DCaseWeaver.Event.CLICK_LINK,
            data: {view: this, x: x, y: y}
          });
      }

      function onResize () {
        var rect = mHeader.getElement().getBoundingClientRect();
        var width = innerWidth;
        var height = innerHeight - rect.height;

        $(mBody.getElement()).css({width: width, height: height});

        self.sendEvent({what: DCaseWeaver.Event.RESIZE});
      }

      constructor();
    }

    DCaseWeaver.Event = {
      RESIZE: 1,
      DCASE_CHANGED: 2,
      OPEN_DCASE_TREE_MENU: 4,
      OPEN_DCASE_NODE_MENU: 6,
      OPEN_DCASE_LINK_MENU: 7,
      CLICK_NODE:8,
      CLICK_LINK:9,
      OPEN_PREFERENCE_DIALOG: 10
    };

    var readyCallbacks = [];

    DCaseWeaver.ready = function (callback) {
      readyCallbacks.push(callback);
    };


    function MenuBar (base) {
      var self = this, mMenuBase, mMenuBarItems, mActive;

      deosc.View.call(self);

      self.getMenuBarItem = function (label) {
        var i, item;
        for (i = 0; i < self.getChildCount(); i++) {
          item = self.getChildAt(i);
          if (item.getLabel() == label) return item;
        }
        item = new MenuBarItem(label);
        self.addMenuBarItem(item);
        return item;
      };

      self.addMenuBarItem = function (item) {
        item.setOnClickListener(
          function () {
            onItemClick(item);
          });
        item.setOnMouseOverListener(
          function () {
            onItemMouseOver(item);
          });
        self.addChild(item);
      };

      function constructor (base) {
        mMenuBase = base;
        mMenuBarItems =[];
        mActive = false;

        $(self.getElement()).addClass("menu-bar");
      }

      function onItemMouseOver (item) {
        if (mActive) {
          selectItem(item);
        }
      }

      function onItemClick (item) {
        if (mActive) {
          mMenuBase.close();
        } else {
          mActive = true;
          $(self.getElement()).addClass("active");
          selectItem(item);
        }
      }

      function selectItem (item) {
        var i, itm, menu, rect;
        for (i = 0; i < self.getChildCount(); i++) {
          itm = self.getChildAt(i);
          if (itm != item) {
            itm.selected(false);
          }
        }
        item.selected(true);
        menu = item.getMenu();

        rect = item.getElement().getBoundingClientRect();
        mMenuBase.open(
          menu,
          {position: {left: rect.left, top: rect.bottom}},
          onClose);
      }

      function onClose () {
        var i, item;

        mActive = false;

        $(self.getElement()).removeClass("active");

        for (i = 0; i < self.getChildCount(); i++) {
          item = self.getChildAt(i);
          item.selected(false);
        }
      }

      constructor(base);
    }

    function MenuBarItem (label) {
      var self = this, mMenu;

      deosc.View.call(self);

      self.setLabel = function (label) {
        self.setText(label);
      };

      self.getLabel = function () {
        return self.getText();
      };

      self.addMenuItem = function (item) {
        mMenu.addMenuItem(item);
      };

      self.getMenu = function () {
        return mMenu;
      };

      function constructor (label) {
        self.setLabel(label);

        $(self.getElement()).addClass("menu-bar-item");

        mMenu = new deosc.Menu();
      }

      constructor(label);
    }


    function UndoToolBarItem (weaver) {
      var self = this, mWeaver, mUndoButton, mRedoButton;

      deosc.ToolBarItem.call(self);

      self.update = function () {
        var status = mWeaver.getUndoStatus();
        if (status.undo) {
          mUndoButton.disabled(false);
        } else {
          mUndoButton.disabled(true);
        }
        if (status.redo) {
          mRedoButton.disabled(false);
        } else {
          mRedoButton.disabled(true);
        }
      };

      function constructor (weaver) {
        mWeaver = weaver;

        mUndoButton = new deosc.View($("<button>")[0]);
        mUndoButton.setOnClickListener(onUndo);
        $(mUndoButton.getElement()).addClass("undo-button");
        self.addChild(mUndoButton);

        mRedoButton = new deosc.View($("<button>")[0]);
        mRedoButton.setOnClickListener(onRedo);
        $(mRedoButton.getElement()).addClass("redo-button");
        self.addChild(mRedoButton);

        self.update();
      }

      function onUndo () {
        mWeaver.undo();
      }

      function onRedo () {
        mWeaver.redo();
      }

      constructor(weaver);
    }

    function UndoToolBarItemPrototype () {}
    UndoToolBarItemPrototype.prototype = deosc.ToolBarItem.prototype;
    UndoToolBarItem.prototype = new UndoToolBarItemPrototype;
    UndoToolBarItem.constructor = UndoToolBarItem;


    function UndoPlugin (tree) {
      var self = this, mWeaver, mTree, mToolBarItem,
      mUndoMenuItem, mRedoMenuItem;

      self.setDCaseWeaver = function (weaver) {
        mWeaver = weaver;
      };

      self.onCreate = function () {
        var menuBarItem;

        mToolBarItem = new UndoToolBarItem(mWeaver);
        mWeaver.getToolBar().addItem(mToolBarItem);

        menuBarItem = mWeaver.getMenuBar().getMenuBarItem("Edit");

        mUndoMenuItem = new deosc.MenuItem("Undo", onUndo);
        menuBarItem.addMenuItem(mUndoMenuItem);

        mRedoMenuItem = new deosc.MenuItem("Redo", onRedo);
        menuBarItem.addMenuItem(mRedoMenuItem);

        updateMenuItem();
      };

      self.onEvent = function (event) {
        if (event.what == DCaseWeaver.Event.DCASE_CHANGED) {
          onDCaseChanged(event.data);
        }
      };

      function constructor () {
      }

      function onDCaseChanged () {
        mToolBarItem.update();
        updateMenuItem();
      }

      function updateMenuItem () {
        var status = mWeaver.getUndoStatus();
        if (status.undo) {
          mUndoMenuItem.disabled(false);
        } else {
          mUndoMenuItem.disabled(true);
        }
        if (status.redo) {
          mRedoMenuItem.disabled(false);
        } else {
          mRedoMenuItem.disabled(true);
        }
      }

      function onUndo () {
        mWeaver.undo();
      }

      function onRedo () {
        mWeaver.redo();
      }

      constructor();
    }


    function DCaseWeaverPlugin () {
    }
    $.extend(
      DCaseWeaverPlugin.prototype,
      {
        init: function (weaver) {
          this.setDCaseWeaver(weaver);
        },
        setDCaseWeaver: function (weaver) {
          this.weaver = weaver;
        },
        getDCaseWeaver: function () {
          return this.weaver;
        },
        createNodeMenuItem: function (node) {
          return null;
        },
        createLinkMenuItem: function (link) {
          return null;
        },
        createToolBarItem: function () {
          return null;
        }
      });

    var Util = {
      getUID: function () {
        var i, index, time4, ret = "";
        time4 = new Date().getTime().toString(4);
        for (i = 0; i < time4.length; i++) {
          index = parseInt(
            Math.floor(Math.random() * 4).toString(4) + time4[i], 4);
          ret += index.toString(16);
        }

        while (ret.length < 32) {
          ret += Math.floor(Math.random() * 15).toString(16);
        }

        return ret;
      }
    };

    $.extend(
      deosc,
      {
        DCaseWeaver: DCaseWeaver,
        DCaseWeaverPlugin: DCaseWeaverPlugin,
        Util: Util
      });
  }
) (window);

$(document).ready(
  function () {
    var weaver;

    function getOptions () {
      var ret = {}, search, list, i;

      search = window.location.search;
      if (search.length == 0) return ret;

      if (!search.match(/^\?([^?]+)$/)) return ret;
      search = RegExp.$1;

      list = search.split("&");

      for (i = 0; i < list.length; i++) {
        if (list[i].match(/([^=]+)=(.+)/)) {
          ret[RegExp.$1] = RegExp.$2;
        }
      }

      return ret;
    }

    $("#disable-javascript").hide();

    if (navigator.userAgent.match(/chrome/i)) {
      $("html").addClass("chrome");
    } else if (navigator.userAgent.match(/firefox/i)) {
      $("html").addClass("firefox");
    }

    weaver = new deosc.DCaseWeaver(getOptions());
    weaver.create();
  }
);
(
  function (window) {

    function DCaseTreeView (weaver) {
      var self = this, mWeaver, mTree, mOnClickDCaseTreeListener,
      mOnClickDCaseNodeListener, mOnClickDCaseLinkListener,
      mDCaseTreeMenu, mDCaseNodeMenu, mDCaseLinkMenu;
      var mDCase;
      var mCanvasViewRoot, mBaseCanvasView, mTreeCanvasView;

      deosc.View.call(self);

      self.setDCase = function (dcase) {
        mDCase = dcase;
        mTreeCanvasView.setDCase(dcase);

        self.onChange();

        setTimeout(
          function () {
            var nodes, nodeView;
            nodes = mDCase.getRootNodes();
            if (0 < nodes.length) {
              nodeView = mTreeCanvasView.findDCaseNodeViewByID(nodes[0].getID());
              self.moveTo(nodeView, "center", true);
            } else {
              mTreeCanvasView.setPosition(0, 0);
              mTreeCanvasView.arrange(); //for reset canvas size
            }
          }, 0);

      };

      self.getDCase = function () {
        return mDCase;
      };

      self.getDCaseTreeCanvasView = function () {
        return mTreeCanvasView;
      };

      self.getBaseCanvasView = function () {
        return mBaseCanvasView;
      };

      self.update = function () {
        if (!mDCase) return;
        mTreeCanvasView.update();
      };

      self.onEvent = function (event) {
        switch (event.what) {
        case deosc.DCaseWeaver.Event.RESIZE:
        case deosc.DCaseWeaver.Event.DCASE_CHANGED:
          self.update();
          break;
        }
      };

      self.onChange = function () {
        if (!mWeaver) return;

        mWeaver.sendEvent(
          {
            what: deosc.DCaseWeaver.Event.DCASE_CHANGED,
            data: mWeaver.getDCase()
          });
      };

      self.addDCaseNodeView = function (nodeView, index) {
        mDCase.addDCaseNode(nodeView.node, index);
        mTreeCanvasView.addDCaseNodeView(nodeView);
      };

      self.removeDCaseNodeView = function (nodeView) {
        mDCase.removeDCaseNode(nodeView.node);
        mTreeCanvasView.removeDCaseNodeView(nodeView);
      };

      self.addDCaseLinkView = function (linkView, index) {
        mDCase.addDCaseLink(linkView.link, index);
        mTreeCanvasView.addDCaseLinkView(linkView);
      };

      self.removeDCaseLinkView = function (linkView) {
        mDCase.removeDCaseLink(linkView.link);
        mTreeCanvasView.removeDCaseLinkView(linkView);
      };

      self.arrange = function () {
        mTreeCanvasView.arrange();
      };

      self.moveTo = function (nodeView, position, noanimation) {
        var left = false, top = false, right = false, bottom = false,
        center = false, x0, y0, x1, y1,
        self = this,
        count = 0,
        max = 10,
        interval = 50;

        if (position) {
          $(position.split("-")).each(
            function () {
              if (this == "left") left = true;
              else if (this == "top") top = true;
              else if (this == "right") right = true;
              else if (this == "bottom") bottom = true;
              else if (this == "center") center = true;
            });
        } else {
          center = true;
        }

        x0 = mTreeCanvasView.left;
        y0 = mTreeCanvasView.top;

        if (center) {
          x1 = -nodeView.left + (mCanvasViewRoot.width - nodeView.width()) / 2;
        } else if (left) {
          x1 = -nodeView.left + 16;
        } else if (right) {
          x1 = -nodeView.right + mCanvasViewRoot.width - 16;
        } else {
          x1 = x0;
        }

        if (0 < x1) {
          x1 = 0;
        } else if (x1 + mTreeCanvasView.width() < mCanvasViewRoot.width) {
          x1 = mCanvasViewRoot.width - mTreeCanvasView.width();
        }

        if (center) {
          y1 = -nodeView.top + (mCanvasViewRoot.height - nodeView.height()) / 2;
        } else if (top) {
          y1 = -nodeView.top + 16;
        } else if (bottom) {
          y1 = -nodeView.bottom + mCanvasViewRoot.height - 16;
        } else {
          y1 = y0;
        }

        if (0 < y1) {
          y1 = 0;
        } else if (y1 + mTreeCanvasView.height() < mCanvasViewRoot.height) {
          y1 = mCanvasViewRoot.height - mTreeCanvasView.height();
        }

        function animate() {
          setTimeout(
            function () {
              var l = ((max - count) * x0 + count * x1) / max;
              var t = ((max - count) * y0 + count * y1) / max;

              mTreeCanvasView.setPosition(l, t);
              if (count < max) {
                count++;
                animate();
              }
            },
            interval
          );
        };

        if (!noanimation) {
          animate();
        } else {
          mTreeCanvasView.setPosition(x1, y1);
        }
      };

      function constructor (weaver) {
        mWeaver = weaver;

        $(self.getElement()).css(
          {
            width: "100%", height: "100%",
           overflow: "hidden", "border-width": 0
          });

        mCanvasViewRoot = deosc.canvas.createViewRoot(self.getElement());
        mBaseCanvasView = deosc.canvas.createAbsoluteLayout();
        mTreeCanvasView = deosc.canvas.createDCaseTreeView();

        mCanvasViewRoot.setView(mBaseCanvasView);
        mBaseCanvasView.addView(mTreeCanvasView);

        mBaseCanvasView.setLayoutParams(
          {mode: deosc.canvas.View.LayoutMode.MATCH_PARENT}
        );
        mBaseCanvasView.setBackgroundColor("#ccc");
        mTreeCanvasView.setBackgroundColor("#fff");

      }

      self.setOnContextMenuListener = function (l) {
        mTreeCanvasView.setOnContextMenuListener(l);
      };

      self.setOnNodeContextMenuListener = function (l) {
        mTreeCanvasView.setOnNodeContextMenuListener(l);
      };

      self.setOnLinkContextMenuListener = function (l) {
        mTreeCanvasView.setOnLinkContextMenuListener(l);
      };

      self.setOnClickLinkListener = function (l) {
        mTreeCanvasView.setOnClickLinkListener(l);
      };

      constructor(weaver);
    }

    function DCaseTreeViewPrototype () {}
    DCaseTreeViewPrototype.prototype = deosc.View.prototype;
    DCaseTreeView.prototype = new DCaseTreeViewPrototype;
    DCaseTreeView.constructor = DCaseTreeView;


    function ArrangeCommand (tree) {
      var self = this, mTree, mNodeViewInfo;

      self.execute = function () {
        mTree.arrange();
        mTree.onChange();
      };

      self.undo = function () {
        var i, info;
        for (i = 0; i < mNodeViewInfo.length; i++) {
          info = mNodeViewInfo[i];
          info.nodeView.setPosition(info.x, info.y);
        }
        mTree.onChange();
      };

      function constructor () {
        var i, v, views, lp, dcase;

        mTree = tree;

        mNodeViewInfo = [];

        views = mTree.getDCaseTreeCanvasView().childViews;
        for (i = 0; i < views.length; i++) {
          v = views[i];
          if (v instanceof deosc.canvas.DCaseNodeView) {
            lp = v.getLayoutParams();
            mNodeViewInfo.push({ nodeView: v, x: lp.x, y: lp.y });
          }
        }
      }

      constructor();
    }


    function DCaseTreeViewPlugin () {
      var self = this, mWeaver, mWidget;

      self.setDCaseWeaver = function (weaver) {
        mWeaver = weaver;
      };

      self.onCreate = function () {
        var menu;
        menu = mWeaver.getDCaseTreeMenu();
        menu.addMenuItem(new deosc.MenuItem("Arrange", onArrange));
      };

      function onArrange (label, data) {
        var tree = mWeaver.getDCaseTreeView();
        mWeaver.command(new ArrangeCommand(tree));
      }
    }

    deosc.DCaseWeaver.ready(
      function (weaver) {
        weaver.addPlugin(new DCaseTreeViewPlugin());
      });

    window.deosc.DCaseTreeView = DCaseTreeView;
    window.deosc.ArrangeCommand = ArrangeCommand;
  }
) (window);(
  function (window) {
    function meta_comp (meta1, meta2) {
      return meta1.path == meta2.path;
    }


    function Metadata (meta) {
      meta.mtime = new Date(meta.mtime * 1000);
      meta.ctime = new Date(meta.ctime * 1000);
      meta.atime = new Date(meta.atime * 1000);
      return meta;
    }


    function FileManager () {
      var self = this, mWeaver, mRepo, mMetadata;

      self.setDCaseWeaver = function (weaver) {
        mWeaver = weaver;
      };

      self.onCreate = function () {
        var menuBarItem, opt;

        mWeaver.getToolBar().addItem(new ToolBarItem(self));

        menuBarItem = mWeaver.getMenuBar().getMenuBarItem("File");
        menuBarItem.addMenuItem(new deosc.MenuItem("New", onClickNew));
        menuBarItem.addMenuItem(new deosc.MenuItem("Open", onClickOpen));
        menuBarItem.addMenuItem(new deosc.MenuItem("Save", onClickSave));
        menuBarItem.addMenuItem(new deosc.MenuItem("Save As", onClickSaveAs));

        menuBarItem = mWeaver.getMenuBar().getMenuBarItem("Document");
        menuBarItem.addMenuItem(new deosc.MenuItem("Browse", onClickBrowse));
        menuBarItem.addMenuItem(new deosc.MenuItem("Search", onClickSearch));
        menuBarItem.addMenuItem(new deosc.MenuItem("Compare To", onClickCompareTo));

        mWeaver.setDCase(new deosc.DCase());

        opt = mWeaver.getOptions();
        if ("path" in opt) {
          mRepo.getMetadata(
            {path: opt.path},
            function (metadata) {
              mRepo.getFile(
                {path: opt.path},
                function (doc) {
                  mMetadata = metadata;
                  mWeaver.setDCase(new deosc.DCase(doc), mMetadata);
                  if ("node" in opt) {
                    setTimeout(
                      function () {
                        var tree, canvasTree, view;

                        tree = mWeaver.getDCaseTreeView();
                        view = tree.getDCaseTreeCanvasView()
                          .findDCaseNodeViewByID(opt.node);
                        tree.moveTo(view, "center", true);
                      }, 0);
                  }
                });
            });
        }
      };

      self.browse = function (option) {
        var dialog;

        dialog = new Browser(mWeaver, mRepo);

        if (option) {
          if (option.filter) dialog.setFilter(option.filter);
          if (option.success) option.success(dialog);
        }

        mWeaver.dialogPopup(dialog);
      };

      self.getMetadata = function (metadata, success, error) {
        mRepo.getMetadata(metadata, success, error);
      };

      self.getFile = function (metadata, success, error) {
        mRepo.getFile(metadata, success, error);
      };

      self.postFile = function (metadata, file, name, success, error) {
        mRepo.postFile(metadata, file, name, success, error);
      };

      self.putFile = function (metadata, file, success, error) {
        mRepo.putFile(metadata, file, success, error);
      };

      self.getParents = function (metadata, success, error) {
        mRepo.getParents(metadata, success, error);
      };

      self.getChildren = function (metadata, success, error) {
        mRepo.getChildren(metadata, success, error);
      };

      self.mkdir = function (metadata, name, success, error) {
        mRepo.mkdir(metadata, name, success, error);
      };

      self.comp = function (meta1, meta2) {
        return meta_comp(meta1, meta2);
      };

      function constructor () {
        mRepo = new Repository();
      }
      constructor();

      function openDCase (success, error) {
        var dialog = new OpenDCaseBrowser(mWeaver, mRepo);
        dialog.setOnLoadDCaseDocumentListener(success);
        mWeaver.dialogPopup(dialog);
      };

      function saveDCase (dcase, metadata, success, error) {
        var xml, browser;
        if (metadata) {
          xml = dcase.toFormatXML();
          mRepo.putFile(metadata, xml, success, error);
        } else {
          browser = new SaveAsDCaseBrowser(mWeaver, mRepo, dcase);
          browser.setOnSaveDCaseListener(success);
          browser.setOnErrorListener(error);
          mWeaver.dialogPopup(browser);
        }
      };

      function compareDCase () {
        self.browse(
          {
            success:function (dialog) {
              dialog.setOnOKListener(
                function (meta) {
                  self.getFile(
                    meta,
                    function (doc) {
                      var dcase = new deosc.DCase(doc);
                      deosc.compareDCase(dcase);
                      dialog.close();
                    },
                    function () {
                      console.error("error");
                    });
                });
            },
            filter: function (meta) {
              return meta.isdir || meta.path.match(/\.dcase$/) != null;
            }
          });
      }

      function searchDCase () {
        var dialog, modificationDate;

        function SearchResultWidget () {
          var self = this, mListModel, mListView, mButtonBox,
          mOnItemChangedListener;

          deosc.View.call(self);

          self.setOnItemChangedListener = function (l) {
            mOnItemChangedListener = l;
          };

          self.addButton = function (button) {
            button.setWidth("100%");
            mButtonBox.addChild(button);
          };

          self.getCurrentItem = function () {
            return mListView.getCurrentItem();
          };

          self.setHeaderItem = function (item) {
            mListModel.setHeaderItem(item);
          };

          self.addItem = function (item) {
            mListModel.add(item);
            mListView.update();
          };

          function constructor () {
            var box, header, i;

            mListModel = new deosc.ListModel();

            mListView = new deosc.ListView();
            mListView.setSize(400, 200);
            mListView.setModel(mListModel);
            mListView.setOnClickItemListener(onItemChanged);

            mButtonBox = new deosc.View();
            mButtonBox.addStyle("vbox");

            self.addStyle("hbox");
            self.addChild(mListView);
            self.addChild(mButtonBox);
          }

          function onItemChanged (item) {
            if (mOnItemChangedListener) mOnItemChangedListener(item);
          }

          constructor();
        }

        function SearchResultDialog (results) {
          var self = this, mWidget, mOpen, mCompare;

          deosc.Dialog.call(self);

          self.onCreate = function () {
            var i, header;

            mOpen = new deosc.View($("<button>"));
            mOpen.setText("Open");
            mOpen.setOnClickListener(onOpen);

            mCompare = new deosc.View($("<button>"));
            mCompare.setText("Compare");
            mCompare.setOnClickListener(onCompare);

            mWidget = new SearchResultWidget();
            mWidget.addButton(mOpen);
            mWidget.addButton(mCompare);

            header = new deosc.ListItem();
            header.addColumns(["Name", "Date Modified", "Location"]);

            mWidget.setHeaderItem(header);

            function createListItem (metadata) {
              var item = new deosc.ListItem();
              item.addColumn(metadata.name);
              item.addColumn(metadata.mtime.toLocaleString());
              item.addColumn(deosc.path.dirname(metadata.path));
              item.setData(metadata);
              return item;
            }

            for (i = 0; i < results.length; i++) {
              mWidget.addItem(createListItem(results[i]));
            }

            self.setContent(mWidget);
            self.setTitle("Search Result");
            self.getFooter().hide();
          };

          function onOpen () {
            var item = mWidget.getCurrentItem();
            if (item) {
              mRepo.open(item.getData());
            }
          }

          function onCompare () {
            var item;

            function success (doc) {
              deosc.compareDCase(new deosc.DCase(doc));
            }

            item = mWidget.getCurrentItem();
            if (item) {
              mRepo.getFile(item.getData(), success);
            }
          }
        }

        function onSearchSuccess (results) {
          var searchResultDialog;

          dialog.close();

          searchResultDialog = new SearchResultDialog(results);
          mWeaver.dialogPopup(searchResultDialog);
        }

        function onSearch () {
          var conditions = [], i;

          for (i = 0; i < dialog.getConditionCount(); i++) {
            conditions.push(dialog.getConditionAt(i));
          }
          conditions.push(
            {attr: "File name", keyword: ".dcase", operator: "="});

          mRepo.search(conditions, onSearchSuccess);
        }

        function checkDate () {
          var timestamp, date;
          timestamp = Date.parse(this.getValue());
          if (isNaN(timestamp)) {
            alert("Input type is not Date type");
            return false;
          } else {
            return true;
          }
        }

        function generateListItem (condition) {
          var date = new Date(condition.keyword);
          var item = new deosc.ListItem();
          item.addColumn(condition.attr);
          item.addColumn(date.toLocaleString());
          item.addColumn(condition.operator);
          condition.keyword = date.toISOString();
          item.setData(condition);
          return item;
        }

        modificationDate = new deosc.ConditionController("Modification date");
        modificationDate.setOnAddListener(checkDate);
        modificationDate.setOnGenerateListItemListener(generateListItem);

        dialog = new deosc.SearchConditionDialog();

        dialog.addConditionController(
          new deosc.ConditionController("File name", ["=", "<>"]));
        dialog.addConditionController(
          new deosc.ConditionController("Directory path", ["="]));
        dialog.addConditionController(modificationDate);

        dialog.setOnSearchListener(onSearch);

        mWeaver.dialogPopup(dialog);
      }

      function onClickNew () {
        mMetadata = null;
        mWeaver.setDCase(new deosc.DCase());
      }

      function onClickOpen () {
        openDCase(onLoadDCaseDocument);
      }

      function onClickSave () {
        saveDCase(mWeaver.getDCase(), mMetadata, onSaveDCase);
      }

      function onClickSaveAs () {
        saveDCase(mWeaver.getDCase(), null, onSaveDCase);
      }

      function onClickBrowse () {
        var dialog = new BrowseBrowser(mWeaver, mRepo);
        mWeaver.dialogPopup(dialog);
      }

      function onClickSearch () {
        searchDCase(mWeaver, mRepo);
      }

      function onClickCompareTo () {
        compareDCase();
      }

      function onLoadDCaseDocument (metadata, doc) {
        mMetadata = metadata;
        mWeaver.setDCase(new deosc.DCase(doc), mMetadata);
      }

      function onSaveDCase (metadata) {
        mMetadata = metadata;
        mWeaver.setMetadata(metadata);
      }

      function ToolBarItem () {
        var self = this, mNewButton, mOpenButton, mSaveButton;

        deosc.ToolBarItem.call(self);

        function constructor () {
          mNewButton = new deosc.View($("<button>"));
          mNewButton.setText("New");
          mNewButton.setOnClickListener(onClickNew);
          self.addChild(mNewButton);

          mOpenButton = new deosc.View($("<button>"));
          mOpenButton.setText("Open");
          mOpenButton.setOnClickListener(onClickOpen);
          self.addChild(mOpenButton);

          mSaveButton = new deosc.View($("<button>"));
          mSaveButton.setText("Save");
          mSaveButton.setOnClickListener(onClickSave);
          self.addChild(mSaveButton);
        }

        constructor();
      }
    }


    function Repository () {
      var self = this;

      self.getUrl = function (path) {
        return "cgi-bin/documents.wsgi/file" + path;
      };

      self.open = function (metadata) {
        var url = "cgi-bin/documents.wsgi/file" + metadata.path;
        if (metadata.path.match(/\.dcase$/)) {
          window.open("?path="+metadata.path, "_blank", "");
        } else {
          window.open(url, "_blank", "");
        }
      };

      self.getRoot = function (success, error) {
        //TODO
      };

      function default_error (jqXHR, textStatus, errorThrown) {
        if (textStatus == "parsererror") {
          alert("Parser Error");
        } else {
          alert(jqXHR.responseText);
        }
      }

      self.getMetadata = function (metadata, success, error) {
        function _success (data, textStatus, jqXHR) {
          if (success) success(new Metadata(data));
        }

        function _error (jqXHR, textStatus, errorThrown) {
          if (error) {
            error();
          } else {
            default_error(jqXHR, textStatus, errorThrown);
          }
        }

        var opt = {
          url: "cgi-bin/documents.wsgi/metadata" + metadata.path,
          type: "GET",
          success: _success,
          error: _error
        };
        $.ajax(opt);
      };

      self.getParents = function (metadata, success, error) {
        function _success (data, textStatus, jqXHR) {
          var i, ret = [];
          if (success) {
            for (i = 0; i < data.length; i++) {
              ret.push(new Metadata(data[i]));
            }
            success(ret);
          }
        }

        function _error (jqXHR, textStatus, errorThrown) {
          if (error) {
            error();
          } else {
            default_error(jqXHR, textStatus, errorThrown);
          }
        }

        var opt = {
          url: "cgi-bin/documents.wsgi/parents" + metadata.path,
          type: "GET",
          success: _success,
          error: _error
        };
        $.ajax(opt);
      };

      self.getChildren = function (metadata, success, error) {
        function _success (data, textStatus, jqXHR) {
          var i, ret = [];
          if (success) {
            for (i = 0; i < data.length; i++) {
              ret.push(new Metadata(data[i]));
            }
            success(ret);
          }
        }

        function _error (jqXHR, textStatus, errorThrown) {
          if (error) {
            error();
          } else {
            default_error(jqXHR, textStatus, errorThrown);
          }
        }

        var opt = {
          url: "cgi-bin/documents.wsgi/children" + metadata.path,
          type: "GET",
          success: _success,
          error: _error
        };
        $.ajax(opt);
      };

      self.getFile = function (metadata, success, error) {
        function _error (jqXHR, textStatus, errorThrown) {
          if (error) {
            error();
          } else {
            default_error(jqXHR, textStatus, errorThrown);
          }
        }

        var opt = {
          url: "cgi-bin/documents.wsgi/file" + metadata.path,
          type: "GET",
          success: success,
          error: _error
        };
        $.ajax(opt);
      };

      self.putFile = function (metadata, data, success, error) {
        function _success (data, textStatus, jqXHR) {
          if (success) success(new Metadata(data));
        }

        function _error (jqXHR, textStatus, errorThrown) {
          if (error) {
            error();
          } else {
            default_error(jqXHR, textStatus, errorThrown);
          }
        }

        var opt = {
          url: "cgi-bin/documents.wsgi/file" + metadata.path,
          type: "PUT",
          data: data,
          processData: false,
          success: _success,
          error: _error
        };
        $.ajax(opt);
      };

      self.postFile = function (metadata, data, name, success, error) {
        function _success (data, textStatus, jqXHR) {
          if (success) success(new Metadata(data));
        }

        function _error (jqXHR, textStatus, errorThrown) {
          if (error) {
            error();
          } else {
            default_error(jqXHR, textStatus, errorThrown);
          }
        }

        function _postFile () {
          var opt = {
            url: "cgi-bin/documents.wsgi/file" + metadata.path
              + "?name=" + name,
            type: "POST",
            data: data,
            processData: false,
            success: _success,
            error: _error
          };
          $.ajax(opt);
        }

        self.getChildren(
          metadata,
          function (children) {
            var i, meta;

            for (i = 0; i < children.length; i++) {
              if (children[i].name == name) {
                meta = children[i];
                break;
              }
            }

            if (meta) {
              if (confirm(name + "\n\noverwrite ?")) {
                self.putFile(meta, data, success, error);
              }
            } else {
              _postFile();
            }
          }, error);
      };

      self.remove = function (metadata, success, error) {
        function _success (data, textStatus, jqXHR) {
          if (success) success(new Metadata(data));
        }

        function _error (jqXHR, textStatus, errorThrown) {
          if (error) {
            error();
          } else {
            default_error(jqXHR, textStatus, errorThrown);
          }
        }

        var opt = {
          url: "cgi-bin/documents.wsgi/delete" + metadata.path,
          type: "POST",
          success: _success,
          error: _error
        };
        $.ajax(opt);
      };

      self.mkdir = function (metadata, name, success, error) {
        function _success (data, textStatus, jqXHR) {
          if (success) success(new Metadata(data));
        }

        function _error (jqXHR, textStatus, errorThrown) {
          if (error) {
            error();
          } else {
            default_error(jqXHR, textStatus, errorThrown);
          }
        }

        var opt = {
          url: "cgi-bin/documents.wsgi/mkdir" + metadata.path + "?name=" + name,
          type: "POST",
          success: _success,
          error: _error
        };
        $.ajax(opt);
      };

      self.download = function (metadata) {
        var url = "cgi-bin/documents.wsgi/download" + metadata.path;
        url = encodeURI(url);
        window.open(url, "_blank", "");
      };

      self.search = function (conditions, success, error) {
        function _success (data, textStatus, jqXHR) {
          var i, ret = [];
          if (success) {
            for (i = 0; i < data.length; i++) {
              ret.push(new Metadata(data[i]));
            }
            success(ret);
          }
        }

        function _error (jqXHR, textStatus, errorThrown) {
          if (error) {
            error();
          } else {
            default_error(jqXHR, textStatus, errorThrown);
          }
        }

        var opt = {
          url: "cgi-bin/documents.wsgi/search?conditions="
            + JSON.stringify(conditions),
          type: "POST",
          success: _success,
          error: _error
        };
        $.ajax(opt);
      };
    }


    function BrowseBrowser (weaver, repo) {
      var self = this, mWeaver, mRepo;

      Browser.call(self, weaver, repo);

      self._super = {
        onCreate: self.onCreate
      };

      self.onCreate = function () {
        self._super.onCreate();

        self.setTitle("Browser");
        self.setPath("/");

        self.addAction("Open", onOpen, BrowseWidget.Target.FILE);
        self.addAction("Delete", onDelete,
                       BrowseWidget.Target.FILE |
                       BrowseWidget.Target.DIR);
        self.addAction("Make Directory", onMkdir,
                       BrowseWidget.Target.FILE |
                       BrowseWidget.Target.DIR |
                       BrowseWidget.Target.ROOTDIR);
        self.addAction("Upload", onUpload,
                       BrowseWidget.Target.FILE |
                       BrowseWidget.Target.DIR |
                       BrowseWidget.Target.ROOTDIR);
        self.addAction("DownLoad", onDownload,
                       BrowseWidget.Target.FILE);

        self.getFooter().hide();
      };

      function constructor (weaver, repo) {
        mWeaver = weaver;
        mRepo = repo;
      }

      function onOpen (meta) {
        self.openFile(meta);
      }

      function onUpload (meta) {
        self.openUploadDialog(meta);
      }

      function onDownload (meta) {
        self.download(meta);
      }

      function onMkdir (meta) {
        self.openMkdirDialog(meta);
      }

      function onDelete (meta) {
        self.remove(meta);
      }

      constructor(weaver, repo);
    }


    function OpenDCaseBrowser (weaver, repo) {
      var self = this, mWeaver, mRepo, mBrowse, mOnLoadDCaseDocumentListener,
      mOnErrorListener;

      Browser.call(self, weaver, repo);

      self.setOnLoadDCaseDocumentListener = function (l) {
        mOnLoadDCaseDocumentListener = l;
      };

      self.setOnErrorListener = function (l) {
        mOnErrorListener = l;
      };

      function constructor (weaver, repo) {
        mWeaver = weaver;
        mRepo = repo;
        self.setTitle("Open D-Case");
        self.setFilter(
          function (meta) {
            return meta.isdir || meta.name.match(/\.dcase$/);
          });
        self.setOnOKListener(onOK);
      }

      function onOK (meta) {

        function onLoadDCaseDocument (doc) {
          if (mOnLoadDCaseDocumentListener)
            mOnLoadDCaseDocumentListener(meta, doc);
          self.close();
        }

        function error () {
          if (mOnErrorListener) mOnErrorListener();
        }

        if (meta) {
          mRepo.getFile(meta, onLoadDCaseDocument, error);
        } else {
          error();
        }
      }

      constructor(weaver, repo);
    }


    function SaveAsDCaseBrowser (weaver, repo, dcase) {
      var self = this, mWeaver, mRepo, mDCase, mNameInput, mDirInput,
      mOnSaveDCaseListener, mOnErrorListener;

      Browser.call(self, weaver, repo);

      self._super = {
        onCreate: self.onCreate
      };

      self.onCreate = function () {
        var main, nameLabel, dirLabel;

        self._super.onCreate();

        main = new deosc.View();
        main.addStyle("vbox");

        dirLabel = new deosc.View();
        dirLabel.addStyle("label");
        dirLabel.setText("Directory");
        main.addChild(dirLabel);

        mDirInput = new deosc.InputText();
        mDirInput.readonly(true);
        mDirInput.setText("/");
        mDirInput.setWidth("100%");
        main.addChild(mDirInput);

        nameLabel = new deosc.View();
        nameLabel.addStyle("label");
        nameLabel.setText("File Name");
        main.addChild(nameLabel);

        mNameInput = new deosc.InputText();
        mNameInput.setWidth("100%");
        main.addChild(mNameInput);

        self.setOnClickItemListener(onClickItem);

        self.getContent().addChild(main, 0);
      };

      self.setOnSaveDCaseListener = function (l) {
        mOnSaveDCaseListener = l;
      };

      self.setOnErrorListener = function (l) {
        mOnErrorListener = l;
      };

      function constructor (weaver, repo, dcase) {
        mWeaver = weaver;
        mRepo = repo;
        mDCase = dcase;

        self.setTitle("Save D-Case");
        self.setFilter(
          function filter (meta) {
            return meta.isdir || meta.name.match(/\.dcase$/);
          });
        self.setOnOKListener(onOK);
      }

      function onClickItem (meta) {
        if (!meta.isdir) {
          mDirInput.setText(deosc.path.dirname(meta.path));
          mNameInput.setText(meta.name);
        } else {
          mDirInput.setText(meta.path);
        }
      }

      function onOK (meta) {
        var name, path, xml;

        function onSaveDCase (metadata) {
          if (mOnSaveDCaseListener)
            mOnSaveDCaseListener(metadata);
          mWeaver.sendEvent(
            {
              what: deosc.DCaseWeaver.Event.DCASE_CHANGED,
              data: mWeaver.getDCase()
            });
          self.close();
        }

        function error () {
          if (mOnErrorListener)
            mOnErrorListener();
        }

        name = mNameInput.getText();
        if (!name.match(/\.dcase$/)) {
          name += ".dcase";
        }

        path = mDirInput.getText();

        xml = mDCase.toFormatXML();

        mRepo.postFile({path: path}, xml, name, onSaveDCase, error);
      }

      constructor(weaver, repo, dcase);
    }


    function Browser (weaver, repo) {
      var self = this, mWeaver, mRepo, mPath, mFilter, mBrowseWidget,
      mOnOKListener, mOnClickItemListener;

      deosc.Dialog.call(self);

      self.onCreate = function () {
        var main;

        main = new deosc.View();
        main.addChild(mBrowseWidget);

        self.setContent(main);

        mBrowseWidget.setRepository(mRepo, mPath, filter);
      };

      self.setPath = function (path) {
        mPath = path;
      };

      self.setFilter = function (filter) {
        mFilter = filter;
      };

      self.setOnOKListener = function (l) {
        mOnOKListener = l;
      };

      self.setOnClickItemListener = function (l) {
        mBrowseWidget.setOnClickItemListener(l);
      };

      self.addAction = function (label, action, target) {
        mBrowseWidget.addAction(label, action, target);
      };

      self.onOK = function () {
        var meta;
        if (mOnOKListener) {
          try {
            meta = mBrowseWidget.getCurrentItem().getData();
          } catch (e) {
            meta = null;
          }
          mOnOKListener(meta);
        } else {
          self.close();
        }
      };

      self.openFile = function (meta) {
        mRepo.open(meta);
      };

      self.openUploadDialog = function (meta) {
        var dialog, view, uploadLabel, fileInput, nameLabel, nameInput, path;

        view = new deosc.View();
        view.addStyle("vbox");

        uploadLabel = new deosc.View();
        uploadLabel.addStyle("label");
        uploadLabel.setText("Upload File:");

        fileInput = new deosc.View($("<input type='file'>")[0]);
        fileInput.setWidth(300);
        fileInput.setOnChangeListener(
          function () {
            var file;
            try {
              file = fileInput.getElement().files[0];
              nameInput.setText(file.name);
            } catch (e) {
            }
          }
        );

        nameLabel = new deosc.View();
        nameLabel.addStyle("label");
        nameLabel.setText("File Name:");

        nameInput = new deosc.InputText();
        nameInput.setWidth(300);

        view.addChild(uploadLabel);
        view.addChild(fileInput);
        view.addChild(nameLabel);
        view.addChild(nameInput);

        dialog = new deosc.Dialog();
        dialog.setTitle("Upload");
        dialog.setContent(view);
        dialog.setOnOKListener(
          function () {
            var file, path, name;

            function success (metadata) {
              dialog.close();
            }

            try {
              file = fileInput.getElement().files[0];
            } catch (e) {
              alert("Please select file.");
              return;
            }

            path = "";
            if (meta) {
              path = meta.isdir ? meta.path : deosc.path.dirname(meta.path);
            }

            name = nameInput.getText();

            mBrowseWidget.postFile({path: path}, file, name, success);
          });

        mWeaver.dialogPopup(dialog);
      };

      self.download = function (meta) {
        mRepo.download(meta);
      };

      self.openMkdirDialog = function (meta) {
        var view, dialog, nameLabel, nameInput, path;

        if (meta.isdir) {
          path = meta.path;
        } else {
          path = deosc.path.dirname(meta.path);
        }

        view = new deosc.View();
        view.addStyle("vbox");

        nameLabel = new deosc.View();
        nameLabel.addStyle("label");
        nameLabel.setText("Directory Name:");

        nameInput = new deosc.InputText();
        nameInput.setWidth(400);

        view.addChild(nameLabel);
        view.addChild(nameInput);

        dialog = new deosc.Dialog();
        dialog.setContent(view);
        dialog.setTitle("Make Directory");
        dialog.setOnOKListener(
          function () {
            var name;

            function success (metadata) {
              dialog.close();
            }

            name = nameInput.getText();
            mBrowseWidget.mkdir({path: path}, name, success);
          });

        mWeaver.dialogPopup(dialog);
      };

      self.remove = function (meta) {
        mBrowseWidget.remove(meta);
      };

      function constructor (weaver, repo) {
        mWeaver = weaver;
        mRepo = repo;
        mPath = "/";
        mBrowseWidget = new BrowseWidget(mWeaver);
        self.setTitle("Browser");
      }

      function filter (item) {
        return mFilter ? mFilter(item.getData()) : true;
      }

      constructor(weaver, repo);
    }


    function BrowseWidget (weaver) {
      var self = this, mWeaver, mRepo, mModel, mView, mRootPath, mButtonBox,
      mButtons, mOnClickItemListener;

      deosc.View.call(self);

      self.setRepository = function (repo, rootPath, filter) {
        mRepo = repo;
        mRootPath = rootPath;

        mModel = new deosc.TreeListModel();
        if (filter) mModel.setFilter(filter);
        mView.setModel(mModel);

        getMetadata({path: mRootPath}, onGetRootMetadata);
      };

      self.setOnClickItemListener = function (l) {
        mOnClickItemListener = l;
      };

      self.addAction = function (label, action, target) {
        var button;
        button = new deosc.View($("<button>")[0]);
        button.setText(label);
        button.setWidth("100%");
        button.setOnClickListener(
          function () {
            var item;

            item = mView.getCurrentItem();
            if (!item) item = mModel.getRootItem();

            action(item.getData());
          });
        mButtonBox.addChild(button);

        mButtons.push({button: button, action: action, target: target});

        update();
      };

      self.putFile = function (meta, file, success, error) {
        function _success (metadata) {
          onGetMetadata(metadata);
          if (success) success(metadata);
        }

        mRepo.putFile(meta, file, _success, error);
      };

      self.postFile = function (meta, file, name, success, error) {
        function _success (metadata) {
          onGetMetadata(metadata);
          if (success) success(metadata);
        }

        mRepo.postFile(meta, file, name, _success, error);
      };

      self.mkdir = function (metadata, name, success, error) {
        function _success (metadata) {
          onGetMetadata(metadata);
          if (success) success(metadata);
        }

        mRepo.mkdir(metadata, name, _success, error);
      };

      self.remove = function (metadata, success, error) {

        function _success (metadata) {
          mModel.foreach(
            function (index) {
              if (meta_comp(this.getData(), metadata)) {
                mModel.remove(this);
                mView.setCurrent(-1);
                mView.update();
                mModel.exitForeach();
              }
            });
          update();
          if (success) success(metadata);
        }

        if (confirm("Delete " + metadata.name)) {
          mRepo.remove(metadata, _success, error);
        }
      };

      self.getCurrentItem = function () {
        return mView.getCurrentItem();
      };

      function constructor (weaver) {
        mWeaver = weaver;
        mButtons = [];

        mView = new deosc.TreeListView();
        mView.setSize(400, 200);
        mView.setOnClickItemListener(onClickItem);
        mView.setOnCollapseItemListener(onCollapseItem);
        mView.setOnExpandItemListener(onExpandItem);
        self.addChild(mView);

        mButtonBox = new deosc.View();
        mButtonBox.addStyle("vbox");
        self.addChild(mButtonBox);

        self.addStyle("hbox");
      }

      function update () {
        var i, button, target, flag, item;

        item = mView.getCurrentItem();

        if (item) {
          if (item.getData().isdir) {
            flag = BrowseWidget.Target.DIR;
          } else {
            flag = BrowseWidget.Target.FILE;
          }
        } else {
          flag = BrowseWidget.Target.ROOTDIR;
        }

        for (i = 0; i < mButtons.length; i++) {
          button = mButtons[i].button;
          target = mButtons[i].target;

          button.disabled((flag & target) != flag);
        }
      }

      function getMetadata (metadata, callback) {
        var exists = false, fm;

        mModel.foreach(
          function () {
            if (meta_comp(this.getData(), metadata)) {
              exists = true;
              mModel.exitForeach();
            }
          });

        if (exists) return;

        mRepo.getMetadata(metadata, callback);
      }

      function onGetRootMetadata (metadata) {
        var i;

        mModel.getRootItem().setData(metadata);

        for (i = 0; i < metadata.children.length; i++) {
          getMetadata(metadata.children[i], onGetMetadata);
        }
      }

      function onGetMetadata (meta) {
        var parentItem, hasChildren, dirname, item, exists = false;

        mModel.foreach(
          function () {
            if (meta_comp(this.getData(), meta)) {
              exists = true;
              mModel.exitForeach();
            }
          });

        if (exists) return;

        if (meta.isdir && 0 < meta.children.length) {
          hasChildren = true;
        } else {
          hasChildren = false;
        }

        item = new deosc.TreeListItem();
        item.addColumn(meta.name);
        item.setData(meta);
        item.setIcon(meta.isdir ? "icon-dir" : "icon-file");
        item.collapsable(meta.isdir);

        dirname = deosc.path.dirname(meta.path);

        mModel.foreach(
          function () {
            if (this.getData().path == dirname) {
              parentItem = this;
              mModel.exitForeach();
            }
          });

        mModel.add(parentItem, item);

        mModel.sort(
          function (a, b) {
            if (a.getData().isdir == b.getData().isdir) {
              return a.getData().name < b.getData().name ? -1 : 1;
            } else {
              return a.getData().isdir ? -1 : 1;
            }
          });

        mView.update();
      }

      function onClickItem (index) {
        if (mOnClickItemListener)
          mOnClickItemListener(mView.getCurrentItem().getData());
        update();
      }

      function onCollapseItem (index) {
      }

      function onExpandItem (index) {
        var i, info, meta, item;

        item = mModel.getItemAt(index);
        meta = item.getData();

        for (i = 0; i < meta.children.length; i++) {
          getMetadata(meta.children[i], onGetMetadata);
        }
      }

      constructor(weaver);
    }

    BrowseWidget.Target = {
      DIR: 1,
      FILE: 2,
      ROOTDIR: 4
    };


    deosc.DCaseWeaver.ready(
      function (weaver) {
        var fm;

        fm = new FileManager();
        weaver.setFileManager(fm);
      });
  }
) (window);(
  function (window) {

    function DCaseNodeEditorPlugin () {
      var self = this, mWeaver, mWidget;

      self.setDCaseWeaver = function (weaver) {
        mWeaver = weaver;
      };

      self.onCreate = function () {
        var menu, item;

        menu = mWeaver.getDCaseTreeMenu();

        item = new deosc.MenuItem("New Node");
        item.setMenu(new DCaseTreeNewNodeMenu(mWeaver));
        menu.addMenuItem(item, 0);

        menu = mWeaver.getDCaseNodeMenu();

        item = new deosc.MenuItem("Add Child");
        item.setMenu(new DCaseNodeAddChildMenu(mWeaver));
        menu.addMenuItem(item);

        menu.addMenuItem(new deosc.MenuItem("Edit Node", onEditNode));
        menu.addMenuItem(new deosc.MenuItem("Delete Node", onDeleteNode));
      };

      function onEditNode (data) {
        var tree, editor, nodeView;

        function onOK () {
          var attrs;

          attrs = data.target.node.getAttributes();
          attrs.name = this.getName();
          attrs.type = this.getType();
          attrs.description = this.getDescription();
          attrs.parameterizedDesc = this.getParameterizedDesc();
          attrs.responsibility = {
            name: this.getResponsibilityName(),
            address:this.getResponsibilityEmail(),
            icon: this.getResponsibilityIcon()
          };
          attrs.validUntil = this.getValidUntil();

          mWeaver.command(new EditNodeCommand(tree, nodeView, attrs));

          this.close();
        }

        nodeView = data.target;
        tree = mWeaver.getDCaseTreeView();

        editor = new DCaseNodeEditor();
        editor.setDCaseNode(nodeView.node);
        editor.setOnOKListener(onOK);
        mWeaver.dialogPopup(editor);
      }

      function onDeleteNode (data) {
        var tree, nodeView, dcase, links, i, commands = [], linkView;

        nodeView = data.target;

        tree = mWeaver.getDCaseTreeView();

        dcase = tree.getDCase();

        commands.push(new DeleteNodeCommand(tree, nodeView));

        links = dcase.findDCaseLinksBySource(nodeView.node.getID());
        for (i = 0; i < links.length; i++) {
          linkView = tree.getDCaseTreeCanvasView()
            .findDCaseLinkViewByID(links[i].getID());
          commands.push(new deosc.DeleteLinkCommand(tree, linkView));
        }

        links = dcase.findDCaseLinksByTarget(nodeView.node.getID());
        for (i = 0; i < links.length; i++) {
          linkView = tree.getDCaseTreeCanvasView()
            .findDCaseLinkViewByID(links[i].getID());
          commands.push(new deosc.DeleteLinkCommand(tree, linkView));
        }

        mWeaver.command(commands);
      };

    }

    function DCaseTreeNewNodeMenu (weaver) {
      var self = this, mWeaver, mTree;

      deosc.Menu.call(self);

      function constructor (weaver) {
        var i, type, item;

        mWeaver = weaver;
        mTree = weaver.getDCaseTreeView();

        for (i = 0; i < deosc.DCase.NODE_TYPE.length; i++) {
          type = deosc.DCase.NODE_TYPE[i];
          item = new deosc.MenuItem(type, onNewNode);
          self.addMenuItem(item);
        }
      }

      function onNewNode (data) {
        var treeView, rootRect, left, top, node, nodeView, type;

        treeView = mTree.getDCaseTreeCanvasView();
        rootRect = mTree.getElement().getBoundingClientRect();
        left = data.position.left - rootRect.left - treeView.left;
        top = data.position.top - rootRect.top - treeView.top;

        type = this.getLabel();

        node = new deosc.DCaseNode(mWeaver.getDCase());
        node.setAttributes(
          {
            id: mTree.getDCase().getNewID(),
            name: mTree.getDCase().getNewName(
              deosc.DCase.getNodeNamePrefix(type)),
            type: type
          });

        nodeView = deosc.canvas.createDCaseNodeView(node);
        nodeView.setPosition(left, top);

        mWeaver.command(new AddNodeCommand(mTree, nodeView));
      }

      constructor(weaver);
    }


    function DCaseNodeAddChildMenu (weaver) {
      var self = this, mWeaver, mTree, mParentType;

      deosc.Menu.call(self);

      self.onInit = function (data) {
        mParentType = data.target.getType();
        update();
      };

      function constructor (weaver) {
        var i, type;

        mWeaver = weaver;
        mTree = weaver.getDCaseTreeView();

        for (i = 0; i < deosc.DCase.NODE_TYPE.length; i++) {
          type = deosc.DCase.NODE_TYPE[i];
          self.addMenuItem(new deosc.MenuItem(type, onAddChild));
        }
      }

      function update() {
        var i, items, item, type;

        items = self.getMenuItems();

        for (i = 0; i < items.length; i++) {
          item = items[i];
          type = item.getLabel();
          if (isSelectable(type)) {
            item.show();
            item.disabled(false);
          } else {
            item.disabled(true);
          }
        }
      }

      function isSelectable (type) {
        var i, selectable;
        if (mParentType in deosc.DCase.SELECTABLE_NODE_TYPE) {
          selectable = deosc.DCase.SELECTABLE_NODE_TYPE[mParentType];
          for (i = 0; i < selectable.length; i++) {
            if (selectable[i] == type) {
              return true;
            }
          }
        }
        return false;
      }

      function onAddChild (data) {
        var node, nodeView, link, linkView, type;

        type = this.getLabel();

        node = new deosc.DCaseNode(mWeaver.getDCase());
        node.setAttributes(
          {
            id: mTree.getDCase().getNewID(),
            name: mTree.getDCase().getNewName(
              deosc.DCase.getNodeNamePrefix(type)),
            type: type
          });

        nodeView = deosc.canvas.createDCaseNodeView(node);

        link = new deosc.DCaseLink(mWeaver.getDCase());
        link.setAttributes(
          {
            id: mTree.getDCase().getNewID(),
            name: mTree.getDCase().getNewName(
              deosc.DCase.LINK_NAME_PREFIX),
            type: deosc.DCaseLink.getLinkType(node.getType()),
            source: data.target.node.getID(),
            target: node.getID()
          });

        linkView = deosc.canvas.createDCaseLinkView(link);

        mWeaver.command(
          [
            new AddNodeCommand(mTree, nodeView),
            new deosc.AddLinkCommand(mTree, linkView),
            new deosc.ArrangeCommand(mTree)
          ]);

        setTimeout(
          function () {
            var cv = mTree.getDCaseTreeCanvasView();
            var display = cv.display;
            var dispLeft = -cv.left;
            var dispTop = -cv.top;
            var dispRight = dispLeft + display.width();
            var dispBottom = dispTop + display.height();
            var left = nodeView.left < dispLeft;
            var right = dispRight < nodeView.right;
            var top = nodeView.top < dispTop;
            var bottom = dispBottom < nodeView.bottom;

            if (left && right) {
              if (top && bottom) {
                ;
              } else if (top) {
                mTree.moveTo(nodeView, "top");
              } else if (bottom) {
                mTree.moveTo(nodeView, "bottom");
              } else {
                ;
              }
            } else if (left) {
              if (top && bottom) {
                mTree.moveTo(nodeView, "left");
              } else if (top) {
                mTree.moveTo(nodeView, "top-left");
              } else if (bottom) {
                mTree.moveTo(nodeView, "bottom-left");
              } else {
                mTree.moveTo(nodeView, "left");
              }
            } else if (right) {
              if (top && bottom) {
                mTree.moveTo(nodeView, "right");
              } else if (top) {
                mTree.moveTo(nodeView, "top-right");
              } else if (bottom) {
                mTree.moveTo(nodeView, "bottom-right");
              } else {
                mTree.moveTo(nodeView, "right");
              }
            } else {
              if (top && bottom) {
                ;
              } else if (top) {
                mTree.moveTo(nodeView, "top");
              } else if (bottom) {
                mTree.moveTo(nodeView, "bottom");
              } else {
                ;
              }
            }

          }, 100);

      }

      constructor(weaver);
    }


    function DCaseNodeEditor () {
      var self = this, mNode, mMain, mTable, mName, mType, mId, mDesc, mPDesc,
      mTypeList, mResponsibilityForm, mValidUntilForm;

      deosc.Dialog.call(self, {resizable: false});

      self.setDCaseNode = function (node) {
        mNode = node;
        mName.setText(node.getName());
        mType.setValue(mTypeList.indexOf(node.getType()));
        mId.setText(node.getID());
        mDesc.setText(node.getDescription());
        mPDesc.setText(node.getParameterizedDesc());
        mResponsibilityForm.setDCaseNode(node);
        mValidUntilForm.setDCaseNode(node);
      };

      self.getName = function () {
        return mName.getText();
      };

      self.getType = function () {
        return mTypeList[mType.getValue()];
      };

      self.getDescription = function () {
        return mDesc.getText();
      };

      self.getParameterizedDesc = function () {
        return mPDesc.getText();
      };

      self.getResponsibilityName = function () {
        return mResponsibilityForm.getName();
      };

      self.getResponsibilityEmail = function () {
        return mResponsibilityForm.getEmail();
      };

      self.getResponsibilityIcon = function () {
        return mResponsibilityForm.getIcon();
      };

      self.getValidUntil = function () {
        return mValidUntilForm.getValidUntil();
      };

      function constructor () {
        var i, type, box;

        mTypeList = [];
        for (i = 0; i < deosc.DCase.NODE_TYPE.length; i++) {
          type = deosc.DCase.NODE_TYPE[i];
          mTypeList.push(type);
        }

        self.setTitle("Node Editor");

        mMain = new deosc.View();
        mMain.setWidth(400);
        mMain.setHeight(400);
        $(mMain.getElement()).css({overflow: "auto"});

        mTable = new deosc.TableView();

        mName = new deosc.InputText();
        mName.setWidth(276);

        mType = new deosc.SelectView();
        mType.setWidth(280);
        for (i = 0; i < mTypeList.length; i++) {
          mType.addItem(mTypeList[i], i);
        }

        mId = new deosc.InputText();
        mId.setWidth(276);
        mId.readonly(true);

        mDesc = new deosc.Textarea();
        mDesc.setSize(274, "3em");

        mPDesc = new deosc.Textarea();
        mPDesc.setSize(274, "3em");

        mTable.addStyle("table1");
        mTable.addRow(["Name", mName]);
        mTable.addRow(["Type", mType]);
        mTable.addRow(["ID", mId]);
        mTable.addRow(["Description", mDesc]);
        mTable.addRow(["Paramterized<br>Description", mPDesc]);
        mTable.getItem(0, 0).setWidth(80);

        mResponsibilityForm = new ResponsibilityForm();

        mValidUntilForm = new ValidUntilForm();

        mMain.addChild(mTable);
        mMain.addChild(mResponsibilityForm);
        mMain.addChild(mValidUntilForm);

        self.setContent(mMain);
      }

      constructor();
    }


    function ResponsibilityForm () {
      var self = this, mNameView, mEmailView, mIconView;

      deosc.View.call(self);

      self.getName = function () {
        return mNameView.getText();
      };

      self.getEmail = function () {
        return mEmailView.getText();
      };

      self.getIcon = function () {
        return mIconView.getText();
      };

      self.setDCaseNode = function (node) {
        var res = node.getResponsibility();
        if (res) {
          mNameView.setText(res.name);
          mEmailView.setText(res.address);
          mIconView.setText(res.icon);
        } else {
          mNameView.setText("");
          mEmailView.setText("");
          mIconView.setText("");
        }
      };

      function constructor () {
        var label, table;

        label = new deosc.View();
        label.setText("Responsibility");
        label.addStyle("label");

        mNameView = new deosc.InputText();
        mNameView.setWidth(252);

        mEmailView = new deosc.InputText();
        mEmailView.setWidth(252);

        mIconView = new deosc.InputText();
        mIconView.setWidth(252);

        table = new deosc.TableView();
        table.addRow(["Name", mNameView]);
        table.addRow(["E-Mail", mEmailView]);
        table.addRow(["Icon URL", mIconView]);
        table.setMargin("0 0 0 32px");
        table.getItem(0, 0).setWidth(80);
        table.getItem(1, 0).setWidth(80);

        self.addChild(label);
        self.addChild(table);
        self.setPadding("16px 0 8px 0  ");
      }

      constructor();
    }


    function ValidUntilForm () {
      var self = this, mValidUntilView;

      deosc.View.call(self);

      self.setDCaseNode = function (node) {
        mValidUntilView.setText(node.getValidUntil());
      };

      self.getValidUntil = function () {
        return mValidUntilView.getText();
      };

      function constructor () {
        var label, table;

        label = new deosc.View();
        label.setText("Valid Until");
        label.addStyle("label");

        mValidUntilView = new deosc.InputText();
        mValidUntilView.setWidth(346);
        mValidUntilView.setMargin("0 0 0 32px");

        self.addChild(label);
        self.addChild(mValidUntilView);
        self.setPadding("16px 0 8px 0");
      }

      constructor();
    }


    function AddNodeCommand (tree, nodeView) {
      var self = this, mTree, mNodeView;

      self.execute = function () {
        mTree.addDCaseNodeView(mNodeView);
        mTree.onChange();
      };

      self.undo = function () {
        mTree.removeDCaseNodeView(mNodeView);
        mTree.onChange();
      };

      function constructor () {
        mTree = tree;
        mNodeView = nodeView;
      }

      constructor();
    }


    function EditNodeCommand (tree, nodeView, attrs) {
      var self = this, mTree, mNode, mBefore, mAfter;

      self.execute = function () {
        mNode.setAttributes(mAfter);
        mTree.onChange();
      };

      self.undo = function () {
        mNode.setAttributes(mBefore);
        mTree.onChange();
      };

      function constructor () {
        mTree = tree;
        if (nodeView instanceof deosc.DCaseNode) {
          mNode = nodeView;
        } else {
          mNode = nodeView.node;
        }
        mBefore = mNode.getAttributes();
        mAfter = attrs;
      }

      constructor();
    }


    function DeleteNodeCommand (tree, nodeView) {
      var self = this, mTree, mNode, mNodeView, mIndex;

      self.execute = function () {
        mTree.removeDCaseNodeView(mNodeView);
        mTree.onChange();
      };

      self.undo = function () {
        mTree.addDCaseNodeView(mNodeView, mIndex);
        mTree.onChange();
      };

      function constructor () {
        mTree = tree;
        mNode = nodeView.node;
        mNodeView = nodeView;
        mIndex = mTree.getDCase().getDCaseNodes().indexOf(mNode);
      }

      constructor();
    }


    deosc.DCaseWeaver.ready(
      function (weaver) {
        weaver.addPlugin(new DCaseNodeEditorPlugin());
      });

    window.deosc.AddNodeCommand    = AddNodeCommand;
    window.deosc.EditNodeCommand   = EditNodeCommand;
    window.deosc.DeleteNodeCommand = DeleteNodeCommand;
  }
) (window);
(
  function (window) {

    function DCaseLinkEditorPlugin () {
      var self = this, mWeaver, mWidget;

      self.setDCaseWeaver = function (weaver) {
        mWeaver = weaver;
      };

      self.onCreate = function () {
        var menu, item;

        item = new deosc.MenuItem("Add Link");
        item.setMenu(new DCaseNodeAddLinkMenu(mWeaver));

        menu = mWeaver.getDCaseNodeMenu();
        menu.addMenuItem(item);

        menu = mWeaver.getDCaseLinkMenu();
        menu.addMenuItem(new deosc.MenuItem("Edit Link", onEditLink));
        menu.addMenuItem(new deosc.MenuItem("Delete Link", onDeleteLink));
      };

      self.onEvent = function (event) {
        if (event.what == deosc.DCaseWeaver.Event.CLICK_LINK) {
          onClickLink(event.data);
        }
      };

      function onClickLink (data) {
        editLink(data.view);
      }

      function onEditLink (data) {
        var dialog, tree, view, link, attrs;

        function onOK () {
          attrs = dialog.getAttributes();

          mWeaver.command(new EditLinkCommand(tree, view, attrs));

          dialog.close();
        }

        tree = mWeaver.getDCaseTreeView();
        view = data.target;
        link = data.target.link;
        attrs = link.getAttributes();

        dialog = new LinkEditorDialog(mWeaver, attrs);
        dialog.setOnOKListener(onOK);
        mWeaver.dialogPopup(dialog);
      }

      function onDeleteLink (data) {
        deleteLink(data.target);
      }

      function deleteLink (view) {
        var tree = mWeaver.getDCaseTreeView();
        mWeaver.command(new DeleteLinkCommand(tree, view));
      }

      function editLink (view) {
        var tree, link, attrs, editor;

        tree = mWeaver.getDCaseTreeView();
        link = view.link;
        attrs = link.getAttributes();

        view.hide();

        editor = new LinkEditor(
          tree, attrs,
          function callback (sourceNode, targetNode) {
            if (sourceNode || targetNode) {
              attrs.source = sourceNode.getID();
              attrs.target = targetNode.getID();
              attrs.type = deosc.DCaseLink.getLinkType(targetNode.getType());
              mWeaver.command(new EditLinkCommand(tree, view, attrs));
            }
            view.show();
          });
      }
    }

    function LinkEditor (tree, attrs, callback) {
      var self = this, mTree, mLink, mLinkView, mLinkEditor, mTreeView,
      mBaseView;
      var lastSourceNodeView;
      var lastTargetNodeView;
      var originalSourceNodeView;
      var originalTargetNodeView;

      function constructor () {
        var dcase, link;

        mTree = tree;

        dcase = mTree.getDCase();

        mTreeView = mTree.getDCaseTreeCanvasView();
        mBaseView = mTree.getBaseCanvasView();

        link = new deosc.DCaseLink(dcase);
        link.setAttributes(attrs);
        mLinkView = deosc.canvas.createDCaseLinkView(link);

        mLinkView.hide();

        originalSourceNodeView = lastSourceNodeView
          = findNodeView(
            mTree.getDCase().findDCaseNodeByID(mLinkView.link.getSource()));
        originalTargetNodeView = lastTargetNodeView
          = findNodeView(
            mTree.getDCase().findDCaseNodeByID(mLinkView.link.getTarget()));

        mLinkEditor = deosc.canvas.createDCaseLinkEditor();
        mLinkEditor.setOnCompleteListener(onComplete);
        mLinkEditor.setOnDragStartSourcePointListener(onDragStartSourcePoint);
        mLinkEditor.setOnDragEndSourcePointListener(onDragEndSourcePoint);
        mLinkEditor.setOnDragStartTargetPointListener(onDragStartTargetPoint);
        mLinkEditor.setOnDragEndTargetPointListener(onDragEndTargetPoint);
        mLinkEditor.setLayoutParams(
          {
            x: mTreeView.left, y: mTreeView.top,
            width: mTreeView.width(), height: mTreeView.height()
          });
        mBaseView.addView(mLinkEditor);

        if (originalSourceNodeView && !originalTargetNodeView) {
          updateSourcePosition();
          var pos = mLinkEditor.getSourcePosition();
          mLinkEditor.setTargetPosition(pos.x, pos.y + 16);
          mLinkEditor.fixSourcePoint(true);
        } else if (!originalSourceNodeView && originalTargetNodeView) {
          updateTargetPosition();
          var pos = mLinkEditor.getTargetPosition();
          mLinkEditor.setSourcePosition(pos.x, pos.y - 16);
          mLinkEditor.fixTargetPoint(true);
        } else if (!originalSourceNodeView && !originalTargetNodeView) {
        } else {
          updateSourcePosition();
          updateTargetPosition();
          mLinkEditor.fixSourcePoint(true);
          mLinkEditor.fixTargetPoint(true);
        }
      }

      function findNodeView(node) {
        for (var i = 0; i < mTreeView.childViews.length; i++) {
          var v = mTreeView.childViews[i];
          if (v instanceof deosc.canvas.DCaseNodeView && v.node == node)
            return v;
        }
        return null;
      }

      function onComplete () {
        mBaseView.removeView(mLinkEditor);
        mTreeView.setOnMouseoverNodeListener(null);
        mTreeView.setOnMouseoutNodeListener(null);

        if (callback) {
          var source, target;
          if (lastSourceNodeView) {
            source = lastSourceNodeView.node;
          } else if (originalSourceNodeView) {
            source = originalSourceNodeView.node;
          } else {
            source = null;
          }
          if (lastTargetNodeView) {
            target = lastTargetNodeView.node;
          } else if (originalTargetNodeView) {
            target = originalTargetNodeView.node;
          } else {
            target = null;
          }
          callback(source, target);
        }

        mLinkView.show();
      }

      function onDragStartSourcePoint () {

        function onMouseoverNode () {
          var x, y;
          if (lastTargetNodeView == this) return;

          var nodeView = lastSourceNodeView = this;
          var targetType = "";
          var childNode =
            mTree.getDCase().findDCaseNodeByID(mLinkView.link.getTarget());
          if (childNode)
            targetType = childNode.getType();

          if (["Context", "Justification", "Assumption", "Parameter"]
              .indexOf(targetType) != -1) {
            x = nodeView.right;
            y = nodeView.top + Math.floor(nodeView.height() / 2);
          } else {
            x = nodeView.left +Math.floor(nodeView.width() / 2);
            y = nodeView.bottom;
          }
          mLinkEditor.setSourcePosition(x, y);
          mLinkEditor.fixSourcePoint(true);
        }

        function onMouseoutNode () {
          lastSourceNodeView = null;
          mLinkEditor.fixSourcePoint(false);
        }

        mLinkEditor.fixSourcePoint(false);
        mTreeView.setOnMouseoverNodeListener(onMouseoverNode);
        mTreeView.setOnMouseoutNodeListener(onMouseoutNode);
      }

      function onDragEndSourcePoint () {
        updateSourcePosition();
        mLinkEditor.fixSourcePoint(true);
        mTreeView.setOnMouseoverNodeListener(null);
        mTreeView.setOnMouseoutNodeListener(null);
      }

      function updateSourcePosition () {
        var x, y;

        if (!lastSourceNodeView)
          lastSourceNodeView = originalSourceNodeView;

        if (!lastSourceNodeView) return;

        var nodeView = lastSourceNodeView;
        var targetType = "";
        var childNode =
          mTree.getDCase().findDCaseNodeByID(mLinkView.link.getTarget());
        if (childNode)
          targetType = childNode.getType();

        if (["Context", "Justification", "Assumption", "Parameter"]
            .indexOf(targetType) != -1) {
          x = nodeView.right;
          y = nodeView.top + Math.floor(nodeView.height() / 2);
        } else {
          x = nodeView.left + Math.floor(nodeView.width() / 2);
          y = nodeView.bottom;
        }
        mLinkEditor.setSourcePosition(x, y);
      }

      function onDragStartTargetPoint () {

        function onMouseoverNode () {
          var x, y;

          if (lastSourceNodeView == this) return;

          var nodeView = lastTargetNodeView = this;
          var targetType = "";
          var childNode =
            mTree.getDCase().findDCaseNodeByID(mLinkView.link.getTarget());
          if (childNode)
            targetType = childNode.getType();

          if (["Context", "Justification", "Assumption", "Parameter"]
              .indexOf(targetType) != -1) {
            x = nodeView.left;
            y = nodeView.top + Math.floor(nodeView.height() / 2);
          } else {
            x = nodeView.left +Math.floor(nodeView.width() / 2);
            y = nodeView.top;
          }
          mLinkEditor.setTargetPosition(x, y);
          mLinkEditor.fixTargetPoint(true);
        }

        function onMouseoutNode () {
          lastTargetNodeView = null;
          mLinkEditor.fixTargetPoint(false);
        }

        mLinkEditor.fixTargetPoint(false);
        mTreeView.setOnMouseoverNodeListener(onMouseoverNode);
        mTreeView.setOnMouseoutNodeListener(onMouseoutNode);
      }

      function onDragEndTargetPoint () {
        updateTargetPosition();
        mLinkEditor.fixTargetPoint(true);
        mTreeView.setOnMouseoverNodeListener(null);
        mTreeView.setOnMouseoutNodeListener(null);
      }

      function updateTargetPosition () {
        var x, y;
        if (!lastTargetNodeView)
          lastTargetNodeView = originalTargetNodeView;

        if (!lastTargetNodeView) return;

        var nodeView = lastTargetNodeView;
        var targetType = "";
        var childNode =
          mTree.getDCase().findDCaseNodeByID(mLinkView.link.getTarget());
        if (childNode)
          targetType = childNode.getType();

        if (["Context", "Justification", "Assumption", "Parameter"]
            .indexOf(targetType) != -1) {
          x = nodeView.left;
          y = nodeView.top + Math.floor(nodeView.height() / 2);
        } else {
          x = nodeView.left +Math.floor(nodeView.width() / 2);
          y = nodeView.top;
        }
        mLinkEditor.setTargetPosition(x, y);
      }

      constructor();
    }


    function DCaseNodeAddLinkMenu (weaver) {
      var self = this, mWeaver, mTree, mParentType;

      deosc.Menu.call(self);

      self.setParentType = function (type) {
        mParentType = type;
        update();
      };

      function constructor (weaver) {
        var i, type;

        mWeaver = weaver;
        mTree = weaver.getDCaseTreeView();

        self.addMenuItem(new deosc.MenuItem("Parent", onAddLink));
        self.addMenuItem(new deosc.MenuItem("Child", onAddLink));
      }

      function onAddLink (data) {
        var attrs, editor;

        attrs = {
          id: mTree.getDCase().getNewID(),
          name: mTree.getDCase().getNewName(deosc.DCase.LINK_NAME_PREFIX)
        };

        if (this.getLabel() == "Parent") {
          attrs.target = data.target.node.getID();
        } else {
          attrs.source = data.target.node.getID();
        }

        editor = new LinkEditor(
          mTree, attrs,
          function callback (sourceNode, targetNode) {
            var link, linkView, attrs;

            if (sourceNode && targetNode) {
              attrs = {
                id: mTree.getDCase().getNewID(),
                name: mTree.getDCase().getNewName(
                  deosc.DCase.LINK_NAME_PREFIX),
                type: deosc.DCaseLink.getLinkType(targetNode.getType()),
                source: sourceNode.getID(),
                target: targetNode.getID()
              };

              link = new deosc.DCaseLink(mWeaver.getDCase());
              link.setAttributes(attrs);
              linkView = deosc.canvas.createDCaseLinkView(link);
              mWeaver.command(new AddLinkCommand(mTree, linkView));
            }
          });

      };

      constructor(weaver);
    }

    function LinkEditorDialog (weaver, attrs) {
      var self = this, mWeaver = weaver, mAttributes = attrs,
      mMain, mName, mType, mAttachment, mId, mSource, mTarget;

      deosc.Dialog.call(self);

      self.getAttributes = function () {
        var attrs = {}, name;
        for (name in mAttributes) {
          attrs[name] = mAttributes[name];
        }
        attrs.name = mName.getText();
        attrs.type = mType.getValue();
        attrs.attachment = mAttachment.getText();
        return attrs;
      };

      function constructor () {
        mName = new deosc.InputText();
        mName.setText(attrs.name);

        mId = new deosc.InputText();
        mId.readonly(true);
        mId.setText(attrs.id);

        mType = new deosc.SelectView();
        mType.addItem("SupportedBy", "SupportedBy");
        mType.addItem("InContextOf", "InContextOf");
        mType.addItem("Responsibility", "Responsibility");
        mType.setValue(attrs.type);

        mSource = new deosc.InputText();
        mSource.readonly(true);
        mSource.setText(attrs.source);

        mTarget = new deosc.InputText();
        mTarget.readonly(true);
        mTarget.setText(attrs.target);

        mAttachment = new deosc.InputText();
        mAttachment.setText(attrs.attachment);

        mMain = new deosc.TableView();
        mMain.addRow(["Name", mName]);
        mMain.addRow(["ID", mId]);
        mMain.addRow(["Type", mType]);
        mMain.addRow(["Source", mSource]);
        mMain.addRow(["Target", mTarget]);
        mMain.addRow(["Attachment", mAttachment]);

        self.setContent(mMain);
        self.setTitle("Edit Link");
      }

      constructor();
    }

    function EditLinkCommand (tree, linkView, attrs) {
      var self = this, mTree, mLink, mBefore, mAfter;

      self.execute = function () {
        mLink.setAttributes(mAfter);
        mTree.onChange();
      };

      self.undo = function () {
        mLink.setAttributes(mBefore);
        mTree.onChange();
      };

      function constructor () {
        mTree = tree;
        mLink = linkView.link;
        mBefore = mLink.getAttributes();
        mAfter = attrs;
      }

      constructor();
    }


    function AddLinkCommand (tree, linkView) {
      var self = this, mTree, mLinkView;

      self.execute = function () {
        mTree.addDCaseLinkView(mLinkView);
        mTree.onChange();
      };

      self.undo = function () {
        mTree.removeDCaseLinkView(mLinkView);
        mTree.onChange();
      };

      function constructor () {
        mTree = tree;
        mLinkView = linkView;
      }

      constructor();
    }


    function DeleteLinkCommand (tree, linkView) {
      var self = this, mTree, mLink, mLinkView, mIndex;

      self.execute = function () {
        mTree.removeDCaseLinkView(mLinkView);
        mTree.onChange();
      };

      self.undo = function () {
        mTree.addDCaseLinkView(mLinkView, mIndex);
        mTree.onChange();
      };

      function constructor () {
        mTree = tree;
        mLink = linkView.link;
        mLinkView = linkView;
        mIndex = mTree.getDCase().getDCaseLinks().indexOf(mLink);
      }

      constructor();
    }


    deosc.DCaseWeaver.ready(
      function (weaver) {
        weaver.addPlugin(new DCaseLinkEditorPlugin());
      });

    window.deosc.AddLinkCommand    = AddLinkCommand;
    window.deosc.EditLinkCommand    = EditLinkCommand;
    window.deosc.DeleteLinkCommand = DeleteLinkCommand;
  }
) (window);(
  function (window) {

    function DCaseNodeEditorPlugin () {
      var self = this, mWeaver, mWidget;

      self.setDCaseWeaver = function (weaver) {
        mWeaver = weaver;
      };

      self.onCreate = function () {
        var menu;
        menu = mWeaver.getDCaseNodeMenu();
        menu.addMenuItem(new deosc.MenuItem("Attachment", onAttachment));
      };

      function onAttachment (data) {
        var tree, dialog;

        tree = mWeaver.getDCaseTreeView();

        dialog = new AttachmentDialog();
        dialog.setValue(data.target.node.getAttachment());
        dialog.setOnOKListener(
          function () {
            var attrs;

            attrs = data.target.node.getAttributes();
            attrs.attachment = dialog.getValue();

            mWeaver.command(
              new deosc.EditNodeCommand(tree, data.target, attrs));

            dialog.close();
          });

        mWeaver.dialogPopup(dialog);
      };
    }

    function AttachmentDialog () {
      var self = this, mMain, mLabel, mInput;

      deosc.Dialog.call(self);

      self.onCreate = function () {
      };

      self.setValue = function (value) {
        if (mInput) mInput.setText(value);
      };

      self.getValue = function () {
        return mInput.getText();
      };

      function constructor () {
        self.setTitle("Attachment");
        mMain = new deosc.View();
        mMain.setPadding("8px 8px");
        mLabel = new deosc.View();
        mLabel.addStyle("label");
        mLabel.setText("URL:");
        mInput = new deosc.InputText();
        mInput.setWidth(300);
        mMain.addChild(mLabel);
        mMain.addChild(mInput);
        self.setContent(mMain);
      }

      constructor();
    }

    deosc.DCaseWeaver.ready(
      function (weaver) {
        weaver.addPlugin(new DCaseNodeEditorPlugin());
      });
  }
) (window);(
  function (window) {
    function SiblingOrderPlugin () {
      var self = this, mWeaver;

      self.setDCaseWeaver = function (weaver) {
        mWeaver = weaver;
      };

      self.onCreate = function () {
        var item = new deosc.MenuItem("Sibling Order", onSiblingOrder);
        mWeaver.getDCaseNodeMenu().addMenuItem(item);
      };

      function onSiblingOrder (data) {
        var dialog, node, links;

        function onOK () {
          var links, i, tree, treeCV, view, attrs, cmds = [];
          tree = mWeaver.getDCaseTreeView();
          treeCV = tree.getDCaseTreeCanvasView();
          links = dialog.getDCaseLinks();
          for (i = 0; i < links.length; i++) {
            view = treeCV.findDCaseLinkViewByID(links[i].getID());
            attrs = links[i].getAttributes();
            cmds.push(new deosc.EditLinkCommand(tree, view, attrs));
          }
          cmds.push(new deosc.ArrangeCommand(tree));

          mWeaver.command(cmds);

          dialog.close();
        }

        node = data.target.node;

        dialog = new SiblingOrderDialog(mWeaver, node);
        dialog.setOnOKListener(onOK);

        mWeaver.dialogPopup(dialog);
      }
    }

    function SiblingOrderDialog (weaver, node) {
      var self = this, mWeaver = weaver, mNode = node, mTree, mTreeCV,
      mDCase, mTarget, mNodeLinkTable, mEvent, mNoParent;

      deosc.Dialog.call(self);

      self.getDCaseLinks = function () {
        var links = [], id;
        if (!mNoParent) {
          for (id in mNodeLinkTable) {
            links.push(mNodeLinkTable[id].link.link);
          }
        }
        return links;
      };

      function createDCase () {
        var dcase_org, dcase, node, nodes, link, links, parent, i;

        function cloneNode (node) {
          var clone = new deosc.DCaseNode(dcase);
          clone.setAttributes(node.getAttributes());
          return clone;
        }

        function cloneLink (link) {
          var clone = new deosc.DCaseLink(dcase);
          clone.setAttributes(link.getAttributes());
          return clone;
        }

        dcase = new deosc.DCase();

        dcase_org = mWeaver.getDCase();

        links = dcase_org.findDCaseLinksByTarget(mNode.getID());
        if (0 < links.length) {
          link = links[0];
          parent = dcase_org.findDCaseNodeByID(link.getSource());
        }

        if (parent) {
          dcase.addDCaseNode(cloneNode(parent));
          links = dcase_org.findDCaseLinksBySource(parent.getID());
          for (i = 0; i < links.length; i++) {
            link = links[i];
            node = dcase_org.findDCaseNodeByID(link.getTarget());
            if (node) {
              dcase.addDCaseNode(cloneNode(node));
              dcase.addDCaseLink(cloneLink(link));
            }
          }
        } else {
          dcase.addDCaseNode(cloneNode(mNode));
        }

        return dcase;
      }


      function createNodeLinkTable () {
        var table, nodes, links, link, parent, i;

        table = {};

        nodes = mDCase.getRootNodes();

        if (0 < nodes.length && nodes[0].getID() != mNode.getID()) {
          parent = nodes[0];
          links = mDCase.findDCaseLinksBySource(parent.getID());
          for (i = 0; i < links.length; i++) {
            link = links[i];
            node = mDCase.findDCaseNodeByID(link.getTarget());
            table[node.getID()] = {
              node: mTreeCV.findDCaseNodeViewByID(node.getID()),
              link: mTreeCV.findDCaseLinkViewByID(link.getID()),
              type: link.getType(),
              target: node.getID() == mNode.getID()
            };
          }

        } else {
          node = mTreeCV.findDCaseNodeViewByID(mNode.getID());
          table[mNode.getID()] = {node: node, target: true};
          mNoParent = true;
        }

        return table;
      }


      function onMousedown (x, y) {
        mEvent = {
          mouse: {x: x, y: y},
          node: {x: mTarget.node.left, y: mTarget.node.top}
        };
        mTreeCV.setOnMouseupListener(onMouseup);
        mTreeCV.setOnMouseoutListener(onMouseup);
        mTreeCV.setOnMousemoveListener(onMousemove);
      }


      function onMouseup () {
        mEvent = null;
        mTreeCV.setOnMouseupListener(null);
        mTreeCV.setOnMousemoveListener(null);
        mTree.arrange();
      }


      function onMousemove (x, y) {
        var before = [], after = [], id, i, changed;


        if (mTarget.type == "InContextOf") {
          mTarget.node.setPosition(mEvent.node.x,
                                   mEvent.node.y + y - mEvent.mouse.y);
        } else {
          mTarget.node.setPosition(mEvent.node.x + x - mEvent.mouse.x,
                                   mEvent.node.y);
        }

        for (id in mNodeLinkTable) {
          before.push(mNodeLinkTable[id]);
          after.push(mNodeLinkTable[id]);
        }

        before.sort(
          function (t1, t2) {
            return t1.link.link.getSiblingOrder() - t2.link.link.getSiblingOrder();
          });

        after.sort(
          function (t1, t2) {
            if (t1.type == "InContextOf" && t2.type == "InContextOf") {
              return t1.node.top - t2.node.top;
            } else if (t1.type == "InContextOf" && t2.type != "InContextOf") {
              return 1;
            } else if (t1.type != "InContextOf" && t2.type == "InContextOf") {
              return -1;
            } else {
              return t1.node.left - t2.node.left;
            }
          });

        changed = false;
        for (i = 0; i < after.length; i++) {
          if (before[i].node.node.getID() != after[i].node.node.getID()){
            changed = true;
          }
          after[i].link.link.setSiblingOrder(i);
        }

        if (changed) {
          mTree.arrange();
        }
      }


      function constructor () {
        var id, t;

        mDCase = createDCase();

        mTree = new deosc.DCaseTreeView();
        mTree.setSize(300, 150);
        $(mTree.getElement()).css("border", "1px solid #ddd");
        mTree.setDCase(mDCase);

        mTreeCV = mTree.getDCaseTreeCanvasView();
        mTreeCV.draggable(false);
        mTreeCV.setDescriptionHidden(true);
        mTreeCV.setTypeHidden(true);

        var super_onMeasure = mTreeCV.onMeasure;
        mTreeCV.onMeasure = function (w, h) {
          super_onMeasure.call(mTreeCV, w, h);
          mTree.setSize(mTreeCV.getMeasuredWidth(), mTreeCV.getMeasuredHeight());
          mTreeCV.onMeasure = super_onMeasure;
        };

        mNodeLinkTable = createNodeLinkTable();

        for (id in mNodeLinkTable) {
          t = mNodeLinkTable[id];
          if (t.target) {
            mTarget = t;
            break;
          }
        }

        mTarget.node.setBackgroundColor("#fa4");
        if (!mNoParent)
          mTarget.node.setOnMousedownListener(onMousedown);

        //target node brings to front
        mTreeCV.removeDCaseNodeView(mTarget.node);
        mTreeCV.addDCaseNodeView(mTarget.node);

        self.setContent(mTree);
        self.setTitle("Sibling Order");
      }

      constructor();
    }

    window.deosc.DCaseWeaver.ready(
      function (weaver) {
        weaver.addPlugin(new SiblingOrderPlugin());
      });
  }
) (window);(
  function (window) {
    function sendDCaseChangeEvent (weaver) {
      weaver.sendEvent(
        {
          what: deosc.DCaseWeaver.Event.DCASE_CHANGED,
          data: weaver.getDCase()
        });
    }

    function RemoveDScriptCommand (weaver, node) {
      var self = this, mWeaver, mNode, mDScriptElem;

      self.execute = function () {
        if (mDScriptElem) {
          $(mDScriptElem).remove();
          sendDCaseChangeEvent(mWeaver);
        }
      };

      self.undo = function () {
        if (mDScriptElem) {
          $(mDScriptElem).appendTo(mNode);
          sendDCaseChangeEvent(mWeaver);
        }
      };

      function constructor () {
        var elems;

        mWeaver = weaver;

        mNode = node;

        elems = elem_findNS(mNode, Ns.DRE, "d-script");
        if (0 < elems.length) {
          mDScriptElem = elems[0];
        }
      }

      constructor();
    }

    function AddDScriptCommand (weaver, node, dscript) {
      var self = this, mWeaver, mNode, mDScriptElem;

      self.execute = function () {
        $(mDScriptElem).appendTo(mNode);
        sendDCaseChangeEvent(mWeaver);
      };

      self.undo = function () {
        $(mDScriptElem).remove();
        sendDCaseChangeEvent(mWeaver);
      };

      function constructor () {
        mWeaver = weaver;
        mNode = node;
        mDScriptElem = dscript.getElement();
        sendDCaseChangeEvent(mWeaver);
      }

      constructor();
    }

    function DScript (elem) {
      var self = this, mElem, mFullNameElem, mValuesElem;

      self.setElement = function (el) {
        mElem = el;
      };

      self.getElement = function () {
        return mElem;
      };

      self.setFullName = function (fullname) {

        if (!mFullNameElem) {
          mFullNameElem = document.createElementNS(
            deosc.Namespace.DRE, "dre:full-name");

          $(mElem).append(mFullNameElem);
        }
        if (fullname && 0 < fullname.length) {
          $(mFullNameElem).text(fullname);
        } else {
          $(mFullNameElem).empty();
        }
      };

      self.getFullName = function () {
        return mFullNameElem ? $(mFullNameElem).text() : null;
      };

      self.setValue = function (name, value) {
        var node, i, valueElem;

        if (!mValuesElem) {
          mValuesElem = document.createElementNS(
            deosc.Namespace.DRE, "dre:values");

          $(mElem).append(mValuesElem);
        }

        for (i = 0; i < mValuesElem.childNodes.length; i++) {
          node = mValuesElem.childNodes[i];
          if (node.nodeType == 1 && node.localName == "value" &&
              $(node).attr("name") == name) {
            valueElem = node;
            break;
          }
        }

        if (!valueElem) {
          valueElem = document.createElementNS(
            deosc.Namespace.DRE, "dre:value");
          $(valueElem).attr("name", name);

          $(mValuesElem).append(valueElem);
        }

        if (value && 0 < value.length) {
          $(valueElem).text(value);
        } else {
          $(valueElem).empty();
        }
      };

      self.getValue = function (name) {
        var i, node;
        for (i = 0; i < mValuesElem.childNodes.length; i++) {
          node = mValuesElem.childNodes[i];
          if (node.nodeType == 1 && node.localName == "value" &&
              $(node).attr("name") == name) {
            return $(node).text();
          }
        }
        return null;
      };

      self.removeValue = function (name) {
        var i, node, targetElem;

        for (i = 0; i < mValuesElem.childNodes.length; i++) {
          node = mValuesElem.childNodes[i];
          if (node.nodeType == 1 && node.localName == "value" &&
             $(node).attr("name") == name) {
            targetElem = node;
            break;
          }
        }

        if (targetElem) {
          for (i = 0; i < mValuesElem.childNodes.length; i++) {
            node = mValuesElem.childNodes[i];
            if (targetElem == node &&
                i + 1 < mValuesElem.childNodes.length) {
              node = mValuesElem.childNodes[i + 1];
              if (node.nodeType == 3 && $(node).text() == "\n") {
                $(node).remove();
                break;
              }
            }
          }
        }
      };

      self.setValues = function (values) {
        var i;

        $(mValuesElem).empty();

        for (i = 0; i < values.length; i++) {
          self.setValue(values[i].name, values[i].value);
        }
      };

      self.getValues = function () {
        var i, node, ret = [];
        for (i = 0; i < mValuesElem.childNodes.length; i++) {
          node = mValuesElem.childNodes[i];
          if (node.nodeType == 1 && node.localName == "value") {
            ret.push(
              {
                name: $(node).attr("name"),
                value: $(node).text()
              });
          }
        }
        return ret;
      };

      self.setParameters = function (parameters) {
        self.setFullName(
          "full-name" in parameters ? parameters["full-name"] : "");

        self.setValues(
          "values" in parameters ? parameters.values : []);
      };

      self.getParameters = function () {
        var ret = {}, fullname;

        fullname = self.getFullName();
        ret["full-name"] = fullname ? fullname : "";

        ret.values = self.getValues();

        return ret;
      };

      function constructor (elem) {
        if (!elem) {
          elem = document.createElementNS(
            deosc.Namespace.DRE, "dre:d-script");
        }
        self.setElement(elem);

        update();
      }

      function update () {
        var elems;

        elems = elem_findNS(mElem, Ns.DRE, "full-name");
        if (0 < elems.length) mFullNameElem = elems[0];

        elems = elem_findNS(mElem, Ns.DRE, "values");
        if (0 < elems.length) mValuesElem = elems[0];
      }

      constructor(elem);
    }


    function trim(line) {
      var left = 0;
      while (line.charAt(left) == " ") {
        left++;
      };
      return line.substring(left);
    }

    function wordBreak (str) {
      var lines = str.split("\n");

      for (var i = 0; i < lines.length; i++) {
        lines[i] = trim(lines[i]);
      }

      return lines.join("<br>");
    }

    function EditParameterTable (params) {
      var self = this, mParams, mInputText;

      deosc.TableView.call(self);

      self.getName = function () {
        return mParams.name;
      };

      self.getValue = function () {
        return mInputText.getText();
      };

      self.setValue = function (value) {
        mInputText.setText(value);
      };

      function constructor (params) {
        mParams = params;

        self.setWidth("100%");
        self.setMargin("16px 0");
        self.addStyle("table2");

        if (mParams.type == "password") {
          mInputText = new deosc.InputPassword();
        } else {
          mInputText = new deosc.InputText();
        }
        mInputText.setText(mParams["default"]);

        self.addRow(["name", mParams.name]);
        self.addRow(["type", mParams.type]);
        self.addRow(["description", mParams.description]);
        self.addRow(["default", mParams["default"]]);
        self.addRow(["value", mInputText.getElement()]);
      }

      constructor(params);
    }

    function EditDScriptDialog (weaver, dCaseNodeView) {
      var self = this, mMainView, mWeaver, mFileManager, mDSTemplate,
      mSelectView, mTableView, mParamsView, mLabelView,
      mParamTables, mDCaseNodeView, mDCaseNode, mDScript, mModules;

      deosc.Dialog.call(self);

      self.onCreate = function () {
        mSelectView = new deosc.SelectView();
        mSelectView.setWidth("100%");
        mSelectView.setOnChangeListener(onChange);

        mTableView = new deosc.TableView();
        mTableView.addStyle("table2");
        mTableView.setMargin("16px 0");

        mLabelView = new deosc.View();
        mLabelView.setText("Parameters");
        mLabelView.addStyle("label");

        mParamsView = new deosc.View();

        mMainView = new deosc.View();
        mMainView.setSize(400, 300);
        mMainView.setMargin(8);
        $(mMainView.getElement()).css({overflow: "auto"});
        mMainView.addChild(mSelectView);
        mMainView.addChild(mTableView);
        mMainView.addChild(mLabelView);
        mMainView.addChild(mParamsView);

        self.setContent(mMainView);
        self.setTitle("D-Script");

        loadDScript();
      };

      self.onOK = function () {
        var dscript, i, param, node, elems, module, commands = [];

        dscript = new DScript();

        node = mDCaseNode.getElement();

        commands.push(new RemoveDScriptCommand(mWeaver, node));

        i = mSelectView.getValue();
        if (0 <= i && i < mModules.length) {
          module = mModules[i];

          dscript.setFullName(module.getName());

          for (i = 0; i < mParamTables.length; i++) {
            param = mParamTables[i];
            dscript.setValue(param.getName(), param.getValue());
          }

          commands.push(new AddDScriptCommand(mWeaver, node, dscript));
        }

        mWeaver.command(commands);

        self.close();
      };

      function constructor (weaver, dCaseNodeView) {
        var elems;

        mWeaver = weaver;
        mFileManager = weaver.getFileManager();
        mDCaseNodeView = dCaseNodeView;
        mDCaseNode = dCaseNodeView.node;
        mParamTables = [];

        elems = elem_findNS(mDCaseNode.getElement(), Ns.DRE, "d-script");
        mDScript = elems.length ? new DScript(elems[0]) : null;
      }

      function loadDScript () {
        var pref;

        function error () {
          var dialog = new deosc.Dialog();
          var widget = new PreferenceView(mWeaver, pref);
          dialog.setContent(widget);
          dialog.setTitle("Preferences");
          dialog.setOnOKListener(
            function () {
              var p = widget.getPreference();
              for (key in p) {
                pref.setValue(key, p[key]);
              }
              dialog.close();
              getFile();
            });
          mWeaver.dialogPopup(dialog);
        }

        function getFile () {
          var path = pref.getValue("path");
          mFileManager.getFile({path: path}, onLoadDScript, error);
        }

        pref = mWeaver.getPreferences().getPreference("D-Script");

        getFile();
      }

      function onLoadDScript (doc) {
        var i, parser;

        if (typeof doc == "string" || doc instanceof String) {
          parser = new DOMParser();
          doc = parser.parseFromString(doc, "application/xml");
        }

        mDSTemplate = new DScriptTemplate(doc);

        mSelectView.empty();

        if (mDCaseNode.getType() == "Monitor") {
          mModules = mDSTemplate.getMonitors();
        } else if (mDCaseNode.getType() == "Action") {
          mModules = mDSTemplate.getActions();
        }

        mSelectView.addItem("No Module", -1);
        for (i = 0; i < mModules.length; i++) {
          mSelectView.addItem(mModules[i].getName(), i);
        }

        if (mDScript) {
          for (i = 0; i < mModules.length; i++) {
            if (mDScript.getFullName() == mModules[i].getName()) {
              mSelectView.setValue(i);
              break;
            }
          }
        }

        update();
      }

      function onChange () {
        update();
      }

      function update () {
        var module, obj, l, configPath, explanation, params, values, i,
        elems, dscript;

        mTableView.empty();
        mParamsView.empty();
        mParamTables.length = 0;

        i = mSelectView.getValue();
        if (i == -1) return;

        module = mModules[i];

        obj = module.getConfigPath();
        configPath = obj.path + " (" + obj.extension + ")";

        explanation = module.getExplanation();
        explanation = wordBreak(explanation);
        explanation = new deosc.View($("<div>" + explanation + "</div>")[0]);

        mTableView.addRow(["Name", module.getName()]);
        mTableView.addRow(["Description", module.getDescription()]);
        mTableView.addRow(["Version", module.getVersion()]);
        mTableView.addRow(["License", module.getLicense()]);
        mTableView.addRow(["Author", module.getAuthor()]);
        mTableView.addRow(["config_path", configPath]);
        mTableView.addRow(["Explanation", explanation]);


        $(module.getParameters()).each(
          function () {
            var paramTable = new EditParameterTable(this);
            mParamsView.addChild(paramTable);
            mParamTables.push(paramTable);
          });

        $(mMainView.getElement())
          .find("tr").children("td:nth-child(1)")
          .css({width: 100});


        if (mDScript && mDScript.getFullName() == module.getName()) {
          values = mDScript.getValues();
          for (i = 0; i < values.length; i++) {
            setParameter(values[i].name, values[i].value);
          }
        }

      }

      function setParameter (key, value) {
        var i;
        for (i = 0; i < mParamTables.length; i++) {
          if (mParamTables[i].getName() == key) {
            mParamTables[i].setValue(value);
            break;
          }
        }
      }

      constructor(weaver, dCaseNodeView);
    }


    function elem_findNS (elem, ns, tag) {
      var list = [];

      function scan (elem) {
        var i, e;
        for (i = 0; i < elem.childNodes.length; i++) {
          e = elem.childNodes[i];
          if (e.namespaceURI == ns && e.nodeType == 1 &&
              e.localName == tag) {
            list.push(e);
          }
          scan(e);
        }
      }

      scan(elem);

      return list;
    }

    var Ns = {
      DS: "http://www.dependable-os.net/d-script",
      DRE: deosc.Namespace.DRE
    };

    function DScriptTemplate (doc) {
      var self = this, mDoc, mMonitors, mActions;

      self.getMonitors = function () {
        return mMonitors;
      };

      self.getActions = function () {
        return mActions;
      };

      function constructor (doc) {
        var elems, dScript, monitors, actions, i;

        mDoc = doc;
        mMonitors = [];
        mActions = [];

        dScript = mDoc.documentElement;

        elems = elem_findNS(dScript, Ns.DS, "monitors");
        if (elems.length) {
          monitors = elems[0];
          elems = elem_findNS(monitors, Ns.DS, "monitor");
          for (i = 0; i < elems.length; i++) {
            mMonitors.push(new DScriptTemplateModule(elems[i]));
          }
        }

        elems = elem_findNS(dScript, Ns.DS, "actions");
        if (elems.length) {
          actions = elems[0];
          elems = elem_findNS(actions, Ns.DS, "action");
          for (i = 0; i < elems.length; i++) {
            mActions.push(new DScriptTemplateModule(elems[i]));
          }
        }
      }

      constructor(doc);
    }


    function DScriptTemplateModule (elem) {
      var self = this, mElem, mName, mDescription, mVersion, mLicense,
      mAuthor, mConfigTemplate, mConfigPath, mExplanations, mParameters;

      function find (elem, tag) {
        return elem_findNS(elem, Ns.DS, tag);
      }

      self.getName = function () {
        return $(mName).text();
      };

      self.getDescription = function () {
        return $(mDescription).text();
      };

      self.getVersion = function () {
        return $(mVersion).text();
      };

      self.getLicense = function () {
        return $(mLicense).text();
      };

      self.getAuthor = function () {
        return $(mAuthor).text();
      };

      self.getConfigTemplate = function () {
        return $(mConfigTemplate).text();
      };

      self.getConfigPath = function () {
        return {
          path: $(mConfigPath).text(),
          extension: $(mConfigPath).attr("extension")
        };
      };

      self.getExplanations = function () {
        var i, elems, list = [];

        elems = find(mExplanations, "explanation");

        for (i = 0; i < elems.length; i++) {
          list.push(
            {
              explanation: $(elems[i]).text(),
              lang: $(elems[i]).attr("lang")
            });
        }

        return list;
      };

      self.getExplanation = function (lang) {
        var i, elems, re1, re2, la, exp1, exp2, exp3;

        if (!lang) lang = navigator.language.replace("-", "_");

        elems = find(mExplanations, "explanation");

        if (elems.length == 0) return "";

        exp3 = $(elems[0]).text();

        re1 = new RegExp("^" + lang + "$");
        re2 = new RegExp("^" + lang);

        for (i = 0; i < elems.length; i++) {
          la = $(elems[i]).attr("lang");
          if (!la) continue;
          if (la.match(re1)) {
            exp1 = $(elems[i]).text();
            break;
          } else if (!exp2 && la.match(re2)) {
            exp2 = $(elems[i]).text();
          }
        }

        return exp1 || exp2 || exp3;
      };

      self.getParameters = function () {
        var elems, i, list = [], params;

        elems = find(mParameters, "parameter");

        for (i = 0; i < elems.length; i++) {
          elem = elems[i];
          params = {};
          params.name = $(elem).attr("name");
          params.type = $(elem).attr("type");
          $(elem).children().each(
            function () {
              params[this.tagName] = $(this).text();
            });
          list.push(params);
        }

        return list;
      };

      function constructor (elem) {
        var elems;
        mElem = elem;

        elems = find(elem, "name");
        if (elems.length) mName = elems[0];
        elems = find(elem, "description");
        if (elems.length) mDescription = elems[0];
        elems = find(elem, "version");
        if (elems.length) mVersion = elems[0];
        elems = find(elem, "license");
        if (elems.length) mLicense = elems[0];
        elems = find(elem, "author");
        if (elems.length) mAuthor = elems[0];
        elems = find(elem, "config_template");
        if (elems.length) mConfigTemplate = elems[0];
        elems = find(elem, "config_path");
        if (elems.length) mConfigPath = elems[0];
        elems = find(elem, "explanations");
        if (elems.length) mExplanations = elems[0];
        elems = find(elem, "parameters");
        if (elems.length) mParameters = elems[0];
      }

      constructor(elem);
    }


    function DScriptPlugin () {
      var self = this, mWeaver;

      self.setDCaseWeaver = function (weaver) {
        mWeaver = weaver;
      };

      self.onCreate = function () {
        var menuBarItem, editDScriptMenuItem;

        menuBarItem = mWeaver.getMenuBar().getMenuBarItem("D-Script");
        menuBarItem.addMenuItem(new deosc.MenuItem("Deploy", onClickDeploy));

        editDScriptMenuItem
          = new deosc.MenuItem("Edit D-Script", onEditDScript);
        editDScriptMenuItem.onInit = function (data) {
          var type = data.target.node.getType();
          if (type == "Action" ||type == "Monitor") {
            this.disabled(false);
          } else {
            this.disabled(true);
          }
        };

        mWeaver.getDCaseNodeMenu()
          .addMenuItem(editDScriptMenuItem);

      };

      self.onEvent = function (event) {
        if (event.what == deosc.DCaseWeaver.Event.OPEN_PREFERENCE_DIALOG) {
          onOpenPreferenceDialog(event.data);
        }
      };

      function constructor () {
      }

      function onEditDScript (data) {
        var dialog = new EditDScriptDialog(mWeaver, data.target);
        mWeaver.dialogPopup(dialog);
      }

      function onClickDeploy () {
        var pref, path, deploy, dscript;

        function error () {
          console.error("error");
          alert("Deployment failed");
        }

        function onDeploySuccess (data) {
          alert("Deployment succeeded");
        }

        function onGetDScript (dscriptDoc) {
          var doc, dcaseDoc, parser, serializer;

          parser = new DOMParser();
          serializer = new XMLSerializer();

          if (typeof dscriptDoc == "string" || dscriptDoc instanceof String) {
            dscriptDoc = parser.parseFromString(dscriptDoc, "application/xml");
          }

          dcaseDoc = parser.parseFromString(
            serializer.serializeToString(mWeaver.getDCase().getElement()),
            "text/xml");

          doc = document.implementation.createDocument(
            deosc.Namespace.DRE, "deploy", null);

          $(doc.documentElement)
            .append(dcaseDoc.documentElement)
            .append(dscriptDoc.documentElement);

          $.ajax(
            {
              url: deploy,
              type: "POST",
              data: doc,
              processData: false,
              success: onDeploySuccess,
              error: error
            });
        }

        function onGetDScriptFailure () {
          openPreferenceDialog();
        }

        function openPreferenceDialog() {
          var dialog;

          dialog = new PreferenceDialog(mWeaver);
          dialog.setOnOKListener(
            function () {
              dialog.close();
              checkPreference();
            });

          mWeaver.dialogPopup(dialog);
        }

        function onCheckSuccess () {
          mWeaver.getFileManager()
            .getFile({path: path}, onGetDScript, onGetDScriptFailure);
        }

        function onCheckFailure () {
          openPreferenceDialog();
        }

        function checkPreference () {
          pref = mWeaver.getPreferences().getPreference("D-Script");
          if (pref) {
            path = pref.getValue("path");
            deploy = pref.getValue("deploy");
            if (path && deploy) {
              onCheckSuccess();
              return;
            }
          }
          onCheckFailure();
        }

        checkPreference();
      }

      function onOpenPreferenceDialog (dialog) {
        var pref = dialog.getPreference("D-Script");
        var view = new PreferenceView(mWeaver, pref);

        dialog.addView(view, pref);
        dialog.setOnSaveListener(
          function () {
            var p = view.getPreference();
            for (key in p) {
              pref.setValue(key, p[key]);
            }
          });
      }

      constructor();
    }


    function PreferenceDialog (weaver) {
      var self = this, mWeaver, mPreference, mWidget, mOnOKListener;

      deosc.Dialog.call(self);

      self.onCreate = function () {
        mWidget = new PreferenceView(mWeaver, mPreference);

        self.setContent(mWidget);
        self.setTitle("Preference");
      };

      self.setOnOKListener = function (l) {
        mOnOKListener = l;
      };

      self.onOK = function () {
        var pref = mWidget.getPreference();
        for (key in pref) {
          mPreference.setValue(key, pref[key]);
        }

        if (mOnOKListener) mOnOKListener();
      };

      function constructor (weaver) {
        mWeaver = weaver;
        mPreference = mWeaver.getPreferences()
          .getPreference("D-Script");
      }

      constructor(weaver);
    }


    function PreferenceView (weaver, pref) {
      var self = this, mWeaver = weaver, mPreference = pref, mPath, mDeploy;

      deosc.View.call(self);

      self.getPreference = function () {
        return {
          path: mPath.getText(),
          deploy: mDeploy.getText()
        };
      };

      function constructor () {
        var table, button;

        table = new deosc.TableView();

        mPath = new deosc.InputText();
        mPath.setWidth(300);
        mPath.setText(mPreference.getValue("path", ""));

        mDeploy = new deosc.InputText();
        mDeploy.setWidth(300);
        mDeploy.setText(mPreference.getValue("deploy", ""));

        button = new deosc.View($("<button>")[0]);
        button.setText("Change");
        button.setOnClickListener(onClickChange);

        table.addRow(["Template Path", mPath, button]);
        table.addRow(["Deploy URL", mDeploy, ""]);

        $(table.getElement()).find("td").css("white-space", "nowrap");

        self.addChild(table);
      }

      function onClickChange () {
        mWeaver.getFileManager().browse(
          {
            success:function (dialog) {
              dialog.setOnOKListener(
                function (metadata) {
                  mPath.setText(metadata.path);
                  dialog.close();
                });
            },
            filter: function (metadata) {
              return (metadata.isdir || metadata.name.match(/\.xml$/));
            }
          });
      };

      constructor();
    }


    deosc.DCaseWeaver.ready(
      function (weaver) {
        weaver.addPlugin(new DScriptPlugin());
      });
  }
) (window);
(
  function (window) {

    function DCaseModulePlugin () {
      var self = this, mWeaver;

      self.setDCaseWeaver = function (weaver) {
        mWeaver = weaver;
      };

      function createShowHideModuleMenuItem () {
        var menuItem;

        function createShowHideModuleMenu () {
          var menu, showMenuItem, hideMenuItem;

          showMenuItem = new deosc.MenuItem("Show Module", onShowModule);
          showMenuItem.onInit = function (data) {
            showMenuItem.disabled(data.target.isExpandModule());
          };

          hideMenuItem = new deosc.MenuItem("Hide Module", onHideModule);
          hideMenuItem.onInit = function (data) {
            hideMenuItem.disabled(!data.target.isExpandModule());
          };

          menu = new deosc.Menu();
          menu.addMenuItem(showMenuItem);
          menu.addMenuItem(hideMenuItem);
          return menu;
        }

        menuItem = new deosc.MenuItem("Show/Hide Module");

        menuItem.onInit = function (data) {
          var type = data.target.node.getType();
          if (type == "Goal" || type == "Module") {
            menuItem.disabled(false);
          } else {
            menuItem.disabled(true);
          }
        };

        menuItem.setMenu(createShowHideModuleMenu());

        return menuItem;
      }

      function createModularizeMenuItem () {
        var menuItem;

        menuItem = new deosc.MenuItem("Modularize", onCreateModule);
        menuItem.onInit = function (data) {
          if (data.target.node.getType() != "Module") {
            menuItem.disabled(false);
          } else {
            menuItem.disabled(true);
          }
        };

        return menuItem;
      }

      function createRestoreModuleMenuItem () {
        var menuItem;

        menuItem = new deosc.MenuItem("Un-modularize", onRestoreModule);
        menuItem.onInit = function (data) {
          if (data.target.node.getType() == "Module") {
            menuItem.disabled(false);
          } else {
            menuItem.disabled(true);
          }
        };

        return menuItem;
      }

      function createSelectModuleMenuItem () {
        var menuItem;

        menuItem = new deosc.MenuItem("Select Module", onSelectModule);
        menuItem.onInit = function (data) {
          var type;

          type = data.target.node.getType();
          if (type == "Goal" || type == "Module") {
            menuItem.disabled(false);
          } else {
            menuItem.disabled(true);
          }
        };

        return menuItem;
      }

      function createSetFlagMenuItem () {
        var menuItem, publicMenuItem, privateMenuItem;

        function createShowHideModuleMenu () {
          var menu = new deosc.Menu();
          publicMenuItem = new deosc.MenuItem("Public", onSetFlag);
          privateMenuItem = new deosc.MenuItem("Private", onSetFlag);
          menu.addMenuItem(publicMenuItem);
          menu.addMenuItem(privateMenuItem);
          return menu;
        }

        menuItem = new deosc.MenuItem("Set Flag");

        menuItem.onInit = function (data) {
          var node;

          node = data.target.node;

          if (node.getFlag() == "P") {
            publicMenuItem.disabled(true);
            privateMenuItem.disabled(false);
          } else {
            publicMenuItem.disabled(false);
            privateMenuItem.disabled(true);
          }
        };

        menuItem.setMenu(createShowHideModuleMenu());

        return menuItem;
      }

      self.onCreate = function () {
        var menu, moduleMenuItem, moduleMenu;
        menu = mWeaver.getDCaseNodeMenu();

        moduleMenuItem = new deosc.MenuItem("Module");
        menu.addMenuItem(moduleMenuItem);

        moduleMenu = new deosc.Menu();
        moduleMenuItem.setMenu(moduleMenu);

        moduleMenu.addMenuItem(createModularizeMenuItem());
        moduleMenu.addMenuItem(createRestoreModuleMenuItem());
        moduleMenu.addMenuItem(createSelectModuleMenuItem());
        moduleMenu.addMenuItem(createShowHideModuleMenuItem());
        moduleMenu.addMenuItem(createSetFlagMenuItem());
      };

      self.onEvent = function (event) {
        if (event.what == deosc.DCaseWeaver.Event.DCASE_CHANGED) {
          onDCaseChanged(event.data);
        }
      };

      function constructor () {
      }

      function onCreateModule (data) {
        createModule(mWeaver, data.target);
      }

      function onRestoreModule (data) {
        restoreModule(mWeaver, data.target);
      }

      function onSelectModule (data) {
        selectModule(mWeaver, data.target);
      }

      function onShowModule (data) {
        data.target.expandModule();
      }

      function onHideModule (data) {
        data.target.collapseModule();
      }

      function onSetFlag (data) {
        var nodeView, node, value, attrs, tree;

        nodeView = data.target;
        node = nodeView.node;

        tree = mWeaver.getDCaseTreeView();

        value = this.getLabel() == "Public" ? "P" : null;

        attrs = node.getAttributes();
        attrs.flag = value;

        mWeaver.command(
          new deosc.EditNodeCommand(tree, nodeView, attrs));
      }

      function onDCaseChanged (dcase) {
        updateModuleNode(mWeaver, dcase);
      }

      constructor();
    }


    function refreshIDs (dcase) {
      dcase.foreach(
        null,
        function () {
          var id = deosc.Util.getUID(), old, links, i;
          if (this instanceof deosc.DCaseNode) {
            old = this.getID();
            this.setID(id);
            links = dcase.findDCaseLinksBySource(old);
            for (i = 0; i < links.length; i++) {
              links[i].setSource(id);
            }
            links = dcase.findDCaseLinksByTarget(old);
            for (i = 0; i < links.length; i++) {
              links[i].setTarget(id);
            }
          } else if (this instanceof deosc.DCaseLink) {
            this.setID(id);
          }
        });
    }


    function cloneDCase (dcase) {
      var parser, doc, xml;
      xml = dcase.toXML();
      parser = new DOMParser();
      doc = parser.parseFromString(xml, "application/xml");
      return new deosc.DCase(doc);
    }


    function createModule (weaver, nodeView) {
      var self = this, mWeaver, mTree, mDCase, mNodeView, mNode;

      function constructor (weaver, nodeView) {
        mWeaver = weaver;
        mTree = weaver.getDCaseTreeView();
        mDCase = weaver.getDCase();
        mNodeView = nodeView;
        mNode = nodeView.node;

        if (!mWeaver.getMetadata()) {
          alert("Please save D-Case before creating module");
          return;
        }

        create();
      }

      function create () {
        var module, awayGoals = [];

        module = new deosc.DCase();

        mDCase.foreach(
          mNodeView.node,
          function () {
            var node, link;
            if (this instanceof deosc.DCaseNode) {
              node = new deosc.DCaseNode(module);
              node.setAttributes(this.getAttributes());
              module.addDCaseNode(node);
            } else if (this instanceof deosc.DCaseLink) {
              link = new deosc.DCaseLink(module);
              link.setAttributes(this.getAttributes());
              module.addDCaseLink(link);
            }
          });

        module.foreach(
          null,
          function () {
            var links, i;

            if (!(this instanceof deosc.DCaseNode)) return;
            if (this.getID() == mNodeView.node.getID()) return;

            links = mDCase.findDCaseLinksByTarget(this.getID());
            for (i = 0; i < links.length; i++) {
              if (!module.findDCaseLinkByID(links[i].getID())) {
                awayGoals.push(this);
              }
            }
          });

        openFileNameDialog(module, awayGoals);
      }

      function openFileNameDialog (module, awayGoals) {
        var dialog, view, label, input, moduleName, nodeName;

        function onOK () {
          var meta, name, path, i, node, fm, baseName, curDir, moduleDir,
          ref, parent;

          name = input.getText();
          if (name.length == 0) {
            alert("Please input module name");
            return;
          }

          if (!name.match(/\.dcase$/)) {
            name += ".dcase";
          }

          meta = mWeaver.getMetadata();

          curDir = deosc.path.dirname(meta.path);
          baseName = meta.name.replace(/\.dcase$/, "");
          moduleDir = curDir+"/"+baseName;
          nodeName = name.replace(/\.dcase$/, "");

          ref = (curDir+"/"+baseName+"/"+nodeName).replace(/\/+/, "/");
          parent = (curDir+"/"+baseName).replace(/\/+/, "/");

          module.setModuleRefSource([ref]);
          module.setModuleParent(parent);

          for (i = 0; i < awayGoals.length; i++) {
            node = awayGoals[i];
            ref = (curDir+"/"+baseName+"/"+node.getName()).replace(/\/+/, "/");
            awayGoals[i].setModuleRefSource([ref]);
            awayGoals[i].setFlag("P");
          }


          fm = mWeaver.getFileManager();

          fm.getMetadata(
            {path: moduleDir, name: deosc.path.basename(moduleDir)},
            function (meta) {
              save(meta, module, name, awayGoals);
            },
            function () {
              fm.mkdir(
                {path: curDir, name: deosc.path.basename(curDir)}, baseName,
                function (meta) {
                  save(meta, module, name, awayGoals);
                });
            }
          );

          dialog.close();
        }

        label = new deosc.View();
        label.addStyle("label");
        label.setText("Module Name");

        input = new deosc.InputText();
        input.setWidth(300);

        view = new deosc.View();
        view.addChild(label);
        view.addChild(input);

        dialog = new deosc.Dialog();
        dialog.setTitle("Modularize");
        dialog.setContent(view);
        dialog.setOnOKListener(onOK);
        mWeaver.dialogPopup(dialog);
      }

      function save (metadata, module, name, awayGoals) {
        var m;

        function success (metadata) {
          convertToModuleNode(metadata, module, awayGoals);
        }

        function error () {
          alert("Failed to save D-Case Module");
        }

        m = cloneDCase(module);
        refreshIDs(m);

        mWeaver.getFileManager()
          .postFile(metadata, m.toFormatXML(), name, success, error);
      }

      function convertToModuleNode (metadata, module, awayGoals) {
        var parser, doc, parentLinks, nodes, links,
        id, node, link, i, tree, commands = [], attrs, name, newId, modPath;

        name = metadata.name.replace(/\.dcase$/, "");
        modPath = metadata.path.replace(/\.dcase$/, "");

        newId = mDCase.getNewID();

        attrs = mNode.getAttributes();
        attrs.id = newId;
        attrs.type = "Module";
        attrs.name = name;
        attrs.attachment = modPath;

        commands.push(new deosc.EditNodeCommand(mTree, mNodeView, attrs));

        var treeCV, view;
        treeCV = mTree.getDCaseTreeCanvasView();

        parentLinks = mDCase.findDCaseLinksByTarget(mNode.getID());

        for (i = 0; i < parentLinks.length; i++) {
          attrs = parentLinks[i].getAttributes();
          attrs.type = deosc.DCaseLink.getLinkType("Module");
          attrs.target = newId;
          view = treeCV.findDCaseLinkViewByID(parentLinks[i].getID());
          if (view)
            commands.push(new deosc.EditLinkCommand(mTree, view, attrs));
        }

        function isAwayGoal (id) {
          var i;
          for (i = 0; i < awayGoals.length; i++) {
            if (awayGoals[i].getID() == id) {
              return true;
            }
          }
          return false;
        }

        nodes = module.getDCaseNodes();
        for (i = 0; i < nodes.length; i++) {
          node = nodes[i];
          id = node.getID();
          if (mNode.getID() == id) continue;
          view = treeCV.findDCaseNodeViewByID(id);
          if (view) {
            if (isAwayGoal(id)) {
              attrs = node.getAttributes();
              attrs.type = "Goal";
              attrs.attachment = modPath+"/"+attrs.name;
              commands.push(new deosc.EditNodeCommand(mTree, view, attrs));
            } else {
              commands.push(new deosc.DeleteNodeCommand(mTree, view));
            }
          }
        }

        links = module.getDCaseLinks();
        for (i = 0; i < links.length; i++) {
          id = links[i].getID();
          view = treeCV.findDCaseLinkViewByID(id);
          if (view)
            commands.push(new deosc.DeleteLinkCommand(mTree, view));
        }

        commands.push(new deosc.ArrangeCommand(mTree));

        mWeaver.command(commands);

        setTimeout(
          function () {
            mTree.moveTo(mNodeView, "center", true);
          }, 0);
      }

      constructor(weaver, nodeView);
    }


    function restoreModule (weaver, nodeView) {
      var id, item, tree, dcase, node, fm, path, attachment,
      moduleName, nodeName, type, meta;

      function success (doc) {
        var parser, module, parentLinks, i, moduleRootNode,
        nodes, links, isfind, commands = [], linkView, attrs;

        if (typeof doc == "string" || doc instanceof String) {
          parser = new DOMParser();
          doc = parser.parseFromString(doc, "application/xml");
        }

        module = new deosc.DCase(doc);
        refreshIDs(module);

        moduleRootNode = module.getRootNodes()[0];
        if (nodeName) {
          module.foreach(
            moduleRootNode,
            function () {
              if (this.getName() == nodeName) {
                moduleRootNode = this;
                return false;
              } else {
                return true;
              }
            });
        }

        parentLinks = dcase.findDCaseLinksByTarget(node.getID());
        for (i = 0; i < parentLinks.length; i++) {
          linkView = tree.getDCaseTreeCanvasView()
            .findDCaseLinkViewByID(parentLinks[i].getID());
          attrs = parentLinks[i].getAttributes();
          attrs.type = deosc.DCaseLink.getLinkType(moduleRootNode.getType());
          attrs.target = moduleRootNode.getID();
          commands.push(new deosc.EditLinkCommand(tree, linkView, attrs));
        }

        commands.push(new deosc.DeleteNodeCommand(tree, nodeView));

        module.foreach(
          moduleRootNode,
          function () {
            var nodeView, linkView;
            if (this instanceof deosc.DCaseNode) {
              if (!dcase.findDCaseNodeByID(this.getID())) {
                nodeView = deosc.canvas.createDCaseNodeView(this);
                commands.push(new deosc.AddNodeCommand(tree, nodeView));
              }
            } else if (this instanceof deosc.DCaseLink) {
              if (!dcase.findDCaseLinkByID(this.getID())) {
                linkView = deosc.canvas.createDCaseLinkView(this);
                commands.push(new deosc.AddLinkCommand(tree, linkView));
              }
            }
          });

        commands.push(new deosc.ArrangeCommand(tree));

        weaver.command(commands);

        removeRefSourceFromModule(
          weaver, path, nodeName,
          weaver.getMetadata().path.replace(/\.dcase$/, "")+"/"+node.getName());
      }

      function error () {
        console.error("No such DCase Module");
      }


      if (!weaver.getMetadata()) {
        alert("Please save D-Case before selecting module");
        return;
      }


      tree = weaver.getDCaseTreeView();
      dcase = tree.getDCase();
      node = nodeView.node;
      id = node.getID();
      type = node.getType();
      fm = weaver.getFileManager();

      attachment = node.getAttachment();

      if (type == "Goal" && attachment.match(/^(.+)\/([^/]+)$/)) {
        path = RegExp.$1 + ".dcase";
        nodeName = RegExp.$2;
      } else if (type == "Module") {
        path = attachment + ".dcase";
      } else {
        error();
        return;
      }

      if (!path.match(/.*\/.*/)) {
        path = deosc.path.dirname(weaver.getMetadata().path)+"/"+path;
      }

      path = "/"+path;
      path = path.replace(/\/+/g, "/");

      meta = {path: path, name: deosc.path.basename(path)};

      fm = weaver.getFileManager();

      fm.getFile(meta, success, error);
    }


    function selectModule (weaver, nodeView) {
      var fm, metadata, type;

      metadata = weaver.getMetadata();

      if (!metadata) {
        alert("Please save D-Case before selecting module");
        return;
      }

      fm = weaver.getFileManager();
      type = nodeView.getType();

      listdir();

      function listdir () {
        function error () {
          console.error("error");
        }

        fm.getParents(
          metadata,
          function (parents) {
            var ret = [];
            if (0 < parents.length) {
              fm.getChildren(
                parents[0],
                function (children) {
                  var i = 0;
                  function foreach () {
                    if (i < children.length) {
                      fm.getMetadata(
                        children[i],
                        function (meta) {
                          if (meta.isdir)
                            ret = ret.concat(meta.children);
                          i++;
                          foreach();
                        },
                        error);
                    } else {
                      onListdir(ret);
                    }
                  }
                  foreach();
                },
                error);
            } else {
              onListdir(ret);
            }
          },
          error);
      }

      function onListdir (list) {
        var model, i, parent;

        function complete () {
          openNodeListDialog(model);
        }

        for (i = 0; i < list.length; i++) {
          if (fm.comp(list[i], metadata) || !list[i].name.match(/\.dcase$/)) {
            list.splice(i--, 1);
          }
        }

        model = new deosc.ListModel();

        if (type == "Goal") {
          foreachDCase(
            list,
            function (metadata, dcase) {
              var nodes, node, i, item, name;
              nodes = dcase.getDCaseNodes();

              for (i = 0; i < nodes.length; i++) {
                node = nodes[i];
                if (node.getFlag() == "P") {
                  name = metadata.name.replace(/\.dcase$/,"");
                  item = new deosc.ListItem();
                  item.addColumn(name);
                  item.addColumn(node.getName());
                  item.addColumn(node.getType());
                  item.setData(
                    {
                      metadata: metadata,
                      node: node
                    });
                  model.add(item);
                }
              }

            },
            complete);

        } else if (type == "Module") {
          parent = weaver.getMetadata().path.replace(/\.dcase$/, "");
          foreachDCase(
            list,
            function (metadata, dcase) {
              var name, item;
              if (dcase.getModuleParent() == parent) {
                name = metadata.name.replace(/\.dcase$/, "");
                item = new deosc.ListItem();
                item.addColumn(name);
                item.setData({metadata: metadata});
                model.add(item);
              }
            },
            complete);
        }
      }

      function foreachDCase (list, callback, complete) {
        var i, dcase;

        function success (doc) {
          var parser;
          if (typeof doc == "string" || doc instanceof String) {
            parser = new DOMParser();
            doc = parser.parseFromString(doc, "application/xml");
          }
          dcase = new deosc.DCase(doc);

          callback(list[i++], dcase);

          if (i < list.length) {
            fm.getFile(list[i], success, error);
          } else {
            complete();
          }
        }

        function error () {
          console.error("error");
          complete();
        }

        i = 0;
        fm.getFile(list[i], success, error);
      }


      function openNodeListDialog (model) {
        var view, dialog;

        function onOK () {
          var data, old, uri, path, nodeName, curNode,
          newModuleNodeName, oldModuleNodeName,
          newModulePath, oldModulePath, oldAttachment, newAttachment,
          attrs, tree;

          curNode = nodeView.node;

          data = view.getCurrentItem().getData();
          newModuleNodeName = data.node ? data.node.getName() : null;
          newModulePath = data.metadata.path;
          if (newModuleNodeName) {
            newAttachment = newModulePath.replace(/\.dcase$/, "")+
            "/"+newModuleNodeName;
          } else {
            newAttachment = newModulePath.replace(/\.dcase$/, "");
          }

          attrs = curNode.getAttributes();
          oldAttachment = attrs.attachment;
          attrs.attachment = newAttachment;

          uri = weaver.getMetadata().path.replace(/\.dcase$/, "")+
            "/"+curNode.getName();

          if (oldAttachment) {
            if (type == "Goal" && oldAttachment.match(/^(.+)\/([^/]+)$/)) {
              oldModulePath = RegExp.$1 + ".dcase";
              oldModuleNodeName = RegExp.$2;
            } else if (type == "Module") {
              oldModulePath = oldAttachment + ".dcase";
            }
          }

          addRefSourceToModule(
            weaver, newModulePath, newModuleNodeName, uri,
            function () {
              if (oldModulePath) {
                removeRefSourceFromModule(
                  weaver, oldModulePath, oldModuleNodeName, uri);
              }
            }
          );

          tree = weaver.getDCaseTreeView();
          weaver.command(
            new deosc.EditNodeCommand(tree, nodeView, attrs));

          dialog.close();
        }

        view = new deosc.ListView();
        view.setSize(400, 200);
        view.setModel(model);

        dialog = new deosc.Dialog();
        dialog.setContent(view);
        dialog.setTitle("Select Module");
        dialog.setOnOKListener(onOK);
        weaver.dialogPopup(dialog);
      }
    }


    function removeRefSourceFromModule (weaver, modulePath, moduleNode, uri, callback) {
      var fm, module;

      function onLoad (doc) {
        var parser, nodes, node, refs, i, index, changed = false;
        if (typeof doc == "string" || doc instanceof String) {
          parser = new DOMParser();
          doc = parser.parseFromString(doc, "application/xml");
        }

        module = new deosc.DCase(doc);

        if (moduleNode) {
          nodes = module.findDCaseNodesByName(moduleNode);
          for (i = 0; i < nodes.length; i++) {
            node = nodes[i];
            refs = node.getModuleRefSource();
            index = refs.indexOf(uri);
            if (index != -1) {
              refs.splice(index, 1);
              node.setModuleRefSource(refs);
              changed = true;
            }
          }
        } else {
          refs = module.getModuleRefSource();
          index = refs.indexOf(uri);
          if (index != -1) {
            refs.splice(index, 1);
            module.setModuleRefSource(refs);
            changed = true;
          }
        }
        if (changed) {
          save();
        } else {
          success();
        }
      }

      function save () {
        fm.putFile({path: modulePath, name: deosc.path.basename(modulePath)}, module.toFormatXML(), success, error);
      }

      function success () {
        if (callback) callback();
      }

      function error () {
        console.log("Failed to update module:"+modulePath);
        if (callback) callback();
      }

      fm = weaver.getFileManager();
      fm.getFile({path: modulePath, name: deosc.path.basename(modulePath)}, onLoad, error);
    }


    function addRefSourceToModule (weaver, modulePath, moduleNode, uri, callback) {
      var fm, module;

      function onLoad (doc) {
        var parser, nodes, node, refs, i, index, changed = false;
        if (typeof doc == "string" || doc instanceof String) {
          parser = new DOMParser();
          doc = parser.parseFromString(doc, "application/xml");
        }

        module = new deosc.DCase(doc);

        if (moduleNode) {
          nodes = module.findDCaseNodesByName(moduleNode);
          for (i = 0; i < nodes.length; i++) {
            node = nodes[i];
            refs = node.getModuleRefSource();
            index = refs.indexOf(uri);
            if (index == -1) {
              refs.push(uri);
              node.setModuleRefSource(refs);
              changed = true;
            }
          }
        } else {
          refs = module.getModuleRefSource();
          index = refs.indexOf(uri);
          if (index == -1) {
            refs.push(uri);
            module.setModuleRefSource(refs);
            changed = true;
          }
        }

        if (changed) {
          save();
        } else {
          success();
        }
      }

      function save () {
        fm.putFile({path: modulePath, name: deosc.path.basename(modulePath)}, module.toFormatXML(), success, error);
      }

      function success () {
        if (callback) callback();
      }

      function error () {
        console.log("Failed to update module:"+modulePath);
        if (callback) callback();
      }

      fm = weaver.getFileManager();
      fm.getFile({path: modulePath, name: deosc.path.basename(modulePath)}, onLoad, error);
    }

    function updateModuleNode (weaver, dcase) {
      var meta;

      function update () {
        var tree, nodeView, node;

        if (!(this instanceof deosc.DCaseNode)) return;

        node = this;

        tree = weaver.getDCaseTreeView().getDCaseTreeCanvasView();
        nodeView = tree.findDCaseNodeViewByID(node.getID());
        if (!nodeView) return;

        nodeView.setOnExpandModuleListener(
          function (module) {
            var moduleView = this, fm, path, nodeName, type, attachment;

            function success (doc) {
              var parser;
              if (typeof doc == "string" || doc instanceof String) {
                parser = new DOMParser();
                doc = parser.parseFromString(doc, "application/xml");
              }
              moduleView.setDCase(new deosc.DCase(doc), nodeName);
            }

            function error () {
              console.error("error");
            }

            type = node.getType();
            attachment = node.getAttachment();

            if (type == "Goal" && attachment.match(/^(.+)\/([^/]+)$/)) {
              path = RegExp.$1 + ".dcase";
              nodeName = RegExp.$2;
            } else if (type == "Module") {
              path = attachment + ".dcase";
            } else {
              error();
              return;
            }

            if (!path.match(/.*\/.*/)) {
              path = deosc.path.dirname(weaver.getMetadata().path)+"/"+path;
            }

            path = "/"+path;
            path = path.replace(/\/+/g, "/");

            fm = weaver.getFileManager();

            fm.getFile({path: path, name: deosc.path.basename(path)}, success, error);
          }
        );
      }

      meta = weaver.getMetadata();
      if (!meta) return;

      dcase.foreach(null, update);
    }

    window.deosc.DCaseWeaver.ready(
      function (weaver) {
        var tree = weaver.getDCaseTreeView();
        weaver.addPlugin(new DCaseModulePlugin(tree));
      });
  }
) (window);(
  function (window) {

    function DCaseParameterPlugin () {
      var self = this, mWeaver;

      self.setDCaseWeaver = function (weaver) {
        mWeaver = weaver;
      };

      self.onCreate = function () {
        var menu, item;
        item = new deosc.MenuItem("Parameters", onParameters);
        item.onInit = function (data) {
          var type = data.target.node.getType();
          if (type == "Parameter") {
            item.disabled(false);
          } else {
            item.disabled(true);
          }
        };

        menu = mWeaver.getDCaseNodeMenu();
        menu.addMenuItem(item);
      };

      self.onOpenProerties = function () {

      };

      self.onOKProperties = function () {

      };

      function onParameters (data) {
        var dialog, params = [], attrs, hash, node, p, vals, val, name;

        function onOK () {
          var params, p, hash = {}, i;

          params = dialog.getParameters();

          for (i = 0; i < params.length; i++) {
            p = params[i];
            name = p.name;
            delete p.name;
            hash[name] = p;
          }

          attrs = node.getAttributes();
          attrs.parameters = hash;

          mWeaver.command(new deosc.EditNodeCommand(mWeaver.getDCaseTreeView(),
                                                    node, attrs));

          dialog.close();
        }

        node = data.target.node;

        hash = node.getParameters();

        for (name in hash) {
          p = hash[name];
          p.name = name;
          params.push(p);
        }

        dialog = new ParameterListDialog(mWeaver, params);
        dialog.setOnOKListener(onOK);
        mWeaver.dialogPopup(dialog);
      }
    }


    function ParameterListDialog (weaver, params) {
      var self = this, mWeaver = weaver, mParameters = params,
      mMain, mABox, mListModel, mListView;

      deosc.Dialog.call(self);

      self.getParameters = function () {
        var params = [];
        mListModel.foreach(
          function () {
            params.push(this.getData());
          });
        return params;
      };

      self.addParameter = function (param) {
        var item = new deosc.ListItem();
        item.addColumn(param.name);
        item.addColumn(param.type);
        item.addColumn(param.value);
        item.setData(param);
        mListModel.add(item);
        mListView.update();
      };

      function createButton (label, listener) {
        var button = new deosc.View($("<button>"));
        button.setText(label);
        button.setOnClickListener(listener);
        button.setWidth("100%");
        return button;
      }

      function constructor () {
        var header, i;

        header = new deosc.ListItem();
        header.addColumns(["Name"]);
        header.addColumns(["Type"]);
        header.addColumns(["Value"]);

        mListModel = new deosc.ListModel();
        mListModel.setHeaderItem(header);

        mListView = new deosc.ListView();
        mListView.setModel(mListModel);
        mListView.setSize(400, 150);

        mABox = new deosc.View();
        mABox.addStyle("vbox");
        mABox.addChild(createButton("Add", onAdd));
        mABox.addChild(createButton("Edit", onEdit));
        mABox.addChild(createButton("Delete", onDelete));

        mMain = new deosc.View();
        mMain.addStyle("hbox");
        mMain.addChild(mListView);
        mMain.addChild(mABox);

        self.setContent(mMain);
        self.setTitle("Parameters");

        if (mParameters) {
          for (i = 0; i < mParameters.length; i++) {
            self.addParameter(mParameters[i]);
          }
        }
      }


      function onAdd () {
        var dialog;

        function onOK () {
          var param, item, exists = false;

          param = dialog.getParameter();
          if (!param) return;

          mListModel.foreach(
            function (index) {
              if (this.getData().name == param.name) {
                exists = true;
                return false;
              }
              return true;
            });

          if (exists) {
            alert(param.name+" is exists");
            return;
          }

          self.addParameter(param);

          dialog.close();
        }

        dialog = new ParameterDialog(mWeaver);
        dialog.setOnOKListener(onOK);

        mWeaver.dialogPopup(dialog);
      }

      function onDelete () {
        var item = mListView.getCurrentItem();
        if (item) {
          mListModel.removeItem(item);
          mListView.setCurrent(-1);
          mListView.update();
        }
      }

      function onEdit () {
        var dialog, item;

        function onOK () {
          var param;

          param = dialog.getParameter();
          if (!param) return;

          item.empty();
          item.addColumn(param.name);
          item.addColumn(param.type);
          item.addColumn(param.value);
          item.setData(param);

          mListView.update();

          dialog.close();
        }

        item = mListView.getCurrentItem();
        if (!item) return;

        dialog = new ParameterDialog(mWeaver, item.getData());
        dialog.setOnOKListener(onOK);

        mWeaver.dialogPopup(dialog);
      }

      constructor();
    }


    function ParameterDialog (weaver, param) {
      var self = this, mWeaver = weaver, mParameter = param, mMain,
      mName, mType, mMin, mMax, mDigit, mInc,  mValue, mLength,
      mEnum, mEnumListModel, mEnumListView, mEnumBox, mEnumValue,
      mRange, mHyphon;

      deosc.Dialog.call(self);

      self.getParameter = function getParameter () {
        var type, name, min, max, digit, inc, items, param = {};

        name = mName.getText();
        if (name.length == 0) return null;

        param.name = name;

        type = mType.getValue();

        if (type.length == 0) {
          param.value = mValue.getText();

        } else {
          param.type = type;

          if (type == "enum") {
            items = [];
            mEnumListModel
              .foreach(function () { items.push(this.getData()); });
            if (0 < items.length) param.items = items;

            param.value = mEnumValue.getValue();

          } else {
            if (type == "string") {
              param.min = 0;

              max = mLength.getText();
              if (0 < max.length) param.max = max;

            } else {
              min = mMin.getText();
              if (0 < min.length) param.min = min;

              max = mMax.getText();
              if (0 < max.length) param.max = max;
            }

            param.value = mValue.getText();
          }
        }


        return param;
      };

      function constructor () {
        var i, item, type;

        mName = new deosc.InputText();

        mType = new deosc.SelectView();
        mType.setOnChangeListener(onChangeType);
        mType.addItem("", "");
        mType.addItem("string", "string");
        mType.addItem("int", "int");
        mType.addItem("double", "double");
        mType.addItem("enum", "enum");
        mType.addItem("raw", "raw");

        mHyphon = new deosc.View();
        mHyphon.setText("-");
        mHyphon.setWidth(24);
        $(mHyphon.getElement()).css({"text-align":"center", "vertical-align": "middle"});

        mMin = new deosc.InputText();
        mMin.setWidth(86);
        $(mMin.getElement()).css({"vertical-align": "middle"});

        mMax = new deosc.InputText();
        mMax.setWidth(86);
        $(mMax.getElement()).css({"vertical-align": "middle"});

        mRange = new deosc.View();
        mRange.addStyle("hbox");
        mRange.addChild(mMin);
        mRange.addChild(mHyphon);
        mRange.addChild(mMax);

        mLength = new deosc.InputText();

        mValue = new deosc.InputText();
        mValue.setOnChangeListener(onChangeValue);

        mEnumValue = new deosc.SelectView();

        mEnumListModel = new deosc.ListModel();

        mEnumListView = new deosc.ListView();
        mEnumListView.setSize(200, 80);
        mEnumListView.setModel(mEnumListModel);

        mEnumBox = new deosc.View();
        mEnumBox.addStyle("vbox");
        mEnumBox.addChild(createButton("Add", onAddEnum));
        mEnumBox.addChild(createButton("Delete", onDeleteEnum));

        mEnum = new deosc.View();
        mEnum.addStyle("hbox");
        mEnum.addChild(mEnumListView);
        mEnum.addChild(mEnumBox);

        mMain = new deosc.TableView();
        mMain.addRow(["Name", mName]);
        mMain.addRow(["Type", mType]);
        mMain.addRow(["Range", mRange]);
        mMain.addRow(["Length", mLength]);
        mMain.addRow(["Value", mValue]);
        mMain.addRow(["Enumeration<br>Items", mEnum]);
        mMain.addRow(["Value", mEnumValue]);

        self.setContent(mMain);
        self.setTitle("Edit Parameter");

        if (mParameter) {
          if ("name" in mParameter)
            mName.setText(mParameter.name);

          type = mParameter.type;
          if (!type) {
            if ("value" in mParameter)
              mValue.setText(mParameter.value);

          } else {
            if ("type" in mParameter)
              mType.setValue(mParameter.type);

            if (type == "enum") {
              if ("items" in mParameter) {
                for (i = 0; i < mParameter.items.length; i++) {
                  item = new deosc.ListItem();
                  item.addColumn(mParameter.items[i]);
                  item.setData(mParameter.items[i]);
                  mEnumListModel.add(item);
                }
                mEnumListView.update();
                updateEnumValue();
              }

              if ("value" in mParameter)
                mEnumValue.setValue(mParameter.value);

            } else {
              if (type == "string") {
                if ("max" in mParameter)
                  mLength.setText(mParameter.max);

              } else {
                if ("min" in mParameter)
                  mMin.setText(mParameter.min);

                if ("max" in mParameter)
                  mMax.setText(mParameter.max);
              }
              if ("value" in mParameter)
                mValue.setText(mParameter.value);
            }
          }
        }

        onChangeType();
      }

      function onChangeType () {
        var type = mType.getValue();

        $.each([2, 3, 4, 5, 6],
               function (index, i) { mMain.getRow(i).hide(); });

        if (type.length == 0) {
          mMain.getRow(4).show();
        } else if (type == "string") {
          $.each([3, 4], function (index, i) { mMain.getRow(i).show(); });
        } else if (type == "enum") {
          $.each([5, 6], function (index, i) { mMain.getRow(i).show(); });
          updateEnumValue();
        } else {
          $.each([2, 4], function (index, i) { mMain.getRow(i).show(); });
        }
      }

      function updateEnumValue () {
        var val;
        val = mEnumValue.getValue();
        mEnumValue.removeAllItems();
        mEnumListModel.foreach(
          function () {
            mEnumValue.addItem(this.getData(), this.getData());
          });
        mEnumValue.setValue(val);
      }

      function onChangeValue () {

      }

      function onAddEnum () {
        function onOK () {
          var text, exists, item;

          text = dialog.getItem();
          if (text.length == 0) return;

          mEnumListModel.foreach(
            function (index) {
              if (this.getData() == text) {
                exists = true;
                return false;
              }
              return true;
            });

          if (exists) {
            alert(text+" is exists");
            return;
          }

          item = new deosc.ListItem();
          item.addColumn(text);
          item.setData(text);

          mEnumListModel.add(item);

          mEnumListView.update();

          updateEnumValue();

          dialog.close();
        }

        var dialog = new AddEnumDialog(mWeaver);
        dialog.setOnOKListener(onOK);

        mWeaver.dialogPopup(dialog);
      }

      function onDeleteEnum () {
        var item = mEnumListView.getCurrentItem();
        if (item) {
          mEnumListModel.removeItem(item);
          mEnumListView.setCurrent(-1);
          mEnumListView.update();
          updateEnumValue();
        }
      }

      constructor();
    }


    function AddEnumDialog (weaver) {
      var self = this, mWeaver = weaver, mMain, mItem;
      deosc.Dialog.call(self);

      self.getItem = function () {
        return mItem.getText();
      };

      function constructor () {
        mItem = new deosc.InputText();

        mMain = new deosc.TableView();
        mMain.addRow(["Item", mItem]);

        self.setContent(mMain);
        self.setTitle("Add Enumeration Item");
      }

      constructor();
    }

    function createButton (label, listener) {
      var button = new deosc.View($("<button>"));
      button.setText(label);
      button.setOnClickListener(listener);
      button.setWidth("100%");
      return button;
    }

    window.deosc.DCaseWeaver.ready(
      function (weaver) {
        weaver.addPlugin(new DCaseParameterPlugin());
      });
  }
) (window);(
  function (window) {
    var MAX_WIDTH = 8192;
    var MAX_HEIGHT = 8192;

    function SaveAsImageFilePlugin () {
      var self = this, mWeaver, mDialog;

      self.setDCaseWeaver = function (weaver) {
        mWeaver = weaver;
      };

      self.onCreate = function () {
        var menuBarItem;

        menuBarItem = mWeaver.getMenuBar().getMenuBarItem("File");
        menuBarItem.addMenuItem(
          new deosc.MenuItem("Save As Image File", onSaveAsImageFile));

        mWeaver.getDCaseNodeMenu().addMenuItem(
          new deosc.MenuItem("Save As Image File", onSaveAsImageFile));
      };

      function constructor () {
      }

      function onSaveAsImageFile (data) {
        var basename;
        var dialog = new SaveAsImageFileDialog(mWeaver);

        try {
          basename = mWeaver.getMetadata().name.replace(/\.dcase$/, "");
        } catch (x) {
          basename = "D-Case";
        }

        if (data.target instanceof deosc.canvas.DCaseNodeView) {
          dialog.setRootNode(data.target.node);
          dialog.setFileName(basename+"_"+data.target.node.getName());
        } else {
          dialog.setFileName(basename);
        }

        mWeaver.dialogPopup(dialog);
      }

      constructor();
    }


    function SaveAsImageFileDialog (weaver) {
      var self = this, mWeaver, mTree, mMainView, mSizeView, mScaleInput,
      mCapture, mFileName, mScale = 1, mNode;

      deosc.Dialog.call(self);

      self.onCreate = function () {
        mSizeView = new deosc.View();

        mScaleInput = new deosc.InputText();
        mScaleInput.setText("1.0");
        mScaleInput.setOnChangeListener(onChange);

        mMainView = new deosc.TableView();
        mMainView.addRow(["Size", "", mSizeView]);
        mMainView.addRow(["Scale", "", mScaleInput]);

        if (mNode) {
          self.setTitle("Save As Image File: "+mNode.getName());
        } else {
          self.setTitle("Save As Image File");
        }
        self.setContent(mMainView);

        updateSize();
      };

      self.onOK = function () {
        var url, data, canvas;

        function post (data, name) {
          var form = $("<form>")
            .attr(
              {
                method: "POST",
                target: "_blank",
                action: "cgi-bin/saveasimage.wsgi"
              });

          $("<input type='hidden' name='BaseName'>").val(name).appendTo(form);
          $("<input type='hidden' name='Length'>")
            .val(data.length).appendTo(form);

          for (i = 0; i < data.length; i++) {
            $("<input type='hidden' name='Data"+i+"'>")
              .val(data[i]).appendTo(form);
          }
          $(form).hide().appendTo("body").submit().remove();
        }

        var i, j;
        data = [];

        var width = mCapture.width();
        var height = mCapture.height();
        var maxw = MAX_WIDTH / mScale;
        var maxh = MAX_HEIGHT / mScale;
        var cols = Math.ceil(width / maxw);
        var rows = Math.ceil(height / maxh);

        for (i = 0; i < rows; i++) {
          var y, h;
          y = maxh * i;
          h = y + maxh <= height ? maxh : height - y;
          for (j = 0; j < cols; j++) {
            var x, w;
            x = maxw * j;
            w = x + maxw <= width ? maxw : width - x;
            canvas = $("<canvas>")[0];

            mCapture.capture(canvas, x, y, w, h, w * mScale, h * mScale);

            url = canvas.toDataURL();
            data.push(url.split(",", 2)[1]);
          }
        }

        post(data, mFileName);

        self.close();
      };

      self.setRootNode = function (node) {
        mNode = node;
        self.setTitle("Save As Image File: "+node.getName());
        mCapture.setRootNode(node);
        updateSize();
      };

      self.setFileName = function (name) {
        mFileName = name;
      };

      function constructor (weaver) {
        mWeaver = weaver;
        mTree = weaver.getDCaseTreeView();
        mFileName = "D-Case";
        mCapture = new Capture(mWeaver);
      }

      function onChange () {
        mScale = parseFloat(mScaleInput.getText());
        updateSize();
      }

      function updateSize () {
        var width, height;

        if (!mSizeView) return;

        width = mCapture.width() * mScale;
        height = mCapture.height() * mScale;
        mSizeView.setText(width +" x "+ height);
      }

      constructor(weaver);
    }


    function Capture (weaver) {
      var self = this, mWeaver, mTree, mRootNode;
      var nodeViews = [];
      var linkViews = [];
      var rect = new deosc.Rect();

      self.setRootNode = function (node) {
        mRootNode = node;
        scan();
        measure();
      };

      self.width = function () {
        return rect.width();
      };

      self.height = function () {
        return rect.height();
      };

      self.capture = function (canvas, sx, sy, sw, sh, dw, dh) {
        capture (canvas, sx, sy, sw, sh, dw, dh);
      };

      function constructor (weaver) {
        mWeaver = weaver;
        mTree = weaver.getDCaseTreeView().getDCaseTreeCanvasView();
        self.setRootNode(null);
      }

      function scan () {
        var dcase;

        nodeViews.length = 0;
        linkViews.length = 0;

        dcase = mWeaver.getDCase();
        dcase.foreach(
          mRootNode,
          function () {
            var view;

            if (this instanceof deosc.DCaseNode) {
              view = mTree.findDCaseNodeViewByID(this.getID());
              if (view) nodeViews.push(view);
            } else if (this instanceof deosc.DCaseLink) {
              view = mTree.findDCaseLinkViewByID(this.getID());
              if (view) linkViews.push(view);
            }
          });
      }

      function measure () {
        var l, r, t, b, i, view;

        view = nodeViews[0];
        l = view.left;
        t = view.top;
        r = view.right;
        b = view.bottom;

        for (i = 1; i < nodeViews.length; i++) {
          view = nodeViews[i];
          l = Math.min(l, view.left);
          t = Math.min(t, view.top);
          r = Math.max(r, view.right);
          b = Math.max(b, view.bottom);
        }

        rect = new deosc.Rect(l, t, r, b);
      }

      function capture (canvas, sx, sy, sw, sh, dw, dh) {
        var i, ctx, view;

        $(canvas)
          .attr({ width: dw, height: dh });
        ctx = canvas.getContext("2d");

        ctx.fillStyle = "#fff";
        ctx.fillRect(0, 0, dw, dh);
        ctx.scale(dw/sw, dh/sh);
        ctx.translate(-sx -rect.left, -sy -rect.top);

        for (i = 0; i < nodeViews.length; i++) {
          view = nodeViews[i];
          view.setDisplayArea(rect);
          ctx.save();
          ctx.translate(view.left, view.top);
          view.draw(ctx);
          ctx.restore();
        }

        for(i = 0; i < linkViews.length; i++) {
          view = linkViews[i];
          ctx.save();
          ctx.translate(view.left, view.top);
          view.draw(ctx);
          ctx.restore();
        }
      }

      constructor(weaver);
    }

    deosc.DCaseWeaver.ready(
      function (weaver) {
        weaver.addPlugin(new SaveAsImageFilePlugin());
      });
  }
) (window);
(
  function (window) {
    function meta_comp (meta1, meta2, version) {
      var id1, id2;
      if ("id" in meta1 && "id" in meta2) {
        id1 = meta1.id.replace(/;.*$/,"");
        id2 = meta2.id.replace(/;.*$/,"");
        if (version === false) {
          return id1 == id2;
        } else {
          return (id1 == id2) && (meta1.version == meta2.version);
        }
      } else if ("path" in meta1 && "path" in meta2) {
        if (version === false) {
          return meta1.path == meta2.path;
        } else {
          return (meta1.path == meta2.path) && (meta1.version == meta2.version);
        }
      } else {
        return false;
      }
    }


    function Metadata (cmis) {
      var self = this, i, child;

      self.path = cmis.path;
      self.id = cmis["cmis:objectId"];
      self.name = cmis["cmis:name"];
      self.version = cmis["cmis:versionLabel"];
      self.isdir = cmis["cmis:baseTypeId"] == "cmis:folder";
      self.crtime = new Date(cmis["cmis:creationDate"]);
      self.mtime = new Date(cmis["cmis:lastModificationDate"]);
      self.isLatestVersion = cmis["cmis:isLatestVersion"];
      self.checkinComment = cmis["cmis:checkinComment"];

      if (self.isdir && cmis.children) {
        self.children = [];
        for (i = 0; i < cmis.children.length; i++) {
          self.children.push(new Metadata(cmis.children[i]));
        }
      }
    }


    function createListItem (metadata) {
      var item = new deosc.ListItem();
      item.addColumn(metadata.name);
      item.addColumn(metadata.version);
      item.addColumn(metadata.mtime.toLocaleString());
      item.addColumn(deosc.path.dirname(metadata.path));
      item.setData(metadata);
      return item;
    }


    function login (weaver, repo, option) {
      function onCheckSuccess () {
        if (option.success) option.success();
      }

      function onCheckFailure () {
        var dialog = new LoginDialog(repo);
        dialog.setOnAuthenticationSuccessListener(onAuthenticationSuccess);
        dialog.setOnAuthenticationFailureListener(onAuthenticationFailure);
        weaver.dialogPopup(dialog);
      }

      function onAuthenticationSuccess () {
        if (option.success) option.success();
      }

      function onAuthenticationFailure () {
        if (option.error) option.error();
      }

      repo.check(onCheckSuccess, onCheckFailure);
    }

    function default_error (jqXHR, textStatus, errorThrown) {
      if (textStatus == "parsererror") {
        alert("Parser Error");
      } else {
        alert(jqXHR.responseText);
      }
    }

    function getMetadata (weaver, repo, metadata, success, error) {
      function _error (jqXHR, textStatus, errorThrown) {
        if (error) {
          error();
        } else {
          default_error(jqXHR, textStatus, errorThrown);
        }
      }

      function onLoginSuccess () {
        repo.getMetadata(
          {
            id: metadata.id,
            path: metadata.path,
            version: metadata.version,
            success: success,
            error: _error
          });
      }

      login(weaver, repo, {success: onLoginSuccess, error: error});
    }

    function getFile (weaver, repo, metadata, success, error) {
      function _error (jqXHR, textStatus, errorThrown) {
        if (error) {
          error();
        } else {
          default_error(jqXHR, textStatus, errorThrown);
        }
      }

      function onLoginSuccess () {
        repo.getMetadata(
          {
            id: metadata.id,
            path: metadata.path,
            version: metadata.version,
            success: function (metadata) {
              repo.getFile(
                {
                  id: metadata.id,
                  success: function (doc) {
                    if (success) success(doc, metadata);
                  },
                  error: _error
                });
            },
            error: _error
          });
      }

      login(weaver, repo, {success: onLoginSuccess, error: error});
    }

    function putFile (weaver, repo, metadata, file, success, error) {
      function _error (jqXHR, textStatus, errorThrown) {
        if (error) {
          error();
        } else {
          default_error(jqXHR, textStatus, errorThrown);
        }
      }

      function onLoginSuccess () {
        var dialog = new CommentDialog();
        dialog.setOnOKListener(
          function () {
            repo.putFile(metadata, file, dialog.getComment(), success, _error);
            dialog.close();
          });
        weaver.dialogPopup(dialog);
      }

      login(weaver, repo, {success: onLoginSuccess, error: error});
    }

    function postFile (weaver, repo, metadata, file, name, success, error) {
      function _error (jqXHR, textStatus, errorThrown) {
        if (error) {
          error();
        } else {
          default_error(jqXHR, textStatus, errorThrown);
        }
      }

      function onCheckExistance (meta) {
        if (meta) {
          if (confirm(meta.name + "\n\noverwrite ?")) {
            putFile(weaver, repo, meta, file, success, error);
          } else {
            if (error) error();
          }
        } else {
          repo.postFile(metadata, file, name, null, success, _error);
        }
      }

      function checkExistence (callback) {
        repo.getChildren(
          metadata,
          function (children) {
            var i, exists = false;
            for (i = 0; i < children.length; i++) {
              if (children[i].name == name) {
                callback(children[i]);
                return;
              }
            }
            callback(null);
          },
          _error);
      }

      function onLoginSuccess () {
        checkExistence(onCheckExistance);
      }

      if (name.match(/　/)) {
        alert(name + "\n\nDouble-byte space is not supported.");
        return;
      }

      login(weaver, repo, {success: onLoginSuccess, error: error});
    }

    function mkdir (weaver, repo, metadata, name, success, error) {
      function _error (jqXHR, textStatus, errorThrown) {
        if (error) {
          error();
        } else {
          default_error(jqXHR, textStatus, errorThrown);
        }
      }

      function onCheckExistance (meta) {
        if (!meta)
          repo.mkdir(metadata, name, success, _error);
      }

      function checkExistence (callback) {
        repo.getChildren(
          metadata,
          function (children) {
            var i, exists = false;
            for (i = 0; i < children.length; i++) {
              if (children[i].name == name) {
                callback(children[i]);
                return;
              }
            }
            callback(null);
          },
          _error);
      }

      function onLoginSuccess () {
        checkExistence(onCheckExistance);
      }

      if (name.match(/　/)) {
        alert(name + "\n\nDouble-byte space is not supported.");
        return;
      }

      login(weaver, repo, {success: onLoginSuccess, error: error});
    }

    function getParents (weaver, repo, metadata, success, error) {
      function _error (jqXHR, textStatus, errorThrown) {
        if (error) {
          error();
        } else {
          default_error(jqXHR, textStatus, errorThrown);
        }
      }

      function onLoginSuccess () {
        repo.getParents(metadata, success, _error);
      }

      login(weaver, repo, {success: onLoginSuccess, error: error});
    }

    function getChildren (weaver, repo, metadata, success, error) {
      function _error (jqXHR, textStatus, errorThrown) {
        if (error) {
          error();
        } else {
          default_error(jqXHR, textStatus, errorThrown);
        }
      }

      function onLoginSuccess () {
        repo.getChildren(metadata, success, _error);
      }

      login(weaver, repo, {success: onLoginSuccess, error: error});
    }

    function openDCase (weaver, repo, option) {
      function error () {
        if (option.error) option.error();
      }

      function onLoginSuccess () {
        var dialog = new OpenDCaseDialog(weaver, repo);
        dialog.setOnOKListener(
          function () {
            var meta = dialog.getMetadata();
            repo.getFile(
              {
                id: meta.id,
                success: function (doc) {
                  if (option.success) option.success(doc, meta);
                },
                error: error
              });
            dialog.close();
          });
        weaver.dialogPopup(dialog);
      }

      login(weaver, repo, {success: onLoginSuccess, error: error});
    }

    function openSaveAsDCaseDialog (weaver, repo, dcase, success, error) {
      function _success (data, textStatus, jqXHR) {
        weaver.sendEvent(
          {
            what: deosc.DCaseWeaver.Event.DCASE_CHANGED,
            data: weaver.getDCase()
          });
        if (success) success(data, textStatus, jqXHR);
      }

      function _error (jqXHR, textStatus, errorThrown) {
        if (error) {
          error();
        } else {
          default_error(jqXHR, textStatus, errorThrown);
        }
      }

      function onLoginSuccess () {
        var dialog = new SaveAsDCaseDialog(weaver, repo);
        dialog.setOnOKListener(
          function () {
            var metadata, name;
            metadata = dialog.getMetadata();
            name = dialog.getName();
            if (!name.match(/\.dcase$/))
              name += ".dcase";
            postFile(
              weaver, repo, metadata, dcase.toFormatXML(), name,
              _success, _error);
            dialog.close();
          });
        weaver.dialogPopup(dialog);
      }

      login(weaver, repo, {success: onLoginSuccess, error: error});
    }


    function openCommitDocumentDialog (weaver, repo, metadata, success, error) {
      function _error (jqXHR, textStatus, errorThrown) {
        if (error) {
          error();
        } else {
          default_error(jqXHR, textStatus, errorThrown);
        }
      }

      function onLoginSuccess () {
        var dialog;

        dialog = new CommitDocumentDialog(metadata.path);
        dialog.setOnOKListener(
          function () {
            var file;

            function _success (metadata) {
              dialog.close();
              if (success) success(metadata);
            }

            file = dialog.getFile();
            if (!file) {
              alert("Please select file.");
              return;
            }

            repo.putFile(metadata, file, dialog.getComment(), _success, error);
          });
        weaver.dialogPopup(dialog);
      }

      login(weaver, repo, {success: onLoginSuccess, error: error});
    }


    function openUploadDocumentDialog (weaver, repo, parent, success, error) {
      function onLoginSuccess () {
        var dialog;

        dialog = new UploadDocumentDialog(parent.path);
        dialog.setOnOKListener(
          function() {
            var file, name;

            function _success (metadata) {
              if (success) success(metadata);
              dialog.close();
            }

            file = dialog.getFile();
            if (!file) {
              alert("Please select file.");
              return;
            }

            name = dialog.getName();
            if (name.length == 0) {
              alert("Please input file name.");
              return;
            }

            postFile(weaver, repo, parent, file, name, _success, error);
          });
        weaver.dialogPopup(dialog);
      }

      login(weaver, repo, {success: onLoginSuccess, error: error});
    }


    function history (weaver, repo, metadata) {
      function onLoginSuccess () {
        var dialog;
        dialog = new HistoryDialog(weaver, repo, metadata);
        weaver.dialogPopup(dialog);
      }

      login(weaver, repo, {success: onLoginSuccess});
    }


    function checkForModifications (weaver, repo) {
      function onLoginSuccess () {
        var dialog = new CheckForModificationsDialog(weaver, repo);
        weaver.dialogPopup(dialog);
      }

      login(weaver, repo, {success: onLoginSuccess});
    }


    function searchLinkageNodes (weaver, repo, metadata) {
      function onLoginSuccess () {
        var dialog = new SearchLinkageNodesDialog(weaver, repo, metadata);
        weaver.dialogPopup(dialog);
      }

      login(weaver, repo, {success: onLoginSuccess});
    }


    function CheckForModificationsDialog (weaver, repo) {
      var self = this, mWeaver, mRepo, mDCase, mView, mModel;

      deosc.Dialog.call(self);

      self.onCreate = function () {
        var header;

        header = new deosc.ListItem();
        header.addColumns(["Node", "Name", "Version", "Status", "Date Modified", "Location"]);

        mModel = new deosc.ListModel();
        mModel.setHeaderItem(header);

        mView = new deosc.ListView();
        mView.setSize(400, 200);
        mView.setModel(mModel);

        self.setContent(mView);
        self.setTitle("Check for modifications");
        self.getFooter().hide();

        mDCase.foreach(null, check);
      };

      function constructor (weaver, repo) {
        mWeaver = weaver;
        mRepo = repo;
        mDCase = weaver.getDCase();
      }

      function check () {
        var node = this, udef010, list, i;

        function success (metadata) {
          var status, item, view;

          status = metadata.isLatestVersion ? "latest" : "changed";

          item = new deosc.ListItem();
          item.addColumn(node.getName());
          item.addColumn(metadata.name);
          item.addColumn(metadata.version);
          item.addColumn(status);
          item.addColumn(metadata.mtime.toLocaleString());
          item.addColumn(deosc.path.dirname(metadata.path));
          item.setData({metadata: metadata, node: node});
          mModel.add(item);

          mView.update();

          if (status == "changed") {
            view = mWeaver.getDCaseTreeView().getDCaseTreeCanvasView()
              .findDCaseNodeViewByID(node.getID());
            view.setBackgroundColor("#ff8");
          }
        }

        if (!(node instanceof deosc.DCaseNode)) return;

        list = node.getCMISInfo();
        if (!list) return;

        for (i = 0; i < list.length; i++) {
          mRepo.getMetadata(
            {
              id: list[i].id,
              version: list[i].version,
              success: success
            });
        }
      }

      constructor(weaver, repo);
    }

    function compareDCase (weaver, repo) {
      browse(
        weaver, repo,
        {
          allVersions: true,
          success: function (dialog) {
            dialog.setOnOKListener(
              function (meta) {
                getFile(
                  weaver, repo, meta,
                  function (doc) {
                    var parser, dcase;
                    if (typeof doc == "string" || doc instanceof String) {
                      parser = new DOMParser();
                      doc = parser.parseFromString(doc, "application/xml");
                    }
                    dcase = new deosc.DCase(doc);
                    deosc.compareDCase(dcase);
                    dialog.close();
                  },
                  function () {
                    console.error("error");
                  });
              });
          },
          filter: function (meta) {
            return meta.isdir || meta.path.match(/\.dcase$/) != null;
          }
        });
    }

    function approvalDCase (weaver, repo, success, error) {
      function onPutFileSuccess (metadata) {
        if (success) success();
      }

      function onLoginSuccess () {
        repo.approve(
          dialog.getMetadata(), dialog.getComment(), onPutFileSuccess, error);
      }

      function onApprove () {
        login(weaver, repo, {success: onLoginSuccess, error: error});
      }

      var dialog = new deosc.ApprovalDialog();
      dialog.setOnApproveListener(onApprove);
      weaver.dialogPopup(dialog);
    }


    function searchDCase (weaver, repo) {
      var dialog, creationDate, modificationDate;

      function createItem (meta) {
        var item = new deosc.TreeListItem();
        item.addColumn(meta.name);
        item.addColumn(meta.version);
        item.addColumn(meta.mtime.toLocaleString());
        item.addColumn(deosc.path.dirname(meta.path));
        item.setData(meta);
        return item;
      }

      function SearchResultDialog (results) {
        var self = this, mWidget, mTreeListView, mTreeListModel,
        mOpenButton, mCompareButton;

        deosc.Dialog.call(self);

        self.onCreate = function () {
          var box, i, j, header, item;

          header = new deosc.ListItem();
          header.addColumns(["Name", "Version", "Date Modified", "Location"]);

          mTreeListModel = new deosc.TreeListModel();
          mTreeListModel.setHeaderItem(header);
          for (i = 0; i < results.length; i++) {
            item = createItem(results[i][0]);
            item.collapsable(true);
            mTreeListModel.add(null, item);
            for (j = 1; j < results[i].length; j++) {
              mTreeListModel.add(item, createItem(results[i][j]));
            }
          }

          mTreeListView = new deosc.TreeListView();
          mTreeListView.setSize(400, 200);
          mTreeListView.setModel(mTreeListModel);
          mTreeListView.setOnClickItemListener(onClickItem);

          mOpenButton = createButton("Open", onOpen);
          mCompareButton = createButton("Compare", onCompare);

          box = new deosc.View();
          box.addStyle("vbox");
          box.addChild(mOpenButton);
          box.addChild(mCompareButton);

          mWidget = new deosc.View();
          mWidget.addStyle("hbox");
          mWidget.addChild(mTreeListView);
          mWidget.addChild(box);

          self.setContent(mWidget);
          self.setTitle("Search Result");
          self.getFooter().hide();

          onClickItem(-1);
        };

        function createButton (label, listener) {
          var button = new deosc.View($("<button>"));
          button.setText(label);
          button.setOnClickListener(listener);
          button.setWidth("100%");
          return button;
        }

        function getCurrent () {
          var item = mTreeListView.getCurrentItem();
          return item ? item.getData() : null;
        }

        function onClickItem (index) {
          if (index == -1) {
            mOpenButton.disabled(true);
            mCompareButton.disabled(true);
          } else {
            mOpenButton.disabled(false);
            mCompareButton.disabled(false);
          }
        }

        function onOpen () {
          var metadata = getCurrent();
          if (metadata)
            repo.open(metadata);
        }

        function onCompare () {
          var metadata = getCurrent();
          if (metadata)
            compare(weaver, repo, metadata);
        }
      }

      function onSearch () {
        login(weaver, repo, {success: onLoginSuccess});
      }

      function onLoginSuccess () {
        var conditions = [], i;

        for (i = 0; i < dialog.getConditionCount(); i++) {
          conditions.push(dialog.getConditionAt(i));
        }
        conditions.push({attr: "File name", keyword: ".dcase", operator: "="});

        repo.search(conditions, onSearchSuccess, onSearchError);
      }

      function onSearchSuccess (results) {
        var searchResultDialog;
        dialog.close();

        searchResultDialog = new SearchResultDialog(results);
        weaver.dialogPopup(searchResultDialog);
      }

      function onSearchError (jqXHR, textStatus, errorThrown) {
        if (textStatus == "parsererror") {
          alert("Parser Error");
        } else {
          alert(jqXHR.responseText);
        }
      }

      function checkDate () {
        var timestamp, date;
        timestamp = Date.parse(this.getValue());
        if (isNaN(timestamp)) {
          alert("Input type is not Date type");
          return false;
        } else {
          return true;
        }
      }

      function generateListItem (condition) {
        var date = new Date(condition.keyword);
        var item = new deosc.ListItem();
        item.addColumn(condition.attr);
        item.addColumn(date.toLocaleString());
        item.addColumn(condition.operator);
        condition.keyword = date.toISOString();
        item.setData(condition);
        return item;
      }

      creationDate = new deosc.ConditionController("Creation date");
      creationDate.setOnAddListener(checkDate);
      creationDate.setOnGenerateListItemListener(generateListItem);

      modificationDate = new deosc.ConditionController("Modification date");
      modificationDate.setOnAddListener(checkDate);
      modificationDate.setOnGenerateListItemListener(generateListItem);

      dialog = new deosc.SearchConditionDialog();

      dialog.addConditionController(
        new deosc.ConditionController("File name", ["=", "<>"]));
      dialog.addConditionController(
        new deosc.ConditionController("Directory path", ["="]));
      dialog.addConditionController(creationDate);
      dialog.addConditionController(
        new deosc.ConditionController("Created by", ["=", "<>"]));
      dialog.addConditionController(modificationDate);
      dialog.addConditionController(
        new deosc.ConditionController("Modified by", ["=", "<>"]));
      dialog.addConditionController(
        new deosc.ConditionController("Comment", ["=", "<>"]));

      dialog.setOnSearchListener(onSearch);

      weaver.dialogPopup(dialog);
    }

    function compare (weaver, repo, metadata, success, error) {
      function onGetFileSuccess (doc) {
        var parser, dcase;
        if (typeof doc == "string" || doc instanceof String) {
          parser = new DOMParser();
          doc = parser.parseFromString(doc, "application/xml");
        }
        dcase = new deosc.DCase(doc);
        deosc.compareDCase(dcase);
        if (success) success();
      }

      getFile(weaver, repo, metadata, onGetFileSuccess, error);
    }


    function browse (weaver, repo, option) {
      function error () {
        if (option.error) option.error();
      }

      function onLoginSuccess () {
        var dialog = new BrowseDialog(weaver, repo, option);
        weaver.dialogPopup(dialog);
        if (option) {
          if (option.filter) dialog.setFilter(option.filter);
          if (option.success) option.success(dialog);
        }
      }

      login(weaver, repo, {success: onLoginSuccess, error: error});
    }


    function FileManager () {
      var self = this, mWeaver, mRepo, mMetadata;

      self.setDCaseWeaver = function (weaver) {
        mWeaver = weaver;
      };

      self.setRepository = function (repo) {
        mRepo = repo;
      };

      self.onCreate = function () {
        var menuBarItem, menu, docMenuItem, docMenu, opt, meta;

        mRepo = new Repository(mWeaver);

        mWeaver.getToolBar().addItem(new ToolBarItem());

        menuBarItem = mWeaver.getMenuBar().getMenuBarItem("File");
        menuBarItem.addMenuItem(new deosc.MenuItem("New", onClickNew));
        menuBarItem.addMenuItem(new deosc.MenuItem("Open", onClickOpen));
        menuBarItem.addMenuItem(new deosc.MenuItem("Save", onClickSave));
        menuBarItem.addMenuItem(new deosc.MenuItem("Save As", onClickSaveAs));

        menuBarItem = mWeaver.getMenuBar().getMenuBarItem("Document");
        menuBarItem.addMenuItem(new deosc.MenuItem("Browse", onClickDocument));
        menuBarItem.addMenuItem(new deosc.MenuItem("Check for modifications", onClickCheckForModifications));
        menuBarItem.addMenuItem(new deosc.MenuItem("Search", onClickSearch));
        menuBarItem.addMenuItem(new deosc.MenuItem("Approval", onClickApproval));
        menuBarItem.addMenuItem(new deosc.MenuItem("Compare To", onClickCompareTo));
        mWeaver.setDCase(new deosc.DCase());

        opt = mWeaver.getOptions();
        if ("path" in opt) {
          meta = {path: opt.path};
          if ("version" in opt) {
            meta.version = opt.version;
          }
          getFile(
            mWeaver, mRepo, meta,
            function (doc, metadata) {
              onLoadDCaseDocument(doc, metadata);
              if ("node" in opt) {
                setTimeout(
                  function () {
                    var tree, canvasTree, view;

                    tree = mWeaver.getDCaseTreeView();
                    view = tree.getDCaseTreeCanvasView()
                      .findDCaseNodeViewByID(opt.node);
                    tree.moveTo(view, "center", true);
                  }, 0);
              }
            });
        }
      };

      self.browse = function (option) {
        browse(mWeaver, mRepo, option);
      };

      self.getMetadata = function (metadata, success, error) {
        getMetadata(mWeaver, mRepo, metadata, success, error);
      };

      self.getFile = function (metadata, success, error) {
        getFile(mWeaver, mRepo, metadata, success, error);
      };

      self.postFile = function (metadata, file, name, success, error) {
        postFile(mWeaver, mRepo, metadata, file, name, success, error);
      };

      self.putFile = function (metadata, file, success, error) {
        putFile(mWeaver, mRepo, metadata, file, success, error);
      };

      self.mkdir = function (metadata, name, success, error) {
        mkdir(mWeaver, mRepo, metadata, name, success, error);
      };

      self.getParents = function (metadata, success, error) {
        getParents(mWeaver, mRepo, metadata, success, error);
      };

      self.getChildren = function (metadata, success, error) {
        getChildren(mWeaver, mRepo, metadata, success, error);
      };

      self.comp = function (meta1, meta2) {
        return meta_comp(meta1, meta2);
      };

      self.openNodeDocumentDialog = function (node) {
        function onCheckSuccess () {
          var dialog = new NodeDocumentDialog(mWeaver, mRepo, node);
          mWeaver.dialogPopup(dialog);
        }

        function onCheckFailure () {
          var dialog = new LoginDialog(mRepo);
          dialog.setOnAuthenticationSuccessListener(onCheckSuccess);
          mWeaver.dialogPopup(dialog);
        }

        mRepo.check(onCheckSuccess, onCheckFailure);
      };

      function onClickNew () {
        mMetadata = null;
        mWeaver.setDCase(new deosc.DCase());
      }

      function onClickOpen () {
        function error () {
          console.error("error");
        }
        openDCase(mWeaver, mRepo, {success: onLoadDCaseDocument, error: error});
      }

      function onClickSave () {
        function error () {
          console.error("error");
        }

        if (mMetadata) {
          self.putFile(
            mMetadata, mWeaver.getDCase().toFormatXML(), onSaveDCase, error);
        } else {
          openSaveAsDCaseDialog(
            mWeaver, mRepo, mWeaver.getDCase(), onSaveDCase, error);
        }
      }

      function onClickSaveAs () {
        function error () {
          console.error("error");
        }

        openSaveAsDCaseDialog(
          mWeaver, mRepo, mWeaver.getDCase(), onSaveDCase, error);
      }

      function onClickDocument() {
        function onCheckSuccess () {
          var dialog = new DocumentDialog(mWeaver, mRepo);
          mWeaver.dialogPopup(dialog);
        }

        function onCheckFailure () {
          var dialog = new LoginDialog(mRepo);
          dialog.setOnAuthenticationSuccessListener(onCheckSuccess);
          mWeaver.dialogPopup(dialog);
        }

        mRepo.check(onCheckSuccess, onCheckFailure);
      }

      function onClickCheckForModifications () {
        checkForModifications(mWeaver, mRepo);
      }

      function onClickSearch () {
        searchDCase(mWeaver, mRepo);
      }

      function onClickApproval () {
        approvalDCase(mWeaver, mRepo);
      }

      function onClickCompareTo () {
        compareDCase(mWeaver, mRepo);
      }

      function onLoadDCaseDocument (doc, metadata) {
        var parser;
        if (typeof doc == "string" || doc instanceof String) {
          parser = new DOMParser();
          doc = parser.parseFromString(doc, "application/xml");
        }
        mMetadata = metadata;
        mWeaver.setDCase(new deosc.DCase(doc), metadata);
      }

      function onSaveDCase (metadata) {
        mMetadata = metadata;
        mWeaver.setMetadata(metadata);
      }

      function ToolBarItem () {
        var self = this, mNewButton, mOpenButton, mSaveButton;

        deosc.ToolBarItem.call(self);

        function constructor () {
          mNewButton = new deosc.View($("<button>"));
          mNewButton.setText("New");
          mNewButton.setOnClickListener(onClickNew);
          self.addChild(mNewButton);

          mOpenButton = new deosc.View($("<button>"));
          mOpenButton.setText("Open");
          mOpenButton.setOnClickListener(onClickOpen);
          self.addChild(mOpenButton);

          mSaveButton = new deosc.View($("<button>"));
          mSaveButton.setText("Save");
          mSaveButton.setOnClickListener(onClickSave);
          self.addChild(mSaveButton);
        }

        constructor();
      }
    };


    function Repository (weaver) {
      var self = this, mWeaver;

      var mUrl = "cgi-bin/alfresco.wsgi";

      function unpackId (id) {
        if (id.match(/(^[^:]*):\/\/([^\/]*)\/([^;]*);([.0-9]*)/)) {
          return {
            store: RegExp.$1 + ":" + RegExp.$2,
            id: RegExp.$3,
            version: RegExp.$4
          };
        } else if (id.match(/(^[^:]*):\/\/([^\/]*)\/(.*)/)) {
          return {
            store: RegExp.$1 + ":" + RegExp.$2,
            id: RegExp.$3
          };
        }
        return null;
      }

      function ba () {
        return btoa(self.getUsername() + ":" + self.getPassword());
      }

      self.open = function (option) {
        var url, id, i, query = [];

        if (option.path && option.path.match(/\.dcase$/)) {
          url = "?path="+option.path;
          if ("version" in option) {
            url += "&version="+option.version;
          } else if (id && "version" in id) {
            url += "&version="+id.version;
          }
          window.open(url, "_blank", "");
          return;
        }

        url = mUrl;

        if (option.id) {
          id = unpackId(option.id);
          url += "/files/" + self.getRepositoryId()
            + "/i/" + id.store + "/" + id.id;
        } else if (option.path) {
          url += "/files/" + self.getRepositoryId()
            + "/p"+ option.path;
        } else {
          if (option.error) option.error();
          return;
        }

        if ("version" in option) {
          query.push({key: "version", value: option.version});
        } else if (id && "version" in id) {
          query.push({key: "version", value: id.version});
        }

        query.push({key: "ba", value: ba()});
        query.push({key: "repositoryUrl", value: self.getRepositoryUrl()});

        for (i = 0; i < query.length; i++) {
          url += (i == 0 ? "?" : "&") + query[i].key + "=" + query[i].value;
        }

        window.open(url, "_blank", "");
      };

      self.login = function (option) {
        var url = mUrl + "/login/", query = [], i;

        query.push({key: "ba", value: ba()});
        query.push({key: "repositoryUrl", value: self.getRepositoryUrl()});

        for (i = 0; i < query.length; i++) {
          url += (i == 0 ? "?" : "&") + query[i].key + "=" + query[i].value;
        }

        $.ajax(
          {
            url: url,
            type: "GET",
            success: option.success,
            error: option.error
          });
      };

      self.logout = function (option) {
        var url = mUrl + "/logout/";

        delete window.sessionStorage.weaver_alfresco_password;

        $.ajax(
          {
            url: url,
            type: "GET",
            success: option ? option.success : null,
            error: option ? option.error : null
          });
      };

      self.getRepository = function (option) {
        var url, query = [], i;

        url = mUrl + "/repository/";
        if (option.id) url += option.id;

        query.push({key: "ba", value: ba()});
        query.push({key: "repositoryUrl", value: self.getRepositoryUrl()});

        for (i = 0; i < query.length; i++) {
          url += (i == 0 ? "?" : "&") + query[i].key + "=" + query[i].value;
        }

        $.ajax(
          {
            url: url,
            type: "GET",
            success: option.success,
            error: option.error
          });
      };

      self.getMetadata = function (option) {
        var repoId, id, pathinfo, url, query, i;

        repoId = self.getRepositoryId();
        if (!repoId) {
          console.error("cannot get repositoryId");
          if (option.error) option.error();
          return;
        }

        if (option.id) {
          id = unpackId(option.id);
          pathinfo = "/metadata/" + repoId + "/i/" + id.store + "/" + id.id;
        } else if (option.path) {
          pathinfo = "/metadata/" + repoId + "/p"+ option.path;
        } else {
          if (option.error) option.error();
          return;
        }

        url = mUrl + pathinfo;

        query = [];
        if ("allVersions" in option)
          query.push({key: "allVersions", value: option.allVersions});

        if ("version" in option) {
          query.push({key: "version", value: option.version});
        } else if (id && "version" in id) {
          query.push({key: "version", value: id.version});
        }

        query.push({key: "ba", value: ba()});
        query.push({key: "repositoryUrl", value: self.getRepositoryUrl()});

        for (i = 0; i < query.length; i++) {
          url += (i == 0 ? "?" : "&") + query[i].key + "=" + query[i].value;
        }

        $.ajax(
          {
            url: url,
            type: "GET",
            success: function (data) {
              var i, child, ret;

              if (option.allVersions) {
                ret = [];
                for (i = 0; i < data.length; i++) {
                  ret.push(new Metadata(data[i]));
                }
              } else {
                ret = new Metadata(data);
              }
              if (option.success) option.success(ret);
            },
            error: option.error
          });
      };

      self.getParents = function (metadata, success, error) {
        var repoId, id, pathinfo, url, query, i;

        repoId = self.getRepositoryId();
        if (!repoId) {
          console.error("cannot get repositoryId");
          if (error) error();
          return;
        }

        if (metadata.id) {
          id = unpackId(metadata.id);
          pathinfo = "/parents/" + repoId + "/i/" + id.store + "/" + id.id;
        } else if (metadata.path) {
          pathinfo = "/parents/" + repoId + "/p"+ metadata.path;
        } else {
          if (error) error();
          return;
        }

        url = mUrl + pathinfo;

        query = [];

        query.push({key: "ba", value: ba()});
        query.push({key: "repositoryUrl", value: self.getRepositoryUrl()});

        for (i = 0; i < query.length; i++) {
          url += (i == 0 ? "?" : "&") + query[i].key + "=" + query[i].value;
        }

        $.ajax(
          {
            url: url,
            type: "GET",
            success: function (data) {
              var i, ret = [];
              for (i = 0; i < data.length; i++) {
                ret.push(new Metadata(data[i]));
              }
              if (success) success(ret);
            },
            error: error
          });
      };

      self.getChildren = function (metadata, success, error) {
        var repoId, id, pathinfo, url, query, i;

        repoId = self.getRepositoryId();
        if (!repoId) {
          console.error("cannot get repositoryId");
          if (error) error();
          return;
        }

        if (metadata.id) {
          id = unpackId(metadata.id);
          pathinfo = "/children/" + repoId + "/i/" + id.store + "/" + id.id;
        } else if (metadata.path) {
          pathinfo = "/children/" + repoId + "/p"+ metadata.path;
        } else {
          if (error) error();
          return;
        }

        url = mUrl + pathinfo;

        query = [];

        query.push({key: "ba", value: ba()});
        query.push({key: "repositoryUrl", value: self.getRepositoryUrl()});

        for (i = 0; i < query.length; i++) {
          url += (i == 0 ? "?" : "&") + query[i].key + "=" + query[i].value;
        }

        $.ajax(
          {
            url: url,
            type: "GET",
            success: function (data) {
              var i, ret = [];
              for (i = 0; i < data.length; i++) {
                ret.push(new Metadata(data[i]));
              }
              if (success) success(ret);
            },
            error: error
          });
      };

      self.getFile = function (option) {
        var ajax, repoId, id, pathinfo, url, query = [], i;

        repoId = self.getRepositoryId();
        if (!repoId) {
          console.error("cannot get repositoryId");
          if (option.error) option.error();
          return;
        }

        if (option.id) {
          id = unpackId(option.id);
          pathinfo = "/files/" + repoId + "/i/" + id.store + "/" + id.id;
        } else if (option.path) {
          pathinfo = "/files/" + repoId + "/p"+ option.path;
        } else {
          if (option.error) option.error();
          return;
        }

        url = mUrl + pathinfo;

        query.push({key: "ba", value: ba()});
        query.push({key: "repositoryUrl", value: self.getRepositoryUrl()});
        if ("version" in option) {
          query.push({key: "version", value: option.version});
        } else if (id && "version" in id) {
          query.push({key: "version", value: id.version});
        }

        for (i = 0; i < query.length; i++) {
          url += (i == 0 ? "?" : "&") + query[i].key + "=" + query[i].value;
        }

        $.ajax(
          {
            url: url,
            type: "GET",
            success: option.success,
            error: option.error
          });
      };

      self.putFile = function (metadata, file, comment, success, error) {
        var ajax, repoId, id, pathinfo, url, query = [], i;

        repoId = self.getRepositoryId();
        if (!repoId) {
          console.error("cannot get repositoryId");
          if (error) error();
          return;
        }

        if (metadata.id) {
          id = unpackId(metadata.id);
          pathinfo = "/files/" + repoId + "/i/" + id.store + "/" + id.id;
        } else if (metadata.path) {
          pathinfo = "/files/" + repoId + "/p"+ metadata.path;
        } else {
          if (error) error();
          return;
        }

        url = mUrl + pathinfo;

        query = [];

        if (comment)
          query.push({key: "comment", value: comment});

        if (file.type)
          query.push({key: "mimeType", value: file.type});

        if (metadata.name.match(/\.dcase$/))
          query.push({key: "mimeType", value: "text/xml"});

        query.push({key: "ba", value: ba()});
        query.push({key: "repositoryUrl", value: self.getRepositoryUrl()});

        for (i = 0; i < query.length; i++) {
          url += (i == 0 ? "?" : "&") + query[i].key + "=" + query[i].value;
        }

        url = encodeURI(url);

        $.ajax(
          {
            url: url,
            type: "PUT",
            data: file,
            processData: false,
            success: function (data) {
              if (success) success(new Metadata(data));
            },
            error: error
          });
      };

      self.postFile = function (metadata, file, name, comment, success, error) {
        var ajax, repoId, id, pathinfo, url, query = [], i;

        repoId = self.getRepositoryId();
        if (!repoId) {
          console.error("cannot get repositoryId");
          if (error) error();
          return;
        }

        if (metadata.id) {
          id = unpackId(metadata.id);
          pathinfo = "/files/" + repoId + "/i/" + id.store + "/" + id.id;
        } else if (metadata.path) {
          pathinfo = "/files/" + repoId + "/p"+ metadata.path;
        } else {
          if (error) error();
          return;
        }

        url = mUrl + pathinfo;

        query = [];

        query.push({key: "name", value: name});

        if (comment)
          query.push({key: "comment", value: comment});

        if (file && file.type)
          query.push({key: "mimeType", value: file.type});

        if (metadata.name.match(/\.dcase$/))
          query.push({key: "mimeType", value: "text/xml"});

        query.push({key: "ba", value: ba()});
        query.push({key: "repositoryUrl", value: self.getRepositoryUrl()});

        for (i = 0; i < query.length; i++) {
          url += (i == 0 ? "?" : "&") + query[i].key + "=" + query[i].value;
        }

        url = encodeURI(url);

        $.ajax(
          {
            url: url,
            type: "POST",
            data: file,
            processData: false,
            success: function (data) {
              if (success) success(new Metadata(data));
            },
            error: error
          });
      };

      self.mkdir = function (metadata, name, success, error) {
        var ajax, repoId, id, pathinfo, url, query = [], i;

        repoId = self.getRepositoryId();
        if (!repoId) {
          console.error("cannot get repositoryId");
          if (error) error();
          return;
        }

        if (metadata.id) {
          id = unpackId(metadata.id);
          pathinfo = "/mkdir/" + repoId + "/i/" + id.store + "/" + id.id;
        } else if (metadata.path) {
          pathinfo = "/mkdir/" + repoId + "/p"+ metadata.path;
        } else {
          if (error) error();
          return;
        }

        url = mUrl + pathinfo;

        query = [];

        query.push({key: "name", value: name});

        query.push({key: "ba", value: ba()});
        query.push({key: "repositoryUrl", value: self.getRepositoryUrl()});

        for (i = 0; i < query.length; i++) {
          url += (i == 0 ? "?" : "&") + query[i].key + "=" + query[i].value;
        }

        url = encodeURI(url);

        $.ajax(
          {
            url: url,
            type: "GET",
            success: function (data) {
              if (success) success(new Metadata(data));
            },
            error: error
          });
      };

      self.search = function (conditions, success, error) {
        var repoId, query = [], i, url;

        repoId = self.getRepositoryId();
        if (!repoId) {
          console.error("cannot get repositoryId");
          if (error) error();
          return;
        }

        url = mUrl + "/search/" + repoId;

        query.push({key: "conditions", value: JSON.stringify(conditions)});
        query.push({key: "ba", value: ba()});
        query.push({key: "repositoryUrl", value: self.getRepositoryUrl()});

        for (i = 0; i < query.length; i++) {
          url += (i == 0 ? "?" : "&") + query[i].key + "=" + query[i].value;
        }

        $.ajax(
          {
            url: url,
            type: "GET",
            success: function (data) {
              var i, j, ret = [], l;
              for (i = 0; i < data.length; i++) {
                ret[i] = [];
                for (j = 0; j < data[i].length; j++) {
                  ret[i].push(new Metadata(data[i][j]));
                }
              }
              if (success) success(ret);
            },
            error: error
          });
      };

      self.approve = function (metadata, comment, success, error) {
        var ajax, repoId, id, pathinfo, url, query = [], i;

        repoId = self.getRepositoryId();
        if (!repoId) {
          console.error("cannot get repositoryId");
          if (error) error();
          return;
        }

        if (metadata.id) {
          id = unpackId(metadata.id);
          pathinfo = "/approve/" + repoId + "/i/" + id.store + "/" + id.id;
        } else if (metadata.path) {
          pathinfo = "/approve/" + repoId + "/p"+ metadata.path;
        } else {
          if (error) error();
          return;
        }

        url = mUrl + pathinfo;

        query = [];

        if (comment)
          query.push({key: "comment", value: comment});

        if (metadata.name.match(/\.dcase$/))
          query.push({key: "mimeType", value: "text/xml"});

        query.push({key: "ba", value: ba()});
        query.push({key: "repositoryUrl", value: self.getRepositoryUrl()});

        for (i = 0; i < query.length; i++) {
          url += (i == 0 ? "?" : "&") + query[i].key + "=" + query[i].value;
        }

        url = encodeURI(url);

        $.ajax(
          {
            url: url,
            type: "GET",
            processData: false,
            success: function (data) {
              if (success) success(new Metadata(data));
            },
            error: error
          });
      };

      function constructor (weaver) {
        mWeaver = weaver;
      }

      self.check = function (success, error) {
        if (self.getRepositoryUrl() && self.getUsername()
            && self.getPassword()) {

          self.getRepository(
            {
              success: success,
              error: error
            });

        } else {
          error();
        }
      };

      self.setRepositoryUrl = function (url) {
        mWeaver.getPreferences().getPreference("Alfresco")
          .setValue("cmisUrl", url);
      };

      self.getRepositoryUrl = function () {
        return mWeaver.getPreferences().getPreference("Alfresco")
          .getValue("cmisUrl", "");
      };

      self.setUsername = function (user) {
        mWeaver.getPreferences().getPreference("Alfresco")
          .setValue("user", user);
      };

      self.getUsername = function () {
        return mWeaver.getPreferences().getPreference("Alfresco")
          .getValue("user", "");
      };

      self.setPassword = function (password) {
        window.sessionStorage.weaver_alfresco_password = password;
      };

      self.getPassword = function () {
        return window.sessionStorage.weaver_alfresco_password;
      };

      self.setRepositoryId = function (id) {
        mWeaver.getPreferences().getPreference("Alfresco")
          .setValue("repositoryId", id);
      };

      self.getRepositoryId = function () {
        return mWeaver.getPreferences().getPreference("Alfresco")
          .getValue("repositoryId", "");
      };

      self.setRepositoryName = function (name) {
        mWeaver.getPreferences().getPreference("Alfresco")
          .setValue("repositoryName", name);
      };

      self.getRepositoryName = function () {
        return mWeaver.getPreferences().getPreference("Alfresco")
          .getValue("repositoryName", "");
      };

      constructor(weaver);
    }


    function LoginView (repo) {
      var self = this, mRepo, mUrlInput, mUserInput, mPasswordInput;

      deosc.TableView.call(self);

      function constructor(repo) {
        mRepo = repo;

        mUrlInput = new deosc.InputText();
        mUserInput = new deosc.InputText();
        mPasswordInput = new deosc.InputPassword();

        self.addRow(["CMIS URL", mUrlInput]);
        self.addRow(["User", mUserInput]);
        self.addRow(["Password", mPasswordInput]);
      }

      self.setRepositoryUrl = function (url) {
        mUrlInput.setText(url);
      };

      self.getRepositoryUrl = function () {
        return mUrlInput.getText();
      };

      self.setUser = function (user) {
        mUserInput.setText(user);
      };

      self.getUser = function () {
        return mUserInput.getText();
      };

      self.setPassword = function (password) {
        mPasswordInput.setText(password);
      };

      self.getPassword = function () {
        return mPasswordInput.getText();
      };

      constructor(repo);
    }


    function LoginDialog (repo) {
      var self = this, mRepo, mMainView, mTableView, mUrlInput, mUserInput,
      mPasswordInput, mMessageView,
      mOnAuthenticationSuccessListener, mOnAuthenticationFailureListener;

      deosc.Dialog.call(self);

      self.setOnAuthenticationSuccessListener = function (l) {
        mOnAuthenticationSuccessListener = l;
      };

      self.setOnAuthenticationFailureListener = function (l) {
        mOnAuthenticationFailureListener = l;
      };

      self.onOK = function () {
        mRepo.setRepositoryUrl(mUrlInput.getText());
        mRepo.setUsername(mUserInput.getText());
        mRepo.setPassword(mPasswordInput.getText());

        mRepo.login(
          {
            success: function () {
              mMessageView.setText("Authentication Succeeded.");
              if (mOnAuthenticationSuccessListener)
                mOnAuthenticationSuccessListener();
              self.close();
            },
            error: function () {
              if (mOnAuthenticationFailureListener)
                mOnAuthenticationFailureListener();
              mMessageView.setText("Authentication Failed.");
            }
          });
      };

      self.onCreate = function () {
        mMainView = new deosc.View();

        mTableView = new deosc.TableView();
        mMainView.addChild(mTableView);

        mUrlInput = new deosc.InputText();
        mUserInput = new deosc.InputText();
        mPasswordInput = new deosc.InputPassword();

        mUrlInput.setText(mRepo.getRepositoryUrl());
        mUrlInput.setWidth(300);

        mUserInput.setText(mRepo.getUsername());
        mUserInput.setWidth(300);

        mPasswordInput.setText(mRepo.getPassword());
        mPasswordInput.setWidth(300);

        mTableView.addRow(["CMIS URL", mUrlInput]);
        mTableView.addRow(["User", mUserInput]);
        mTableView.addRow(["Password", mPasswordInput]);

        mMessageView = new deosc.View();
        mMessageView.addStyle("label");
        mMainView.addChild(mMessageView);

        self.setTitle("Login");
        self.setContent(mMainView);
      };

      function constructor (repo) {
        mRepo = repo;
      }

      constructor (repo);
    }


    function SelectRepositoryWidget () {
      var self = this, mRepo, mItems, mOnSelectRepositoryListener;

      deosc.SelectView.call(self);

      self.setRepository = function (repo) {
        mRepo = repo;
        self.update();
      };

      self.getSelectedItem = function () {
        var i = self.getValue();
        return mItems[i];
      };

      self.update = function () {
        self.removeAllItems();
        mItems.length = 0;

        mRepo.getRepository({success: onRepository});
      };

      self.onError = function () {
      };

      function constructor () {
        mItems = [];
      }

      function onRepository (items) {
        var i;

        for (i = 0; i < items.length; i++) {
          self.addItem(items[i].repositoryName, i);
          mItems.push(items[i]);
        }
        onChange();
      }

      self.getItemAt = function (index) {
        return mItems[index];
      };

      self.setOnSelectRepositoryListener = function (l) {
        mOnSelectRepositoryListener = l;
      };

      function onChange () {
        var item = self.getSelectedItem();
        mRepo.setRepositoryId(item.repositoryId);
        mRepo.setRepositoryName(item.repositoryName);

        if (mOnSelectRepositoryListener)
          mOnSelectRepositoryListener(self.getValue());
      }

      constructor();
    }


    function OpenDCaseDialog (weaver, repo) {
      var self = this, mWeaver, mRepo, mMetadata, mMain, mSelectRepoWidget,
      mRepoDocList;

      deosc.Dialog.call(self);

      self.getMetadata = function () {
        return mMetadata;
      };

      self.onCreate = function () {
        mSelectRepoWidget = new SelectRepositoryWidget();
        mSelectRepoWidget.setWidth(400);
        mSelectRepoWidget.setOnSelectRepositoryListener(onSelectRepository);
        mSelectRepoWidget.setRepository(mRepo);

        mRepoDocList = new BrowseWidget(mWeaver);
        mRepoDocList.setSize(400, 200);
        mRepoDocList.setOnClickItemListener(onClickItem);

        mMain = new deosc.View();
        mMain.addStyle("vbox");
        mMain.addChild(mSelectRepoWidget);
        mMain.addChild(mRepoDocList);

        self.setContent(mMain);
        self.setTitle("Open D-Case");
      };

      function constructor (weaver, repo) {
        mWeaver = weaver;
        mRepo = repo;
      }

      function filter (item) {
        var meta = item.getData();
        return meta.isdir || meta.path.match(/\.dcase$/) != null;
      }

      function onSelectRepository (index) {
        var item = mSelectRepoWidget.getItemAt(index);
        mRepo.setRepositoryId(item.repositoryId);
        mRepo.setRepositoryName(item.repositoryName);

        mRepoDocList.setRepository(mRepo, "/", filter);
      }

      function onClickItem (item) {
        mMetadata = item.getData();
      }

      constructor(weaver, repo);
    }


    function SaveAsDCaseDialog (weaver, repo) {
      var self = this, mWeaver, mRepo, mMain, mSelectRepoWidget,
      mRepoDocList, mDir, mName, mOnSaveDCaseListener, mOnErrorListener,
      mMetadata;

      deosc.Dialog.call(self);

      self.getMetadata = function () {
        return mMetadata;
      };

      self.getName = function () {
        return mName.getText();
      };

      self.onCreate = function () {
        var label1, label2;

        label1 = new deosc.View();
        label1.addStyle("label");
        label1.setText("Directory");

        label2 = new deosc.View();
        label2.addStyle("label");
        label2.setText("File Name");

        mDir = new deosc.InputText();
        mDir.readonly(true);
        mDir.setText("/");
        mDir.setWidth(400);

        mName = new deosc.InputText();
        mName.setWidth(400);

        mSelectRepoWidget = new SelectRepositoryWidget();
        mSelectRepoWidget.setWidth(400);
        mSelectRepoWidget.setOnSelectRepositoryListener(onSelectRepository);
        mSelectRepoWidget.setRepository(mRepo);

        mRepoDocList = new BrowseWidget(mWeaver);
        mRepoDocList.setSize(400, 100);
        mRepoDocList.setOnClickItemListener(onRepoClickItem);

        mMain = new deosc.View();
        mMain.addStyle("vbox");
        mMain.addChild(label1);
        mMain.addChild(mDir);
        mMain.addChild(label2);
        mMain.addChild(mName);
        mMain.addChild(mSelectRepoWidget);
        mMain.addChild(mRepoDocList);

        self.setContent(mMain);
        self.setTitle("Save D-Case");
      };

      function constructor (weaver, repo) {
        mWeaver = weaver;
        mRepo = repo;
      }

      function filter (item) {
        var meta = item.getData();
        return meta.isdir || meta.path.match(/\.dcase$/) != null;
      }

      function onSelectRepository (index) {
        var item = mSelectRepoWidget.getItemAt(index);
        mRepo.setRepositoryId(item.repositoryId);
        mRepo.setRepositoryName(item.repositoryName);

        mRepoDocList.setRepository(mRepo, "/", filter);
      }

      function onRepoClickItem (item) {
        var name;

        mMetadata = item.getData();

        if (!mMetadata.isdir) {
          name = mMetadata.name;
          mMetadata = item.getParentItem().getData();
          mDir.setText(mMetadata.path);
          mName.setText(name);
        } else {
          mDir.setText(mMetadata.path);
        }
      }

      constructor(weaver, repo);
    }


    function DocumentDialog (weaver, repo, node) {
      var self = this, mWeaver, mRepo, mNode, mSelectRepoWidget,
      mRepoDocList, mRepoABox;

      deosc.Dialog.call(self);

      self.onCreate = function () {
        var main, label, box;

        mSelectRepoWidget = new SelectRepositoryWidget();
        mSelectRepoWidget.setWidth(400);
        mSelectRepoWidget.setOnSelectRepositoryListener(onSelectRepository);
        mSelectRepoWidget.setRepository(mRepo);

        mRepoDocList = new BrowseWidget(mWeaver);
        mRepoDocList.setSize(400, 200);
        mRepoDocList.setOnClickItemListener(onClickItem);

        mRepoABox = new ActionBox();
        mRepoABox.addAction(
          new Action({label: "Open", action: onClickOpen, isEnable: isFile}));
        mRepoABox.addAction(
          new Action({label: "Commit", action: onClickCommit, isEnable: isFile}));
        mRepoABox.addAction(
          new Action({label: "Upload", action: onClickUpload}));
        mRepoABox.addAction(
          new Action({label: "History", action: onClickHistory, isEnable: isFile}));
        mRepoABox.addAction(
          new Action({label: "Linkage Nodes", action: onClickLinkageNodes, isEnable: isFile}));

        box = new deosc.View();
        box.addStyle("hbox");
        box.addChild(mRepoDocList);
        box.addChild(mRepoABox);

        main = new deosc.View();
        main.addChild(mSelectRepoWidget);
        main.addChild(box);

        self.setContent(main);
        self.setTitle("Document");
        self.getFooter().hide();
      };

      function constructor (weaver, repo, node) {
        mWeaver = weaver;
        mRepo = repo;
        mNode = node;
      }

      function onSelectRepository (index) {
        var item = mSelectRepoWidget.getItemAt(index);
        mRepo.setRepositoryId(item.repositoryId);
        mRepo.setRepositoryName(item.repositoryName);

        mRepoDocList.setRepository(mRepo, "/");
      }

      function isFile (metadata) {
        return !metadata.isdir;
      }

      function isDir (metadata) {
        return metadata.isdir;
      }

      function onClickItem (item) {
        mRepoABox.setData(item.getData());
      }

      function onClickOpen (metadata) {
        mRepo.open(metadata);
      }

      function onClickCommit (metadata) {
        function success (metadata) {
          mRepoDocList.updateMetadata(metadata);
        }
        openCommitDocumentDialog(mWeaver, mRepo, metadata, success);
      }

      function onClickUpload (metadata) {
        var parent;

        function success (metadata) {
          mRepoDocList.addMetadata(parent, metadata);
        }

        if (metadata.isdir) {
          parent = metadata;
          openUploadDocumentDialog(mWeaver, mRepo, parent, success);
        } else {
          getParents(
            mWeaver, mRepo, metadata,
            function (parents) {
              parent = parents[0];
              openUploadDocumentDialog(mWeaver, mRepo, parent, success);
            });
        }
      }

      function onClickHistory (metadata) {
        history(mWeaver, mRepo, metadata);
      }

      function onClickLinkageNodes (metadata) {
        searchLinkageNodes(mWeaver, mRepo, metadata);
      }

      constructor(weaver, repo, node);
    }


    function NodeDocumentDialog (weaver, repo, node) {
      var self = this, mWeaver, mRepo, mNode, mMain, mSelectRepoWidget,
      mRepoDocList, mRepoABox, mNodeABox, mNodeDocList;

      deosc.Dialog.call(self);

      self.onCreate = function () {
        var label1, label2, box1, box2;

        label1 = new deosc.View();
        label1.addStyle("label");
        label1.setText("Repository");

        label2 = new deosc.View();
        label2.addStyle("label");
        label2.setText("Documents");

        mSelectRepoWidget = new SelectRepositoryWidget();
        mSelectRepoWidget.setWidth(400);
        mSelectRepoWidget.setOnSelectRepositoryListener(onSelectRepository);
        mSelectRepoWidget.setRepository(mRepo);

        mRepoDocList = new BrowseWidget(mWeaver);
        mRepoDocList.setSize(400, 150);
        mRepoDocList.setOnClickItemListener(onRepoClickItem);

        mRepoABox = new ActionBox();
        mRepoABox.addAction(
          new Action({label: "Add", action: onRepoActionAdd, isEnable: isFile}));
        mRepoABox.addAction(
          new Action({label: "Open", action: onActionOpen, isEnable: isFile}));
        mRepoABox.addAction(
          new Action({label: "Commit", action: onActionCommit, isEnable: isFile}));
        mRepoABox.addAction(
          new Action({label: "Upload", action: onActionUpload}));
        mRepoABox.addAction(
          new Action({label: "History", action: onActionHistory, isEnable: isFile}));
        mRepoABox.addAction(
          new Action({label: "Linkage Nodes", action: onActionLinkageNodes, isEnable: isFile}));

        box1 = new deosc.View();
        box1.addStyle("hbox");
        box1.addChild(mRepoDocList);
        box1.addChild(mRepoABox);

        mNodeDocList = new NodeDocumentList(mWeaver, mRepo, mNode);
        mNodeDocList.setSize(400, 150);
        mNodeDocList.setOnClickItemListener(onNodeClickItem);

        mNodeABox = new ActionBox();
        mNodeABox.addAction(
          new Action({label: "Delete", action: onNodeActionDelete}));
        mNodeABox.addAction(
          new Action({label: "Open", action: onActionOpen}));
        mNodeABox.addAction(
          new Action({label: "Commit", action: onActionCommit}));
        mNodeABox.addAction(
          new Action({label: "History", action: onActionHistory}));
        mNodeABox.addAction(
          new Action({label: "Linkage Nodes", action: onActionLinkageNodes}));

        box2 = new deosc.View();
        box2.addStyle("hbox");
        box2.addChild(mNodeDocList);
        box2.addChild(mNodeABox);

        mMain = new deosc.View();
        mMain.addChild(label1);
        mMain.addChild(mSelectRepoWidget);
        mMain.addChild(box1);
        mMain.addChild(label2);
        mMain.addChild(box2);

        self.setContent(mMain);
        self.setTitle("Node Documents");
        self.getFooter().hide();
      };

      function constructor (weaver, repo, node) {
        mWeaver = weaver;
        mRepo = repo;
        mNode = node;
      }

      function onSelectRepository (index) {
        var item = mSelectRepoWidget.getItemAt(index);
        mRepo.setRepositoryId(item.repositoryId);
        mRepo.setRepositoryName(item.repositoryName);

        mRepoDocList.setRepository(mRepo, "/");
      }

      function isFile (metadata) {
        return !metadata.isdir;
      }

      function isDir (metadata) {
        return metadata.isdir;
      }

      function onRepoClickItem (item) {
        mRepoABox.setData(item.getData());
      }

      function onNodeClickItem (item) {
        mNodeABox.setData(item.getData());
      }

      function onRepoActionAdd (metadata) {
        mNodeDocList.addDocument(metadata);
      }

      function onNodeActionDelete (metadata) {
        mNodeDocList.removeDocument(metadata);
      }

      function onActionOpen (metadata) {
        mRepo.open(metadata);
      }

      function onActionCommit (metadata) {
        function success (metadata) {
          mRepoDocList.updateMetadata(metadata);
          mNodeDocList.updateDocument(metadata);
        }
        openCommitDocumentDialog(mWeaver, mRepo, metadata, success);
      }

      function onActionUpload (metadata) {
        var parent;

        function success (metadata) {
          mRepoDocList.addMetadata(parent, metadata);
        }

        if (metadata.isdir) {
          parent = metadata;
          openUploadDocumentDialog(mWeaver, mRepo, parent, success);
        } else {
          getParents(
            mWeaver, mRepo, metadata,
            function (parents) {
              parent = parents[0];
              openUploadDocumentDialog(mWeaver, mRepo, parent, success);
            });
        }
      }

      function onActionHistory (metadata) {
        history(mWeaver, mRepo, metadata);
      }

      function onActionLinkageNodes (metadata) {
        searchLinkageNodes(mWeaver, mRepo, metadata);
      }

      constructor(weaver, repo, node);
    }


    function Action (options) {
      var self = this, mData, mLabel, mAction, mIsEnable, mButton;

      deosc.View.call(self, $("<button>")[0]);

      self.setData = function (data) {
        mData = data;
        update();
      };

      self.setButton = function (button) {
        mButton = button;
        mButton.setText(mLabel);
        mButton.setWidth("100%");
        mButton.setOnClickListener(
          function () {
            mAction(mData);
          });
        update();
      };

      function constructor (options) {
        mLabel = options.label;
        mAction = options.action;
        mIsEnable = options.isEnable;
      }

      function update () {
        if (!mData) {
          mButton.disabled(true);
        } else if (!mIsEnable || mIsEnable(mData)) {
          mButton.disabled(false);
        } else {
          mButton.disabled(true);
        }
      }

      constructor(options);
    }


    function ActionBox () {
      var self = this, mActions, mData;

      deosc.View.call(self);

      self.addAction = function (action) {
        var button;

        button = new deosc.View($("<button>")[0]);

        self.addChild(button);

        action.setButton(button);

        mActions.push(action);

        update();
      };

      self.setData = function (data) {
        mData = data;
        update();
      };

      function constructor () {
        mActions = [];
        self.addStyle("vbox");
      }

      function update () {
        var i;

        for (i = 0; i < mActions.length; i++) {
          mActions[i].setData(mData);
        }
      }

      constructor();
    }


    function NodeDocumentList (weaver, repo, node) {
      var self = this, mWeaver, mRepo, mTree, mNode, mModel,
      mOnClickItemListener;

      deosc.ListView.call(self);

      self._super = {
        setOnClickItemListener: self.setOnClickItemListener
      };

      self.setOnClickItemListener = function (l) {
        mOnClickItemListener = l;
      };

      self.addDocument = function (metadata) {
        var exists = false, item;

        mModel.foreach(
          function (index) {
            if (this.getData().id == metadata.id) {
              exists = true;
              return false;
            }
            return true;
          });
        if (exists) return;

        mModel.add(createListItem(metadata));

        self.update();

        var list = mNode.getCMISInfo();
        if (!list) list = [];

        list.push(
          {id: metadata.id, name: metadata.name, workspace: "D-CaseWeaver"});

        var attrs = mNode.getAttributes();
        attrs.cmisInfo = list;

        mWeaver.command(new deosc.EditNodeCommand(mTree, mNode, attrs));
      };

      self.removeDocument = function (metadata) {
        mModel.foreach(
          function (index) {
            if (this.getData().id == metadata.id) {
              mModel.removeItem(this);
              self.setCurrent(-1);
              return false;
            }
            return true;
          });
        self.update();

        var list = mNode.getCMISInfo();
        if (!list) return;

        for (var i = 0; i < list.length; i++) {
          if (list[i].id == metadata.id) {
            list.splice(i, 1);
            break;
          }
        }

        var attrs = mNode.getAttributes();
        attrs.cmisInfo = list;

        mWeaver.command(new deosc.EditNodeCommand(mTree, mNode, attrs));
      };

      self.updateDocument = function (metadata) {
        mModel.foreach(
          function (index) {
            if (meta_comp(this.getData(), metadata, false)) {
              this.setColumns([metadata.name, metadata.version, metadata.mtime.toLocaleString(), deosc.path.dirname(metadata.path)]);
              this.setData(metadata);
              return false;
            }
            return true;
          });
        self.update();

        var list = mNode.getCMISInfo();
        if (!list) return;

        for (var i = 0; i < list.length; i++) {
          if (meta_comp(list[i], metadata, false)) {
            list.splice(i, 1, {id: metadata.id,
                               name: metadata.name,
                               workspace: "D-CaseWeaver"});
            break;
          }
        }

        var attrs = mNode.getAttributes();
        attrs.cmisInfo = list;

        mWeaver.command(new deosc.EditNodeCommand(mTree, mNode, attrs));
      };

      self.updateNodeDocuments = function () {
        var i, list;

        mModel = new deosc.ListModel();
        self.setModel(mModel);

        list = mNode.getCMISInfo();
        if (!list) list = [];

        function success (metadata) {
          mModel.add(createListItem(metadata));

          if (i < list.length) {
            mRepo.getMetadata(
              {id: list[i++].id, success: success, error: error});
          } else {
            self.update();
          }
        }

        function error () {
          console.log("error");
        }

        i = 0;

        if (i < list.length) {
          mRepo.getMetadata({id: list[i++].id, success: success, error: error});
        } else {
          self.update();
        }
      };

      function constructor (weaver, repo, node) {
        var i, list, header;

        mWeaver = weaver;
        mRepo = repo;
        mTree = weaver.getDCaseTreeView();
        mNode = node;

        self._super.setOnClickItemListener(onClickItem);

        header = new deosc.ListItem();
        header.addColumns(["Name", "Version", "Date Modified", "Location"]);


        mModel = new deosc.ListModel();
        mModel.setHeaderItem(header);
        self.setModel(mModel);

        list = mNode.getCMISInfo();
        if (!list) list = [];

        function success (metadata) {
          mModel.add(createListItem(metadata));

          if (i < list.length) {
            mRepo.getMetadata(
              {id: list[i++].id, success: success, error: error});
          } else {
            self.update();
          }
        }

        function error () {
          console.log("error");
          if (i < list.length) {
            mRepo.getMetadata(
              {id: list[i++].id, success: success, error: error});
          } else {
            self.update();
          }
        }

        i = 0;

        if (i < list.length) {
          mRepo.getMetadata(
            {
              id: list[i].id,
              version: list[i].version,
              success: success,
              error: error
            });
          i++;
        } else {
          self.update();
        }
      }

      function onClickItem (index) {
        if (mOnClickItemListener)
          mOnClickItemListener(mModel.getItemAt(index));
      }

      constructor(weaver, repo, node);
    }


    function BrowseDialog (weaver, repo, option) {
      var self = this, mWeaver, mRepo, mSelectRepoWidget,
      mDocList, mAbox, mFilter, mItem, mOnOKListener, mOption;

      deosc.Dialog.call(self);

      self.setFilter = function (filter) {
        mFilter = filter;
      };

      self.setOnOKListener = function (l) {
        mOnOKListener = l;
      };

      self.onCreate = function () {
        var main, box;

        mSelectRepoWidget = new SelectRepositoryWidget();
        mSelectRepoWidget.setWidth(400);
        mSelectRepoWidget.setOnSelectRepositoryListener(onSelectRepository);
        mSelectRepoWidget.setRepository(mRepo);

        mDocList = new BrowseWidget(mWeaver, mOption);
        mDocList.setSize(400, 200);
        mDocList.setOnClickItemListener(onClickItem);

        mAbox = new ActionBox();

        box = new deosc.View();
        box.addStyle("hbox");
        box.addChild(mDocList);
        box.addChild(mAbox);

        main = new deosc.View();
        main.addChild(mSelectRepoWidget);
        main.addChild(box);

        self.setContent(main);
        self.setTitle("Document Browser");
      };

      self.onOK = function () {
        if (mOnOKListener) {
          mOnOKListener(mItem ? mItem.getData() : null);
        } else {
          self.close();
        }
      };

      function constructor (weaver, repo, option) {
        mWeaver = weaver;
        mRepo = repo;
        mOption = option;
      }

      function filter (item) {
        if (!mFilter) return true;
        return mFilter(item ? item.getData() : null);
      }

      function onSelectRepository (index) {
        var item = mSelectRepoWidget.getItemAt(index);
        mRepo.setRepositoryId(item.repositoryId);
        mRepo.setRepositoryName(item.repositoryName);

        mDocList.setRepository(mRepo, "/", filter);
      }

      function onClickItem (item) {
        mItem = item;
      }

      constructor(weaver, repo, option);
    }


    function BrowseWidget (weaver, option) {
      var self = this, mRepo, mModel, mRootPath, mButtonBox,
      mOnClickItemListener, mIsAllVersions;

      deosc.TreeListView.call(self);

      self._super = {
        setOnClickItemListener: self.setOnClickItemListener
      };

      self.setRepository = function (repo, rootPath, filter) {
        var header;

        mRepo = repo;
        mRootPath = rootPath;

        header = new deosc.ListItem();
        header.addColumns(["Name", "Version", "Date Modified"]);

        mModel = new deosc.TreeListModel();
        mModel.setHeaderItem(header);


        if (filter) mModel.setFilter(filter);
        self.setModel(mModel);

        getMetadata(
          {
            path: mRootPath,
            callback: onGetRootMetadata
          });
      };

      self.setOnClickItemListener = function (l) {
        mOnClickItemListener = l;
      };

      self.addMetadata = function (parent, metadata) {
        onGetMetadata(parent, metadata);
      };

      self.updateMetadata = function (metadata) {
        mModel.foreach(
          function () {
            if (meta_comp(metadata, this.getData(), false)) {
              this.clearColumns();
              this.addColumn(metadata.name);
              this.addColumn(metadata.version);
              this.addColumn(metadata.mtime.toLocaleString());
              this.setData(metadata);
              mModel.exitForeach();
            }
          });
        self.update();
      };

      function constructor (weaver, option) {
        self._super.setOnClickItemListener(onClickItem);
        self.setOnCollapseItemListener(onCollapseItem);
        self.setOnExpandItemListener(onExpandItem);
        mIsAllVersions = option && option.allVersions;
      }

      function getMetadata (option) {
        var exists = false;

        function error () {
          console.error("getMetadata");
        }

        mModel.foreach(
          function () {
            if (meta_comp(option, this.getData())) {
              exists = true;
              mModel.exitForeach();
            }
          });

        if (exists) return;

        mRepo.getMetadata(
          {
            id: option.id,
            path: option.path,
            success: option.callback,
            error: error
          });
      }

      function onGetRootMetadata (root) {
        mRepo.getChildren(
          root,
          function (children) {
            var i;
            for (i = 0; i < children.length; i++) {
              onGetMetadata(root, children[i]);
            }
          });
      }

      function onGetMetadata (parent, meta) {
        var parentItem, dirname, item, exists = false;

        mModel.foreach(
          function () {
            if (meta_comp(this.getData(), meta)) {
              exists = true;
              mModel.exitForeach();
            }
          });

        if (exists) return;

        item = new deosc.TreeListItem();
        item.addColumn(meta.name);
        item.addColumn(meta.version);
        item.addColumn(meta.mtime.toLocaleString());
        item.setData(meta);
        item.setIcon(meta.isdir ? "icon-dir" : "icon-file");
        item.collapsable(meta.isdir || (mIsAllVersions && meta.isLatestVersion));

        mModel.foreach(
          function () {
            if (meta_comp(this.getData(), parent)) {
              parentItem = this;
              mModel.exitForeach();
            }
          });

        mModel.add(parentItem, item);

        mModel.sort(
          function (a, b) {
            var metaA = a.getData();
            var metaB = b.getData();
            if (metaA.isdir == metaB.isdir) {
              if (metaA.name == metaB.name) {
                return 0;
              } else if (metaA.name < metaB.name) {
                return -1;
              } else {
                return 1;
              }
            } else {
              return metaA.isdir ? -1 : 1;
            }
          });

        self.update();
      }

      function onClickItem (index) {
        if (mOnClickItemListener)
          mOnClickItemListener(self.getCurrentItem());
      }

      function onCollapseItem (index) {
      }

      function onExpandItem (index) {
        var item, metadata;

        item = mModel.getItemAt(index);
        metadata = item.getData();

        if (metadata.isdir) {
          mRepo.getChildren(
            metadata,
            function (children) {
              var i;
              for (i = 0; i < children.length; i++) {
                onGetMetadata(metadata, children[i]);
              }
            });
        } else if (mIsAllVersions && metadata.isLatestVersion) {
          mRepo.getMetadata(
            {
              id: metadata.id,
              path: metadata.path,
              allVersions: true,
              success: function (allVersions) {
                var i;
                for (i = 1; i < allVersions.length; i++) {
                  onGetMetadata(allVersions[0], allVersions[i]);
                }
              }
            });
        }
      }

      constructor(weaver, option);
    }

    BrowseWidget.Target = {
      DIR: 1,
      FILE: 2,
      FILE_DIR: 3,
      ROOT_DIR: 4
    };


    function CommentDialog () {
      var self = this, mMain, mComment;

      deosc.Dialog.call(self);

      self.getComment = function () {
        return mComment.getText();
      };

      self.onCreate = function () {
        var label;

        label = new deosc.View();
        label.addStyle("label");
        label.setText("Comment");

        mComment = new deosc.Textarea();
        mComment.setWidth(400, 200);

        mMain = new deosc.View();
        mMain.addStyle("vbox");
        mMain.addChild(label);
        mMain.addChild(mComment);

        self.setContent(mMain);
        self.setTitle("Comment");
      };
    }


    function CommitDocumentDialog (path) {
      var self = this, mPath, mFileInput, mPathInput, mComment;

      deosc.Dialog.call(self);

      self.getFile = function () {
        return mFileInput.getElement().files[0];
      };

      self.getComment = function () {
        return mComment.getText();
      };

      self.onCreate = function () {
        var main, label1, label2, label3;

        label1 = new deosc.View();
        label1.addStyle("label");
        label1.setText("Commit File");

        label2 = new deosc.View();
        label2.addStyle("label");
        label2.setText("Path");

        label3 = new deosc.View();
        label3.addStyle("label");
        label3.setText("Comment");

        mFileInput = new deosc.View($("<input type='file'>")[0]);
        mFileInput.setWidth(400);

        mPathInput = new deosc.InputText();
        mPathInput.setText(mPath);
        mPathInput.setWidth(400);
        mPathInput.readonly(true);

        mComment = new deosc.Textarea();
        mComment.setSize(400, "5em");

        main = new deosc.View();
        main.addStyle("vbox");
        main.addChild(label1);
        main.addChild(mFileInput);
        main.addChild(label2);
        main.addChild(mPathInput);
        main.addChild(label3);
        main.addChild(mComment);

        self.setTitle("Commit");
        self.setContent(main);
      };

      function constructor (path) {
        mPath = path;
      }

      constructor(path);
    }


    function UploadDocumentDialog (path) {
      var self = this, mPath, mDir, mFile, mName;

      deosc.Dialog.call(self);

      self.getFile = function () {
        return mFile.getElement().files[0];
      };

      self.getName = function () {
        return mName.getText();
      };

      self.onCreate = function () {
        var main, label1, label2, label3;

        label1 = new deosc.View();
        label1.addStyle("label");
        label1.setText("Upload File");

        label2 = new deosc.View();
        label2.addStyle("label");
        label2.setText("Directory");

        label3 = new deosc.View();
        label3.addStyle("label");
        label3.setText("File Name");

        mFile = new deosc.View($("<input type='file'>")[0]);
        mFile.setWidth(400);
        mFile.setOnChangeListener(onChange);

        mDir = new deosc.InputText();
        mDir.setWidth(400);
        mDir.readonly(true);
        mDir.setText(mPath);

        mName = new deosc.InputText();
        mName.setWidth(400);

        main = new deosc.View();
        main.addStyle("vbox");
        main.addChild(label1);
        main.addChild(mFile);
        main.addChild(label2);
        main.addChild(mDir);
        main.addChild(label3);
        main.addChild(mName);

        self.setContent(main);
        self.setTitle("Upload");
      };

      function constructor (path) {
        mPath = path;
      }

      function onChange () {
        var file = mFile.getElement().files[0];
        if (file) mName.setText(file.name);
      }

      constructor(path);
    }


    function HistoryDialog (weaver, repo, metadata) {
      var self = this, mWeaver, mRepo, mMetadata, mView, mModel, mABox;

      deosc.Dialog.call(self);

      self.onCreate = function () {
        var main, box, path;

        path = new deosc.InputText();
        path.setWidth(400);
        path.readonly(true);
        path.setText(mMetadata.path);

        mView = new deosc.ListView();
        mView.setSize(400, 200);
        mView.setOnClickItemListener(onClickItem);

        mABox = new ActionBox();
        mABox.setWidth(80);
        mABox.addAction(new Action({label: "Open", action: onClickOpen}));

        box = new deosc.View();
        box.addStyle("hbox");
        box.addChild(mView);
        box.addChild(mABox);

        main = new deosc.View();
        main.addStyle("vbox");
        main.addChild(path);
        main.addChild(box);

        self.setContent(main);
        self.setTitle("History");
        self.getFooter().hide();

        mRepo.getMetadata(
          {
            id: mMetadata.id,
            allVersions: true,
            success: onGetMetadata
          });
      };

      function constructor (weaver, repo, metadata) {
        mWeaver = weaver;
        mRepo = repo;
        mMetadata = metadata;
      }

      function onGetMetadata (metadatas) {
        var i, meta, item, header;

        header = new deosc.ListItem();
        header.addColumns(["Version", "Comment", "Date Modified"]);

        mModel = new deosc.ListModel();
        mModel.setHeaderItem(header);

        for (i = 0; i < metadatas.length; i++) {
          meta = metadatas[i];

          item = new deosc.ListItem();
          item.addColumn(meta.version);
          item.addColumn(meta.checkinComment);
          item.addColumn(meta.mtime.toLocaleString());
          item.setData(meta);
          mModel.add(item);
        }

        mView.setModel(mModel);
      }

      function onClickItem (index) {
        var item = mModel.getItemAt(index);
        mABox.setData(item.getData());
      }

      function onClickOpen (metadata) {
        mRepo.open(
          {
            id: metadata.id,
            path: metadata.path,
            version: metadata.version
          });
      }

      constructor(weaver, repo, metadata);
    }


    function SearchLinkageNodesDialog (weaver, repo, metadata) {
      var self = this, mWeaver, mDCase, mRepo, mMetadata, mView, mModel;

      deosc.Dialog.call(self);

      self.onCreate = function () {
        var main, path, nodes, i, label1, label2, header;

        header = new deosc.ListItem();
        header.addColumns(["Name", "Node", "Date Modified", "Location"]);

        mModel = new deosc.ListModel();
        mModel.setHeaderItem(header);

        label1 = new deosc.View();
        label1.addStyle("label");
        label1.setText("Document:");

        path = new deosc.InputText();
        path.setText(mMetadata.path);
        path.setWidth(400);
        path.readonly(true);

        label2 = new deosc.View();
        label2.addStyle("label");
        label2.setText("Linkage Nodes:");

        mView = new deosc.ListView();
        mView.setSize(400, 200);
        mView.setModel(mModel);

        main = new deosc.View();
        main.addStyle("vbox");
        main.addChild(label1);
        main.addChild(path);
        main.addChild(label2);
        main.addChild(mView);

        self.setContent(main);
        self.setTitle("Linkage Node List");
        self.getFooter().hide();

        mRepo.getParents(
          metadata,
          function (parents) {
            if (0 < parents.length) {
              mRepo.getChildren(parents[0], onListdir);
            } else {
              onListdir([]);
            }
          });
      };

      function constructor (weaver, repo, metadata) {
        mWeaver = weaver;
        mRepo = repo;
        mMetadata = metadata;
      }

      function onListdir (list) {
        var i, meta;

        for (i = 0; i < list.length; i++) {
          meta = list[i];
          if (meta.isdir) continue;
          if (!meta.name.match(/\.dcase/)) continue;
          getFile(meta);
        }
      }

      function getFile (meta) {
        mRepo.getFile(
          {
            id: meta.id,
            success: function (doc) {
              onGetFile(doc, meta);
            }
          });
      }

      function onGetFile (doc, meta) {
        var dcase;

        var parser;
        if (typeof doc == "string" || doc instanceof String) {
          parser = new DOMParser();
          doc = parser.parseFromString(doc, "application/xml");
        }

        dcase = new deosc.DCase(doc);

        dcase.foreach(
          null,
          function () {
            var node = this, item, udef010, list, i;

            if (!(node instanceof deosc.DCaseNode)) return;

            list = node.getCMISInfo();
            if (!list) return;

            for (i = 0; i < list.length; i++) {
              if (!meta_comp(list[i], mMetadata, false)) continue;

              item = new deosc.ListItem();
              item.addColumn(meta.name);
              item.addColumn(node.getName());
              item.addColumn(meta.mtime.toLocaleString());
              item.addColumn(deosc.path.dirname(meta.path));
              item.setData({node: node, ecm: list[i]});
              mModel.add(item);
            }

            mView.update();
          });
      }

      constructor(weaver, repo, metadata);
    }

    function PreferenceView (weaver, pref) {
      var self = this, mWeaver = weaver, mPreference = pref,
      mUse, mCmisUrl, mUser, mEditMember;

      deosc.View.call(self);

      self.getPreference = function () {
        return {
          cmisUrl: mCmisUrl.getText(),
          user: mUser.getText(),
          use: mUse.isChecked(),
          members: mEditMember.getMembers()
        };
      };

      function constructor () {
        var label1, label2, label3;

        mUse = new Use();
        mUse.setMargin("16px 0 8px 0");

        label1 = new deosc.View();
        label1.setMargin("16px 0 8px 0");
        label1.setText("CMIS URL");

        mCmisUrl = new deosc.InputText();
        mCmisUrl.setWidth(300);
        mCmisUrl.setMargin("0 0 0 4px");
        mCmisUrl.setText(mPreference.getValue("cmisUrl", ""));

        label2 = new deosc.View();
        label2.setMargin("16px 0 8px 0");
        label2.setText("User");

        mUser = new deosc.InputText();
        mUser.setWidth(300);
        mUser.setMargin("0 0 0 4px");
        mUser.setText(mPreference.getValue("user", ""));

        label3 = new deosc.View();
        label3.setMargin("16px 0 4px 0");
        label3.setText("Member");

        mEditMember = new deosc.EditMemberWidget();

        self.addChild(mUse);
        self.addChild(label1);
        self.addChild(mCmisUrl);
        self.addChild(label2);
        self.addChild(mUser);
        self.addChild(label3);
        self.addChild(mEditMember);
      }

      constructor();

      function Use () {
        var self = this, mCheckBox;

        deosc.View.call(self);

        self.isChecked = function () {
          return mCheckBox.isChecked();
        };

        function constructor () {
          var label, use;

          self.addStyle("hbox");

          use = mPreference.getValue("use", false);

          mCheckBox = new deosc.View($("<input type='checkbox'>")[0]);
          mCheckBox.checked(use);

          label = new deosc.View();
          label.setText("Use Alfresco");
          label.setPadding("0 0 0 8px");

          self.addChild(mCheckBox);
          self.addChild(label);
        }

        constructor();
      }
    }


    function AlfrescoPlugin () {
      var self = this, mWeaver;

      self.setDCaseWeaver = function (weaver) {
        mWeaver = weaver;
      };

      self.onEvent = function (event) {
        if (event.what == deosc.DCaseWeaver.Event.OPEN_PREFERENCE_DIALOG) {
          onOpenPreferenceDialog(event.data);
        }
      };

      self.onCreate = function () {
        var menu, pref;
        pref = mWeaver.getPreferences().getPreference("Alfresco");

        if (pref.getValue("use", false)) {
          menu = mWeaver.getDCaseNodeMenu();
          menu.addMenuItem(new DocumentMenuItem());
        }
      };

      function onOpenPreferenceDialog (dialog) {
        var pref = dialog.getPreference("Alfresco");
        var view = new PreferenceView(mWeaver, pref);

        dialog.addView(view, pref);
        dialog.setOnSaveListener(
          function () {
            var p = view.getPreference();
            for (key in p) {
              pref.setValue(key, p[key]);
            }
          });
      }

      function DocumentMenuItem () {
        var self = this;

        deosc.MenuItem.call(self, "Document");

        self.onInit = function (data) {
          var type = data.target.node.getType();
          if (type == "Context" || type == "Evidence") {
            self.disabled(false);
          } else {
            self.disabled(true);
          }
        };

        function constructor () {
          var menu = new deosc.Menu();
          menu.addMenuItem(new deosc.MenuItem("Browse", onClickDocument));
          self.setMenu(menu);
        }

        function onClickDocument (data) {
          var fm = mWeaver.getFileManager();
          fm.openNodeDocumentDialog(data.target.node);
        }

        constructor();
      }
    }


    deosc.DCaseWeaver.ready(
      function (weaver) {
        var pref;

        pref = weaver.getPreferences().getPreference("Alfresco");

        if (pref.getValue("use", false)) {
          weaver.setFileManager(new FileManager());
        }

        weaver.addPlugin(new AlfrescoPlugin());
      }
    );

  }
) (window);


(
  function (window) {
    var PREFERENCE_NAME = "DS-Bench";

    function openScenarioListDialog (weaver, nodeView) {
      var dialog;

      function success () {
        dialog = new ScenarioListDialog(weaver, nodeView);
        weaver.dialogPopup(dialog);
      }

      if (checkPreference(weaver)) {
        success();
      } else {
        dialog = new PreferenceDialog(weaver);
        dialog.setOnOKListener(
          function () {
            if (checkPreference(weaver)) success();
          });
        weaver.dialogPopup(dialog);
      }
    }


    function openParametersDialog (weaver, nodeView) {
      var dialog = new ParametersDialog(weaver, nodeView);
      weaver.dialogPopup(dialog);
    }


    function executeScenario (weaver, node) {
      var dsbench, scenario, before, tree;

      function onStart () {
        //todo set view id
      }

      function onSuccess (status) {
        var successNode, nodeView, link, linkView, desc, tree, dcase, pref, url,
        after, msg, i;

        tree = weaver.getDCaseTreeView();
        dcase = tree.getDCase();
        desc = "";
        for (i = 0; i < status.scores.length; i++) {
          desc += status.scores[i].name + ":"
            + status.scores[i].actual
            + "(" +  status.scores[i].expect + ")\n";
        }

        pref = weaver.getPreferences().getPreference("DS-Bench");
        url = pref.getValue("url") + "/view_result_detail.cgi?view_id="
          + scenario.getViewId();

        successNode = new deosc.DCaseNode(weaver.getDCase());
        successNode.setAttributes(
          {
            name: dcase.getNewName(deosc.DCase.getNodeNamePrefix("Evidence")),
            id: dcase.getNewID(),
            type: "Evidence",
            description: desc,
            attachment: url
          });

        nodeView = deosc.canvas.createDCaseNodeView(successNode);
        nodeView.setBackgroundColor("#88f");

        link = new deosc.DCaseLink(weaver.getDCase());
        link.setAttributes(
          {
            name: dcase.getNewName(deosc.DCase.LINK_NAME_PREFIX),
            id: dcase.getNewID(),
            type: deosc.DCaseLink.getLinkType(successNode.getType()),
            source: node.getID(),
            target: successNode.getID()
          });
        linkView = deosc.canvas.createDCaseLinkView(link);

        msg = status.status + " " + status.progress + "%";
        node.setMessage(msg);
        after = node.getAttributes();
        node.setAttributes(before);

        weaver.command(
          [
            new deosc.EditNodeCommand(tree, node, after),
            new deosc.AddNodeCommand(tree, nodeView),
            new deosc.AddLinkCommand(tree, linkView),
            new deosc.ArrangeCommand(tree)
          ]);
      }

      function onFailure (status) {
        var failureNode, nodeView, link, linkView, desc, tree, dcase, pref, url,
        after, msg;

        tree = weaver.getDCaseTreeView();
        dcase = tree.getDCase();
        pref = weaver.getPreferences().getPreference("DS-Bench");
        url = pref.getValue("url") + "/view_result_detail.cgi?view_id="
          + scenario.getViewId();

        failureNode = new deosc.DCaseNode(weaver.getDCase());
        failureNode.setAttributes(
          {
            name: dcase.getNewName(
              deosc.DCase.getNodeNamePrefix("Undeveloped")),
            id: dcase.getNewID(),
            type: "Undeveloped",
            attachment: url
          });

        nodeView = deosc.canvas.createDCaseNodeView(failureNode);
        nodeView.setBackgroundColor("#f88");

        link = new deosc.DCaseLink(weaver.getDCase());
        link.setAttributes(
          {
            name: dcase.getNewName(deosc.DCase.LINK_NAME_PREFIX),
            id: dcase.getNewID(),
            type: deosc.DCaseLink.getLinkType(failureNode.getType()),
            source: node.getID(),
            target: failureNode.getID()
          });
        linkView = deosc.canvas.createDCaseLinkView(link);

        msg = status.status + " " + status.progress + "%";
        node.setMessage(msg);
        after = node.getAttributes();
        node.setAttributes(before);

        weaver.command(
          [
            new deosc.EditNodeCommand(tree, node, after),
            new deosc.AddNodeCommand(tree, nodeView),
            new deosc.AddLinkCommand(tree, linkView),
            new deosc.ArrangeCommand(tree)
          ]);
      }

      tree = weaver.getDCaseTreeView();

      dsbench = new DSBench(weaver);

      before = node.getAttributes();

      scenario = new Scenario(dsbench, tree, node);
      scenario.setOnStartListener(onStart);
      scenario.setOnSuccessListener(onSuccess);
      scenario.setOnFailureListener(onFailure);
      scenario.start();
    }


    function stopScenario (node) {
      var scenario;
      scenario = Scenario.getScenario(node.getID());
      if (scenario) scenario.stop();
    }


    function addSuccessNode () {

    }

    function addFailureNode () {

    }

    function DSBench (weaver) {
      var self = this, mWeaver, mPreference;

      self.getAttrs = function (e) {
        var i, attr, attrs = {};
        for (i = 0; i < e.attributes.length; i++) {
          attr = e.attributes[i];
          attrs[attr.nodeName] = attr.nodeValue;
        }
        return attrs;
      };

      self.makeFilter = function (ns, tag) {
        return function () {
          return this.namespaceURI == ns && this.localName == tag;
        };
      };

      self.getScenarioList = function (callback) {
        var url = mPreference.getValue("url", null);

        $.ajax(
          {
            url: "cgi-bin/dsbench-proxy.wsgi/scenario_list",
            data: {url: url},
            success: function (doc) {
              if (callback) {
                callback(self.parseScenarioList(doc));
              }
            },
            error: function (jqXHR, textStatus, errorThrown) {
              if (textStatus == "parsererror") {
                alert("Parser Error");
              } else {
                alert(jqXHR.responseText);
              }
            }
          });
      };

      self.parseScenarioList = function (doc) {
        var scenarios = [];
        var ns = "http://www.dependable-os.net/2011/06/ds-bench/configurations";
        $(doc)
          .children().filter(self.makeFilter(ns, "Configurations"))
          .children().filter(self.makeFilter(ns, "Scenario"))
          .each(
            function () {
              scenarios.push(self.getAttrs(this));
            });

        scenarios.sort(
          function (a, b) {
            if (a.id == b.id) {
              return 0;
            } else if (a.id < b.id) {
              return -1;
            } else {
              return 1;
            }
          });

        return scenarios;
      };

      self.getScenarioDetail = function (name, callback) {
        var url = mPreference.getValue("url", null);

        $.ajax(
          {
            url: "cgi-bin/dsbench-proxy.wsgi/scenario_detail",
            data: {url: url, name: name},
            success: function (doc) {
              if (callback) callback(self.parseScenarioDetail(doc));
            },
            error: function () {
            }
          });
      };

      self.parseScenarioDetail = function (doc) {
        var ns = "http://www.dependable-os.net/2011/06/ds-bench/scenario";
        var scenario = {};
        var parameters = [];

        var scenarioElement = $(doc)
          .children().filter(self.makeFilter(ns, "Scenario"))[0];

        scenario = self.getAttrs(scenarioElement);

        $(scenarioElement)
          .children().filter(self.makeFilter(ns, "Parameter"))
          .each(
            function () {
              parameters.push(self.getAttrs(this));
            });

        return {scenario: scenario, parameters: parameters};
      };

      self.runScenario = function (name, parameters, callback) {
        var key, p, ns, doc, scenarioElem, paramElem, url;

        url = mPreference.getValue("url", null);
        ns = "http://www.dependable-os.net/2011/06/ds-bench/exec/request";

        doc = document.implementation.createDocument(ns, "Scenario", null);

        scenarioElem = doc.documentElement;
        $(scenarioElem).attr({name: name});

        for (key in parameters) {
          p = parameters[key];
          paramElem = doc.createElementNS(ns, "xs:Parameter");
          $(paramElem)
            .attr({name: key, value: p.value})
            .appendTo(scenarioElem);
        }

        var opt = {
          url: "cgi-bin/dsbench-proxy.wsgi/run_scenario?url=" + url,
          data: doc,
          type: "POST",
          processData: false,
          success: function (doc) {
            var result = self.parseRunResult(doc);
            if (callback) callback(result);
          },
          error: function (jqXHR, textStatus, errorThrown) {
            if (textStatus == "parsererror") {
              alert("Parser Error");
            } else {
              alert(jqXHR.responseText);
            }
          }
        };
        $.ajax(opt);
      };

      self.parseRunResult = function (doc) {
        var ns = "http://www.dependable-os.net/2011/06/ds-bench/exec/response";

        //TODO: fix bug that run_scenario.cgi return content type "text/html"
        var parser = new DOMParser();
        doc = parser.parseFromString(doc, "application/xml");

        return self.getAttrs(doc.documentElement);
      };

      self.getStateResult = function (scenario_name, exec_id, parameters, callback) {
        var url = mPreference.getValue("url", null);
        var data = {
          url: url,
          scenario_name: scenario_name,
          exec_id: exec_id
        };
        var name;
        for (name in parameters) {
          data[name] = parameters[name].value;
        }
        var opt = {
          url: "cgi-bin/dsbench-proxy.wsgi/status",
          data: data,
          success: function (doc) {
            var result = self.parseStateResult(doc);
            if (callback) callback(result);
          },
          error: function (jqXHR, textStatus, errorThrown) {
            if (textStatus == "parsererror") {
              alert("Parser Error");
            } else {
              alert(jqXHR.responseText);
            }
          }
        };
        $.ajax(opt);
      };

      self.parseStateResult = function (doc) {
        var ns = "http://www.dependable-os.net/2011/06/ds-bench/search";
        var result = {
          scenarios: []
        };

        $(doc)
          .children().filter(self.makeFilter(ns, "Result"))
          .each(
            function () {
              $.extend(result, self.getAttrs(this));
            })
          .children().filter(self.makeFilter(ns, "Scenario"))
          .each(
            function () {
              var scenario = self.getAttrs(this);
              scenario.scores = [];
              $(this).children().filter(self.makeFilter(ns, "Score"))
                .each(
                  function () {
                    scenario.scores.push(self.getAttrs(this));
                  });
              result.scenarios.push(scenario);
            });

        return result;
      };

      self.stopScenario = function (name, exec_id, callback) {
        var i, p, ns, doc, scenarioElem, paramElem, url;

        url = mPreference.getValue("url", null);
        ns = "http://www.dependable-os.net/2011/06/ds-bench/exec/stop/request";

        doc = document.implementation.createDocument(ns, "Scenario", null);

        scenarioElem = doc.documentElement;
        $(scenarioElem).attr({name: name, exec_id: exec_id});

        var opt = {
          url: "cgi-bin/dsbench-proxy.wsgi/stop_scenario?url=" + url,
          data: doc,
          type: "POST",
          processData: false,
          success: function (doc) {
            var result = self.parseStopResult(doc);
            if (callback) callback(result);
          },
          error: function (jqXHR, textStatus, errorThrown) {
            if (textStatus == "parsererror") {
              alert("Parser Error");
            } else {
              alert(jqXHR.responseText);
            }
          }
        };
        $.ajax(opt);
      };

      self.parseStopResult = function (doc) {
        var ns = "http://www.dependable-os.net/2011/06/ds-bench/exec/response";

        //TODO: fix bug that run_scenario.cgi return content type "text/html"
        var parser = new DOMParser();
        doc = parser.parseFromString(doc, "application/xml");

        return self.getAttrs(doc.documentElement);
      };

      function constructor (weaver) {
        mWeaver = weaver;
        mPreference = weaver.getPreferences().getPreference("DS-Bench");
      }

      constructor(weaver);
    }

    DSBench.Status = {
      SUCCESS: 0,
      NOT_SATISFIED: 1,
      READY: 2,
      RUN: 3,
      RESERVE: 4,
      INSTALL: 5,
      STOPPING: 6,
      STOPPED: 7,
      ERROR: 8,
      ST_ERROR:9
    };

    DSBench.StatusText = [
      "finish(success)",
      "finish(failed)",
      "ready",
      "run",
      "reserve",
      "install",
      "stopping",
      "stopped",
      "error",
      "st error"
    ];


    function ScenarioListDialog (weaver, nodeView) {
      var self = this, mWeaver, mNodeView, mDSBench, mListView, mModel;

      deosc.Dialog.call(self);

      self.onCreate = function () {
        mListView = new deosc.ListView();
        mListView.setSize(400, 200);
        self.setContent(mListView);
        self.setTitle("DS-Bench Scenario");

        mDSBench.getScenarioList(onGetScenarioList);
      };

      self.onOK = function () {
        var item, data;
        item = mListView.getCurrentItem();
        if (item) {
          data = item.getData();
          mDSBench.getScenarioDetail(data.name, onLoadScenario);
        } else {
          alert("Please select a scenario.");
        }
      };

      function constructor (weaver, nodeView) {
        mWeaver = weaver;
        mNodeView = nodeView;
        mDSBench = new DSBench(weaver);
      }

      function onGetScenarioList (list) {
        var i, item;

        mModel = new deosc.ListModel();

        for (i = 0; i < list.length; i++) {
          item = new deosc.ListItem();
          item.addColumns([list[i].name, list[i].description]);
          item.setData(list[i]);
          mModel.add(item);
        }

        mListView.setModel(mModel);
      }

      function onLoadScenario (detail) {
        var node, nodeView, link, linkView, parentNode, name, tree, pref,
        i, p, params = {};

        tree = mWeaver.getDCaseTreeView();

        parentNode = mNodeView.node;

        pref = mWeaver.getPreferences().getPreference("DS-Bench");

        name = pref.getValue("name", "");

        for (i = 0; i < detail.parameters.length; i++) {
          p = detail.parameters[i];
          params[p.name] = p;
        }

        node = new deosc.DCaseNode(mWeaver.getDCase());
        node.setAttributes(
          {
            type: "Goal",
            name: tree.getDCase().getNewName(
              deosc.DCase.getNodeNamePrefix("Goal")),
            id: tree.getDCase().getNewID(),
            description: detail.scenario.description_format,
            parameterizedDesc: detail.scenario.description_format,
            parameters: params,
            dsBenchProject:{
              project: "DS-Bench",
              server: name,
              scenario: detail.scenario.name
            }
          });
        nodeView = deosc.canvas.createDCaseNodeView(node);

        link = new deosc.DCaseLink(mWeaver.getDCase());
        link.setAttributes(
          {
            name: tree.getDCase().getNewName(deosc.DCase.LINK_NAME_PREFIX),
            id: tree.getDCase().getNewID(),
            type: deosc.DCaseLink.getLinkType(node.getType()),
            source: parentNode.getID(),
            target: node.getID()
          });
        linkView = deosc.canvas.createDCaseLinkView(link);

        mWeaver.command(
          [
            new deosc.AddNodeCommand(tree, nodeView),
            new deosc.AddLinkCommand(tree, linkView),
            new deosc.ArrangeCommand(tree)
          ]);

        self.close();
      }

      constructor(weaver, nodeView);
    }


    function ParametersDialog (weaver, nodeView) {
      var self = this, mWeaver, mNodeView, mNode, mTree, mListView, mModel,
      mCaption, mTable, mMain, mParameters;

      deosc.Dialog.call(self);

      self.onCreate = function () {
        var name, proj;

        mMain = new deosc.View();

        mTable = new deosc.TableView();
        mMain.addChild(mTable);
        self.setContent(mMain);
        self.setTitle("Scenario Parameters");

        mParameters = mNode.getParameters();
        proj = mNode.getDSBenchProject();

        for (name in  mParameters) {
          (
            function (name, param) {
              var item, input;

              input = new deosc.InputText();
              input.setText(param.value);
              input.setOnChangeListener(
                function () {
                  param.value = input.getText();
                });

              mTable.addRow([name, input]);
            }
          ) (name, mParameters[name]);
        }

      };

      self.onOK = function () {
        var params;

        params = mNode.getAttributes();
        params.parameters = mParameters;

        mWeaver.command(new deosc.EditNodeCommand(mTree, mNodeView, params));

        self.close();
      };

      function constructor (weaver, nodeView) {
        mWeaver = weaver;
        mTree = weaver.getDCaseTreeView();
        mNodeView = nodeView;
        mNode = nodeView.node;
      }

      constructor(weaver, nodeView);
    }


    function Scenario (dsbench, tree, node) {
      var self = this, mDSBench, mTree, mNode,
      mOnStartListener, mOnStopListener, mOnUpdateListener,
      mOnSuccessListener, mOnFailureListener,
      mParameters, mScenario, mStatus, mInterval,
      mRunResult, mStatusResult;

      self.setOnStartListener = function (l) {
        mOnStartListener = l;
      };

      self.setOnStopListener = function (l) {
        mOnStopListener = l;
      };

      self.setOnUpdateListener = function (l) {
        mOnUpdateListener = l;
      };

      self.setOnSuccessListener = function (l) {
        mOnSuccessListener = l;
      };

      self.setOnFailureListener = function (l) {
        mOnFailureListener = l;
      };

      self.start = function () {
        var proj, params;

        params = mNode.getParameters();
        mParameters = params ? params : {};

        proj = mNode.getDSBenchProject();
        mScenario = proj ? proj.scenario : null;

        mDSBench.runScenario(mScenario, mParameters, onStart);
      };

      self.stop = function () {
        mDSBench.stopScenario(mScenario, mRunResult.exec_id);
      };

      self.status = function () {
        return mStatus;
      };

      self.getViewId = function () {
        if (mRunResult) {
          return mRunResult.view_id;
        }
        return null;
      };

      function constructor (dsbench, tree, node) {
        mDSBench = dsbench;
        mTree = tree;
        mNode = node;
        mInterval = 5000;
        mStatus = {
          status: "idle",
          progress: 0,
          scores: [],
          isRunning: false
        };

        Scenario.register(node.getID(), self);

        function loop () {
          function onGetStateResult (data) {
            mStatusResult = data;
            if (data.code == 0) {
              var s = data.scenarios[0];
              var status = s.status;
              var desc = (0 <= status && status <= DSBench.Status.ST_ERROR) ?
                DSBench.StatusText[status] :
                ((status < 0) ? "failed" : "unknown");

              mStatus = {
                status: desc,
                progress: s.progress,
                scores: s.scores,
                isRunning: false
              };

              onUpdate();

              if (status == DSBench.Status.SUCCESS) {
                onSuccess();
              } else if (status == DSBench.Status.NOT_SATISFIED
                         || status == DSBench.Status.STOPPED
                         || status == DSBench.Status.ERROR
                         || status < 0
                         || DSBench.Status.ST_ERROR < status) {
                onFailure();
              } else {
                mStatus.isRunning = true;
                setTimeout(loop, mInterval);
              }
            } else {
              console.error(data.message);
              onFailure();
            }
          }

          if (!mRunResult) {
            setTimeout(loop, mInterval);
            return;
          }

          mDSBench.getStateResult(
            mRunResult.scenario_name,
            mRunResult.exec_id,
            mParameters,
            onGetStateResult
          );
        }
        loop();
      }

      function onStart (runResult) {
        var proj;
        proj = mNode.getDSBenchProject();
        proj.execId = runResult.exec_id;
        proj.viewId = runResult.view_id;
        mNode.setDSBenchProject(proj);
        mRunResult = runResult;
      }

      function onUpdate () {
        var msg = mStatus.status + " " + mStatus.progress + "%";

        mNode.setMessage(msg);

        mTree.onChange(); //TODO weaver.sendEvent();
      }

      function onSuccess () {
        Scenario.unregister(mNode.getID());
        if (mOnSuccessListener) mOnSuccessListener(mStatus);
      }

      function onFailure () {
        Scenario.unregister(mNode.getID());
        if (mOnFailureListener) mOnFailureListener(mStatus);
      }

      constructor(dsbench, tree, node);
    }
    Scenario.scenario_list = {};
    Scenario.getScenario = function (id) {
      return Scenario.scenario_list[id];
    };
    Scenario.register = function (id, scenario) {
      Scenario.scenario_list[id] = scenario;
    };
    Scenario.unregister = function (id) {
      delete Scenario.scenario_list[id];
    };

    function DSBenchPlugin () {
      var self = this, mWeaver, mTree, mDSBench, mPreference;

      self.setDCaseWeaver = function (weaver) {
        mWeaver = weaver;
        mTree = weaver.getDCaseTreeView();
        mDSBench = new DSBench(weaver);
        mPreference = weaver.getPreferences().getPreference("DS-Bench");
      };

      self.onCreate = function () {
        var menuItem;

        menuItem = new deosc.MenuItem("DS-Bench");
        menuItem.onInit = function (data) {
          var type;
          type = data.target.getType();
          if (type == "Goal" || type == "Strategy") {
            menuItem.disabled(false);
          } else {
            menuItem.disabled(true);
          }
        };
        menuItem.setMenu(new DSBenchMenu(mWeaver));

        mWeaver.getDCaseNodeMenu()
          .addMenuItem(menuItem);
      };

      self.onEvent = function (event) {
        if (event.what == deosc.DCaseWeaver.Event.OPEN_PREFERENCE_DIALOG) {
          onOpenPreferenceDialog(event.data);
        }
      };

      function constructor () {
      }

      function onOpenPreferenceDialog (dialog) {
        var pref = dialog.getPreference(PREFERENCE_NAME);
        var view = new PreferenceView(mWeaver, pref);

        dialog.addView(view, pref);
        dialog.setOnSaveListener(
          function () {
            var p = view.getPreference();
            for (key in p) {
              pref.setValue(key, p[key]);
            }
          });
      }

      constructor();
    }

    function DSBenchMenu (weaver) {
      var self = this, mWeaver, mDSBench, mPreference, mTree, mSelectScenario,
      mSetParameters, mExecute, mCancel;

      deosc.Menu.call(self);

      self.onInit = function (data) {
        var type, msg, hash, name, params = {}, proj, scenario;

        msg = data.target.node.getMessage();
        hash = data.target.node.getParameters();
        for (name in hash) {
          params[name] = hash[name].value;
        }
        proj = data.target.node.getDSBenchProject();

        type = data.target.getType();
        if (type == "Strategy") {
          mSelectScenario.show();
        } else {
          mSelectScenario.hide();
        }

        if (type == "Goal") {

          scenario = Scenario.getScenario(data.target.node.getID());
          if (scenario) {
            mSetParameters.hide();
            mExecute.hide();
            if (scenario.status().isRunning) {
              mCancel.show();
            } else {
              mCancel.hide();
            }
          } else if (msg) {
            mSetParameters.hide();
            mExecute.hide();
            mCancel.hide();
          } else {
            mSetParameters.show();
            mExecute.show();
            mCancel.hide();

            if (proj) {
              mExecute.disabled(false);
            } else {
              mExecute.disabled(true);
            }

            if (params && proj) {
              mSetParameters.disabled(false);
            } else {
              mSetParameters.disabled(true);
            }
          }
        } else {
          mSetParameters.hide();
          mExecute.hide();
          mCancel.hide();
        }
      };

      function constructor (weaver) {
        mWeaver = weaver;
        mTree = weaver.getDCaseTreeView();
        mDSBench = new DSBench(weaver);
        mPreference = weaver.getPreferences().getPreference("DS-Bench");

        mSelectScenario
          = new deosc.MenuItem("Select Scenario", onClickSelectScenario);

        mSetParameters
          = new deosc.MenuItem("Set Parameters", onClickSetParameters);

        mExecute = new deosc.MenuItem("Execute", onClickExecute);

        mCancel = new deosc.MenuItem("Cancel", onClickCancel);

        self.addMenuItem(mSelectScenario);
        self.addMenuItem(mSetParameters);
        self.addMenuItem(mExecute);
        self.addMenuItem(mCancel);
      }

      function onClickSelectScenario (data) {
        openScenarioListDialog(mWeaver, data.target);
      }

      function onClickSetParameters (data) {
        openParametersDialog(mWeaver, data.target);
      }

      function onClickExecute (data) {
        executeScenario(mWeaver, data.target.node);
      }

      function onClickCancel (data) {
        stopScenario(data.target.node);
      }

      constructor(weaver);
    }


    function checkPreference (weaver) {
      var preference = weaver.getPreferences().getPreference(PREFERENCE_NAME);
      if (!preference.getValue("name")) return false;
      if (!preference.getValue("url")) return false;
      return true;
    }


    function PreferenceDialog (weaver) {
      var self = this, mWeaver = weaver, mWidget, mPreference, mOnOKListener;

      deosc.Dialog.call(self);

      self.setOnOKListener = function (l) {
        mOnOKListener = l;
      };

      self.onCreate = function () {
        mPreference = mWeaver.getPreferences().getPreference(PREFERENCE_NAME);
        mWidget = new PreferenceView(mWeaver, mPreference);
        self.setContent(mWidget);
        self.setTitle("Preference");
      };

      self.onOK = function () {
        var pref = mWidget.getPreference();
        for (key in pref) {
          mPreference.setValue(key, pref[key]);
        }

        if (mOnOKListener) mOnOKListener.call(self);

        self.close();
      };
    }


    function PreferenceView (weaver, pref) {
      var self = this, mWeaver = weaver, mPreference = pref,
      mNameInput, mUrlInput;

      deosc.View.call(self);

      self.getPreference = function () {
        return {
          name: mNameInput.getText(),
          url: mUrlInput.getText()
        };
      };

      function constructor () {
        var table;

        mNameInput = new deosc.InputText();
        mNameInput.setSize(300);

        mUrlInput = new deosc.InputText();
        mUrlInput.setSize(300);

        table = new deosc.TableView();
        table.addRow(["Name", mNameInput]);
        table.addRow(["URL", mUrlInput]);

        self.addChild(table);

        mNameInput.setText(mPreference.getValue("name", ""));
        mUrlInput.setText(mPreference.getValue("url", ""));
      }

      constructor();
    }


    deosc.DCaseWeaver.ready(
      function (weaver) {
        weaver.addPlugin(new DSBenchPlugin());
      }
    );

  }
) (window);
(
  function (window) {

    function NodeStatusMonitorPlugin (isstart) {
      var self = this, mWeaver, mStartMenuItem, mStopMenuItem, mTimerId,
      mInterval, mUrl, mErrorCount = 0;

      self.setDCaseWeaver = function (weaver) {
        mWeaver = weaver;
      };

      self.onCreate = function () {
        var menuBarItem;

        menuBarItem = mWeaver.getMenuBar().getMenuBarItem("Monitor");
        mStartMenuItem = new deosc.MenuItem("Start", onClickStart);
        mStopMenuItem = new deosc.MenuItem("Stop", onClickStop);
        menuBarItem.addMenuItem(mStartMenuItem);
        menuBarItem.addMenuItem(mStopMenuItem);

        update();

        if (isstart) onClickStart();
      };

      self.onEvent = function (event) {
        if (event.what == deosc.DCaseWeaver.Event.OPEN_PREFERENCE_DIALOG) {
          onOpenPreferenceDialog(event.data);
        }
      };

      function constructor () {
      }

      function update () {
        mStartMenuItem.disabled(mTimerId);
        mStopMenuItem.disabled(!mTimerId);
      }

      function onClickStart () {
        monitor();
      }

      function onClickStop () {
        clearInterval(mTimerId);
        mTimerId = null;
        update();
      }

      function onOpenPreferenceDialog (dialog) {
        var pref = dialog.getPreference("Node Status Monitor");
        var view = new PreferenceView(mWeaver, pref);

        dialog.addView(view, pref);
        dialog.setOnSaveListener(
          function () {
            var p = view.getPreference();
            for (key in p) {
              pref.setValue(key, p[key]);
            }
          });
      }

      function parse (text) {
        var i, lines, status, id, message, ret = {};

        lines = text.split("\n");

        for (i = 0; i < lines.length; i++) {
          id = null, status = "normal", message = "";
          if (lines[i].match(/id\s*=\s*\"([^\"]*)\"/)) {
            id = RegExp.$1;
            if (lines[i].match(/status\s*=\s*\"([^\"]*)\"/)) {
              status = RegExp.$1;
            }
            if (lines[i].match(/message\s*=\s*\"([^\"]*)\"/)) {
              message = RegExp.$1;
            }
            ret[id] = {status: status, message: message};
          }
        }

        return ret;
      }


      function monitor () {
        var pref;

        pref = mWeaver.getPreferences().getPreference("Node Status Monitor");

        mInterval = pref.getValue("interval", 10000);
        mUrl = pref.getValue("url", "");
        mUrl += "/" + mWeaver.getDCase().getID();

        $.ajax(
          {
            url: mUrl,
            success: function (text) {
              var map, i, views, v, index, item, msg;
              mErrorCount = 0;
              views = mWeaver.getDCaseTreeView()
                .getDCaseTreeCanvasView().childViews;
              map = parse(text);
              for (i = 0; i < views.length; i++) {
                v = views[i];
                if (v instanceof deosc.canvas.DCaseNodeView) {
                  if (v.node.getID() in map) {
                    item = map[v.node.getID()];
                    v.setStatus(item.status);
                    v.setMessage(item.message);
                  } else {
                    v.setStatus("normal");
                    v.setMessage(null);
                  }
                }
              }
              mTimerId = setTimeout(monitor, mInterval);
            },
            error: function () {
              mErrorCount++;
              if (mErrorCount < 10) {
                mTimerId = setTimeout(monitor, 10000);
              } else {
                console.log("Node Status Monitor stopped");
                mTimerId = null;
                mErrorCount = 0;
              }
            },
            complete: update
          });
      }

      constructor();
    }

    function PreferenceView (weaver, pref) {
      var self = this, mWeaver = weaver, mPreference = pref,
      mIntervalInput, mUrlInput;

      deosc.View.call(self);

      self.getPreference = function () {
        return {
          url: mUrlInput.getText(),
          interval: mIntervalInput.getText()
        };
      };

      function constructor () {
        var table;

        mUrlInput = new deosc.InputText();
        mUrlInput.setWidth(300);

        mIntervalInput = new deosc.InputText();
        mIntervalInput.setWidth(300);

        table = new deosc.TableView();
        table.addRow(["URL", mUrlInput]);
        table.addRow(["Interval[ms]", mIntervalInput]);

        self.addChild(table);

        mUrlInput.setText(mPreference.getValue("url", ""));
        mIntervalInput.setText(mPreference.getValue("interval", 10000));
      }

      constructor();
    }


    deosc.DCaseWeaver.ready(
      function (weaver) {
        var isstart;

        isstart = weaver.getOptions().node_status_monitor == 'start';

        weaver.addPlugin(new NodeStatusMonitorPlugin(isstart));
      }
    );

  }
) (window);(
  function (window) {
    function sendDCaseChangeEvent (weaver) {
      weaver.sendEvent(
        {
          what: deosc.DCaseWeaver.Event.DCASE_CHANGED,
          data: weaver.getDCase()
        });
    }

    function EditPropertiesCommand (weaver, dcase, description) {
      var self = this, mWeaver, mDCase, mBefore, mAfter;

      self.execute = function () {
        mDCase.setDescription(mAfter);
        sendDCaseChangeEvent(mWeaver);
      };

      self.undo = function () {
        mDCase.setDescription(mBefore);
        sendDCaseChangeEvent(mWeaver);
      };

      function constructor () {
        mWeaver = weaver;
        mDCase = dcase;
        mBefore = mDCase.getDescription();
        mAfter = description;
      }

      constructor();
    }


    function PropertiesPlugin () {
      var self = this, mWeaver, mWidget;

      self.setDCaseWeaver = function (weaver) {
        mWeaver = weaver;
      };

      self.onCreate = function () {
        var menuBarItem;

        menuBarItem = mWeaver.getMenuBar().getMenuBarItem("File");
        menuBarItem.addMenuItem(
          new deosc.MenuItem("Properties", onClickProperties));

        mWidget = new PropertiesWidget(mWeaver);
        mWidget.setOnEditListener(onClickProperties);
        mWeaver.getBody().addChild(mWidget);
      };

      self.onEvent = function (event) {
        if (event.what == deosc.DCaseWeaver.Event.DCASE_CHANGED) {
          onDCaseChanged(event.data);
        }
      };

      function constructor () {
      }

      function onClickProperties () {
        var dcase = mWeaver.getDCase();
        var metadata = mWeaver.getMetadata();
        var dialog = new PropertiesDialog(dcase, metadata);
        dialog.setOnOKListener(
          function () {
            var description = dialog.getDescription();
            if (description != dcase.getDescription()) {
              mWeaver.command(
                new EditPropertiesCommand(mWeaver, dcase, description));
            }
            dialog.close();
          });
        mWeaver.dialogPopup(dialog);
      }

      function onDCaseChanged () {
        mWidget.update();
      }

      constructor();
    }


    function PropertiesWidget (weaver) {
      var self = this, mWeaver, mNameView, mDescView, mEditButton,
      mOnEditListener;

      deosc.View.call(self);

      self.update = function () {
        var dcase, meta, name, desc;

        dcase = mWeaver.getDCase();
        if (!dcase) {
          self.hide();
          return;
        } else {
          self.show();
        }

        desc = dcase.getDescription(true);
        desc = desc.replace("\n","<br>");

        meta = mWeaver.getMetadata();
        name = meta ? meta.name : "Untitled";

        mNameView.setText(name);
        if (desc.length == 0) {
          mDescView.hide();
        } else {
          mDescView.show();
          mDescView.setHtml(desc);
        }
      };

      self.setOnEditListener = function (l) {
        mOnEditListener = l;
      };

      function constructor (weaver) {
        mWeaver = weaver;

        mNameView = new deosc.View();
        mNameView.addStyle("properties-widget-name");

        mDescView = new deosc.View();
        mDescView.addStyle("properties-widget-desc");

        mEditButton = new deosc.View();
        mEditButton.addStyle("properties-widget-edit-button");
        mEditButton.setOnClickListener(onClickEditButton);

        self.addStyle("properties-widget");
        self.addChild(mNameView);
        self.addChild(mDescView);
        self.addChild(mEditButton);

        self.update();
      }

      function onClickEditButton () {
        if (mOnEditListener) mOnEditListener();
      }

      constructor(weaver);
    }

    function PropertiesDialog (dcase, metadata) {
      var self = this, mDCase, mMetadata, mID, mDescription, mFileName,
      mLocation, mCreated, mModified;

      deosc.Dialog.call(self);

      self.getDescription = function () {
        return mDescription;
      };

      self.onCreate = function () {
        var widget, filename, table, desclabel, desc;

        filename = new deosc.View();
        filename.setText(mFileName);
        filename.addStyle("properties-dialog-name");

        table = new deosc.TableView();
        table.addRow(["Location:", mLocation]);
        if (mCreated)
          table.addRow(["Created:", mCreated]);
        table.addRow(["Modified:", mModified]);
        table.addRow(["ID:", mID]);
        table.addStyle("properties-dialog-table");

        desclabel = new deosc.View();
        desclabel.setText("Description:");
        desclabel.addStyle("properties-dialog-desc-label");

        desc = new deosc.Textarea();
        desc.setText(mDescription);
        desc.setOnChangeListener(
          function () {
            mDescription = this.getText();
          });
        desc.addStyle("properties-dialog-desc");

        widget = new deosc.View();
        widget.addChild(filename);
        widget.addChild(table);
        widget.addChild(desclabel);
        widget.addChild(desc);

        self.setContent(widget);
        self.setWidth(400);
        self.setTitle("Properties");
      };

      function constructor (dcase, metadata) {
        mDCase = dcase;
        mID = mDCase.getID();
        mDescription = mDCase.getDescription();

        mFileName = "Untitled";
        mLocation = "---";
        mCreated = null;
        mModified = "---";

        mMetadata = metadata;
        if (mMetadata) {
          mFileName = mMetadata.name;
          mLocation = deosc.path.dirname(mMetadata.path);
          if (mMetadata.crtime)
            mCreated = mMetadata.crtime.toLocaleString();
          mModified = mMetadata.mtime.toLocaleString();
        }
      }

      constructor(dcase, metadata);
    }

    deosc.DCaseWeaver.ready(
      function (weaver) {
        weaver.addPlugin(new PropertiesPlugin());
      });
  }
) (window);
(
  function (window) {
    var mWeaver;

    function compareDCase (target) {
      var nodelist = [], linklist = [], source, serializer, tnodes, tlinks,
      snodes, slinks, snode, tnode, slink, tlink, i, ss, ts, v, tree, parser;

      parser = new DOMParser();
      serializer = new XMLSerializer();

      source = mWeaver.getDCase();

      var s = source.toFormatXML();
      var t = target.toFormatXML();

      source = new deosc.DCase(
        parser.parseFromString(source.toFormatXML(), "application/xml"));

      target = new deosc.DCase(
        parser.parseFromString(target.toFormatXML(), "application/xml"));

      snodes = source.getDCaseNodes();
      tnodes = target.getDCaseNodes();

      for (i = 0; i < snodes.length; i++) {
        snode = snodes[i];
        tnode = target.findDCaseNodeByID(snode.getID());
        if (tnode) {
          ss = serializer.serializeToString(snode.getElement());
          ts = serializer.serializeToString(tnode.getElement());
          if (ss != ts) {
            nodelist.push({id: snode.getID(), status: "modified"});
          }
        } else {
          nodelist.push({id: snode.getID(), status: "created"});
        }
      }

      for (i = 0; i < tnodes.length; i++) {
        tnode = tnodes[i];
        snode = source.findDCaseNodeByID(tnode.getID());
        if (!snode) {
          nodelist.push({id: tnode.getID(), status: "deleted"});
        }
      }


      slinks = source.getDCaseLinks();
      tlinks = target.getDCaseLinks();

      for (i = 0; i < slinks.length; i++) {
        slink = slinks[i];
        tlink = target.findDCaseLinkByID(slink.getID());
        if (tlink) {
          ss = serializer.serializeToString(slink.getElement());
          ts = serializer.serializeToString(tlink.getElement());
          if (ss != ts) {
            linklist.push({id: slink.getID(), status: "modified"});
          }
        } else {
          linklist.push({id: slink.getID(), status: "created"});
        }
      }

      for (i = 0; i < tlinks.length; i++) {
        tlink = tlinks[i];
        slink = source.findDCaseLinkByID(tlink.getID());
        if (!slink) {
          linklist.push({id: tlink.getID(), status: "deleted"});
        }
      }


      tree = mWeaver.getDCaseTreeView().getDCaseTreeCanvasView();

      for (i = 0; i < tree.childViews.length; i++) {
        v = tree.childViews[i];
        if (v instanceof deosc.canvas.DCaseNodeView) {
          v.setBorderColor("#000");
        } else if (v instanceof deosc.canvas.DCaseLinkView) {
          v.setColor("#000");
        }
      }

      for (i = 0; i < nodelist.length; i++) {
        v = tree.findDCaseNodeViewByID(nodelist[i].id);
        if (nodelist[i].status == "created") {
          v.setBorderColor("#00f");
        } else if (nodelist[i].status == "modified") {
          v.setBorderColor("#f00");
        }
      }

      for (i = 0; i < linklist.length; i++) {
        v = tree.findDCaseLinkViewByID(linklist[i].id);
        if (linklist[i].status == "created") {
          v.setColor("#00f");
        } else if (linklist[i].status == "modified") {
          v.setColor("#f00");
        }
      }
    }

    deosc.DCaseWeaver.ready(
      function (weaver) {
        mWeaver = weaver;
      });

    window.deosc.compareDCase = compareDCase;
  }
) (window);(
  function (window) {
    function DropDown () {
      var self = this, mItems = [];

      deosc.View.call(self, $("<select>"));

      self.addItem = function (item) {
        var val;
        val = mItems.length;
        $(item.getElement()).val(val);
        self.addChild(item);
        mItems.push(item);
      };

      self.clearItems = function () {
        self.empty();
        mItems.length = 0;
      };

      self.getItem = function () {
        var val;
        val = $(self.getElement()).val();
        return mItems[val];
      };

      self.getLabel = function () {
        var item = self.getItem();
        return item ? item.getLabel() : null;
      };

      self.getData = function () {
        var item = self.getItem();
        return item ? item.getData() : null;
      };
    }

    function DropDownItem (label, data) {
      var self = this, mData;

      deosc.View.call(self, $("<option>"));

      self.setData = function (data) {
        mData = data;
      };

      self.getData = function () {
        return mData;
      };

      self.setLabel = function (label) {
        self.setText(label);
      };

      self.getLabel = function () {
        return self.getText();
      };

      function constructor (label, data) {
        if (label) self.setLabel(label);
        if (data) self.setData(data);
      }

      constructor(label, data);
    }

    function SearchConditionDialog () {
      var self = this, mWidget, mControllers = [];

      deosc.Dialog.call(self);

      self.getConditionAt = function (i) {
        return mWidget.getConditionAt(i);
      };

      self.getConditionCount = function () {
        return mWidget.getConditionCount();
      };

      self.addConditionController = function (controller) {
        mControllers.push(controller);
        if (mWidget) mWidget.addController(controller);
      };

      self.setOnSearchListener = function (l) {
        self.setOnOKListener(l);
      };

      self.onCreate = function () {
        var i;

        mWidget = new SearchConditionWidget();
        for (i = 0; i < mControllers.length; i++) {
          mWidget.addController(mControllers[i]);
        }

        self.setContent(mWidget);
        self.setTitle("Search D-Case");

        self.getFooter().getChildAt(0).setText("Search");
      };

      function constructor () {
      }

      constructor();
    }

    function SearchConditionWidget () {
      var self = this, mConditions, mAttrDD, mKeywordInput, mOperatorDD,
      mListModel, mListView, mController;

      deosc.View.call(self);

      self.getConditionAt = function (i) {
        var item = mListModel.getItemAt(i);
        return item ? item.getData() : null;
      };

      self.getConditionCount = function () {
        return mListModel.getItemCount();
      };

      self.addController = function (controller) {
        mAttrDD.addItem(new DropDownItem(controller.getName(), controller));
        if (mAttrDD.getChildCount() == 1) {
          onChangeAttribute();
        }
      };

      function onChangeAttribute () {
        var operators, i;

        if (mController) mController.onDetach();

        mOperatorDD.clearItems();

        mController = mAttrDD.getItem().getData();
        mController.onAttach(mKeywordInput, mOperatorDD);
      }

      function constructor () {
        var label1, op_eq, op_ne, op_le, op_ge, add, header;

        mConditions = [];

        label1 = new deosc.View();
        label1.setMargin("4px 0");
        label1.setText("Search keyword");

        mAttrDD = new DropDown();
        mAttrDD.setOnChangeListener(onChangeAttribute);
        mAttrDD.setMargin("8px 4px 8px 0");

        mKeywordInput = new deosc.InputText();
        mKeywordInput.setMargin("8px 4px");

        mOperatorDD = new DropDown();
        mOperatorDD.setWidth(40);
        mOperatorDD.setMargin("8px 4px");

        add = new deosc.View($("<button>"));
        add.setText("Add");
        add.setWidth("4em");
        add.setMargin("8px 0 8px 4px");
        add.setOnClickListener(onAdd);


        header = new deosc.ListItem();
        header.addColumns(["Attribute", "Keyword", "Opearator"]);

        mListModel = new deosc.ListModel();
        mListModel.setHeaderItem(header);

        mListView = new deosc.ListView();
        mListView.setHeight(200);
        mListView.setModel(mListModel);

        self.addChild(label1);
        self.addChild(mAttrDD);
        self.addChild(mKeywordInput);
        self.addChild(mOperatorDD);
        self.addChild(add);
        self.addChild(mListView);
      }

      function onAdd () {
        var condition, item, ts, date;

        if (!mController && !mController.onAdd()) return;

        item = mController.generateListItem();

        mListModel.add(item);
        mListView.update();

        mKeywordInput.setText("");
      }

      constructor();
    }


    function ConditionController (name, operators) {
      var self = this, mName, mOperators, mKeywordView, mOperatorView,
      mOnInitListener, mOnChangeListener, mOnAddListener,
      mOnGenerateListItemListener;

      self.setName = function (name) {
        mName = name;
      };

      self.getName = function () {
        return mName;
      };

      self.setOperators = function (operators) {
        mOperators = operators;
      };

      self.getOperators = function () {
        return mOperators;
      };

      self.setValue = function (value) {
        if (mView) mView.setText(value);
      };

      self.getValue = function () {
        return mView ? mView.getText() : null;
      };

      self.setOnInitListener = function (l) {
        mOnInitListener = l;
      };

      self.setOnAddListener = function (l) {
        mOnAddListener = l;
      };

      self.setOnChangeListener = function (l) {
        mOnChangeListener = l;
      };

      self.setOnGenerateListItemListener = function (l) {
        mOnGenerateListItemListener = l;
      };

      self.generateListItem = function () {
        var item, condition;

        condition = {
          "attr": mName,
          "keyword": mKeywordView.getText(),
          "operator": mOperatorView.getLabel()
        };

        if (mOnGenerateListItemListener) {
          item = mOnGenerateListItemListener(condition);
        } else {
          item = new deosc.ListItem();
          item.addColumn(condition.attr);
          item.addColumn(condition.keyword);
          item.addColumn(condition.operator);
          item.setData(condition);
        }

        return item;
      };

      self.onAttach  = function (keywordView, operatorView) {
        var i;

        mKeywordView = keywordView;
        mOperatorView = operatorView;

        for (i = 0; i < mOperators.length; i++) {
          mOperatorView.addItem(new DropDownItem(mOperators[i]));
        }
        mOperatorView.disabled(mOperators.length < 2);

        mKeywordView.setOnChangeListener(self.onChange);

        if (mOnInitListener) mOnInitListener.call(self);
      };

      self.onChange = function () {
        if (mOnChangeListener) mOnChangeListener.call(self);
      };

      self.onAdd = function () {
        if (mOnAddListener) {
          return mOnAddListener.call(self);
        } else {
          return 0 < mView.getText().length;
        }
      };

      self.onDetach = function () {
        mKeywordView = null;
        mOperatorView = null;
      };

      function constructor (name, operators) {
        mName = name;
        mOperators = operators ? operators : ["=", "<>", ">=", "<="];
      }

      constructor (name, operators);
    }


    window.deosc.SearchConditionDialog = SearchConditionDialog;
    window.deosc.SearchConditionWidget = SearchConditionWidget;
    window.deosc.ConditionController = ConditionController;
  }
) (window);
(
  function (window) {
    var mWeaver;

    function ApprovalDialog () {
      var self = this, mWidget, mPath, mComment, mEditApprovalMemberWidget,
      mMetadata, mOnApproveListener;

      deosc.Dialog.call(self);

      self.setOnApproveListener = function (l) {
        mOnApproveListener = l;
      };


      self.getMetadata = function () {
        return mMetadata;
      };

      self.getComment = function () {
        var i, count, member, comment;

        comment = "Approve the version. ";

        comment += new Date().toLocaleString() + '\n';

        count = mEditApprovalMemberWidget.getApprovalMemberCount();
        for (i = 0; i < count; i++) {
          member = mEditApprovalMemberWidget.getApprovalMemberAt(i);
          comment += member + '\n';
        }
        comment += mComment.getText();

        return comment;
      };

      self.onCreate = function () {
        var label1, label, textInput, chooseButton, box;

        mWidget = new deosc.View();

        label1 = new deosc.View();
        label1.setText("D-Case");
        label1.setMargin("8px 0 4px 0");

        mPath = new deosc.InputText();
        mPath.readonly(true);
        mPath.setWidth("calc(100% - 90px)");

        chooseButton = new deosc.View($("<button>"));
        chooseButton.setText("Choose");
        chooseButton.setWidth(80);
        chooseButton.setOnClickListener(onClickChoose);

        box = new deosc.View();
        box.addChild(mPath);
        box.addChild(chooseButton);

        label = new deosc.View();
        label.setText("Comment");
        label.setMargin("8px 0 4px 0");

        mComment = new deosc.Textarea();
        mComment.setSize("calc(100% - 6px)", 50);
        mComment.setMargin("0 0 8px 0");

        mEditApprovalMemberWidget = new EditApprovalMemberWidget();

        mWidget.addChild(label1);
        mWidget.addChild(box);
        mWidget.addChild(label);
        mWidget.addChild(mComment);
        mWidget.addChild(mEditApprovalMemberWidget);

        self.setTitle("Approval D-Case");
        self.setContent(mWidget);

        self.getFooter().getChildAt(0).setText("Approve");
      };

      self.onOK = function () {
        if (check()) {
          mEditApprovalMemberWidget.apply();
          if (mOnApproveListener) mOnApproveListener();
          self.close();
        }
      };

      function onClickChoose () {
        mWeaver.getFileManager().browse(
          {
            filter: function (metadata) {
              return (metadata.isdir || metadata.name.match(/\.dcase$/));
            },
            success: onBrowse
          });
      }

      function onBrowse (dialog) {
        dialog.setOnOKListener(
          function (metadata) {
            if (metadata.isdir) {
              alert("Please select a file.");
            } else {
              mMetadata = metadata;
              mPath.setText(mMetadata.path);
              dialog.close();
            }
          });
      }

      function check () {
        var comment;

        if (!mMetadata) {
          alert("Please choose a D-Case");
          return false;
        }

        comment = mComment.getText();
        if (!comment || comment.length == 0) {
          alert("Please enter a comment.");
          return false;
        }

        return true;
      }
    }

    function EditApprovalMemberWidget () {
      var self = this, mPreference, mMemberInput,
      mMemberListView, mMemberListModel,
      mApprovalMemberListView, mApprovalMemberListModel, mNewMembers = [];

      deosc.View.call(self);

      self.getApprovalMemberCount = function () {
        return mApprovalMemberListModel.getItemCount();
      };

      self.getApprovalMemberAt = function (i) {
        var item = mApprovalMemberListModel.getItemAt(i);
        return item ? item.getData() : null;
      };

      self.getNewMembers = function () {
        return mNewMembers;
      };

      self.apply = function () {
        var members = mPreference.getValue("members", []);
        mPreference.setValue("members", members.concat(mNewMembers));
      };

      function constructor () {
        var box1, box2, label1, label2, button1, button2, button3, table,
        members, i;

        mPreference = mWeaver.getPreferences().getPreference("Alfresco");

        label1 = new deosc.View();
        label1.setWidth(70);
        $(label1.getElement()).css("line-height", "32px");
        label1.setText("Member");

        label2 = new deosc.View();
        label2.setText("Approver member");

        button1 = new deosc.View($("<button>"));
        button1.setWidth(56);
        button1.setText("Add");
        button1.setOnClickListener(onAddMember);

        button2 = new deosc.View($("<button>"));
        button2.setText(">>");
        button2.setOnClickListener(onAddApprovalMember);

        button3 = new deosc.View($("<button>"));
        button3.setText("<<");
        button3.setOnClickListener(onRemoveApprovalMember);

        mMemberInput = new deosc.InputText();
        mMemberInput.setWidth(106);

        mMemberListModel = new deosc.ListModel();

        mMemberListView = new deosc.ListView();
        mMemberListView.setSize(240, 100);
        mMemberListView.setModel(mMemberListModel);

        mApprovalMemberListModel = new deosc.ListModel();

        mApprovalMemberListView = new deosc.ListView();
        mApprovalMemberListView.setSize(240, 100);
        mApprovalMemberListView.setModel(mApprovalMemberListModel);

        box1 = new deosc.View();
        box1.addStyle("hbox");
        box1.addChild(label1);
        box1.addChild(mMemberInput);
        box1.addChild(button1);
        $(box1.getElement()).children().css("vertical-align", "middle");

        box2 = new deosc.View();
        box2.addStyle("vbox");
        box2.addChild(button2);
        box2.addChild(button3);

        table = new deosc.TableView();
        table.addRow([box1, "", label2]);
        table.addRow([mMemberListView, box2, mApprovalMemberListView]);

        self.addChild(table);

        members = mPreference.getValue("members", []);
        for (i = 0; i < members.length; i++) {
          mMemberListModel.add(createItem(members[i]));
        }

        mMemberListView.update();
      }

      function onAddMember () {
        var member = mMemberInput.getText();
        if (!member) return;

        mMemberInput.setText("");

        mNewMembers.push(member);

        mMemberListModel.add(createItem(member));
        mMemberListView.update();
      }

      function onRemoveApprovalMember () {
        var item = mApprovalMemberListView.getCurrentItem();
        if (!item) return;

        mApprovalMemberListView.setCurrent(-1);

        mApprovalMemberListModel.removeItem(item);
        mMemberListModel.add(item);

        mMemberListView.update();
        mApprovalMemberListView.update();
      }

      function onAddApprovalMember () {
        var item = mMemberListView.getCurrentItem();
        if (!item) return;

        mMemberListView.setCurrent(-1);

        mMemberListModel.removeItem(item);
        mApprovalMemberListModel.add(item);

        mMemberListView.update();
        mApprovalMemberListView.update();
      }

      constructor();
    }


    function EditMemberWidget () {
      var self = this, mMemberInput, mPreference,
      mMemberListView, mMemberListModel;

      deosc.View.call(self);

      self.getMembers = function () {
        var list = [];
        mMemberListModel.foreach(
          function (index) {
            list.push(this.getData());
          });
        return list;
      };

      self.apply = function () {
        mPreference.setValue("members", self.getMembers());
      };

      function constructor () {
        var table, button1, button2, members, i;

        mPreference = mWeaver.getPreferences().getPreference("Alfresco");

        button1 = new deosc.View($("<button>"));
        button1.setWidth(60);
        button1.setText("Add");
        button1.setOnClickListener(onAdd);

        button2 = new deosc.View($("<button>"));
        button2.setWidth(60);
        button2.setText("Delete");
        button2.setOnClickListener(onDelete);

        mMemberInput = new deosc.InputText();
        mMemberInput.setSize(300);

        mMemberListModel = new deosc.ListModel();

        mMemberListView = new deosc.ListView();
        mMemberListView.setSize(300, 100);
        mMemberListView.setModel(mMemberListModel);

        table = new deosc.TableView();
        table.addRow([mMemberInput, button1]);
        table.addRow([mMemberListView, button2]);

        $(table.getElement()).find("td").css("padding", 4);
        $($(table.getElement()).find("td")[3]).css("vertical-align", "top");

        self.addChild(table);

        members = mPreference.getValue("members", []);
        for (i = 0; i < members.length; i++) {
          mMemberListModel.add(createItem(members[i]));
        }

        mMemberListView.update();
      }

      function onAdd () {
        var member = mMemberInput.getText();
        mMemberInput.setText("");
        if (!member) return;

        mMemberListModel.add(createItem(member));
        mMemberListView.update();
      }

      function onDelete () {
        mMemberListModel.removeItem(mMemberListView.getCurrentItem());
        mMemberListView.setCurrent(-1);
        mMemberListView.update();
      }

      constructor();
    }


    function createItem (member) {
      var item = new deosc.ListItem();
      item.addColumn(member);
      item.setData(member);
      return item;
    }


    deosc.DCaseWeaver.ready(
      function (weaver) {
        mWeaver = weaver;
      }
    );

    window.deosc.ApprovalDialog = ApprovalDialog;
    window.deosc.EditMemberWidget = EditMemberWidget;
  }
) (window);
(
  function (window) {
    var mWeaver;

    function SearchNodePlugin () {
      var self = this, mSearch;

      self.setDCaseWeaver = function (weaver) {
        mWeaver = weaver;
      };

      self.onCreate = function () {
        mSearch = new SearchToolBarItem();
        mSearch.setID("search");
        mSearch.setOnChangeListener(
          function () {
            var i, ret = [], re, tree;

            tree = mWeaver.getDCaseTreeView();

            re = new RegExp(mSearch.getStatus().input, "i");

            $(tree.getDCaseTreeCanvasView().childViews)
              .each(
                function () {
                  if ((this instanceof deosc.canvas.DCaseNodeView)
                    && re.test(this.node.getName())) {
                    ret.push(this);
                  }
                });

            ret.sort(
              function (a, b) {
                return a.node.getName() > b.node.getName();
              });

            mSearch.clearResultList();

            for (i = 0; i < ret.length; i++) {
              mSearch.addResult(
                ret[i].node.getName(),
                function (nodeView) {
                  return function () {
                    tree.moveTo(nodeView);
                  };
                } (ret[i])
              );
            }
          });

        mWeaver.getToolBar().addItem(mSearch);
      };
    }


    function SearchToolBarItem () {
      var self = this, mLabel, mInput, mResultList, mOnChangeListener;

      deosc.ToolBarItem.call(self);

      self.getStatus = function () {
        return {input: mInput.getText()};
      };

      self.setOnChangeListener = function (l) {
        mOnChangeListener = l;
      };

      self.addResult = function (description, callback) {
        var item = new deosc.View();

        function onClick () {
          self.clear();
          if (callback) callback();
        }

        item.setText(description);
        item.setOnClickListener(onClick);
        $(item.getElement())
            .css({"font-size": 14, padding: 4})
            .hover(
              function () {
                $(this).css({"background-color": "#fC8"});
              },
              function () {
                $(this).css({"background-color": ""});
              });

        mResultList.addChild(item);
        mResultList.show();
      };

      self.clear = function () {
        mInput.setText("");
        self.clearResultList();
      };

      self.clearResultList = function () {
        mResultList.empty();
        mResultList.hide();
      };

      function constructor () {
        mLabel = new deosc.View();
        mLabel.setText("Search");
        $(mLabel.getElement()).css(
          {
            display: "inline-block",
            "font-size": 14,
            "margin-right": 8
          });
        self.addChild(mLabel);

        mInput = new deosc.InputText();
        $(mInput.getElement()).keyup(
          function (event) {
              if (event.keyCode == 13) {
              } else {
                onChange();
              }
          });
        self.addChild(mInput);

        mResultList = new deosc.View();
        $(mResultList.getElement()).css(
          {
            position: "absolute",
            "max-height": 320,
            overflow: "auto",
            "background-color": "#eee",
            "border-left": "1px solid #fff",
            "border-top": "1px solid #fff",
            "border-right": "1px solid #888",
            "border-bottom": "1px solid #888",
            "z-index": 2
          });
        mResultList.hide();
        self.addChild(mResultList);
      }

      function onChange () {
        if (mInput.getText().length == 0){
          mResultList.hide();
          return;
        }

        $(mResultList.getElement()).css(
          {
            left: $(mInput.getElement()).position().left,
            width: $(mInput.getElement()).width()
          });

        if (mOnChangeListener) mOnChangeListener();
      }

      constructor();
    }


    deosc.DCaseWeaver.ready(
      function (weaver) {
        weaver.addPlugin(new SearchNodePlugin());
      });
  }
) (window);
(
  function (window) {
    var mWeaver;

    function StatisticsView () {
      var self = this, mTable, mInfo;

      deosc.View.call(self);

      self.update = function () {
        var dcase;
        var table = mTable.getElement(), i, nodes, type, info = {};
        $(table).empty();

        dcase = mWeaver.getDCase();
        if (!dcase) return;

        nodes = dcase.getDCaseNodes();

        for (i = 0; i < nodes.length; i++) {
          type = nodes[i].getType();
          if (type in info) {
            info[type]++;
          } else {
            info[type] = 1;
          }
        }

        for (type in info) {
          $("<tr>")
            .append($("<td>").text(type))
            .append($("<td>").text(info[type]))
            .appendTo(table);
        }

        mInfo = info;
      };

      self.getInfo = function () {
        return mInfo;
      };

      function constructor () {
        mInfo = {};

        self.addStyle("info-panel");

        mTable = new deosc.View($("<table>")[0]);
        self.addChild(mTable);
      }

      constructor();
    }

    function StatisticsViewPrototype () {}
    StatisticsViewPrototype.prototype = deosc.View.prototype;
    StatisticsView.prototype = new StatisticsViewPrototype;
    StatisticsView.constructor = StatisticsView;


    function StatisticsToolBarItem () {
      var self = this, mLabel, mCheckbox, mOnChangeListener;

      deosc.ToolBarItem.call(self);

      self.setOnChangeListener = function (l) {
        mOnChangeListener = l;
      };

      self.isChecked = function () {
        return mCheckbox.isChecked();
      };

      function constructor () {
        self.addStyle("hbox");

        mLabel = new deosc.View();
        mLabel.setText("INFO");
        $(mLabel.getElement())
          .css(
            {
              display: "inline-block",
              "margin-right": 8,
              "vertical-align": "middle"
            });
        self.addChild(mLabel);

        mCheckbox = new deosc.View($("<input type='checkbox'>")[0]);
        mCheckbox.checked(true);
        $(mCheckbox.getElement())
          .css({"vertical-align": "middle"})
          .change(onChange);
        self.addChild(mCheckbox);
      }

      function onChange () {
        if (mOnChangeListener) mOnChangeListener();
      }

      constructor();
    }

    function StatisticsToolBarItemPrototype () {}
    StatisticsToolBarItemPrototype.prototype = deosc.ToolBarItem.prototype;
    StatisticsToolBarItem.prototype = new StatisticsToolBarItemPrototype;
    StatisticsToolBarItem.constructor = StatisticsToolBarItem;


    function StatisticsPlugin () {
      var self = this, mStatisticsView, mToolBarItem;

      self.setDCaseWeaver = function (weaver) {
        mWeaver = weaver;
      };

      self.onCreate = function () {
        mStatisticsView = new StatisticsView();

        mToolBarItem = new StatisticsToolBarItem();
        mToolBarItem.setOnChangeListener(onChange);

        mWeaver.getBody().addChild(mStatisticsView);
        mWeaver.getToolBar().addItem(mToolBarItem);

        update();
      };

      self.onEvent = function (event) {
        if (event.what == deosc.DCaseWeaver.Event.DCASE_CHANGED) {
          onChange(event.data);
        }
      };

      function constructor () {
      }

      function update () {
        if (mStatisticsView) {
          mStatisticsView.update();
          var info = mStatisticsView.getInfo();
          if (mToolBarItem.isChecked()) {
            if (Object.keys(info).length == 0) {
              mStatisticsView.hide();
            } else {
              mStatisticsView.show();
            }
          } else {
            mStatisticsView.hide();
          }
        }
      }

      function onChange (dcase) {
        update();
      };

      constructor();
    }

    deosc.DCaseWeaver.ready(
      function (weaver) {
        weaver.addPlugin(new StatisticsPlugin());
      });
  }
) (window);
